// The Module object: Our interface to the outside world. We import
// and export values on it, and do the work to get that through
// closure compiler if necessary. There are various ways Module can be used:
// 1. Not defined. We create it here
// 2. A function parameter, function(Module) { ..generated code.. }
// 3. pre-run appended it, var Module = {}; ..generated code..
// 4. External script tag defines var Module.
// We need to do an eval in order to handle the closure compiler
// case, where this code here is minified but Module was defined
// elsewhere (e.g. case 4 above). We also need to check if Module
// already exists (e.g. case 3 above).
// Note that if you want to run closure, and also to use Module
// after the generated code, you will need to define   var Module = {};
// before the code. Then that object will be used in the code, and you
// can continue to use Module afterwards as well.
var Module;
if (!Module) Module = (typeof Module !== 'undefined' ? Module : null) || {};

// Sometimes an existing Module object exists with properties
// meant to overwrite the default module functionality. Here
// we collect those properties and reapply _after_ we configure
// the current environment's defaults to avoid having to be so
// defensive during initialization.
var moduleOverrides = {};
for (var key in Module) {
  if (Module.hasOwnProperty(key)) {
    moduleOverrides[key] = Module[key];
  }
}

// The environment setup code below is customized to use Module.
// *** Environment setup code ***
var ENVIRONMENT_IS_NODE = typeof process === 'object' && typeof require === 'function';
var ENVIRONMENT_IS_WEB = typeof window === 'object';
// Three configurations we can be running in:
// 1) We could be the application main() thread running in the main JS UI thread. (ENVIRONMENT_IS_WORKER == false and ENVIRONMENT_IS_PTHREAD == false)
// 2) We could be the application main() thread proxied to worker. (with Emscripten -s PROXY_TO_WORKER=1) (ENVIRONMENT_IS_WORKER == true, ENVIRONMENT_IS_PTHREAD == false)
// 3) We could be an application pthread running in a worker. (ENVIRONMENT_IS_WORKER == true and ENVIRONMENT_IS_PTHREAD == true)
var ENVIRONMENT_IS_WORKER = typeof importScripts === 'function';
if (typeof ENVIRONMENT_IS_PTHREAD === 'undefined') ENVIRONMENT_IS_PTHREAD = false; // ENVIRONMENT_IS_PTHREAD=true will have been preset in pthread-main.js.
var ENVIRONMENT_IS_SHELL = !ENVIRONMENT_IS_WEB && !ENVIRONMENT_IS_NODE && !ENVIRONMENT_IS_WORKER;

if (ENVIRONMENT_IS_NODE) {
  // Expose functionality in the same simple way that the shells work
  // Note that we pollute the global namespace here, otherwise we break in node
  if (!Module['print']) Module['print'] = function print(x) {
    process['stdout'].write(x + '\n');
  };
  if (!Module['printErr']) Module['printErr'] = function printErr(x) {
    process['stderr'].write(x + '\n');
  };

  var nodeFS = require('fs');
  var nodePath = require('path');

  Module['read'] = function read(filename, binary) {
    filename = nodePath['normalize'](filename);
    var ret = nodeFS['readFileSync'](filename);
    // The path is absolute if the normalized version is the same as the resolved.
    if (!ret && filename != nodePath['resolve'](filename)) {
      filename = path.join(__dirname, '..', 'src', filename);
      ret = nodeFS['readFileSync'](filename);
    }
    if (ret && !binary) ret = ret.toString();
    return ret;
  };

  Module['readBinary'] = function readBinary(filename) { return Module['read'](filename, true) };

  Module['load'] = function load(f) {
    globalEval(read(f));
  };

  if (process['argv'].length > 1) {
    Module['thisProgram'] = process['argv'][1].replace(/\\/g, '/');
  } else {
    Module['thisProgram'] = 'unknown-program';
  }

  Module['arguments'] = process['argv'].slice(2);

  if (typeof module !== 'undefined') {
    module['exports'] = Module;
  }

  process['on']('uncaughtException', function(ex) {
    // suppress ExitStatus exceptions from showing an error
    if (!(ex instanceof ExitStatus)) {
      throw ex;
    }
  });
}
else if (ENVIRONMENT_IS_SHELL) {
  if (!Module['print']) Module['print'] = print;
  if (typeof printErr != 'undefined') Module['printErr'] = printErr; // not present in v8 or older sm

  if (typeof read != 'undefined') {
    Module['read'] = read;
  } else {
    Module['read'] = function read() { throw 'no read() available (jsc?)' };
  }

  Module['readBinary'] = function readBinary(f) {
    if (typeof readbuffer === 'function') {
      return new Uint8Array(readbuffer(f));
    }
    var data = read(f, 'binary');
    assert(typeof data === 'object');
    return data;
  };

  if (typeof scriptArgs != 'undefined') {
    Module['arguments'] = scriptArgs;
  } else if (typeof arguments != 'undefined') {
    Module['arguments'] = arguments;
  }

}
else if (ENVIRONMENT_IS_WEB || ENVIRONMENT_IS_WORKER) {
  Module['read'] = function read(url) {
    var xhr = new XMLHttpRequest();
    xhr.open('GET', url, false);
    xhr.send(null);
    return xhr.responseText;
  };

  if (typeof arguments != 'undefined') {
    Module['arguments'] = arguments;
  }

  if (typeof console !== 'undefined') {
    if (!Module['print']) Module['print'] = function print(x) {
      console.log(x);
    };
    if (!Module['printErr']) Module['printErr'] = function printErr(x) {
      console.log(x);
    };
  } else {
    // Probably a worker, and without console.log. We can do very little here...
    var TRY_USE_DUMP = false;
    if (!Module['print']) Module['print'] = (TRY_USE_DUMP && (typeof(dump) !== "undefined") ? (function(x) {
      dump(x);
    }) : (function(x) {
      // self.postMessage(x); // enable this if you want stdout to be sent as messages
    }));
  }

  if (ENVIRONMENT_IS_WORKER) {
    Module['load'] = importScripts;
  }
}
else {
  // Unreachable because SHELL is dependant on the others
  throw 'Unknown runtime environment. Where are we?';
}

function globalEval(x) {
  eval.call(null, x);
}
if (!Module['load'] && Module['read']) {
  Module['load'] = function load(f) {
    globalEval(Module['read'](f));
  };
}
if (!Module['print']) {
  Module['print'] = function(){};
}
if (!Module['printErr']) {
  Module['printErr'] = Module['print'];
}
if (!Module['arguments']) {
  Module['arguments'] = [];
}
if (!Module['thisProgram']) {
  Module['thisProgram'] = './this.program';
}

// *** Environment setup code ***

// Closure helpers
Module.print = Module['print'];
Module.printErr = Module['printErr'];

// Callbacks
Module['preRun'] = [];
Module['postRun'] = [];

// Merge back in the overrides
for (var key in moduleOverrides) {
  if (moduleOverrides.hasOwnProperty(key)) {
    Module[key] = moduleOverrides[key];
  }
}



// === Preamble library stuff ===

// Documentation for the public APIs defined in this file must be updated in: 
//    site/source/docs/api_reference/preamble.js.rst
// A prebuilt local version of the documentation is available at: 
//    site/build/text/docs/api_reference/preamble.js.txt
// You can also build docs locally as HTML or other formats in site/
// An online HTML version (which may be of a different version of Emscripten)
//    is up at http://kripken.github.io/emscripten-site/docs/api_reference/preamble.js.html

//========================================
// Runtime code shared with compiler
//========================================

var Runtime = {
  setTempRet0: function (value) {
    tempRet0 = value;
  },
  getTempRet0: function () {
    return tempRet0;
  },
  stackSave: function () {
    return STACKTOP;
  },
  stackRestore: function (stackTop) {
    STACKTOP = stackTop;
  },
  getNativeTypeSize: function (type) {
    switch (type) {
      case 'i1': case 'i8': return 1;
      case 'i16': return 2;
      case 'i32': return 4;
      case 'i64': return 8;
      case 'float': return 4;
      case 'double': return 8;
      default: {
        if (type[type.length-1] === '*') {
          return Runtime.QUANTUM_SIZE; // A pointer
        } else if (type[0] === 'i') {
          var bits = parseInt(type.substr(1));
          assert(bits % 8 === 0);
          return bits/8;
        } else {
          return 0;
        }
      }
    }
  },
  getNativeFieldSize: function (type) {
    return Math.max(Runtime.getNativeTypeSize(type), Runtime.QUANTUM_SIZE);
  },
  STACK_ALIGN: 16,
  getAlignSize: function (type, size, vararg) {
    // we align i64s and doubles on 64-bit boundaries, unlike x86
    if (!vararg && (type == 'i64' || type == 'double')) return 8;
    if (!type) return Math.min(size, 8); // align structures internally to 64 bits
    return Math.min(size || (type ? Runtime.getNativeFieldSize(type) : 0), Runtime.QUANTUM_SIZE);
  },
  dynCall: function (sig, ptr, args) {
    if (args && args.length) {
      assert(args.length == sig.length-1);
      if (!args.splice) args = Array.prototype.slice.call(args);
      args.splice(0, 0, ptr);
      assert(('dynCall_' + sig) in Module, 'bad function pointer type - no table for sig \'' + sig + '\'');
      return Module['dynCall_' + sig].apply(null, args);
    } else {
      assert(sig.length == 1);
      assert(('dynCall_' + sig) in Module, 'bad function pointer type - no table for sig \'' + sig + '\'');
      return Module['dynCall_' + sig].call(null, ptr);
    }
  },
  functionPointers: [],
  addFunction: function (func) {
    for (var i = 0; i < Runtime.functionPointers.length; i++) {
      if (!Runtime.functionPointers[i]) {
        Runtime.functionPointers[i] = func;
        return 2*(1 + i);
      }
    }
    throw 'Finished up all reserved function pointers. Use a higher value for RESERVED_FUNCTION_POINTERS.';
  },
  removeFunction: function (index) {
    Runtime.functionPointers[(index-2)/2] = null;
  },
  getAsmConst: function (code, numArgs) {
    // code is a constant string on the heap, so we can cache these
    if (!Runtime.asmConstCache) Runtime.asmConstCache = {};
    var func = Runtime.asmConstCache[code];
    if (func) return func;
    var args = [];
    for (var i = 0; i < numArgs; i++) {
      args.push(String.fromCharCode(36) + i); // $0, $1 etc
    }
    var source = Pointer_stringify(code);
    if (source[0] === '"') {
      // tolerate EM_ASM("..code..") even though EM_ASM(..code..) is correct
      if (source.indexOf('"', 1) === source.length-1) {
        source = source.substr(1, source.length-2);
      } else {
        // something invalid happened, e.g. EM_ASM("..code($0)..", input)
        abort('invalid EM_ASM input |' + source + '|. Please use EM_ASM(..code..) (no quotes) or EM_ASM({ ..code($0).. }, input) (to input values)');
      }
    }
    try {
      // Module is the only 'upvar', which we provide directly. We also provide FS for legacy support.
      var evalled = eval('(function(Module, FS) { return function(' + args.join(',') + '){ ' + source + ' } })')(Module, typeof FS !== 'undefined' ? FS : null);
    } catch(e) {
      Module.printErr('error in executing inline EM_ASM code: ' + e + ' on: \n\n' + source + '\n\nwith args |' + args + '| (make sure to use the right one out of EM_ASM, EM_ASM_ARGS, etc.)');
      throw e;
    }
    return Runtime.asmConstCache[code] = evalled;
  },
  warnOnce: function (text) {
    if (!Runtime.warnOnce.shown) Runtime.warnOnce.shown = {};
    if (!Runtime.warnOnce.shown[text]) {
      Runtime.warnOnce.shown[text] = 1;
      Module.printErr(text);
    }
  },
  funcWrappers: {},
  getFuncWrapper: function (func, sig) {
    assert(sig);
    if (!Runtime.funcWrappers[sig]) {
      Runtime.funcWrappers[sig] = {};
    }
    var sigCache = Runtime.funcWrappers[sig];
    if (!sigCache[func]) {
      sigCache[func] = function dynCall_wrapper() {
        return Runtime.dynCall(sig, func, arguments);
      };
    }
    return sigCache[func];
  },
  getCompilerSetting: function (name) {
    throw 'You must build with -s RETAIN_COMPILER_SETTINGS=1 for Runtime.getCompilerSetting or emscripten_get_compiler_setting to work';
  },
  stackAlloc: function (size) { var ret = STACKTOP;STACKTOP = (STACKTOP + size)|0;STACKTOP = (((STACKTOP)+15)&-16);(assert((((STACKTOP|0) < (STACK_MAX|0))|0))|0); return ret; },
  staticAlloc: function (size) { var ret = STATICTOP;STATICTOP = (STATICTOP + (assert(!staticSealed),size))|0;STATICTOP = (((STATICTOP)+15)&-16); return ret; },
  dynamicAlloc: function (size) { var ret = DYNAMICTOP;DYNAMICTOP = (DYNAMICTOP + (assert(DYNAMICTOP > 0),size))|0;DYNAMICTOP = (((DYNAMICTOP)+15)&-16); if (DYNAMICTOP >= TOTAL_MEMORY) enlargeMemory();; return ret; },
  alignMemory: function (size,quantum) { var ret = size = Math.ceil((size)/(quantum ? quantum : 16))*(quantum ? quantum : 16); return ret; },
  makeBigInt: function (low,high,unsigned) { var ret = (unsigned ? ((+((low>>>0)))+((+((high>>>0)))*4294967296.0)) : ((+((low>>>0)))+((+((high|0)))*4294967296.0))); return ret; },
  GLOBAL_BASE: 8,
  QUANTUM_SIZE: 4,
  __dummy__: 0
}


Module['Runtime'] = Runtime;









//========================================
// Runtime essentials
//========================================

var __THREW__ = 0; // Used in checking for thrown exceptions.

var ABORT = false; // whether we are quitting the application. no code should run after this. set in exit() and abort()
var EXITSTATUS = 0;

var undef = 0;
// tempInt is used for 32-bit signed values or smaller. tempBigInt is used
// for 32-bit unsigned values or more than 32 bits. TODO: audit all uses of tempInt
var tempValue, tempInt, tempBigInt, tempInt2, tempBigInt2, tempPair, tempBigIntI, tempBigIntR, tempBigIntS, tempBigIntP, tempBigIntD, tempDouble, tempFloat;
var tempI64, tempI64b;
var tempRet0, tempRet1, tempRet2, tempRet3, tempRet4, tempRet5, tempRet6, tempRet7, tempRet8, tempRet9;

function assert(condition, text) {
  if (!condition) {
    abort('Assertion failed: ' + text);
  }
}

var globalScope = this;

// Returns the C function with a specified identifier (for C++, you need to do manual name mangling)
function getCFunc(ident) {
  var func = Module['_' + ident]; // closure exported function
  if (!func) {
    try {
      func = eval('_' + ident); // explicit lookup
    } catch(e) {}
  }
  assert(func, 'Cannot call unknown function ' + ident + ' (perhaps LLVM optimizations or closure removed it?)');
  return func;
}

var cwrap, ccall;
(function(){
  var JSfuncs = {
    // Helpers for cwrap -- it can't refer to Runtime directly because it might
    // be renamed by closure, instead it calls JSfuncs['stackSave'].body to find
    // out what the minified function name is.
    'stackSave': function() {
      Runtime.stackSave()
    },
    'stackRestore': function() {
      Runtime.stackRestore()
    },
    // type conversion from js to c
    'arrayToC' : function(arr) {
      var ret = Runtime.stackAlloc(arr.length);
      writeArrayToMemory(arr, ret);
      return ret;
    },
    'stringToC' : function(str) {
      var ret = 0;
      if (str !== null && str !== undefined && str !== 0) { // null string
        // at most 4 bytes per UTF-8 code point, +1 for the trailing '\0'
        ret = Runtime.stackAlloc((str.length << 2) + 1);
        writeStringToMemory(str, ret);
      }
      return ret;
    }
  };
  // For fast lookup of conversion functions
  var toC = {'string' : JSfuncs['stringToC'], 'array' : JSfuncs['arrayToC']};

  // C calling interface. 
  ccall = function ccallFunc(ident, returnType, argTypes, args) {
    var func = getCFunc(ident);
    var cArgs = [];
    var stack = 0;
    assert(returnType !== 'array', 'Return type should not be "array".');
    if (args) {
      for (var i = 0; i < args.length; i++) {
        var converter = toC[argTypes[i]];
        if (converter) {
          if (stack === 0) stack = Runtime.stackSave();
          cArgs[i] = converter(args[i]);
        } else {
          cArgs[i] = args[i];
        }
      }
    }
    var ret = func.apply(null, cArgs);
    if (returnType === 'string') ret = Pointer_stringify(ret);
    if (stack !== 0) Runtime.stackRestore(stack);
    return ret;
  }

  var sourceRegex = /^function\s*\(([^)]*)\)\s*{\s*([^*]*?)[\s;]*(?:return\s*(.*?)[;\s]*)?}$/;
  function parseJSFunc(jsfunc) {
    // Match the body and the return value of a javascript function source
    var parsed = jsfunc.toString().match(sourceRegex).slice(1);
    return {arguments : parsed[0], body : parsed[1], returnValue: parsed[2]}
  }
  var JSsource = {};
  for (var fun in JSfuncs) {
    if (JSfuncs.hasOwnProperty(fun)) {
      // Elements of toCsource are arrays of three items:
      // the code, and the return value
      JSsource[fun] = parseJSFunc(JSfuncs[fun]);
    }
  }

  
  cwrap = function cwrap(ident, returnType, argTypes) {
    argTypes = argTypes || [];
    var cfunc = getCFunc(ident);
    // When the function takes numbers and returns a number, we can just return
    // the original function
    var numericArgs = argTypes.every(function(type){ return type === 'number'});
    var numericRet = (returnType !== 'string');
    if ( numericRet && numericArgs) {
      return cfunc;
    }
    // Creation of the arguments list (["$1","$2",...,"$nargs"])
    var argNames = argTypes.map(function(x,i){return '$'+i});
    var funcstr = "(function(" + argNames.join(',') + ") {";
    var nargs = argTypes.length;
    if (!numericArgs) {
      // Generate the code needed to convert the arguments from javascript
      // values to pointers
      funcstr += 'var stack = ' + JSsource['stackSave'].body + ';';
      for (var i = 0; i < nargs; i++) {
        var arg = argNames[i], type = argTypes[i];
        if (type === 'number') continue;
        var convertCode = JSsource[type + 'ToC']; // [code, return]
        funcstr += 'var ' + convertCode.arguments + ' = ' + arg + ';';
        funcstr += convertCode.body + ';';
        funcstr += arg + '=' + convertCode.returnValue + ';';
      }
    }

    // When the code is compressed, the name of cfunc is not literally 'cfunc' anymore
    var cfuncname = parseJSFunc(function(){return cfunc}).returnValue;
    // Call the function
    funcstr += 'var ret = ' + cfuncname + '(' + argNames.join(',') + ');';
    if (!numericRet) { // Return type can only by 'string' or 'number'
      // Convert the result to a string
      var strgfy = parseJSFunc(function(){return Pointer_stringify}).returnValue;
      funcstr += 'ret = ' + strgfy + '(ret);';
    }
    if (!numericArgs) {
      // If we had a stack, restore it
      funcstr += JSsource['stackRestore'].body.replace('()', '(stack)') + ';';
    }
    funcstr += 'return ret})';
    return eval(funcstr);
  };
})();
Module["cwrap"] = cwrap;
Module["ccall"] = ccall;


function setValue(ptr, value, type, noSafe) {
  type = type || 'i8';
  if (type.charAt(type.length-1) === '*') type = 'i32'; // pointers are 32-bit
    switch(type) {
      case 'i1': HEAP8[((ptr)>>0)]=value; break;
      case 'i8': HEAP8[((ptr)>>0)]=value; break;
      case 'i16': HEAP16[((ptr)>>1)]=value; break;
      case 'i32': HEAP32[((ptr)>>2)]=value; break;
      case 'i64': (tempI64 = [value>>>0,(tempDouble=value,(+(Math_abs(tempDouble))) >= 1.0 ? (tempDouble > 0.0 ? ((Math_min((+(Math_floor((tempDouble)/4294967296.0))), 4294967295.0))|0)>>>0 : (~~((+(Math_ceil((tempDouble - +(((~~(tempDouble)))>>>0))/4294967296.0)))))>>>0) : 0)],HEAP32[((ptr)>>2)]=tempI64[0],HEAP32[(((ptr)+(4))>>2)]=tempI64[1]); break;
      case 'float': HEAPF32[((ptr)>>2)]=value; break;
      case 'double': HEAPF64[((ptr)>>3)]=value; break;
      default: abort('invalid type for setValue: ' + type);
    }
}
Module['setValue'] = setValue;


function getValue(ptr, type, noSafe) {
  type = type || 'i8';
  if (type.charAt(type.length-1) === '*') type = 'i32'; // pointers are 32-bit
    switch(type) {
      case 'i1': return HEAP8[((ptr)>>0)];
      case 'i8': return HEAP8[((ptr)>>0)];
      case 'i16': return HEAP16[((ptr)>>1)];
      case 'i32': return HEAP32[((ptr)>>2)];
      case 'i64': return HEAP32[((ptr)>>2)];
      case 'float': return HEAPF32[((ptr)>>2)];
      case 'double': return HEAPF64[((ptr)>>3)];
      default: abort('invalid type for setValue: ' + type);
    }
  return null;
}
Module['getValue'] = getValue;

var ALLOC_NORMAL = 0; // Tries to use _malloc()
var ALLOC_STACK = 1; // Lives for the duration of the current function call
var ALLOC_STATIC = 2; // Cannot be freed
var ALLOC_DYNAMIC = 3; // Cannot be freed except through sbrk
var ALLOC_NONE = 4; // Do not allocate
Module['ALLOC_NORMAL'] = ALLOC_NORMAL;
Module['ALLOC_STACK'] = ALLOC_STACK;
Module['ALLOC_STATIC'] = ALLOC_STATIC;
Module['ALLOC_DYNAMIC'] = ALLOC_DYNAMIC;
Module['ALLOC_NONE'] = ALLOC_NONE;

// allocate(): This is for internal use. You can use it yourself as well, but the interface
//             is a little tricky (see docs right below). The reason is that it is optimized
//             for multiple syntaxes to save space in generated code. So you should
//             normally not use allocate(), and instead allocate memory using _malloc(),
//             initialize it with setValue(), and so forth.
// @slab: An array of data, or a number. If a number, then the size of the block to allocate,
//        in *bytes* (note that this is sometimes confusing: the next parameter does not
//        affect this!)
// @types: Either an array of types, one for each byte (or 0 if no type at that position),
//         or a single type which is used for the entire block. This only matters if there
//         is initial data - if @slab is a number, then this does not matter at all and is
//         ignored.
// @allocator: How to allocate memory, see ALLOC_*
function allocate(slab, types, allocator, ptr) {
  var zeroinit, size;
  if (typeof slab === 'number') {
    zeroinit = true;
    size = slab;
  } else {
    zeroinit = false;
    size = slab.length;
  }

  var singleType = typeof types === 'string' ? types : null;

  var ret;
  if (allocator == ALLOC_NONE) {
    ret = ptr;
  } else {
    ret = [_malloc, Runtime.stackAlloc, Runtime.staticAlloc, Runtime.dynamicAlloc][allocator === undefined ? ALLOC_STATIC : allocator](Math.max(size, singleType ? 1 : types.length));
  }

  if (zeroinit) {
    var ptr = ret, stop;
    assert((ret & 3) == 0);
    stop = ret + (size & ~3);
    for (; ptr < stop; ptr += 4) {
      HEAP32[((ptr)>>2)]=0;
    }
    stop = ret + size;
    while (ptr < stop) {
      HEAP8[((ptr++)>>0)]=0;
    }
    return ret;
  }

  if (singleType === 'i8') {
    if (slab.subarray || slab.slice) {
      HEAPU8.set(slab, ret);
    } else {
      HEAPU8.set(new Uint8Array(slab), ret);
    }
    return ret;
  }

  var i = 0, type, typeSize, previousType;
  while (i < size) {
    var curr = slab[i];

    if (typeof curr === 'function') {
      curr = Runtime.getFunctionIndex(curr);
    }

    type = singleType || types[i];
    if (type === 0) {
      i++;
      continue;
    }
    assert(type, 'Must know what type to store in allocate!');

    if (type == 'i64') type = 'i32'; // special case: we have one i32 here, and one i32 later

    setValue(ret+i, curr, type);

    // no need to look up size unless type changes, so cache it
    if (previousType !== type) {
      typeSize = Runtime.getNativeTypeSize(type);
      previousType = type;
    }
    i += typeSize;
  }

  return ret;
}
Module['allocate'] = allocate;

function Pointer_stringify(ptr, /* optional */ length) {
  if (length === 0 || !ptr) return '';
  // TODO: use TextDecoder
  // Find the length, and check for UTF while doing so
  var hasUtf = 0;
  var t;
  var i = 0;
  while (1) {
    assert(ptr + i < TOTAL_MEMORY);
    t = HEAPU8[(((ptr)+(i))>>0)];
    hasUtf |= t;
    if (t == 0 && !length) break;
    i++;
    if (length && i == length) break;
  }
  if (!length) length = i;

  var ret = '';

  if (hasUtf < 128) {
    var MAX_CHUNK = 1024; // split up into chunks, because .apply on a huge string can overflow the stack
    var curr;
    while (length > 0) {
      curr = String.fromCharCode.apply(String, HEAPU8.subarray(ptr, ptr + Math.min(length, MAX_CHUNK)));
      ret = ret ? ret + curr : curr;
      ptr += MAX_CHUNK;
      length -= MAX_CHUNK;
    }
    return ret;
  }
  return Module['UTF8ToString'](ptr);
}
Module['Pointer_stringify'] = Pointer_stringify;

// Given a pointer 'ptr' to a null-terminated ASCII-encoded string in the emscripten HEAP, returns
// a copy of that string as a Javascript String object.

function AsciiToString(ptr) {
  var str = '';
  while (1) {
    var ch = HEAP8[((ptr++)>>0)];
    if (!ch) return str;
    str += String.fromCharCode(ch);
  }
}
Module['AsciiToString'] = AsciiToString;

// Copies the given Javascript String object 'str' to the emscripten HEAP at address 'outPtr',
// null-terminated and encoded in ASCII form. The copy will require at most str.length+1 bytes of space in the HEAP.

function stringToAscii(str, outPtr) {
  return writeAsciiToMemory(str, outPtr, false);
}
Module['stringToAscii'] = stringToAscii;

// Given a pointer 'ptr' to a null-terminated UTF8-encoded string in the a given array that contains uint8 values, returns
// a copy of that string as a Javascript String object.

function UTF8ArrayToString(u8Array, idx) {
  var u0, u1, u2, u3, u4, u5;

  var str = '';
  while (1) {
    // For UTF8 byte structure, see http://en.wikipedia.org/wiki/UTF-8#Description and https://www.ietf.org/rfc/rfc2279.txt and https://tools.ietf.org/html/rfc3629
    u0 = u8Array[idx++];
    if (!u0) return str;
    if (!(u0 & 0x80)) { str += String.fromCharCode(u0); continue; }
    u1 = u8Array[idx++] & 63;
    if ((u0 & 0xE0) == 0xC0) { str += String.fromCharCode(((u0 & 31) << 6) | u1); continue; }
    u2 = u8Array[idx++] & 63;
    if ((u0 & 0xF0) == 0xE0) {
      u0 = ((u0 & 15) << 12) | (u1 << 6) | u2;
    } else {
      u3 = u8Array[idx++] & 63;
      if ((u0 & 0xF8) == 0xF0) {
        u0 = ((u0 & 7) << 18) | (u1 << 12) | (u2 << 6) | u3;
      } else {
        u4 = u8Array[idx++] & 63;
        if ((u0 & 0xFC) == 0xF8) {
          u0 = ((u0 & 3) << 24) | (u1 << 18) | (u2 << 12) | (u3 << 6) | u4;
        } else {
          u5 = u8Array[idx++] & 63;
          u0 = ((u0 & 1) << 30) | (u1 << 24) | (u2 << 18) | (u3 << 12) | (u4 << 6) | u5;
        }
      }
    }
    if (u0 < 0x10000) {
      str += String.fromCharCode(u0);
    } else {
      var ch = u0 - 0x10000;
      str += String.fromCharCode(0xD800 | (ch >> 10), 0xDC00 | (ch & 0x3FF));
    }
  }
}
Module['UTF8ArrayToString'] = UTF8ArrayToString;

// Given a pointer 'ptr' to a null-terminated UTF8-encoded string in the emscripten HEAP, returns
// a copy of that string as a Javascript String object.

function UTF8ToString(ptr) {
  return UTF8ArrayToString(HEAPU8, ptr);
}
Module['UTF8ToString'] = UTF8ToString;

// Copies the given Javascript String object 'str' to the given byte array at address 'outIdx',
// encoded in UTF8 form and null-terminated. The copy will require at most str.length*4+1 bytes of space in the HEAP.
// Use the function lengthBytesUTF8() to compute the exact number of bytes (excluding null terminator) that this function will write.
// Parameters:
//   str: the Javascript string to copy.
//   outU8Array: the array to copy to. Each index in this array is assumed to be one 8-byte element.
//   outIdx: The starting offset in the array to begin the copying.
//   maxBytesToWrite: The maximum number of bytes this function can write to the array. This count should include the null 
//                    terminator, i.e. if maxBytesToWrite=1, only the null terminator will be written and nothing else.
//                    maxBytesToWrite=0 does not write any bytes to the output, not even the null terminator.
// Returns the number of bytes written, EXCLUDING the null terminator.

function stringToUTF8Array(str, outU8Array, outIdx, maxBytesToWrite) {
  if (!(maxBytesToWrite > 0)) // Parameter maxBytesToWrite is not optional. Negative values, 0, null, undefined and false each don't write out any bytes.
    return 0;

  var startIdx = outIdx;
  var endIdx = outIdx + maxBytesToWrite - 1; // -1 for string null terminator.
  for (var i = 0; i < str.length; ++i) {
    // Gotcha: charCodeAt returns a 16-bit word that is a UTF-16 encoded code unit, not a Unicode code point of the character! So decode UTF16->UTF32->UTF8.
    // See http://unicode.org/faq/utf_bom.html#utf16-3
    // For UTF8 byte structure, see http://en.wikipedia.org/wiki/UTF-8#Description and https://www.ietf.org/rfc/rfc2279.txt and https://tools.ietf.org/html/rfc3629
    var u = str.charCodeAt(i); // possibly a lead surrogate
    if (u >= 0xD800 && u <= 0xDFFF) u = 0x10000 + ((u & 0x3FF) << 10) | (str.charCodeAt(++i) & 0x3FF);
    if (u <= 0x7F) {
      if (outIdx >= endIdx) break;
      outU8Array[outIdx++] = u;
    } else if (u <= 0x7FF) {
      if (outIdx + 1 >= endIdx) break;
      outU8Array[outIdx++] = 0xC0 | (u >> 6);
      outU8Array[outIdx++] = 0x80 | (u & 63);
    } else if (u <= 0xFFFF) {
      if (outIdx + 2 >= endIdx) break;
      outU8Array[outIdx++] = 0xE0 | (u >> 12);
      outU8Array[outIdx++] = 0x80 | ((u >> 6) & 63);
      outU8Array[outIdx++] = 0x80 | (u & 63);
    } else if (u <= 0x1FFFFF) {
      if (outIdx + 3 >= endIdx) break;
      outU8Array[outIdx++] = 0xF0 | (u >> 18);
      outU8Array[outIdx++] = 0x80 | ((u >> 12) & 63);
      outU8Array[outIdx++] = 0x80 | ((u >> 6) & 63);
      outU8Array[outIdx++] = 0x80 | (u & 63);
    } else if (u <= 0x3FFFFFF) {
      if (outIdx + 4 >= endIdx) break;
      outU8Array[outIdx++] = 0xF8 | (u >> 24);
      outU8Array[outIdx++] = 0x80 | ((u >> 18) & 63);
      outU8Array[outIdx++] = 0x80 | ((u >> 12) & 63);
      outU8Array[outIdx++] = 0x80 | ((u >> 6) & 63);
      outU8Array[outIdx++] = 0x80 | (u & 63);
    } else {
      if (outIdx + 5 >= endIdx) break;
      outU8Array[outIdx++] = 0xFC | (u >> 30);
      outU8Array[outIdx++] = 0x80 | ((u >> 24) & 63);
      outU8Array[outIdx++] = 0x80 | ((u >> 18) & 63);
      outU8Array[outIdx++] = 0x80 | ((u >> 12) & 63);
      outU8Array[outIdx++] = 0x80 | ((u >> 6) & 63);
      outU8Array[outIdx++] = 0x80 | (u & 63);
    }
  }
  // Null-terminate the pointer to the buffer.
  outU8Array[outIdx] = 0;
  return outIdx - startIdx;
}
Module['stringToUTF8Array'] = stringToUTF8Array;

// Copies the given Javascript String object 'str' to the emscripten HEAP at address 'outPtr',
// null-terminated and encoded in UTF8 form. The copy will require at most str.length*4+1 bytes of space in the HEAP.
// Use the function lengthBytesUTF8() to compute the exact number of bytes (excluding null terminator) that this function will write.
// Returns the number of bytes written, EXCLUDING the null terminator.

function stringToUTF8(str, outPtr, maxBytesToWrite) {
  assert(typeof maxBytesToWrite == 'number', 'stringToUTF8(str, outPtr, maxBytesToWrite) is missing the third parameter that specifies the length of the output buffer!');
  return stringToUTF8Array(str, HEAPU8, outPtr, maxBytesToWrite);
}
Module['stringToUTF8'] = stringToUTF8;

// Returns the number of bytes the given Javascript string takes if encoded as a UTF8 byte array, EXCLUDING the null terminator byte.

function lengthBytesUTF8(str) {
  var len = 0;
  for (var i = 0; i < str.length; ++i) {
    // Gotcha: charCodeAt returns a 16-bit word that is a UTF-16 encoded code unit, not a Unicode code point of the character! So decode UTF16->UTF32->UTF8.
    // See http://unicode.org/faq/utf_bom.html#utf16-3
    var u = str.charCodeAt(i); // possibly a lead surrogate
    if (u >= 0xD800 && u <= 0xDFFF) u = 0x10000 + ((u & 0x3FF) << 10) | (str.charCodeAt(++i) & 0x3FF);
    if (u <= 0x7F) {
      ++len;
    } else if (u <= 0x7FF) {
      len += 2;
    } else if (u <= 0xFFFF) {
      len += 3;
    } else if (u <= 0x1FFFFF) {
      len += 4;
    } else if (u <= 0x3FFFFFF) {
      len += 5;
    } else {
      len += 6;
    }
  }
  return len;
}
Module['lengthBytesUTF8'] = lengthBytesUTF8;

// Given a pointer 'ptr' to a null-terminated UTF16LE-encoded string in the emscripten HEAP, returns
// a copy of that string as a Javascript String object.

function UTF16ToString(ptr) {
  var i = 0;

  var str = '';
  while (1) {
    var codeUnit = HEAP16[(((ptr)+(i*2))>>1)];
    if (codeUnit == 0)
      return str;
    ++i;
    // fromCharCode constructs a character from a UTF-16 code unit, so we can pass the UTF16 string right through.
    str += String.fromCharCode(codeUnit);
  }
}
Module['UTF16ToString'] = UTF16ToString;

// Copies the given Javascript String object 'str' to the emscripten HEAP at address 'outPtr',
// null-terminated and encoded in UTF16 form. The copy will require at most str.length*4+2 bytes of space in the HEAP.
// Use the function lengthBytesUTF16() to compute the exact number of bytes (excluding null terminator) that this function will write.
// Parameters:
//   str: the Javascript string to copy.
//   outPtr: Byte address in Emscripten HEAP where to write the string to.
//   maxBytesToWrite: The maximum number of bytes this function can write to the array. This count should include the null 
//                    terminator, i.e. if maxBytesToWrite=2, only the null terminator will be written and nothing else.
//                    maxBytesToWrite<2 does not write any bytes to the output, not even the null terminator.
// Returns the number of bytes written, EXCLUDING the null terminator.

function stringToUTF16(str, outPtr, maxBytesToWrite) {
  assert(typeof maxBytesToWrite == 'number', 'stringToUTF16(str, outPtr, maxBytesToWrite) is missing the third parameter that specifies the length of the output buffer!');
  // Backwards compatibility: if max bytes is not specified, assume unsafe unbounded write is allowed.
  if (maxBytesToWrite === undefined) {
    maxBytesToWrite = 0x7FFFFFFF;
  }
  if (maxBytesToWrite < 2) return 0;
  maxBytesToWrite -= 2; // Null terminator.
  var startPtr = outPtr;
  var numCharsToWrite = (maxBytesToWrite < str.length*2) ? (maxBytesToWrite / 2) : str.length;
  for (var i = 0; i < numCharsToWrite; ++i) {
    // charCodeAt returns a UTF-16 encoded code unit, so it can be directly written to the HEAP.
    var codeUnit = str.charCodeAt(i); // possibly a lead surrogate
    HEAP16[((outPtr)>>1)]=codeUnit;
    outPtr += 2;
  }
  // Null-terminate the pointer to the HEAP.
  HEAP16[((outPtr)>>1)]=0;
  return outPtr - startPtr;
}
Module['stringToUTF16'] = stringToUTF16;

// Returns the number of bytes the given Javascript string takes if encoded as a UTF16 byte array, EXCLUDING the null terminator byte.

function lengthBytesUTF16(str) {
  return str.length*2;
}
Module['lengthBytesUTF16'] = lengthBytesUTF16;

function UTF32ToString(ptr) {
  var i = 0;

  var str = '';
  while (1) {
    var utf32 = HEAP32[(((ptr)+(i*4))>>2)];
    if (utf32 == 0)
      return str;
    ++i;
    // Gotcha: fromCharCode constructs a character from a UTF-16 encoded code (pair), not from a Unicode code point! So encode the code point to UTF-16 for constructing.
    // See http://unicode.org/faq/utf_bom.html#utf16-3
    if (utf32 >= 0x10000) {
      var ch = utf32 - 0x10000;
      str += String.fromCharCode(0xD800 | (ch >> 10), 0xDC00 | (ch & 0x3FF));
    } else {
      str += String.fromCharCode(utf32);
    }
  }
}
Module['UTF32ToString'] = UTF32ToString;

// Copies the given Javascript String object 'str' to the emscripten HEAP at address 'outPtr',
// null-terminated and encoded in UTF32 form. The copy will require at most str.length*4+4 bytes of space in the HEAP.
// Use the function lengthBytesUTF32() to compute the exact number of bytes (excluding null terminator) that this function will write.
// Parameters:
//   str: the Javascript string to copy.
//   outPtr: Byte address in Emscripten HEAP where to write the string to.
//   maxBytesToWrite: The maximum number of bytes this function can write to the array. This count should include the null 
//                    terminator, i.e. if maxBytesToWrite=4, only the null terminator will be written and nothing else.
//                    maxBytesToWrite<4 does not write any bytes to the output, not even the null terminator.
// Returns the number of bytes written, EXCLUDING the null terminator.

function stringToUTF32(str, outPtr, maxBytesToWrite) {
  assert(typeof maxBytesToWrite == 'number', 'stringToUTF32(str, outPtr, maxBytesToWrite) is missing the third parameter that specifies the length of the output buffer!');
  // Backwards compatibility: if max bytes is not specified, assume unsafe unbounded write is allowed.
  if (maxBytesToWrite === undefined) {
    maxBytesToWrite = 0x7FFFFFFF;
  }
  if (maxBytesToWrite < 4) return 0;
  var startPtr = outPtr;
  var endPtr = startPtr + maxBytesToWrite - 4;
  for (var i = 0; i < str.length; ++i) {
    // Gotcha: charCodeAt returns a 16-bit word that is a UTF-16 encoded code unit, not a Unicode code point of the character! We must decode the string to UTF-32 to the heap.
    // See http://unicode.org/faq/utf_bom.html#utf16-3
    var codeUnit = str.charCodeAt(i); // possibly a lead surrogate
    if (codeUnit >= 0xD800 && codeUnit <= 0xDFFF) {
      var trailSurrogate = str.charCodeAt(++i);
      codeUnit = 0x10000 + ((codeUnit & 0x3FF) << 10) | (trailSurrogate & 0x3FF);
    }
    HEAP32[((outPtr)>>2)]=codeUnit;
    outPtr += 4;
    if (outPtr + 4 > endPtr) break;
  }
  // Null-terminate the pointer to the HEAP.
  HEAP32[((outPtr)>>2)]=0;
  return outPtr - startPtr;
}
Module['stringToUTF32'] = stringToUTF32;

// Returns the number of bytes the given Javascript string takes if encoded as a UTF16 byte array, EXCLUDING the null terminator byte.

function lengthBytesUTF32(str) {
  var len = 0;
  for (var i = 0; i < str.length; ++i) {
    // Gotcha: charCodeAt returns a 16-bit word that is a UTF-16 encoded code unit, not a Unicode code point of the character! We must decode the string to UTF-32 to the heap.
    // See http://unicode.org/faq/utf_bom.html#utf16-3
    var codeUnit = str.charCodeAt(i);
    if (codeUnit >= 0xD800 && codeUnit <= 0xDFFF) ++i; // possibly a lead surrogate, so skip over the tail surrogate.
    len += 4;
  }

  return len;
}
Module['lengthBytesUTF32'] = lengthBytesUTF32;

function demangle(func) {
  var hasLibcxxabi = !!Module['___cxa_demangle'];
  if (hasLibcxxabi) {
    try {
      var buf = _malloc(func.length);
      writeStringToMemory(func.substr(1), buf);
      var status = _malloc(4);
      var ret = Module['___cxa_demangle'](buf, 0, 0, status);
      if (getValue(status, 'i32') === 0 && ret) {
        return Pointer_stringify(ret);
      }
      // otherwise, libcxxabi failed, we can try ours which may return a partial result
    } catch(e) {
      // failure when using libcxxabi, we can try ours which may return a partial result
    } finally {
      if (buf) _free(buf);
      if (status) _free(status);
      if (ret) _free(ret);
    }
  }
  var i = 3;
  // params, etc.
  var basicTypes = {
    'v': 'void',
    'b': 'bool',
    'c': 'char',
    's': 'short',
    'i': 'int',
    'l': 'long',
    'f': 'float',
    'd': 'double',
    'w': 'wchar_t',
    'a': 'signed char',
    'h': 'unsigned char',
    't': 'unsigned short',
    'j': 'unsigned int',
    'm': 'unsigned long',
    'x': 'long long',
    'y': 'unsigned long long',
    'z': '...'
  };
  var subs = [];
  var first = true;
  function dump(x) {
    //return;
    if (x) Module.print(x);
    Module.print(func);
    var pre = '';
    for (var a = 0; a < i; a++) pre += ' ';
    Module.print (pre + '^');
  }
  function parseNested() {
    i++;
    if (func[i] === 'K') i++; // ignore const
    var parts = [];
    while (func[i] !== 'E') {
      if (func[i] === 'S') { // substitution
        i++;
        var next = func.indexOf('_', i);
        var num = func.substring(i, next) || 0;
        parts.push(subs[num] || '?');
        i = next+1;
        continue;
      }
      if (func[i] === 'C') { // constructor
        parts.push(parts[parts.length-1]);
        i += 2;
        continue;
      }
      var size = parseInt(func.substr(i));
      var pre = size.toString().length;
      if (!size || !pre) { i--; break; } // counter i++ below us
      var curr = func.substr(i + pre, size);
      parts.push(curr);
      subs.push(curr);
      i += pre + size;
    }
    i++; // skip E
    return parts;
  }
  function parse(rawList, limit, allowVoid) { // main parser
    limit = limit || Infinity;
    var ret = '', list = [];
    function flushList() {
      return '(' + list.join(', ') + ')';
    }
    var name;
    if (func[i] === 'N') {
      // namespaced N-E
      name = parseNested().join('::');
      limit--;
      if (limit === 0) return rawList ? [name] : name;
    } else {
      // not namespaced
      if (func[i] === 'K' || (first && func[i] === 'L')) i++; // ignore const and first 'L'
      var size = parseInt(func.substr(i));
      if (size) {
        var pre = size.toString().length;
        name = func.substr(i + pre, size);
        i += pre + size;
      }
    }
    first = false;
    if (func[i] === 'I') {
      i++;
      var iList = parse(true);
      var iRet = parse(true, 1, true);
      ret += iRet[0] + ' ' + name + '<' + iList.join(', ') + '>';
    } else {
      ret = name;
    }
    paramLoop: while (i < func.length && limit-- > 0) {
      //dump('paramLoop');
      var c = func[i++];
      if (c in basicTypes) {
        list.push(basicTypes[c]);
      } else {
        switch (c) {
          case 'P': list.push(parse(true, 1, true)[0] + '*'); break; // pointer
          case 'R': list.push(parse(true, 1, true)[0] + '&'); break; // reference
          case 'L': { // literal
            i++; // skip basic type
            var end = func.indexOf('E', i);
            var size = end - i;
            list.push(func.substr(i, size));
            i += size + 2; // size + 'EE'
            break;
          }
          case 'A': { // array
            var size = parseInt(func.substr(i));
            i += size.toString().length;
            if (func[i] !== '_') throw '?';
            i++; // skip _
            list.push(parse(true, 1, true)[0] + ' [' + size + ']');
            break;
          }
          case 'E': break paramLoop;
          default: ret += '?' + c; break paramLoop;
        }
      }
    }
    if (!allowVoid && list.length === 1 && list[0] === 'void') list = []; // avoid (void)
    if (rawList) {
      if (ret) {
        list.push(ret + '?');
      }
      return list;
    } else {
      return ret + flushList();
    }
  }
  var parsed = func;
  try {
    // Special-case the entry point, since its name differs from other name mangling.
    if (func == 'Object._main' || func == '_main') {
      return 'main()';
    }
    if (typeof func === 'number') func = Pointer_stringify(func);
    if (func[0] !== '_') return func;
    if (func[1] !== '_') return func; // C function
    if (func[2] !== 'Z') return func;
    switch (func[3]) {
      case 'n': return 'operator new()';
      case 'd': return 'operator delete()';
    }
    parsed = parse();
  } catch(e) {
    parsed += '?';
  }
  if (parsed.indexOf('?') >= 0 && !hasLibcxxabi) {
    Runtime.warnOnce('warning: a problem occurred in builtin C++ name demangling; build with  -s DEMANGLE_SUPPORT=1  to link in libcxxabi demangling');
  }
  return parsed;
}

function demangleAll(text) {
  return text.replace(/__Z[\w\d_]+/g, function(x) { var y = demangle(x); return x === y ? x : (x + ' [' + y + ']') });
}

function jsStackTrace() {
  var err = new Error();
  if (!err.stack) {
    // IE10+ special cases: It does have callstack info, but it is only populated if an Error object is thrown,
    // so try that as a special-case.
    try {
      throw new Error(0);
    } catch(e) {
      err = e;
    }
    if (!err.stack) {
      return '(no stack trace available)';
    }
  }
  return err.stack.toString();
}

function stackTrace() {
  return demangleAll(jsStackTrace());
}
Module['stackTrace'] = stackTrace;

// Memory management

var PAGE_SIZE = 4096;
function alignMemoryPage(x) {
  return (x+4095)&-4096;
}

var HEAP;
var HEAP8, HEAPU8, HEAP16, HEAPU16, HEAP32, HEAPU32, HEAPF32, HEAPF64;

var STATIC_BASE = 0, STATICTOP = 0, staticSealed = false; // static area
var STACK_BASE = 0, STACKTOP = 0, STACK_MAX = 0; // stack area
var DYNAMIC_BASE = 0, DYNAMICTOP = 0; // dynamic area handled by sbrk

function enlargeMemory() {
  abort('Cannot enlarge memory arrays, since compiling with pthreads support enabled (-lpthread).');
}


var TOTAL_STACK = Module['TOTAL_STACK'] || 5242880;
var TOTAL_MEMORY = Module['TOTAL_MEMORY'] || 16777216;
var FAST_MEMORY = Module['FAST_MEMORY'] || 2097152;

var totalMemory = 64*1024;
while (totalMemory < TOTAL_MEMORY || totalMemory < 2*TOTAL_STACK) {
  if (totalMemory < 16*1024*1024) {
    totalMemory *= 2;
  } else {
    totalMemory += 16*1024*1024
  }
}
if (totalMemory !== TOTAL_MEMORY) {
  Module.printErr('increasing TOTAL_MEMORY to ' + totalMemory + ' to be compliant with the asm.js spec (and given that TOTAL_STACK=' + TOTAL_STACK + ')');
  TOTAL_MEMORY = totalMemory;
}

// Initialize the runtime's memory
// check for full engine support (use string 'subarray' to avoid closure compiler confusion)
assert(typeof Int32Array !== 'undefined' && typeof Float64Array !== 'undefined' && !!(new Int32Array(1)['subarray']) && !!(new Int32Array(1)['set']),
       'JS engine does not provide full typed array support');

// Pthreads don't allocate their own HEAP, but will share the one from the main thread.
var buffer;
if (!ENVIRONMENT_IS_PTHREAD) buffer = (typeof SharedArrayBuffer != 'undefined') ? new SharedArrayBuffer(TOTAL_MEMORY) : new ArrayBuffer(TOTAL_MEMORY);

if (typeof SharedArrayBuffer != 'undefined') {
  HEAP8 = new SharedInt8Array(buffer);
  HEAP16 = new SharedInt16Array(buffer);
  HEAP32 = new SharedInt32Array(buffer);
  HEAPU8 = new SharedUint8Array(buffer);
  HEAPU16 = new SharedUint16Array(buffer);
  HEAPU32 = new SharedUint32Array(buffer);
  HEAPF32 = new SharedFloat32Array(buffer);
  HEAPF64 = new SharedFloat64Array(buffer);
} else {
  HEAP8 = new Int8Array(buffer);
  HEAP16 = new Int16Array(buffer);
  HEAP32 = new Int32Array(buffer);
  HEAPU8 = new Uint8Array(buffer);
  HEAPU16 = new Uint16Array(buffer);
  HEAPU32 = new Uint32Array(buffer);
  HEAPF32 = new Float32Array(buffer);
  HEAPF64 = new Float64Array(buffer);
}

// Endianness check (note: assumes compiler arch was little-endian)
HEAP32[0] = 255;
assert(HEAPU8[0] === 255 && HEAPU8[3] === 0, 'Typed arrays 2 must be run on a little-endian system');

Module['HEAP'] = HEAP;
Module['buffer'] = buffer;
Module['HEAP8'] = HEAP8;
Module['HEAP16'] = HEAP16;
Module['HEAP32'] = HEAP32;
Module['HEAPU8'] = HEAPU8;
Module['HEAPU16'] = HEAPU16;
Module['HEAPU32'] = HEAPU32;
Module['HEAPF32'] = HEAPF32;
Module['HEAPF64'] = HEAPF64;

function callRuntimeCallbacks(callbacks) {
  while(callbacks.length > 0) {
    var callback = callbacks.shift();
    if (typeof callback == 'function') {
      callback();
      continue;
    }
    var func = callback.func;
    if (typeof func === 'number') {
      if (callback.arg === undefined) {
        Runtime.dynCall('v', func);
      } else {
        Runtime.dynCall('vi', func, [callback.arg]);
      }
    } else {
      func(callback.arg === undefined ? null : callback.arg);
    }
  }
}

var __ATPRERUN__  = []; // functions called before the runtime is initialized
var __ATINIT__    = []; // functions called during startup
var __ATMAIN__    = []; // functions called when main() is to be run
var __ATEXIT__    = []; // functions called during shutdown
var __ATPOSTRUN__ = []; // functions called after the runtime has exited

var runtimeInitialized = ENVIRONMENT_IS_PTHREAD; // The runtime is hosted in the main thread, and bits shared to pthreads via SharedArrayBuffer. No need to init again in pthread.
var runtimeExited = false;

function preRun() {
  if (ENVIRONMENT_IS_PTHREAD) return; // PThreads reuse the runtime from the main thread.
  // compatibility - merge in anything from Module['preRun'] at this time
  if (Module['preRun']) {
    if (typeof Module['preRun'] == 'function') Module['preRun'] = [Module['preRun']];
    while (Module['preRun'].length) {
      addOnPreRun(Module['preRun'].shift());
    }
  }
  callRuntimeCallbacks(__ATPRERUN__);
}

function ensureInitRuntime() {
  if (runtimeInitialized) return;
  runtimeInitialized = true;
  callRuntimeCallbacks(__ATINIT__);
}

function preMain() {
  if (ENVIRONMENT_IS_PTHREAD) return; // PThreads reuse the runtime from the main thread.
  callRuntimeCallbacks(__ATMAIN__);
}

function exitRuntime() {
  if (ENVIRONMENT_IS_PTHREAD) return; // PThreads reuse the runtime from the main thread.
  callRuntimeCallbacks(__ATEXIT__);
  runtimeExited = true;
}

function postRun() {
  if (ENVIRONMENT_IS_PTHREAD) return; // PThreads reuse the runtime from the main thread.
  // compatibility - merge in anything from Module['postRun'] at this time
  if (Module['postRun']) {
    if (typeof Module['postRun'] == 'function') Module['postRun'] = [Module['postRun']];
    while (Module['postRun'].length) {
      addOnPostRun(Module['postRun'].shift());
    }
  }
  callRuntimeCallbacks(__ATPOSTRUN__);
}

function addOnPreRun(cb) {
  __ATPRERUN__.unshift(cb);
}
Module['addOnPreRun'] = Module.addOnPreRun = addOnPreRun;

function addOnInit(cb) {
  __ATINIT__.unshift(cb);
}
Module['addOnInit'] = Module.addOnInit = addOnInit;

function addOnPreMain(cb) {
  __ATMAIN__.unshift(cb);
}
Module['addOnPreMain'] = Module.addOnPreMain = addOnPreMain;

function addOnExit(cb) {
  __ATEXIT__.unshift(cb);
}
Module['addOnExit'] = Module.addOnExit = addOnExit;

function addOnPostRun(cb) {
  __ATPOSTRUN__.unshift(cb);
}
Module['addOnPostRun'] = Module.addOnPostRun = addOnPostRun;

// Tools


function intArrayFromString(stringy, dontAddNull, length /* optional */) {
  var len = length > 0 ? length : lengthBytesUTF8(stringy)+1;
  var u8array = new Array(len);
  var numBytesWritten = stringToUTF8Array(stringy, u8array, 0, u8array.length);
  if (dontAddNull) u8array.length = numBytesWritten;
  return u8array;
}
Module['intArrayFromString'] = intArrayFromString;

function intArrayToString(array) {
  var ret = [];
  for (var i = 0; i < array.length; i++) {
    var chr = array[i];
    if (chr > 0xFF) {
        assert(false, 'Character code ' + chr + ' (' + String.fromCharCode(chr) + ')  at offset ' + i + ' not in 0x00-0xFF.');
      chr &= 0xFF;
    }
    ret.push(String.fromCharCode(chr));
  }
  return ret.join('');
}
Module['intArrayToString'] = intArrayToString;

function writeStringToMemory(string, buffer, dontAddNull) {
  var array = intArrayFromString(string, dontAddNull);
  var i = 0;
  while (i < array.length) {
    var chr = array[i];
    HEAP8[(((buffer)+(i))>>0)]=chr;
    i = i + 1;
  }
}
Module['writeStringToMemory'] = writeStringToMemory;

function writeArrayToMemory(array, buffer) {
  for (var i = 0; i < array.length; i++) {
    HEAP8[((buffer++)>>0)]=array[i];
  }
}
Module['writeArrayToMemory'] = writeArrayToMemory;

function writeAsciiToMemory(str, buffer, dontAddNull) {
  for (var i = 0; i < str.length; ++i) {
    assert(str.charCodeAt(i) === str.charCodeAt(i)&0xff);
    HEAP8[((buffer++)>>0)]=str.charCodeAt(i);
  }
  // Null-terminate the pointer to the HEAP.
  if (!dontAddNull) HEAP8[((buffer)>>0)]=0;
}
Module['writeAsciiToMemory'] = writeAsciiToMemory;

function unSign(value, bits, ignore) {
  if (value >= 0) {
    return value;
  }
  return bits <= 32 ? 2*Math.abs(1 << (bits-1)) + value // Need some trickery, since if bits == 32, we are right at the limit of the bits JS uses in bitshifts
                    : Math.pow(2, bits)         + value;
}
function reSign(value, bits, ignore) {
  if (value <= 0) {
    return value;
  }
  var half = bits <= 32 ? Math.abs(1 << (bits-1)) // abs is needed if bits == 32
                        : Math.pow(2, bits-1);
  if (value >= half && (bits <= 32 || value > half)) { // for huge values, we can hit the precision limit and always get true here. so don't do that
                                                       // but, in general there is no perfect solution here. With 64-bit ints, we get rounding and errors
                                                       // TODO: In i64 mode 1, resign the two parts separately and safely
    value = -2*half + value; // Cannot bitshift half, as it may be at the limit of the bits JS uses in bitshifts
  }
  return value;
}

// check for imul support, and also for correctness ( https://bugs.webkit.org/show_bug.cgi?id=126345 )
if (!Math['imul'] || Math['imul'](0xffffffff, 5) !== -5) Math['imul'] = function imul(a, b) {
  var ah  = a >>> 16;
  var al = a & 0xffff;
  var bh  = b >>> 16;
  var bl = b & 0xffff;
  return (al*bl + ((ah*bl + al*bh) << 16))|0;
};
Math.imul = Math['imul'];


if (!Math['clz32']) Math['clz32'] = function(x) {
  x = x >>> 0;
  for (var i = 0; i < 32; i++) {
    if (x & (1 << (31 - i))) return i;
  }
  return 32;
};
Math.clz32 = Math['clz32']

var Math_abs = Math.abs;
var Math_cos = Math.cos;
var Math_sin = Math.sin;
var Math_tan = Math.tan;
var Math_acos = Math.acos;
var Math_asin = Math.asin;
var Math_atan = Math.atan;
var Math_atan2 = Math.atan2;
var Math_exp = Math.exp;
var Math_log = Math.log;
var Math_sqrt = Math.sqrt;
var Math_ceil = Math.ceil;
var Math_floor = Math.floor;
var Math_pow = Math.pow;
var Math_imul = Math.imul;
var Math_fround = Math.fround;
var Math_min = Math.min;
var Math_clz32 = Math.clz32;

// A counter of dependencies for calling run(). If we need to
// do asynchronous work before running, increment this and
// decrement it. Incrementing must happen in a place like
// PRE_RUN_ADDITIONS (used by emcc to add file preloading).
// Note that you can add dependencies in preRun, even though
// it happens right before run - run will be postponed until
// the dependencies are met.
var runDependencies = 0;
var runDependencyWatcher = null;
var dependenciesFulfilled = null; // overridden to take different actions when all run dependencies are fulfilled
var runDependencyTracking = {};

function addRunDependency(id) {
  runDependencies++;
  if (Module['monitorRunDependencies']) {
    Module['monitorRunDependencies'](runDependencies);
  }
  if (id) {
    assert(!runDependencyTracking[id]);
    runDependencyTracking[id] = 1;
    if (runDependencyWatcher === null && typeof setInterval !== 'undefined') {
      // Check for missing dependencies every few seconds
      runDependencyWatcher = setInterval(function() {
        if (ABORT) {
          clearInterval(runDependencyWatcher);
          runDependencyWatcher = null;
          return;
        }
        var shown = false;
        for (var dep in runDependencyTracking) {
          if (!shown) {
            shown = true;
            Module.printErr('still waiting on run dependencies:');
          }
          Module.printErr('dependency: ' + dep);
        }
        if (shown) {
          Module.printErr('(end of list)');
        }
      }, 10000);
    }
  } else {
    Module.printErr('warning: run dependency added without ID');
  }
}
Module['addRunDependency'] = addRunDependency;
function removeRunDependency(id) {
  runDependencies--;
  if (Module['monitorRunDependencies']) {
    Module['monitorRunDependencies'](runDependencies);
  }
  if (id) {
    assert(runDependencyTracking[id]);
    delete runDependencyTracking[id];
  } else {
    Module.printErr('warning: run dependency removed without ID');
  }
  if (runDependencies == 0) {
    if (runDependencyWatcher !== null) {
      clearInterval(runDependencyWatcher);
      runDependencyWatcher = null;
    }
    if (dependenciesFulfilled) {
      var callback = dependenciesFulfilled;
      dependenciesFulfilled = null;
      callback(); // can add another dependenciesFulfilled
    }
  }
}
Module['removeRunDependency'] = removeRunDependency;

Module["preloadedImages"] = {}; // maps url to image data
Module["preloadedAudios"] = {}; // maps url to audio data


var memoryInitializer = null;

// To work around https://bugzilla.mozilla.org/show_bug.cgi?id=1049079, warm up a worker pool before starting up the application.
if (!ENVIRONMENT_IS_PTHREAD) addOnPreRun(function() { addRunDependency('pthreads'); PThread.allocateUnusedWorkers(8, function() { removeRunDependency('pthreads'); }); });

// === Body ===




STATIC_BASE = 8;

STATICTOP = STATIC_BASE + 2400;
  /* global initializers */ __ATINIT__.push();
  

if (!ENVIRONMENT_IS_PTHREAD) {
/* memory initializer */ allocate([2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,70,97,105,108,101,100,32,116,111,32,99,114,101,97,116,101,32,116,104,114,101,97,100,33,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,123,32,99,111,110,115,111,108,101,46,108,111,103,40,39,65,108,108,32,116,104,114,101,97,100,115,32,102,105,110,105,115,104,101,100,46,32,67,111,117,110,116,101,114,32,61,32,39,32,43,32,36,48,32,43,32,39,32,97,115,32,101,120,112,101,99,116,101,100,46,39,41,59,32,125,0,0,0,0,0,123,32,99,111,110,115,111,108,101,46,101,114,114,111,114,40,39,65,108,108,32,116,104,114,101,97,100,115,32,102,105,110,105,115,104,101,100,44,32,98,117,116,32,99,111,117,110,116,101,114,32,61,32,39,32,43,32,36,48,32,43,32,39,32,33,61,32,39,32,43,32,36,49,32,43,32,39,33,39,41,59,32,125,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,99,97,108,108,0,0,0,0,47,85,115,101,114,115,47,106,106,121,108,97,110,107,105,47,101,109,115,100,107,95,103,105,116,47,101,109,115,99,114,105,112,116,101,110,47,105,110,99,111,109,105,110,103,47,115,121,115,116,101,109,47,108,105,98,47,112,116,104,114,101,97,100,47,108,105,98,114,97,114,121,95,112,116,104,114,101,97,100,46,99,0,0,0,0,0,0,101,109,115,99,114,105,112,116,101,110,95,115,121,110,99,95,114,117,110,95,105,110,95,109,97,105,110,95,116,104,114,101,97,100,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,99,97,108,108,95,113,117,101,117,101,95,108,101,110,103,116,104,32,60,32,67,65,76,76,95,81,85,69,85,69,95,83,73,90,69,0,0,0,0,0,112,111,115,116,77,101,115,115,97,103,101,40,123,32,99,109,100,58,32,39,112,114,111,99,101,115,115,81,117,101,117,101,100,77,97,105,110,84,104,114,101,97,100,87,111,114,107,39,32,125,41,0,0,0,0,0,101,109,115,99,114,105,112,116,101,110,95,105,115,95,109,97,105,110,95,114,117,110,116,105,109,101,95,116,104,114,101,97,100,40,41,32,38,38,32,34,101,109,115,99,114,105,112,116,101,110,95,109,97,105,110,95,116,104,114,101,97,100,95,112,114,111,99,101,115,115,95,113,117,101,117,101,100,95,99,97,108,108,115,32,109,117,115,116,32,98,101,32,99,97,108,108,101,100,32,102,114,111,109,32,116,104,101,32,109,97,105,110,32,116,104,114,101,97,100,33,34,0,0,0,0,0,0,0,101,109,115,99,114,105,112,116,101,110,95,109,97,105,110,95,116,104,114,101,97,100,95,112,114,111,99,101,115,115,95,113,117,101,117,101,100,95,99,97,108,108,115,0,0,0,0,0,48,32,38,38,32,34,73,110,118,97,108,105,100,32,69,109,115,99,114,105,112,116,101,110,32,112,116,104,114,101,97,100,32,95,100,111,95,99,97,108,108,32,111,112,99,111,100,101,33,34,0,0,0,0,0,0,95,100,111,95,99,97,108,108,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0], "i8", ALLOC_NONE, Runtime.GLOBAL_BASE);
}





var tempDoublePtr = ENVIRONMENT_IS_PTHREAD ? 0 : Runtime.alignMemory(allocate(12, "i8", ALLOC_STATIC), 8);

assert(tempDoublePtr % 8 == 0);

function copyTempFloat(ptr) { // functions, because inlining this code increases code size too much

  if (!tempDoublePtr) throw "tempDoublePtr not allocated!"; 
  HEAP8[tempDoublePtr] = HEAP8[ptr];

  HEAP8[tempDoublePtr+1] = HEAP8[ptr+1];

  HEAP8[tempDoublePtr+2] = HEAP8[ptr+2];

  HEAP8[tempDoublePtr+3] = HEAP8[ptr+3];

}

function copyTempDouble(ptr) {

  if (!tempDoublePtr) throw "tempDoublePtr not allocated!"; 
  HEAP8[tempDoublePtr] = HEAP8[ptr];

  HEAP8[tempDoublePtr+1] = HEAP8[ptr+1];

  HEAP8[tempDoublePtr+2] = HEAP8[ptr+2];

  HEAP8[tempDoublePtr+3] = HEAP8[ptr+3];

  HEAP8[tempDoublePtr+4] = HEAP8[ptr+4];

  HEAP8[tempDoublePtr+5] = HEAP8[ptr+5];

  HEAP8[tempDoublePtr+6] = HEAP8[ptr+6];

  HEAP8[tempDoublePtr+7] = HEAP8[ptr+7];

}


  
  
  var ERRNO_CODES={EPERM:1,ENOENT:2,ESRCH:3,EINTR:4,EIO:5,ENXIO:6,E2BIG:7,ENOEXEC:8,EBADF:9,ECHILD:10,EAGAIN:11,EWOULDBLOCK:11,ENOMEM:12,EACCES:13,EFAULT:14,ENOTBLK:15,EBUSY:16,EEXIST:17,EXDEV:18,ENODEV:19,ENOTDIR:20,EISDIR:21,EINVAL:22,ENFILE:23,EMFILE:24,ENOTTY:25,ETXTBSY:26,EFBIG:27,ENOSPC:28,ESPIPE:29,EROFS:30,EMLINK:31,EPIPE:32,EDOM:33,ERANGE:34,ENOMSG:42,EIDRM:43,ECHRNG:44,EL2NSYNC:45,EL3HLT:46,EL3RST:47,ELNRNG:48,EUNATCH:49,ENOCSI:50,EL2HLT:51,EDEADLK:35,ENOLCK:37,EBADE:52,EBADR:53,EXFULL:54,ENOANO:55,EBADRQC:56,EBADSLT:57,EDEADLOCK:35,EBFONT:59,ENOSTR:60,ENODATA:61,ETIME:62,ENOSR:63,ENONET:64,ENOPKG:65,EREMOTE:66,ENOLINK:67,EADV:68,ESRMNT:69,ECOMM:70,EPROTO:71,EMULTIHOP:72,EDOTDOT:73,EBADMSG:74,ENOTUNIQ:76,EBADFD:77,EREMCHG:78,ELIBACC:79,ELIBBAD:80,ELIBSCN:81,ELIBMAX:82,ELIBEXEC:83,ENOSYS:38,ENOTEMPTY:39,ENAMETOOLONG:36,ELOOP:40,EOPNOTSUPP:95,EPFNOSUPPORT:96,ECONNRESET:104,ENOBUFS:105,EAFNOSUPPORT:97,EPROTOTYPE:91,ENOTSOCK:88,ENOPROTOOPT:92,ESHUTDOWN:108,ECONNREFUSED:111,EADDRINUSE:98,ECONNABORTED:103,ENETUNREACH:101,ENETDOWN:100,ETIMEDOUT:110,EHOSTDOWN:112,EHOSTUNREACH:113,EINPROGRESS:115,EALREADY:114,EDESTADDRREQ:89,EMSGSIZE:90,EPROTONOSUPPORT:93,ESOCKTNOSUPPORT:94,EADDRNOTAVAIL:99,ENETRESET:102,EISCONN:106,ENOTCONN:107,ETOOMANYREFS:109,EUSERS:87,EDQUOT:122,ESTALE:116,ENOTSUP:95,ENOMEDIUM:123,EILSEQ:84,EOVERFLOW:75,ECANCELED:125,ENOTRECOVERABLE:131,EOWNERDEAD:130,ESTRPIPE:86};
  
  var ERRNO_MESSAGES={0:"Success",1:"Not super-user",2:"No such file or directory",3:"No such process",4:"Interrupted system call",5:"I/O error",6:"No such device or address",7:"Arg list too long",8:"Exec format error",9:"Bad file number",10:"No children",11:"No more processes",12:"Not enough core",13:"Permission denied",14:"Bad address",15:"Block device required",16:"Mount device busy",17:"File exists",18:"Cross-device link",19:"No such device",20:"Not a directory",21:"Is a directory",22:"Invalid argument",23:"Too many open files in system",24:"Too many open files",25:"Not a typewriter",26:"Text file busy",27:"File too large",28:"No space left on device",29:"Illegal seek",30:"Read only file system",31:"Too many links",32:"Broken pipe",33:"Math arg out of domain of func",34:"Math result not representable",35:"File locking deadlock error",36:"File or path name too long",37:"No record locks available",38:"Function not implemented",39:"Directory not empty",40:"Too many symbolic links",42:"No message of desired type",43:"Identifier removed",44:"Channel number out of range",45:"Level 2 not synchronized",46:"Level 3 halted",47:"Level 3 reset",48:"Link number out of range",49:"Protocol driver not attached",50:"No CSI structure available",51:"Level 2 halted",52:"Invalid exchange",53:"Invalid request descriptor",54:"Exchange full",55:"No anode",56:"Invalid request code",57:"Invalid slot",59:"Bad font file fmt",60:"Device not a stream",61:"No data (for no delay io)",62:"Timer expired",63:"Out of streams resources",64:"Machine is not on the network",65:"Package not installed",66:"The object is remote",67:"The link has been severed",68:"Advertise error",69:"Srmount error",70:"Communication error on send",71:"Protocol error",72:"Multihop attempted",73:"Cross mount point (not really error)",74:"Trying to read unreadable message",75:"Value too large for defined data type",76:"Given log. name not unique",77:"f.d. invalid for this operation",78:"Remote address changed",79:"Can   access a needed shared lib",80:"Accessing a corrupted shared lib",81:".lib section in a.out corrupted",82:"Attempting to link in too many libs",83:"Attempting to exec a shared library",84:"Illegal byte sequence",86:"Streams pipe error",87:"Too many users",88:"Socket operation on non-socket",89:"Destination address required",90:"Message too long",91:"Protocol wrong type for socket",92:"Protocol not available",93:"Unknown protocol",94:"Socket type not supported",95:"Not supported",96:"Protocol family not supported",97:"Address family not supported by protocol family",98:"Address already in use",99:"Address not available",100:"Network interface is not configured",101:"Network is unreachable",102:"Connection reset by network",103:"Connection aborted",104:"Connection reset by peer",105:"No buffer space available",106:"Socket is already connected",107:"Socket is not connected",108:"Can't send after socket shutdown",109:"Too many references",110:"Connection timed out",111:"Connection refused",112:"Host is down",113:"Host is unreachable",114:"Socket already connected",115:"Connection already in progress",116:"Stale file handle",122:"Quota exceeded",123:"No medium (in tape drive)",125:"Operation canceled",130:"Previous owner died",131:"State not recoverable"};
  
  
  var ___errno_state=0;function ___setErrNo(value) {
      // For convenient setting and returning of errno.
      HEAP32[((___errno_state)>>2)]=value;
      return value;
    }
  
  var PATH={splitPath:function (filename) {
        var splitPathRe = /^(\/?|)([\s\S]*?)((?:\.{1,2}|[^\/]+?|)(\.[^.\/]*|))(?:[\/]*)$/;
        return splitPathRe.exec(filename).slice(1);
      },normalizeArray:function (parts, allowAboveRoot) {
        // if the path tries to go above the root, `up` ends up > 0
        var up = 0;
        for (var i = parts.length - 1; i >= 0; i--) {
          var last = parts[i];
          if (last === '.') {
            parts.splice(i, 1);
          } else if (last === '..') {
            parts.splice(i, 1);
            up++;
          } else if (up) {
            parts.splice(i, 1);
            up--;
          }
        }
        // if the path is allowed to go above the root, restore leading ..s
        if (allowAboveRoot) {
          for (; up--; up) {
            parts.unshift('..');
          }
        }
        return parts;
      },normalize:function (path) {
        var isAbsolute = path.charAt(0) === '/',
            trailingSlash = path.substr(-1) === '/';
        // Normalize the path
        path = PATH.normalizeArray(path.split('/').filter(function(p) {
          return !!p;
        }), !isAbsolute).join('/');
        if (!path && !isAbsolute) {
          path = '.';
        }
        if (path && trailingSlash) {
          path += '/';
        }
        return (isAbsolute ? '/' : '') + path;
      },dirname:function (path) {
        var result = PATH.splitPath(path),
            root = result[0],
            dir = result[1];
        if (!root && !dir) {
          // No dirname whatsoever
          return '.';
        }
        if (dir) {
          // It has a dirname, strip trailing slash
          dir = dir.substr(0, dir.length - 1);
        }
        return root + dir;
      },basename:function (path) {
        // EMSCRIPTEN return '/'' for '/', not an empty string
        if (path === '/') return '/';
        var lastSlash = path.lastIndexOf('/');
        if (lastSlash === -1) return path;
        return path.substr(lastSlash+1);
      },extname:function (path) {
        return PATH.splitPath(path)[3];
      },join:function () {
        var paths = Array.prototype.slice.call(arguments, 0);
        return PATH.normalize(paths.join('/'));
      },join2:function (l, r) {
        return PATH.normalize(l + '/' + r);
      },resolve:function () {
        var resolvedPath = '',
          resolvedAbsolute = false;
        for (var i = arguments.length - 1; i >= -1 && !resolvedAbsolute; i--) {
          var path = (i >= 0) ? arguments[i] : FS.cwd();
          // Skip empty and invalid entries
          if (typeof path !== 'string') {
            throw new TypeError('Arguments to path.resolve must be strings');
          } else if (!path) {
            return ''; // an invalid portion invalidates the whole thing
          }
          resolvedPath = path + '/' + resolvedPath;
          resolvedAbsolute = path.charAt(0) === '/';
        }
        // At this point the path should be resolved to a full absolute path, but
        // handle relative paths to be safe (might happen when process.cwd() fails)
        resolvedPath = PATH.normalizeArray(resolvedPath.split('/').filter(function(p) {
          return !!p;
        }), !resolvedAbsolute).join('/');
        return ((resolvedAbsolute ? '/' : '') + resolvedPath) || '.';
      },relative:function (from, to) {
        from = PATH.resolve(from).substr(1);
        to = PATH.resolve(to).substr(1);
        function trim(arr) {
          var start = 0;
          for (; start < arr.length; start++) {
            if (arr[start] !== '') break;
          }
          var end = arr.length - 1;
          for (; end >= 0; end--) {
            if (arr[end] !== '') break;
          }
          if (start > end) return [];
          return arr.slice(start, end - start + 1);
        }
        var fromParts = trim(from.split('/'));
        var toParts = trim(to.split('/'));
        var length = Math.min(fromParts.length, toParts.length);
        var samePartsLength = length;
        for (var i = 0; i < length; i++) {
          if (fromParts[i] !== toParts[i]) {
            samePartsLength = i;
            break;
          }
        }
        var outputParts = [];
        for (var i = samePartsLength; i < fromParts.length; i++) {
          outputParts.push('..');
        }
        outputParts = outputParts.concat(toParts.slice(samePartsLength));
        return outputParts.join('/');
      }};
  
  var TTY={ttys:[],init:function () {
        // https://github.com/kripken/emscripten/pull/1555
        // if (ENVIRONMENT_IS_NODE) {
        //   // currently, FS.init does not distinguish if process.stdin is a file or TTY
        //   // device, it always assumes it's a TTY device. because of this, we're forcing
        //   // process.stdin to UTF8 encoding to at least make stdin reading compatible
        //   // with text files until FS.init can be refactored.
        //   process['stdin']['setEncoding']('utf8');
        // }
      },shutdown:function () {
        // https://github.com/kripken/emscripten/pull/1555
        // if (ENVIRONMENT_IS_NODE) {
        //   // inolen: any idea as to why node -e 'process.stdin.read()' wouldn't exit immediately (with process.stdin being a tty)?
        //   // isaacs: because now it's reading from the stream, you've expressed interest in it, so that read() kicks off a _read() which creates a ReadReq operation
        //   // inolen: I thought read() in that case was a synchronous operation that just grabbed some amount of buffered data if it exists?
        //   // isaacs: it is. but it also triggers a _read() call, which calls readStart() on the handle
        //   // isaacs: do process.stdin.pause() and i'd think it'd probably close the pending call
        //   process['stdin']['pause']();
        // }
      },register:function (dev, ops) {
        TTY.ttys[dev] = { input: [], output: [], ops: ops };
        FS.registerDevice(dev, TTY.stream_ops);
      },stream_ops:{open:function (stream) {
          var tty = TTY.ttys[stream.node.rdev];
          if (!tty) {
            throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
          }
          stream.tty = tty;
          stream.seekable = false;
        },close:function (stream) {
          // flush any pending line data
          stream.tty.ops.flush(stream.tty);
        },flush:function (stream) {
          stream.tty.ops.flush(stream.tty);
        },read:function (stream, buffer, offset, length, pos /* ignored */) {
          if (!stream.tty || !stream.tty.ops.get_char) {
            throw new FS.ErrnoError(ERRNO_CODES.ENXIO);
          }
          var bytesRead = 0;
          for (var i = 0; i < length; i++) {
            var result;
            try {
              result = stream.tty.ops.get_char(stream.tty);
            } catch (e) {
              throw new FS.ErrnoError(ERRNO_CODES.EIO);
            }
            if (result === undefined && bytesRead === 0) {
              throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
            }
            if (result === null || result === undefined) break;
            bytesRead++;
            buffer[offset+i] = result;
          }
          if (bytesRead) {
            stream.node.timestamp = Date.now();
          }
          return bytesRead;
        },write:function (stream, buffer, offset, length, pos) {
          if (!stream.tty || !stream.tty.ops.put_char) {
            throw new FS.ErrnoError(ERRNO_CODES.ENXIO);
          }
          for (var i = 0; i < length; i++) {
            try {
              stream.tty.ops.put_char(stream.tty, buffer[offset+i]);
            } catch (e) {
              throw new FS.ErrnoError(ERRNO_CODES.EIO);
            }
          }
          if (length) {
            stream.node.timestamp = Date.now();
          }
          return i;
        }},default_tty_ops:{get_char:function (tty) {
          if (!tty.input.length) {
            var result = null;
            if (ENVIRONMENT_IS_NODE) {
              result = process['stdin']['read']();
              if (!result) {
                if (process['stdin']['_readableState'] && process['stdin']['_readableState']['ended']) {
                  return null;  // EOF
                }
                return undefined;  // no data available
              }
            } else if (typeof window != 'undefined' &&
              typeof window.prompt == 'function') {
              // Browser.
              result = window.prompt('Input: ');  // returns null on cancel
              if (result !== null) {
                result += '\n';
              }
            } else if (typeof readline == 'function') {
              // Command line.
              result = readline();
              if (result !== null) {
                result += '\n';
              }
            }
            if (!result) {
              return null;
            }
            tty.input = intArrayFromString(result, true);
          }
          return tty.input.shift();
        },put_char:function (tty, val) {
          if (val === null || val === 10) {
            Module['print'](UTF8ArrayToString(tty.output, 0));
            tty.output = [];
          } else {
            if (val != 0) tty.output.push(val); // val == 0 would cut text output off in the middle.
          }
        },flush:function (tty) {
          if (tty.output && tty.output.length > 0) {
            Module['print'](UTF8ArrayToString(tty.output, 0));
            tty.output = [];
          }
        }},default_tty1_ops:{put_char:function (tty, val) {
          if (val === null || val === 10) {
            Module['printErr'](UTF8ArrayToString(tty.output, 0));
            tty.output = [];
          } else {
            if (val != 0) tty.output.push(val);
          }
        },flush:function (tty) {
          if (tty.output && tty.output.length > 0) {
            Module['printErr'](UTF8ArrayToString(tty.output, 0));
            tty.output = [];
          }
        }}};
  
  var MEMFS={ops_table:null,mount:function (mount) {
        return MEMFS.createNode(null, '/', 16384 | 511 /* 0777 */, 0);
      },createNode:function (parent, name, mode, dev) {
        if (FS.isBlkdev(mode) || FS.isFIFO(mode)) {
          // no supported
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (!MEMFS.ops_table) {
          MEMFS.ops_table = {
            dir: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr,
                lookup: MEMFS.node_ops.lookup,
                mknod: MEMFS.node_ops.mknod,
                rename: MEMFS.node_ops.rename,
                unlink: MEMFS.node_ops.unlink,
                rmdir: MEMFS.node_ops.rmdir,
                readdir: MEMFS.node_ops.readdir,
                symlink: MEMFS.node_ops.symlink
              },
              stream: {
                llseek: MEMFS.stream_ops.llseek
              }
            },
            file: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr
              },
              stream: {
                llseek: MEMFS.stream_ops.llseek,
                read: MEMFS.stream_ops.read,
                write: MEMFS.stream_ops.write,
                allocate: MEMFS.stream_ops.allocate,
                mmap: MEMFS.stream_ops.mmap
              }
            },
            link: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr,
                readlink: MEMFS.node_ops.readlink
              },
              stream: {}
            },
            chrdev: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr
              },
              stream: FS.chrdev_stream_ops
            }
          };
        }
        var node = FS.createNode(parent, name, mode, dev);
        if (FS.isDir(node.mode)) {
          node.node_ops = MEMFS.ops_table.dir.node;
          node.stream_ops = MEMFS.ops_table.dir.stream;
          node.contents = {};
        } else if (FS.isFile(node.mode)) {
          node.node_ops = MEMFS.ops_table.file.node;
          node.stream_ops = MEMFS.ops_table.file.stream;
          node.usedBytes = 0; // The actual number of bytes used in the typed array, as opposed to contents.buffer.byteLength which gives the whole capacity.
          // When the byte data of the file is populated, this will point to either a typed array, or a normal JS array. Typed arrays are preferred
          // for performance, and used by default. However, typed arrays are not resizable like normal JS arrays are, so there is a small disk size
          // penalty involved for appending file writes that continuously grow a file similar to std::vector capacity vs used -scheme.
          node.contents = null; 
        } else if (FS.isLink(node.mode)) {
          node.node_ops = MEMFS.ops_table.link.node;
          node.stream_ops = MEMFS.ops_table.link.stream;
        } else if (FS.isChrdev(node.mode)) {
          node.node_ops = MEMFS.ops_table.chrdev.node;
          node.stream_ops = MEMFS.ops_table.chrdev.stream;
        }
        node.timestamp = Date.now();
        // add the new node to the parent
        if (parent) {
          parent.contents[name] = node;
        }
        return node;
      },getFileDataAsRegularArray:function (node) {
        if (node.contents && node.contents.subarray) {
          var arr = [];
          for (var i = 0; i < node.usedBytes; ++i) arr.push(node.contents[i]);
          return arr; // Returns a copy of the original data.
        }
        return node.contents; // No-op, the file contents are already in a JS array. Return as-is.
      },getFileDataAsTypedArray:function (node) {
        if (!node.contents) return new Uint8Array;
        if (node.contents.subarray) return node.contents.subarray(0, node.usedBytes); // Make sure to not return excess unused bytes.
        return new Uint8Array(node.contents);
      },expandFileStorage:function (node, newCapacity) {
  
        // If we are asked to expand the size of a file that already exists, revert to using a standard JS array to store the file
        // instead of a typed array. This makes resizing the array more flexible because we can just .push() elements at the back to
        // increase the size.
        if (node.contents && node.contents.subarray && newCapacity > node.contents.length) {
          node.contents = MEMFS.getFileDataAsRegularArray(node);
          node.usedBytes = node.contents.length; // We might be writing to a lazy-loaded file which had overridden this property, so force-reset it.
        }
  
        if (!node.contents || node.contents.subarray) { // Keep using a typed array if creating a new storage, or if old one was a typed array as well.
          var prevCapacity = node.contents ? node.contents.buffer.byteLength : 0;
          if (prevCapacity >= newCapacity) return; // No need to expand, the storage was already large enough.
          // Don't expand strictly to the given requested limit if it's only a very small increase, but instead geometrically grow capacity.
          // For small filesizes (<1MB), perform size*2 geometric increase, but for large sizes, do a much more conservative size*1.125 increase to
          // avoid overshooting the allocation cap by a very large margin.
          var CAPACITY_DOUBLING_MAX = 1024 * 1024;
          newCapacity = Math.max(newCapacity, (prevCapacity * (prevCapacity < CAPACITY_DOUBLING_MAX ? 2.0 : 1.125)) | 0);
          if (prevCapacity != 0) newCapacity = Math.max(newCapacity, 256); // At minimum allocate 256b for each file when expanding.
          var oldContents = node.contents;
          node.contents = new Uint8Array(newCapacity); // Allocate new storage.
          if (node.usedBytes > 0) node.contents.set(oldContents.subarray(0, node.usedBytes), 0); // Copy old data over to the new storage.
          return;
        }
        // Not using a typed array to back the file storage. Use a standard JS array instead.
        if (!node.contents && newCapacity > 0) node.contents = [];
        while (node.contents.length < newCapacity) node.contents.push(0);
      },resizeFileStorage:function (node, newSize) {
        if (node.usedBytes == newSize) return;
        if (newSize == 0) {
          node.contents = null; // Fully decommit when requesting a resize to zero.
          node.usedBytes = 0;
          return;
        }
  
        if (!node.contents || node.contents.subarray) { // Resize a typed array if that is being used as the backing store.
          var oldContents = node.contents;
          node.contents = new Uint8Array(new ArrayBuffer(newSize)); // Allocate new storage.
          if (oldContents) {
            node.contents.set(oldContents.subarray(0, Math.min(newSize, node.usedBytes))); // Copy old data over to the new storage.
          }
          node.usedBytes = newSize;
          return;
        }
        // Backing with a JS array.
        if (!node.contents) node.contents = [];
        if (node.contents.length > newSize) node.contents.length = newSize;
        else while (node.contents.length < newSize) node.contents.push(0);
        node.usedBytes = newSize;
      },node_ops:{getattr:function (node) {
          var attr = {};
          // device numbers reuse inode numbers.
          attr.dev = FS.isChrdev(node.mode) ? node.id : 1;
          attr.ino = node.id;
          attr.mode = node.mode;
          attr.nlink = 1;
          attr.uid = 0;
          attr.gid = 0;
          attr.rdev = node.rdev;
          if (FS.isDir(node.mode)) {
            attr.size = 4096;
          } else if (FS.isFile(node.mode)) {
            attr.size = node.usedBytes;
          } else if (FS.isLink(node.mode)) {
            attr.size = node.link.length;
          } else {
            attr.size = 0;
          }
          attr.atime = new Date(node.timestamp);
          attr.mtime = new Date(node.timestamp);
          attr.ctime = new Date(node.timestamp);
          // NOTE: In our implementation, st_blocks = Math.ceil(st_size/st_blksize),
          //       but this is not required by the standard.
          attr.blksize = 4096;
          attr.blocks = Math.ceil(attr.size / attr.blksize);
          return attr;
        },setattr:function (node, attr) {
          if (attr.mode !== undefined) {
            node.mode = attr.mode;
          }
          if (attr.timestamp !== undefined) {
            node.timestamp = attr.timestamp;
          }
          if (attr.size !== undefined) {
            MEMFS.resizeFileStorage(node, attr.size);
          }
        },lookup:function (parent, name) {
          throw FS.genericErrors[ERRNO_CODES.ENOENT];
        },mknod:function (parent, name, mode, dev) {
          return MEMFS.createNode(parent, name, mode, dev);
        },rename:function (old_node, new_dir, new_name) {
          // if we're overwriting a directory at new_name, make sure it's empty.
          if (FS.isDir(old_node.mode)) {
            var new_node;
            try {
              new_node = FS.lookupNode(new_dir, new_name);
            } catch (e) {
            }
            if (new_node) {
              for (var i in new_node.contents) {
                throw new FS.ErrnoError(ERRNO_CODES.ENOTEMPTY);
              }
            }
          }
          // do the internal rewiring
          delete old_node.parent.contents[old_node.name];
          old_node.name = new_name;
          new_dir.contents[new_name] = old_node;
          old_node.parent = new_dir;
        },unlink:function (parent, name) {
          delete parent.contents[name];
        },rmdir:function (parent, name) {
          var node = FS.lookupNode(parent, name);
          for (var i in node.contents) {
            throw new FS.ErrnoError(ERRNO_CODES.ENOTEMPTY);
          }
          delete parent.contents[name];
        },readdir:function (node) {
          var entries = ['.', '..']
          for (var key in node.contents) {
            if (!node.contents.hasOwnProperty(key)) {
              continue;
            }
            entries.push(key);
          }
          return entries;
        },symlink:function (parent, newname, oldpath) {
          var node = MEMFS.createNode(parent, newname, 511 /* 0777 */ | 40960, 0);
          node.link = oldpath;
          return node;
        },readlink:function (node) {
          if (!FS.isLink(node.mode)) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
          return node.link;
        }},stream_ops:{read:function (stream, buffer, offset, length, position) {
          var contents = stream.node.contents;
          if (position >= stream.node.usedBytes) return 0;
          var size = Math.min(stream.node.usedBytes - position, length);
          assert(size >= 0);
          if (size > 8 && contents.subarray) { // non-trivial, and typed array
            buffer.set(contents.subarray(position, position + size), offset);
          } else
          {
            for (var i = 0; i < size; i++) buffer[offset + i] = contents[position + i];
          }
          return size;
        },write:function (stream, buffer, offset, length, position, canOwn) {
          if (!length) return 0;
          var node = stream.node;
          node.timestamp = Date.now();
  
          if (buffer.subarray && (!node.contents || node.contents.subarray)) { // This write is from a typed array to a typed array?
            if (canOwn) { // Can we just reuse the buffer we are given?
              assert(position === 0, 'canOwn must imply no weird position inside the file');
              node.contents = buffer.subarray(offset, offset + length);
              node.usedBytes = length;
              return length;
            } else if (node.usedBytes === 0 && position === 0) { // If this is a simple first write to an empty file, do a fast set since we don't need to care about old data.
              node.contents = new Uint8Array(buffer.subarray(offset, offset + length));
              node.usedBytes = length;
              return length;
            } else if (position + length <= node.usedBytes) { // Writing to an already allocated and used subrange of the file?
              node.contents.set(buffer.subarray(offset, offset + length), position);
              return length;
            }
          }
          // Appending to an existing file and we need to reallocate, or source data did not come as a typed array.
          MEMFS.expandFileStorage(node, position+length);
          if (node.contents.subarray && buffer.subarray) node.contents.set(buffer.subarray(offset, offset + length), position); // Use typed array write if available.
          else
            for (var i = 0; i < length; i++) {
             node.contents[position + i] = buffer[offset + i]; // Or fall back to manual write if not.
            }
          node.usedBytes = Math.max(node.usedBytes, position+length);
          return length;
        },llseek:function (stream, offset, whence) {
          var position = offset;
          if (whence === 1) {  // SEEK_CUR.
            position += stream.position;
          } else if (whence === 2) {  // SEEK_END.
            if (FS.isFile(stream.node.mode)) {
              position += stream.node.usedBytes;
            }
          }
          if (position < 0) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
          return position;
        },allocate:function (stream, offset, length) {
          MEMFS.expandFileStorage(stream.node, offset + length);
          stream.node.usedBytes = Math.max(stream.node.usedBytes, offset + length);
        },mmap:function (stream, buffer, offset, length, position, prot, flags) {
          if (!FS.isFile(stream.node.mode)) {
            throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
          }
          var ptr;
          var allocated;
          var contents = stream.node.contents;
          // Only make a new copy when MAP_PRIVATE is specified.
          if ( !(flags & 2) &&
                (contents.buffer === buffer || contents.buffer === buffer.buffer) ) {
            // We can't emulate MAP_SHARED when the file is not backed by the buffer
            // we're mapping to (e.g. the HEAP buffer).
            allocated = false;
            ptr = contents.byteOffset;
          } else {
            // Try to avoid unnecessary slices.
            if (position > 0 || position + length < stream.node.usedBytes) {
              if (contents.subarray) {
                contents = contents.subarray(position, position + length);
              } else {
                contents = Array.prototype.slice.call(contents, position, position + length);
              }
            }
            allocated = true;
            ptr = _malloc(length);
            if (!ptr) {
              throw new FS.ErrnoError(ERRNO_CODES.ENOMEM);
            }
            buffer.set(contents, ptr);
          }
          return { ptr: ptr, allocated: allocated };
        }}};
  
  var IDBFS={dbs:{},indexedDB:function () {
        if (typeof indexedDB !== 'undefined') return indexedDB;
        var ret = null;
        if (typeof window === 'object') ret = window.indexedDB || window.mozIndexedDB || window.webkitIndexedDB || window.msIndexedDB;
        assert(ret, 'IDBFS used, but indexedDB not supported');
        return ret;
      },DB_VERSION:21,DB_STORE_NAME:"FILE_DATA",mount:function (mount) {
        // reuse all of the core MEMFS functionality
        return MEMFS.mount.apply(null, arguments);
      },syncfs:function (mount, populate, callback) {
        IDBFS.getLocalSet(mount, function(err, local) {
          if (err) return callback(err);
  
          IDBFS.getRemoteSet(mount, function(err, remote) {
            if (err) return callback(err);
  
            var src = populate ? remote : local;
            var dst = populate ? local : remote;
  
            IDBFS.reconcile(src, dst, callback);
          });
        });
      },getDB:function (name, callback) {
        // check the cache first
        var db = IDBFS.dbs[name];
        if (db) {
          return callback(null, db);
        }
  
        var req;
        try {
          req = IDBFS.indexedDB().open(name, IDBFS.DB_VERSION);
        } catch (e) {
          return callback(e);
        }
        req.onupgradeneeded = function(e) {
          var db = e.target.result;
          var transaction = e.target.transaction;
  
          var fileStore;
  
          if (db.objectStoreNames.contains(IDBFS.DB_STORE_NAME)) {
            fileStore = transaction.objectStore(IDBFS.DB_STORE_NAME);
          } else {
            fileStore = db.createObjectStore(IDBFS.DB_STORE_NAME);
          }
  
          if (!fileStore.indexNames.contains('timestamp')) {
            fileStore.createIndex('timestamp', 'timestamp', { unique: false });
          }
        };
        req.onsuccess = function() {
          db = req.result;
  
          // add to the cache
          IDBFS.dbs[name] = db;
          callback(null, db);
        };
        req.onerror = function(e) {
          callback(this.error);
          e.preventDefault();
        };
      },getLocalSet:function (mount, callback) {
        var entries = {};
  
        function isRealDir(p) {
          return p !== '.' && p !== '..';
        };
        function toAbsolute(root) {
          return function(p) {
            return PATH.join2(root, p);
          }
        };
  
        var check = FS.readdir(mount.mountpoint).filter(isRealDir).map(toAbsolute(mount.mountpoint));
  
        while (check.length) {
          var path = check.pop();
          var stat;
  
          try {
            stat = FS.stat(path);
          } catch (e) {
            return callback(e);
          }
  
          if (FS.isDir(stat.mode)) {
            check.push.apply(check, FS.readdir(path).filter(isRealDir).map(toAbsolute(path)));
          }
  
          entries[path] = { timestamp: stat.mtime };
        }
  
        return callback(null, { type: 'local', entries: entries });
      },getRemoteSet:function (mount, callback) {
        var entries = {};
  
        IDBFS.getDB(mount.mountpoint, function(err, db) {
          if (err) return callback(err);
  
          var transaction = db.transaction([IDBFS.DB_STORE_NAME], 'readonly');
          transaction.onerror = function(e) {
            callback(this.error);
            e.preventDefault();
          };
  
          var store = transaction.objectStore(IDBFS.DB_STORE_NAME);
          var index = store.index('timestamp');
  
          index.openKeyCursor().onsuccess = function(event) {
            var cursor = event.target.result;
  
            if (!cursor) {
              return callback(null, { type: 'remote', db: db, entries: entries });
            }
  
            entries[cursor.primaryKey] = { timestamp: cursor.key };
  
            cursor.continue();
          };
        });
      },loadLocalEntry:function (path, callback) {
        var stat, node;
  
        try {
          var lookup = FS.lookupPath(path);
          node = lookup.node;
          stat = FS.stat(path);
        } catch (e) {
          return callback(e);
        }
  
        if (FS.isDir(stat.mode)) {
          return callback(null, { timestamp: stat.mtime, mode: stat.mode });
        } else if (FS.isFile(stat.mode)) {
          // Performance consideration: storing a normal JavaScript array to a IndexedDB is much slower than storing a typed array.
          // Therefore always convert the file contents to a typed array first before writing the data to IndexedDB.
          node.contents = MEMFS.getFileDataAsTypedArray(node);
          return callback(null, { timestamp: stat.mtime, mode: stat.mode, contents: node.contents });
        } else {
          return callback(new Error('node type not supported'));
        }
      },storeLocalEntry:function (path, entry, callback) {
        try {
          if (FS.isDir(entry.mode)) {
            FS.mkdir(path, entry.mode);
          } else if (FS.isFile(entry.mode)) {
            FS.writeFile(path, entry.contents, { encoding: 'binary', canOwn: true });
          } else {
            return callback(new Error('node type not supported'));
          }
  
          FS.chmod(path, entry.mode);
          FS.utime(path, entry.timestamp, entry.timestamp);
        } catch (e) {
          return callback(e);
        }
  
        callback(null);
      },removeLocalEntry:function (path, callback) {
        try {
          var lookup = FS.lookupPath(path);
          var stat = FS.stat(path);
  
          if (FS.isDir(stat.mode)) {
            FS.rmdir(path);
          } else if (FS.isFile(stat.mode)) {
            FS.unlink(path);
          }
        } catch (e) {
          return callback(e);
        }
  
        callback(null);
      },loadRemoteEntry:function (store, path, callback) {
        var req = store.get(path);
        req.onsuccess = function(event) { callback(null, event.target.result); };
        req.onerror = function(e) {
          callback(this.error);
          e.preventDefault();
        };
      },storeRemoteEntry:function (store, path, entry, callback) {
        var req = store.put(entry, path);
        req.onsuccess = function() { callback(null); };
        req.onerror = function(e) {
          callback(this.error);
          e.preventDefault();
        };
      },removeRemoteEntry:function (store, path, callback) {
        var req = store.delete(path);
        req.onsuccess = function() { callback(null); };
        req.onerror = function(e) {
          callback(this.error);
          e.preventDefault();
        };
      },reconcile:function (src, dst, callback) {
        var total = 0;
  
        var create = [];
        Object.keys(src.entries).forEach(function (key) {
          var e = src.entries[key];
          var e2 = dst.entries[key];
          if (!e2 || e.timestamp > e2.timestamp) {
            create.push(key);
            total++;
          }
        });
  
        var remove = [];
        Object.keys(dst.entries).forEach(function (key) {
          var e = dst.entries[key];
          var e2 = src.entries[key];
          if (!e2) {
            remove.push(key);
            total++;
          }
        });
  
        if (!total) {
          return callback(null);
        }
  
        var errored = false;
        var completed = 0;
        var db = src.type === 'remote' ? src.db : dst.db;
        var transaction = db.transaction([IDBFS.DB_STORE_NAME], 'readwrite');
        var store = transaction.objectStore(IDBFS.DB_STORE_NAME);
  
        function done(err) {
          if (err) {
            if (!done.errored) {
              done.errored = true;
              return callback(err);
            }
            return;
          }
          if (++completed >= total) {
            return callback(null);
          }
        };
  
        transaction.onerror = function(e) {
          done(this.error);
          e.preventDefault();
        };
  
        // sort paths in ascending order so directory entries are created
        // before the files inside them
        create.sort().forEach(function (path) {
          if (dst.type === 'local') {
            IDBFS.loadRemoteEntry(store, path, function (err, entry) {
              if (err) return done(err);
              IDBFS.storeLocalEntry(path, entry, done);
            });
          } else {
            IDBFS.loadLocalEntry(path, function (err, entry) {
              if (err) return done(err);
              IDBFS.storeRemoteEntry(store, path, entry, done);
            });
          }
        });
  
        // sort paths in descending order so files are deleted before their
        // parent directories
        remove.sort().reverse().forEach(function(path) {
          if (dst.type === 'local') {
            IDBFS.removeLocalEntry(path, done);
          } else {
            IDBFS.removeRemoteEntry(store, path, done);
          }
        });
      }};
  
  var NODEFS={isWindows:false,staticInit:function () {
        NODEFS.isWindows = !!process.platform.match(/^win/);
      },mount:function (mount) {
        assert(ENVIRONMENT_IS_NODE);
        return NODEFS.createNode(null, '/', NODEFS.getMode(mount.opts.root), 0);
      },createNode:function (parent, name, mode, dev) {
        if (!FS.isDir(mode) && !FS.isFile(mode) && !FS.isLink(mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var node = FS.createNode(parent, name, mode);
        node.node_ops = NODEFS.node_ops;
        node.stream_ops = NODEFS.stream_ops;
        return node;
      },getMode:function (path) {
        var stat;
        try {
          stat = fs.lstatSync(path);
          if (NODEFS.isWindows) {
            // On Windows, directories return permission bits 'rw-rw-rw-', even though they have 'rwxrwxrwx', so 
            // propagate write bits to execute bits.
            stat.mode = stat.mode | ((stat.mode & 146) >> 1);
          }
        } catch (e) {
          if (!e.code) throw e;
          throw new FS.ErrnoError(ERRNO_CODES[e.code]);
        }
        return stat.mode;
      },realPath:function (node) {
        var parts = [];
        while (node.parent !== node) {
          parts.push(node.name);
          node = node.parent;
        }
        parts.push(node.mount.opts.root);
        parts.reverse();
        return PATH.join.apply(null, parts);
      },flagsToPermissionStringMap:{0:"r",1:"r+",2:"r+",64:"r",65:"r+",66:"r+",129:"rx+",193:"rx+",514:"w+",577:"w",578:"w+",705:"wx",706:"wx+",1024:"a",1025:"a",1026:"a+",1089:"a",1090:"a+",1153:"ax",1154:"ax+",1217:"ax",1218:"ax+",4096:"rs",4098:"rs+"},flagsToPermissionString:function (flags) {
        if (flags in NODEFS.flagsToPermissionStringMap) {
          return NODEFS.flagsToPermissionStringMap[flags];
        } else {
          return flags;
        }
      },node_ops:{getattr:function (node) {
          var path = NODEFS.realPath(node);
          var stat;
          try {
            stat = fs.lstatSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
          // node.js v0.10.20 doesn't report blksize and blocks on Windows. Fake them with default blksize of 4096.
          // See http://support.microsoft.com/kb/140365
          if (NODEFS.isWindows && !stat.blksize) {
            stat.blksize = 4096;
          }
          if (NODEFS.isWindows && !stat.blocks) {
            stat.blocks = (stat.size+stat.blksize-1)/stat.blksize|0;
          }
          return {
            dev: stat.dev,
            ino: stat.ino,
            mode: stat.mode,
            nlink: stat.nlink,
            uid: stat.uid,
            gid: stat.gid,
            rdev: stat.rdev,
            size: stat.size,
            atime: stat.atime,
            mtime: stat.mtime,
            ctime: stat.ctime,
            blksize: stat.blksize,
            blocks: stat.blocks
          };
        },setattr:function (node, attr) {
          var path = NODEFS.realPath(node);
          try {
            if (attr.mode !== undefined) {
              fs.chmodSync(path, attr.mode);
              // update the common node structure mode as well
              node.mode = attr.mode;
            }
            if (attr.timestamp !== undefined) {
              var date = new Date(attr.timestamp);
              fs.utimesSync(path, date, date);
            }
            if (attr.size !== undefined) {
              fs.truncateSync(path, attr.size);
            }
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },lookup:function (parent, name) {
          var path = PATH.join2(NODEFS.realPath(parent), name);
          var mode = NODEFS.getMode(path);
          return NODEFS.createNode(parent, name, mode);
        },mknod:function (parent, name, mode, dev) {
          var node = NODEFS.createNode(parent, name, mode, dev);
          // create the backing node for this in the fs root as well
          var path = NODEFS.realPath(node);
          try {
            if (FS.isDir(node.mode)) {
              fs.mkdirSync(path, node.mode);
            } else {
              fs.writeFileSync(path, '', { mode: node.mode });
            }
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
          return node;
        },rename:function (oldNode, newDir, newName) {
          var oldPath = NODEFS.realPath(oldNode);
          var newPath = PATH.join2(NODEFS.realPath(newDir), newName);
          try {
            fs.renameSync(oldPath, newPath);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },unlink:function (parent, name) {
          var path = PATH.join2(NODEFS.realPath(parent), name);
          try {
            fs.unlinkSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },rmdir:function (parent, name) {
          var path = PATH.join2(NODEFS.realPath(parent), name);
          try {
            fs.rmdirSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },readdir:function (node) {
          var path = NODEFS.realPath(node);
          try {
            return fs.readdirSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },symlink:function (parent, newName, oldPath) {
          var newPath = PATH.join2(NODEFS.realPath(parent), newName);
          try {
            fs.symlinkSync(oldPath, newPath);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },readlink:function (node) {
          var path = NODEFS.realPath(node);
          try {
            return fs.readlinkSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        }},stream_ops:{open:function (stream) {
          var path = NODEFS.realPath(stream.node);
          try {
            if (FS.isFile(stream.node.mode)) {
              stream.nfd = fs.openSync(path, NODEFS.flagsToPermissionString(stream.flags));
            }
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },close:function (stream) {
          try {
            if (FS.isFile(stream.node.mode) && stream.nfd) {
              fs.closeSync(stream.nfd);
            }
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },read:function (stream, buffer, offset, length, position) {
          if (length === 0) return 0; // node errors on 0 length reads
          // FIXME this is terrible.
          var nbuffer = new Buffer(length);
          var res;
          try {
            res = fs.readSync(stream.nfd, nbuffer, 0, length, position);
          } catch (e) {
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
          if (res > 0) {
            for (var i = 0; i < res; i++) {
              buffer[offset + i] = nbuffer[i];
            }
          }
          return res;
        },write:function (stream, buffer, offset, length, position) {
          // FIXME this is terrible.
          var nbuffer = new Buffer(buffer.subarray(offset, offset + length));
          var res;
          try {
            res = fs.writeSync(stream.nfd, nbuffer, 0, length, position);
          } catch (e) {
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
          return res;
        },llseek:function (stream, offset, whence) {
          var position = offset;
          if (whence === 1) {  // SEEK_CUR.
            position += stream.position;
          } else if (whence === 2) {  // SEEK_END.
            if (FS.isFile(stream.node.mode)) {
              try {
                var stat = fs.fstatSync(stream.nfd);
                position += stat.size;
              } catch (e) {
                throw new FS.ErrnoError(ERRNO_CODES[e.code]);
              }
            }
          }
  
          if (position < 0) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
  
          return position;
        }}};
  
  var _stdin=ENVIRONMENT_IS_PTHREAD?-1:allocate(1, "i32*", ALLOC_STATIC);
  
  var _stdout=ENVIRONMENT_IS_PTHREAD?-1:allocate(1, "i32*", ALLOC_STATIC);
  
  var _stderr=ENVIRONMENT_IS_PTHREAD?-1:allocate(1, "i32*", ALLOC_STATIC);
  
  function _fflush(stream) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_1(78, stream);
      // int fflush(FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fflush.html
  
      /*
      // Disabled, see https://github.com/kripken/emscripten/issues/2770
      stream = FS.getStreamFromPtr(stream);
      if (stream.stream_ops.flush) {
        stream.stream_ops.flush(stream);
      }
      */
    }var FS={root:null,mounts:[],devices:[null],streams:[],nextInode:1,nameTable:null,currentPath:"/",initialized:false,ignorePermissions:true,trackingDelegate:{},tracking:{openFlags:{READ:1,WRITE:2}},ErrnoError:null,genericErrors:{},handleFSError:function (e) {
        if (!(e instanceof FS.ErrnoError)) throw e + ' : ' + stackTrace();
        return ___setErrNo(e.errno);
      },lookupPath:function (path, opts) {
        path = PATH.resolve(FS.cwd(), path);
        opts = opts || {};
  
        if (!path) return { path: '', node: null };
  
        var defaults = {
          follow_mount: true,
          recurse_count: 0
        };
        for (var key in defaults) {
          if (opts[key] === undefined) {
            opts[key] = defaults[key];
          }
        }
  
        if (opts.recurse_count > 8) {  // max recursive lookup of 8
          throw new FS.ErrnoError(ERRNO_CODES.ELOOP);
        }
  
        // split the path
        var parts = PATH.normalizeArray(path.split('/').filter(function(p) {
          return !!p;
        }), false);
  
        // start at the root
        var current = FS.root;
        var current_path = '/';
  
        for (var i = 0; i < parts.length; i++) {
          var islast = (i === parts.length-1);
          if (islast && opts.parent) {
            // stop resolving
            break;
          }
  
          current = FS.lookupNode(current, parts[i]);
          current_path = PATH.join2(current_path, parts[i]);
  
          // jump to the mount's root node if this is a mountpoint
          if (FS.isMountpoint(current)) {
            if (!islast || (islast && opts.follow_mount)) {
              current = current.mounted.root;
            }
          }
  
          // by default, lookupPath will not follow a symlink if it is the final path component.
          // setting opts.follow = true will override this behavior.
          if (!islast || opts.follow) {
            var count = 0;
            while (FS.isLink(current.mode)) {
              var link = FS.readlink(current_path);
              current_path = PATH.resolve(PATH.dirname(current_path), link);
              
              var lookup = FS.lookupPath(current_path, { recurse_count: opts.recurse_count });
              current = lookup.node;
  
              if (count++ > 40) {  // limit max consecutive symlinks to 40 (SYMLOOP_MAX).
                throw new FS.ErrnoError(ERRNO_CODES.ELOOP);
              }
            }
          }
        }
  
        return { path: current_path, node: current };
      },getPath:function (node) {
        var path;
        while (true) {
          if (FS.isRoot(node)) {
            var mount = node.mount.mountpoint;
            if (!path) return mount;
            return mount[mount.length-1] !== '/' ? mount + '/' + path : mount + path;
          }
          path = path ? node.name + '/' + path : node.name;
          node = node.parent;
        }
      },hashName:function (parentid, name) {
        var hash = 0;
  
  
        for (var i = 0; i < name.length; i++) {
          hash = ((hash << 5) - hash + name.charCodeAt(i)) | 0;
        }
        return ((parentid + hash) >>> 0) % FS.nameTable.length;
      },hashAddNode:function (node) {
        var hash = FS.hashName(node.parent.id, node.name);
        node.name_next = FS.nameTable[hash];
        FS.nameTable[hash] = node;
      },hashRemoveNode:function (node) {
        var hash = FS.hashName(node.parent.id, node.name);
        if (FS.nameTable[hash] === node) {
          FS.nameTable[hash] = node.name_next;
        } else {
          var current = FS.nameTable[hash];
          while (current) {
            if (current.name_next === node) {
              current.name_next = node.name_next;
              break;
            }
            current = current.name_next;
          }
        }
      },lookupNode:function (parent, name) {
        var err = FS.mayLookup(parent);
        if (err) {
          throw new FS.ErrnoError(err, parent);
        }
        var hash = FS.hashName(parent.id, name);
        for (var node = FS.nameTable[hash]; node; node = node.name_next) {
          var nodeName = node.name;
          if (node.parent.id === parent.id && nodeName === name) {
            return node;
          }
        }
        // if we failed to find it in the cache, call into the VFS
        return FS.lookup(parent, name);
      },createNode:function (parent, name, mode, rdev) {
        if (!FS.FSNode) {
          FS.FSNode = function(parent, name, mode, rdev) {
            if (!parent) {
              parent = this;  // root node sets parent to itself
            }
            this.parent = parent;
            this.mount = parent.mount;
            this.mounted = null;
            this.id = FS.nextInode++;
            this.name = name;
            this.mode = mode;
            this.node_ops = {};
            this.stream_ops = {};
            this.rdev = rdev;
          };
  
          FS.FSNode.prototype = {};
  
          // compatibility
          var readMode = 292 | 73;
          var writeMode = 146;
  
          // NOTE we must use Object.defineProperties instead of individual calls to
          // Object.defineProperty in order to make closure compiler happy
          Object.defineProperties(FS.FSNode.prototype, {
            read: {
              get: function() { return (this.mode & readMode) === readMode; },
              set: function(val) { val ? this.mode |= readMode : this.mode &= ~readMode; }
            },
            write: {
              get: function() { return (this.mode & writeMode) === writeMode; },
              set: function(val) { val ? this.mode |= writeMode : this.mode &= ~writeMode; }
            },
            isFolder: {
              get: function() { return FS.isDir(this.mode); }
            },
            isDevice: {
              get: function() { return FS.isChrdev(this.mode); }
            }
          });
        }
  
        var node = new FS.FSNode(parent, name, mode, rdev);
  
        FS.hashAddNode(node);
  
        return node;
      },destroyNode:function (node) {
        FS.hashRemoveNode(node);
      },isRoot:function (node) {
        return node === node.parent;
      },isMountpoint:function (node) {
        return !!node.mounted;
      },isFile:function (mode) {
        return (mode & 61440) === 32768;
      },isDir:function (mode) {
        return (mode & 61440) === 16384;
      },isLink:function (mode) {
        return (mode & 61440) === 40960;
      },isChrdev:function (mode) {
        return (mode & 61440) === 8192;
      },isBlkdev:function (mode) {
        return (mode & 61440) === 24576;
      },isFIFO:function (mode) {
        return (mode & 61440) === 4096;
      },isSocket:function (mode) {
        return (mode & 49152) === 49152;
      },flagModes:{"r":0,"rs":1052672,"r+":2,"w":577,"wx":705,"xw":705,"w+":578,"wx+":706,"xw+":706,"a":1089,"ax":1217,"xa":1217,"a+":1090,"ax+":1218,"xa+":1218},modeStringToFlags:function (str) {
        var flags = FS.flagModes[str];
        if (typeof flags === 'undefined') {
          throw new Error('Unknown file open mode: ' + str);
        }
        return flags;
      },flagsToPermissionString:function (flag) {
        var accmode = flag & 2097155;
        var perms = ['r', 'w', 'rw'][accmode];
        if ((flag & 512)) {
          perms += 'w';
        }
        return perms;
      },nodePermissions:function (node, perms) {
        if (FS.ignorePermissions) {
          return 0;
        }
        // return 0 if any user, group or owner bits are set.
        if (perms.indexOf('r') !== -1 && !(node.mode & 292)) {
          return ERRNO_CODES.EACCES;
        } else if (perms.indexOf('w') !== -1 && !(node.mode & 146)) {
          return ERRNO_CODES.EACCES;
        } else if (perms.indexOf('x') !== -1 && !(node.mode & 73)) {
          return ERRNO_CODES.EACCES;
        }
        return 0;
      },mayLookup:function (dir) {
        var err = FS.nodePermissions(dir, 'x');
        if (err) return err;
        if (!dir.node_ops.lookup) return ERRNO_CODES.EACCES;
        return 0;
      },mayCreate:function (dir, name) {
        try {
          var node = FS.lookupNode(dir, name);
          return ERRNO_CODES.EEXIST;
        } catch (e) {
        }
        return FS.nodePermissions(dir, 'wx');
      },mayDelete:function (dir, name, isdir) {
        var node;
        try {
          node = FS.lookupNode(dir, name);
        } catch (e) {
          return e.errno;
        }
        var err = FS.nodePermissions(dir, 'wx');
        if (err) {
          return err;
        }
        if (isdir) {
          if (!FS.isDir(node.mode)) {
            return ERRNO_CODES.ENOTDIR;
          }
          if (FS.isRoot(node) || FS.getPath(node) === FS.cwd()) {
            return ERRNO_CODES.EBUSY;
          }
        } else {
          if (FS.isDir(node.mode)) {
            return ERRNO_CODES.EISDIR;
          }
        }
        return 0;
      },mayOpen:function (node, flags) {
        if (!node) {
          return ERRNO_CODES.ENOENT;
        }
        if (FS.isLink(node.mode)) {
          return ERRNO_CODES.ELOOP;
        } else if (FS.isDir(node.mode)) {
          if ((flags & 2097155) !== 0 ||  // opening for write
              (flags & 512)) {
            return ERRNO_CODES.EISDIR;
          }
        }
        return FS.nodePermissions(node, FS.flagsToPermissionString(flags));
      },MAX_OPEN_FDS:4096,nextfd:function (fd_start, fd_end) {
        fd_start = fd_start || 0;
        fd_end = fd_end || FS.MAX_OPEN_FDS;
        for (var fd = fd_start; fd <= fd_end; fd++) {
          if (!FS.streams[fd]) {
            return fd;
          }
        }
        throw new FS.ErrnoError(ERRNO_CODES.EMFILE);
      },getStream:function (fd) {
        return FS.streams[fd];
      },createStream:function (stream, fd_start, fd_end) {
        if (!FS.FSStream) {
          FS.FSStream = function(){};
          FS.FSStream.prototype = {};
          // compatibility
          Object.defineProperties(FS.FSStream.prototype, {
            object: {
              get: function() { return this.node; },
              set: function(val) { this.node = val; }
            },
            isRead: {
              get: function() { return (this.flags & 2097155) !== 1; }
            },
            isWrite: {
              get: function() { return (this.flags & 2097155) !== 0; }
            },
            isAppend: {
              get: function() { return (this.flags & 1024); }
            }
          });
        }
        // clone it, so we can return an instance of FSStream
        var newStream = new FS.FSStream();
        for (var p in stream) {
          newStream[p] = stream[p];
        }
        stream = newStream;
        var fd = FS.nextfd(fd_start, fd_end);
        stream.fd = fd;
        FS.streams[fd] = stream;
        return stream;
      },closeStream:function (fd) {
        FS.streams[fd] = null;
      },getStreamFromPtr:function (ptr) {
        return FS.streams[ptr - 1];
      },getPtrForStream:function (stream) {
        return stream ? stream.fd + 1 : 0;
      },chrdev_stream_ops:{open:function (stream) {
          var device = FS.getDevice(stream.node.rdev);
          // override node's stream ops with the device's
          stream.stream_ops = device.stream_ops;
          // forward the open call
          if (stream.stream_ops.open) {
            stream.stream_ops.open(stream);
          }
        },llseek:function () {
          throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
        }},major:function (dev) {
        return ((dev) >> 8);
      },minor:function (dev) {
        return ((dev) & 0xff);
      },makedev:function (ma, mi) {
        return ((ma) << 8 | (mi));
      },registerDevice:function (dev, ops) {
        FS.devices[dev] = { stream_ops: ops };
      },getDevice:function (dev) {
        return FS.devices[dev];
      },getMounts:function (mount) {
        var mounts = [];
        var check = [mount];
  
        while (check.length) {
          var m = check.pop();
  
          mounts.push(m);
  
          check.push.apply(check, m.mounts);
        }
  
        return mounts;
      },syncfs:function (populate, callback) {
        if (typeof(populate) === 'function') {
          callback = populate;
          populate = false;
        }
  
        var mounts = FS.getMounts(FS.root.mount);
        var completed = 0;
  
        function done(err) {
          if (err) {
            if (!done.errored) {
              done.errored = true;
              return callback(err);
            }
            return;
          }
          if (++completed >= mounts.length) {
            callback(null);
          }
        };
  
        // sync all mounts
        mounts.forEach(function (mount) {
          if (!mount.type.syncfs) {
            return done(null);
          }
          mount.type.syncfs(mount, populate, done);
        });
      },mount:function (type, opts, mountpoint) {
        var root = mountpoint === '/';
        var pseudo = !mountpoint;
        var node;
  
        if (root && FS.root) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        } else if (!root && !pseudo) {
          var lookup = FS.lookupPath(mountpoint, { follow_mount: false });
  
          mountpoint = lookup.path;  // use the absolute path
          node = lookup.node;
  
          if (FS.isMountpoint(node)) {
            throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
          }
  
          if (!FS.isDir(node.mode)) {
            throw new FS.ErrnoError(ERRNO_CODES.ENOTDIR);
          }
        }
  
        var mount = {
          type: type,
          opts: opts,
          mountpoint: mountpoint,
          mounts: []
        };
  
        // create a root node for the fs
        var mountRoot = type.mount(mount);
        mountRoot.mount = mount;
        mount.root = mountRoot;
  
        if (root) {
          FS.root = mountRoot;
        } else if (node) {
          // set as a mountpoint
          node.mounted = mount;
  
          // add the new mount to the current mount's children
          if (node.mount) {
            node.mount.mounts.push(mount);
          }
        }
  
        return mountRoot;
      },unmount:function (mountpoint) {
        var lookup = FS.lookupPath(mountpoint, { follow_mount: false });
  
        if (!FS.isMountpoint(lookup.node)) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
  
        // destroy the nodes for this mount, and all its child mounts
        var node = lookup.node;
        var mount = node.mounted;
        var mounts = FS.getMounts(mount);
  
        Object.keys(FS.nameTable).forEach(function (hash) {
          var current = FS.nameTable[hash];
  
          while (current) {
            var next = current.name_next;
  
            if (mounts.indexOf(current.mount) !== -1) {
              FS.destroyNode(current);
            }
  
            current = next;
          }
        });
  
        // no longer a mountpoint
        node.mounted = null;
  
        // remove this mount from the child mounts
        var idx = node.mount.mounts.indexOf(mount);
        assert(idx !== -1);
        node.mount.mounts.splice(idx, 1);
      },lookup:function (parent, name) {
        return parent.node_ops.lookup(parent, name);
      },mknod:function (path, mode, dev) {
        var lookup = FS.lookupPath(path, { parent: true });
        var parent = lookup.node;
        var name = PATH.basename(path);
        if (!name || name === '.' || name === '..') {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var err = FS.mayCreate(parent, name);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        if (!parent.node_ops.mknod) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        return parent.node_ops.mknod(parent, name, mode, dev);
      },create:function (path, mode) {
        mode = mode !== undefined ? mode : 438 /* 0666 */;
        mode &= 4095;
        mode |= 32768;
        return FS.mknod(path, mode, 0);
      },mkdir:function (path, mode) {
        mode = mode !== undefined ? mode : 511 /* 0777 */;
        mode &= 511 | 512;
        mode |= 16384;
        return FS.mknod(path, mode, 0);
      },mkdev:function (path, mode, dev) {
        if (typeof(dev) === 'undefined') {
          dev = mode;
          mode = 438 /* 0666 */;
        }
        mode |= 8192;
        return FS.mknod(path, mode, dev);
      },symlink:function (oldpath, newpath) {
        if (!PATH.resolve(oldpath)) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        }
        var lookup = FS.lookupPath(newpath, { parent: true });
        var parent = lookup.node;
        if (!parent) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        }
        var newname = PATH.basename(newpath);
        var err = FS.mayCreate(parent, newname);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        if (!parent.node_ops.symlink) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        return parent.node_ops.symlink(parent, newname, oldpath);
      },rename:function (old_path, new_path) {
        var old_dirname = PATH.dirname(old_path);
        var new_dirname = PATH.dirname(new_path);
        var old_name = PATH.basename(old_path);
        var new_name = PATH.basename(new_path);
        // parents must exist
        var lookup, old_dir, new_dir;
        try {
          lookup = FS.lookupPath(old_path, { parent: true });
          old_dir = lookup.node;
          lookup = FS.lookupPath(new_path, { parent: true });
          new_dir = lookup.node;
        } catch (e) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        }
        if (!old_dir || !new_dir) throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        // need to be part of the same mount
        if (old_dir.mount !== new_dir.mount) {
          throw new FS.ErrnoError(ERRNO_CODES.EXDEV);
        }
        // source must exist
        var old_node = FS.lookupNode(old_dir, old_name);
        // old path should not be an ancestor of the new path
        var relative = PATH.relative(old_path, new_dirname);
        if (relative.charAt(0) !== '.') {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        // new path should not be an ancestor of the old path
        relative = PATH.relative(new_path, old_dirname);
        if (relative.charAt(0) !== '.') {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTEMPTY);
        }
        // see if the new path already exists
        var new_node;
        try {
          new_node = FS.lookupNode(new_dir, new_name);
        } catch (e) {
          // not fatal
        }
        // early out if nothing needs to change
        if (old_node === new_node) {
          return;
        }
        // we'll need to delete the old entry
        var isdir = FS.isDir(old_node.mode);
        var err = FS.mayDelete(old_dir, old_name, isdir);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        // need delete permissions if we'll be overwriting.
        // need create permissions if new doesn't already exist.
        err = new_node ?
          FS.mayDelete(new_dir, new_name, isdir) :
          FS.mayCreate(new_dir, new_name);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        if (!old_dir.node_ops.rename) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (FS.isMountpoint(old_node) || (new_node && FS.isMountpoint(new_node))) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        }
        // if we are going to change the parent, check write permissions
        if (new_dir !== old_dir) {
          err = FS.nodePermissions(old_dir, 'w');
          if (err) {
            throw new FS.ErrnoError(err);
          }
        }
        try {
          if (FS.trackingDelegate['willMovePath']) {
            FS.trackingDelegate['willMovePath'](old_path, new_path);
          }
        } catch(e) {
          console.log("FS.trackingDelegate['willMovePath']('"+old_path+"', '"+new_path+"') threw an exception: " + e.message);
        }
        // remove the node from the lookup hash
        FS.hashRemoveNode(old_node);
        // do the underlying fs rename
        try {
          old_dir.node_ops.rename(old_node, new_dir, new_name);
        } catch (e) {
          throw e;
        } finally {
          // add the node back to the hash (in case node_ops.rename
          // changed its name)
          FS.hashAddNode(old_node);
        }
        try {
          if (FS.trackingDelegate['onMovePath']) FS.trackingDelegate['onMovePath'](old_path, new_path);
        } catch(e) {
          console.log("FS.trackingDelegate['onMovePath']('"+old_path+"', '"+new_path+"') threw an exception: " + e.message);
        }
      },rmdir:function (path) {
        var lookup = FS.lookupPath(path, { parent: true });
        var parent = lookup.node;
        var name = PATH.basename(path);
        var node = FS.lookupNode(parent, name);
        var err = FS.mayDelete(parent, name, true);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        if (!parent.node_ops.rmdir) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (FS.isMountpoint(node)) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        }
        try {
          if (FS.trackingDelegate['willDeletePath']) {
            FS.trackingDelegate['willDeletePath'](path);
          }
        } catch(e) {
          console.log("FS.trackingDelegate['willDeletePath']('"+path+"') threw an exception: " + e.message);
        }
        parent.node_ops.rmdir(parent, name);
        FS.destroyNode(node);
        try {
          if (FS.trackingDelegate['onDeletePath']) FS.trackingDelegate['onDeletePath'](path);
        } catch(e) {
          console.log("FS.trackingDelegate['onDeletePath']('"+path+"') threw an exception: " + e.message);
        }
      },readdir:function (path) {
        var lookup = FS.lookupPath(path, { follow: true });
        var node = lookup.node;
        if (!node.node_ops.readdir) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTDIR);
        }
        return node.node_ops.readdir(node);
      },unlink:function (path) {
        var lookup = FS.lookupPath(path, { parent: true });
        var parent = lookup.node;
        var name = PATH.basename(path);
        var node = FS.lookupNode(parent, name);
        var err = FS.mayDelete(parent, name, false);
        if (err) {
          // POSIX says unlink should set EPERM, not EISDIR
          if (err === ERRNO_CODES.EISDIR) err = ERRNO_CODES.EPERM;
          throw new FS.ErrnoError(err);
        }
        if (!parent.node_ops.unlink) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (FS.isMountpoint(node)) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        }
        try {
          if (FS.trackingDelegate['willDeletePath']) {
            FS.trackingDelegate['willDeletePath'](path);
          }
        } catch(e) {
          console.log("FS.trackingDelegate['willDeletePath']('"+path+"') threw an exception: " + e.message);
        }
        parent.node_ops.unlink(parent, name);
        FS.destroyNode(node);
        try {
          if (FS.trackingDelegate['onDeletePath']) FS.trackingDelegate['onDeletePath'](path);
        } catch(e) {
          console.log("FS.trackingDelegate['onDeletePath']('"+path+"') threw an exception: " + e.message);
        }
      },readlink:function (path) {
        var lookup = FS.lookupPath(path);
        var link = lookup.node;
        if (!link) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        }
        if (!link.node_ops.readlink) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        return link.node_ops.readlink(link);
      },stat:function (path, dontFollow) {
        var lookup = FS.lookupPath(path, { follow: !dontFollow });
        var node = lookup.node;
        if (!node) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        }
        if (!node.node_ops.getattr) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        return node.node_ops.getattr(node);
      },lstat:function (path) {
        return FS.stat(path, true);
      },chmod:function (path, mode, dontFollow) {
        var node;
        if (typeof path === 'string') {
          var lookup = FS.lookupPath(path, { follow: !dontFollow });
          node = lookup.node;
        } else {
          node = path;
        }
        if (!node.node_ops.setattr) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        node.node_ops.setattr(node, {
          mode: (mode & 4095) | (node.mode & ~4095),
          timestamp: Date.now()
        });
      },lchmod:function (path, mode) {
        FS.chmod(path, mode, true);
      },fchmod:function (fd, mode) {
        var stream = FS.getStream(fd);
        if (!stream) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        FS.chmod(stream.node, mode);
      },chown:function (path, uid, gid, dontFollow) {
        var node;
        if (typeof path === 'string') {
          var lookup = FS.lookupPath(path, { follow: !dontFollow });
          node = lookup.node;
        } else {
          node = path;
        }
        if (!node.node_ops.setattr) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        node.node_ops.setattr(node, {
          timestamp: Date.now()
          // we ignore the uid / gid for now
        });
      },lchown:function (path, uid, gid) {
        FS.chown(path, uid, gid, true);
      },fchown:function (fd, uid, gid) {
        var stream = FS.getStream(fd);
        if (!stream) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        FS.chown(stream.node, uid, gid);
      },truncate:function (path, len) {
        if (len < 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var node;
        if (typeof path === 'string') {
          var lookup = FS.lookupPath(path, { follow: true });
          node = lookup.node;
        } else {
          node = path;
        }
        if (!node.node_ops.setattr) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (FS.isDir(node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EISDIR);
        }
        if (!FS.isFile(node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var err = FS.nodePermissions(node, 'w');
        if (err) {
          throw new FS.ErrnoError(err);
        }
        node.node_ops.setattr(node, {
          size: len,
          timestamp: Date.now()
        });
      },ftruncate:function (fd, len) {
        var stream = FS.getStream(fd);
        if (!stream) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        if ((stream.flags & 2097155) === 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        FS.truncate(stream.node, len);
      },utime:function (path, atime, mtime) {
        var lookup = FS.lookupPath(path, { follow: true });
        var node = lookup.node;
        node.node_ops.setattr(node, {
          timestamp: Math.max(atime, mtime)
        });
      },open:function (path, flags, mode, fd_start, fd_end) {
        if (path === "") {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        }
        flags = typeof flags === 'string' ? FS.modeStringToFlags(flags) : flags;
        mode = typeof mode === 'undefined' ? 438 /* 0666 */ : mode;
        if ((flags & 64)) {
          mode = (mode & 4095) | 32768;
        } else {
          mode = 0;
        }
        var node;
        if (typeof path === 'object') {
          node = path;
        } else {
          path = PATH.normalize(path);
          try {
            var lookup = FS.lookupPath(path, {
              follow: !(flags & 131072)
            });
            node = lookup.node;
          } catch (e) {
            // ignore
          }
        }
        // perhaps we need to create the node
        var created = false;
        if ((flags & 64)) {
          if (node) {
            // if O_CREAT and O_EXCL are set, error out if the node already exists
            if ((flags & 128)) {
              throw new FS.ErrnoError(ERRNO_CODES.EEXIST);
            }
          } else {
            // node doesn't exist, try to create it
            node = FS.mknod(path, mode, 0);
            created = true;
          }
        }
        if (!node) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        }
        // can't truncate a device
        if (FS.isChrdev(node.mode)) {
          flags &= ~512;
        }
        // check permissions, if this is not a file we just created now (it is ok to
        // create and write to a file with read-only permissions; it is read-only
        // for later use)
        if (!created) {
          var err = FS.mayOpen(node, flags);
          if (err) {
            throw new FS.ErrnoError(err);
          }
        }
        // do truncation if necessary
        if ((flags & 512)) {
          FS.truncate(node, 0);
        }
        // we've already handled these, don't pass down to the underlying vfs
        flags &= ~(128 | 512);
  
        // register the stream with the filesystem
        var stream = FS.createStream({
          node: node,
          path: FS.getPath(node),  // we want the absolute path to the node
          flags: flags,
          seekable: true,
          position: 0,
          stream_ops: node.stream_ops,
          // used by the file family libc calls (fopen, fwrite, ferror, etc.)
          ungotten: [],
          error: false
        }, fd_start, fd_end);
        // call the new stream's open function
        if (stream.stream_ops.open) {
          stream.stream_ops.open(stream);
        }
        if (Module['logReadFiles'] && !(flags & 1)) {
          if (!FS.readFiles) FS.readFiles = {};
          if (!(path in FS.readFiles)) {
            FS.readFiles[path] = 1;
            Module['printErr']('read file: ' + path);
          }
        }
        try {
          if (FS.trackingDelegate['onOpenFile']) {
            var trackingFlags = 0;
            if ((flags & 2097155) !== 1) {
              trackingFlags |= FS.tracking.openFlags.READ;
            }
            if ((flags & 2097155) !== 0) {
              trackingFlags |= FS.tracking.openFlags.WRITE;
            }
            FS.trackingDelegate['onOpenFile'](path, trackingFlags);
          }
        } catch(e) {
          console.log("FS.trackingDelegate['onOpenFile']('"+path+"', flags) threw an exception: " + e.message);
        }
        return stream;
      },close:function (stream) {
        try {
          if (stream.stream_ops.close) {
            stream.stream_ops.close(stream);
          }
        } catch (e) {
          throw e;
        } finally {
          FS.closeStream(stream.fd);
        }
      },llseek:function (stream, offset, whence) {
        if (!stream.seekable || !stream.stream_ops.llseek) {
          throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
        }
        stream.position = stream.stream_ops.llseek(stream, offset, whence);
        stream.ungotten = [];
        return stream.position;
      },read:function (stream, buffer, offset, length, position) {
        if (length < 0 || position < 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        if ((stream.flags & 2097155) === 1) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        if (FS.isDir(stream.node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EISDIR);
        }
        if (!stream.stream_ops.read) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var seeking = true;
        if (typeof position === 'undefined') {
          position = stream.position;
          seeking = false;
        } else if (!stream.seekable) {
          throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
        }
        var bytesRead = stream.stream_ops.read(stream, buffer, offset, length, position);
        if (!seeking) stream.position += bytesRead;
        return bytesRead;
      },write:function (stream, buffer, offset, length, position, canOwn) {
        if (length < 0 || position < 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        if ((stream.flags & 2097155) === 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        if (FS.isDir(stream.node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EISDIR);
        }
        if (!stream.stream_ops.write) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        if (stream.flags & 1024) {
          // seek to the end before writing in append mode
          FS.llseek(stream, 0, 2);
        }
        var seeking = true;
        if (typeof position === 'undefined') {
          position = stream.position;
          seeking = false;
        } else if (!stream.seekable) {
          throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
        }
        var bytesWritten = stream.stream_ops.write(stream, buffer, offset, length, position, canOwn);
        if (!seeking) stream.position += bytesWritten;
        try {
          if (stream.path && FS.trackingDelegate['onWriteToFile']) FS.trackingDelegate['onWriteToFile'](stream.path);
        } catch(e) {
          console.log("FS.trackingDelegate['onWriteToFile']('"+path+"') threw an exception: " + e.message);
        }
        return bytesWritten;
      },allocate:function (stream, offset, length) {
        if (offset < 0 || length <= 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        if ((stream.flags & 2097155) === 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        if (!FS.isFile(stream.node.mode) && !FS.isDir(node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
        }
        if (!stream.stream_ops.allocate) {
          throw new FS.ErrnoError(ERRNO_CODES.EOPNOTSUPP);
        }
        stream.stream_ops.allocate(stream, offset, length);
      },mmap:function (stream, buffer, offset, length, position, prot, flags) {
        // TODO if PROT is PROT_WRITE, make sure we have write access
        if ((stream.flags & 2097155) === 1) {
          throw new FS.ErrnoError(ERRNO_CODES.EACCES);
        }
        if (!stream.stream_ops.mmap) {
          throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
        }
        return stream.stream_ops.mmap(stream, buffer, offset, length, position, prot, flags);
      },ioctl:function (stream, cmd, arg) {
        if (!stream.stream_ops.ioctl) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTTY);
        }
        return stream.stream_ops.ioctl(stream, cmd, arg);
      },readFile:function (path, opts) {
        opts = opts || {};
        opts.flags = opts.flags || 'r';
        opts.encoding = opts.encoding || 'binary';
        if (opts.encoding !== 'utf8' && opts.encoding !== 'binary') {
          throw new Error('Invalid encoding type "' + opts.encoding + '"');
        }
        var ret;
        var stream = FS.open(path, opts.flags);
        var stat = FS.stat(path);
        var length = stat.size;
        var buf = new Uint8Array(length);
        FS.read(stream, buf, 0, length, 0);
        if (opts.encoding === 'utf8') {
          ret = UTF8ArrayToString(buf, 0);
        } else if (opts.encoding === 'binary') {
          ret = buf;
        }
        FS.close(stream);
        return ret;
      },writeFile:function (path, data, opts) {
        opts = opts || {};
        opts.flags = opts.flags || 'w';
        opts.encoding = opts.encoding || 'utf8';
        if (opts.encoding !== 'utf8' && opts.encoding !== 'binary') {
          throw new Error('Invalid encoding type "' + opts.encoding + '"');
        }
        var stream = FS.open(path, opts.flags, opts.mode);
        if (opts.encoding === 'utf8') {
          var buf = new Uint8Array(lengthBytesUTF8(data)+1);
          var actualNumBytes = stringToUTF8Array(data, buf, 0, buf.length);
          FS.write(stream, buf, 0, actualNumBytes, 0, opts.canOwn);
        } else if (opts.encoding === 'binary') {
          FS.write(stream, data, 0, data.length, 0, opts.canOwn);
        }
        FS.close(stream);
      },cwd:function () {
        return FS.currentPath;
      },chdir:function (path) {
        var lookup = FS.lookupPath(path, { follow: true });
        if (!FS.isDir(lookup.node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTDIR);
        }
        var err = FS.nodePermissions(lookup.node, 'x');
        if (err) {
          throw new FS.ErrnoError(err);
        }
        FS.currentPath = lookup.path;
      },createDefaultDirectories:function () {
        FS.mkdir('/tmp');
        FS.mkdir('/home');
        FS.mkdir('/home/web_user');
      },createDefaultDevices:function () {
        // create /dev
        FS.mkdir('/dev');
        // setup /dev/null
        FS.registerDevice(FS.makedev(1, 3), {
          read: function() { return 0; },
          write: function() { return 0; }
        });
        FS.mkdev('/dev/null', FS.makedev(1, 3));
        // setup /dev/tty and /dev/tty1
        // stderr needs to print output using Module['printErr']
        // so we register a second tty just for it.
        TTY.register(FS.makedev(5, 0), TTY.default_tty_ops);
        TTY.register(FS.makedev(6, 0), TTY.default_tty1_ops);
        FS.mkdev('/dev/tty', FS.makedev(5, 0));
        FS.mkdev('/dev/tty1', FS.makedev(6, 0));
        // setup /dev/[u]random
        var random_device;
        if (typeof crypto !== 'undefined') {
          // for modern web browsers
          var randomBuffer = new Uint8Array(1);
          random_device = function() { crypto.getRandomValues(randomBuffer); return randomBuffer[0]; };
        } else if (ENVIRONMENT_IS_NODE) {
          // for nodejs
          random_device = function() { return require('crypto').randomBytes(1)[0]; };
        } else {
          // default for ES5 platforms
          random_device = function() { return (Math.random()*256)|0; };
        }
        FS.createDevice('/dev', 'random', random_device);
        FS.createDevice('/dev', 'urandom', random_device);
        // we're not going to emulate the actual shm device,
        // just create the tmp dirs that reside in it commonly
        FS.mkdir('/dev/shm');
        FS.mkdir('/dev/shm/tmp');
      },createStandardStreams:function () {
        // TODO deprecate the old functionality of a single
        // input / output callback and that utilizes FS.createDevice
        // and instead require a unique set of stream ops
  
        // by default, we symlink the standard streams to the
        // default tty devices. however, if the standard streams
        // have been overwritten we create a unique device for
        // them instead.
        if (Module['stdin']) {
          FS.createDevice('/dev', 'stdin', Module['stdin']);
        } else {
          FS.symlink('/dev/tty', '/dev/stdin');
        }
        if (Module['stdout']) {
          FS.createDevice('/dev', 'stdout', null, Module['stdout']);
        } else {
          FS.symlink('/dev/tty', '/dev/stdout');
        }
        if (Module['stderr']) {
          FS.createDevice('/dev', 'stderr', null, Module['stderr']);
        } else {
          FS.symlink('/dev/tty1', '/dev/stderr');
        }
  
        // open default streams for the stdin, stdout and stderr devices
        var stdin = FS.open('/dev/stdin', 'r');
        HEAP32[((_stdin)>>2)]=FS.getPtrForStream(stdin);
        assert(stdin.fd === 0, 'invalid handle for stdin (' + stdin.fd + ')');
  
        var stdout = FS.open('/dev/stdout', 'w');
        HEAP32[((_stdout)>>2)]=FS.getPtrForStream(stdout);
        assert(stdout.fd === 1, 'invalid handle for stdout (' + stdout.fd + ')');
  
        var stderr = FS.open('/dev/stderr', 'w');
        HEAP32[((_stderr)>>2)]=FS.getPtrForStream(stderr);
        assert(stderr.fd === 2, 'invalid handle for stderr (' + stderr.fd + ')');
      },ensureErrnoError:function () {
        if (FS.ErrnoError) return;
        FS.ErrnoError = function ErrnoError(errno, node) {
          this.node = node;
          this.setErrno = function(errno) {
            this.errno = errno;
            for (var key in ERRNO_CODES) {
              if (ERRNO_CODES[key] === errno) {
                this.code = key;
                break;
              }
            }
          };
          this.setErrno(errno);
          this.message = ERRNO_MESSAGES[errno];
          if (this.stack) this.stack = demangleAll(this.stack);
        };
        FS.ErrnoError.prototype = new Error();
        FS.ErrnoError.prototype.constructor = FS.ErrnoError;
        // Some errors may happen quite a bit, to avoid overhead we reuse them (and suffer a lack of stack info)
        [ERRNO_CODES.ENOENT].forEach(function(code) {
          FS.genericErrors[code] = new FS.ErrnoError(code);
          FS.genericErrors[code].stack = '<generic error, no stack>';
        });
      },staticInit:function () {
        FS.ensureErrnoError();
  
        FS.nameTable = new Array(4096);
  
        FS.mount(MEMFS, {}, '/');
  
        FS.createDefaultDirectories();
        FS.createDefaultDevices();
      },init:function (input, output, error) {
        assert(!FS.init.initialized, 'FS.init was previously called. If you want to initialize later with custom parameters, remove any earlier calls (note that one is automatically added to the generated code)');
        FS.init.initialized = true;
  
        FS.ensureErrnoError();
  
        // Allow Module.stdin etc. to provide defaults, if none explicitly passed to us here
        Module['stdin'] = input || Module['stdin'];
        Module['stdout'] = output || Module['stdout'];
        Module['stderr'] = error || Module['stderr'];
  
        FS.createStandardStreams();
      },quit:function () {
        FS.init.initialized = false;
        for (var i = 0; i < FS.streams.length; i++) {
          var stream = FS.streams[i];
          if (!stream) {
            continue;
          }
          FS.close(stream);
        }
      },getMode:function (canRead, canWrite) {
        var mode = 0;
        if (canRead) mode |= 292 | 73;
        if (canWrite) mode |= 146;
        return mode;
      },joinPath:function (parts, forceRelative) {
        var path = PATH.join.apply(null, parts);
        if (forceRelative && path[0] == '/') path = path.substr(1);
        return path;
      },absolutePath:function (relative, base) {
        return PATH.resolve(base, relative);
      },standardizePath:function (path) {
        return PATH.normalize(path);
      },findObject:function (path, dontResolveLastLink) {
        var ret = FS.analyzePath(path, dontResolveLastLink);
        if (ret.exists) {
          return ret.object;
        } else {
          ___setErrNo(ret.error);
          return null;
        }
      },analyzePath:function (path, dontResolveLastLink) {
        // operate from within the context of the symlink's target
        try {
          var lookup = FS.lookupPath(path, { follow: !dontResolveLastLink });
          path = lookup.path;
        } catch (e) {
        }
        var ret = {
          isRoot: false, exists: false, error: 0, name: null, path: null, object: null,
          parentExists: false, parentPath: null, parentObject: null
        };
        try {
          var lookup = FS.lookupPath(path, { parent: true });
          ret.parentExists = true;
          ret.parentPath = lookup.path;
          ret.parentObject = lookup.node;
          ret.name = PATH.basename(path);
          lookup = FS.lookupPath(path, { follow: !dontResolveLastLink });
          ret.exists = true;
          ret.path = lookup.path;
          ret.object = lookup.node;
          ret.name = lookup.node.name;
          ret.isRoot = lookup.path === '/';
        } catch (e) {
          ret.error = e.errno;
        };
        return ret;
      },createFolder:function (parent, name, canRead, canWrite) {
        var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
        var mode = FS.getMode(canRead, canWrite);
        return FS.mkdir(path, mode);
      },createPath:function (parent, path, canRead, canWrite) {
        parent = typeof parent === 'string' ? parent : FS.getPath(parent);
        var parts = path.split('/').reverse();
        while (parts.length) {
          var part = parts.pop();
          if (!part) continue;
          var current = PATH.join2(parent, part);
          try {
            FS.mkdir(current);
          } catch (e) {
            // ignore EEXIST
          }
          parent = current;
        }
        return current;
      },createFile:function (parent, name, properties, canRead, canWrite) {
        var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
        var mode = FS.getMode(canRead, canWrite);
        return FS.create(path, mode);
      },createDataFile:function (parent, name, data, canRead, canWrite, canOwn) {
        var path = name ? PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name) : parent;
        var mode = FS.getMode(canRead, canWrite);
        var node = FS.create(path, mode);
        if (data) {
          if (typeof data === 'string') {
            var arr = new Array(data.length);
            for (var i = 0, len = data.length; i < len; ++i) arr[i] = data.charCodeAt(i);
            data = arr;
          }
          // make sure we can write to the file
          FS.chmod(node, mode | 146);
          var stream = FS.open(node, 'w');
          FS.write(stream, data, 0, data.length, 0, canOwn);
          FS.close(stream);
          FS.chmod(node, mode);
        }
        return node;
      },createDevice:function (parent, name, input, output) {
        var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
        var mode = FS.getMode(!!input, !!output);
        if (!FS.createDevice.major) FS.createDevice.major = 64;
        var dev = FS.makedev(FS.createDevice.major++, 0);
        // Create a fake device that a set of stream ops to emulate
        // the old behavior.
        FS.registerDevice(dev, {
          open: function(stream) {
            stream.seekable = false;
          },
          close: function(stream) {
            // flush any pending line data
            if (output && output.buffer && output.buffer.length) {
              output(10);
            }
          },
          read: function(stream, buffer, offset, length, pos /* ignored */) {
            var bytesRead = 0;
            for (var i = 0; i < length; i++) {
              var result;
              try {
                result = input();
              } catch (e) {
                throw new FS.ErrnoError(ERRNO_CODES.EIO);
              }
              if (result === undefined && bytesRead === 0) {
                throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
              }
              if (result === null || result === undefined) break;
              bytesRead++;
              buffer[offset+i] = result;
            }
            if (bytesRead) {
              stream.node.timestamp = Date.now();
            }
            return bytesRead;
          },
          write: function(stream, buffer, offset, length, pos) {
            for (var i = 0; i < length; i++) {
              try {
                output(buffer[offset+i]);
              } catch (e) {
                throw new FS.ErrnoError(ERRNO_CODES.EIO);
              }
            }
            if (length) {
              stream.node.timestamp = Date.now();
            }
            return i;
          }
        });
        return FS.mkdev(path, mode, dev);
      },createLink:function (parent, name, target, canRead, canWrite) {
        var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
        return FS.symlink(target, path);
      },forceLoadFile:function (obj) {
        if (obj.isDevice || obj.isFolder || obj.link || obj.contents) return true;
        var success = true;
        if (typeof XMLHttpRequest !== 'undefined') {
          throw new Error("Lazy loading should have been performed (contents set) in createLazyFile, but it was not. Lazy loading only works in web workers. Use --embed-file or --preload-file in emcc on the main thread.");
        } else if (Module['read']) {
          // Command-line.
          try {
            // WARNING: Can't read binary files in V8's d8 or tracemonkey's js, as
            //          read() will try to parse UTF8.
            obj.contents = intArrayFromString(Module['read'](obj.url), true);
            obj.usedBytes = obj.contents.length;
          } catch (e) {
            success = false;
          }
        } else {
          throw new Error('Cannot load without read() or XMLHttpRequest.');
        }
        if (!success) ___setErrNo(ERRNO_CODES.EIO);
        return success;
      },createLazyFile:function (parent, name, url, canRead, canWrite) {
        // Lazy chunked Uint8Array (implements get and length from Uint8Array). Actual getting is abstracted away for eventual reuse.
        function LazyUint8Array() {
          this.lengthKnown = false;
          this.chunks = []; // Loaded chunks. Index is the chunk number
        }
        LazyUint8Array.prototype.get = function LazyUint8Array_get(idx) {
          if (idx > this.length-1 || idx < 0) {
            return undefined;
          }
          var chunkOffset = idx % this.chunkSize;
          var chunkNum = (idx / this.chunkSize)|0;
          return this.getter(chunkNum)[chunkOffset];
        }
        LazyUint8Array.prototype.setDataGetter = function LazyUint8Array_setDataGetter(getter) {
          this.getter = getter;
        }
        LazyUint8Array.prototype.cacheLength = function LazyUint8Array_cacheLength() {
          // Find length
          var xhr = new XMLHttpRequest();
          xhr.open('HEAD', url, false);
          xhr.send(null);
          if (!(xhr.status >= 200 && xhr.status < 300 || xhr.status === 304)) throw new Error("Couldn't load " + url + ". Status: " + xhr.status);
          var datalength = Number(xhr.getResponseHeader("Content-length"));
          var header;
          var hasByteServing = (header = xhr.getResponseHeader("Accept-Ranges")) && header === "bytes";
          var chunkSize = 1024*1024; // Chunk size in bytes
  
          if (!hasByteServing) chunkSize = datalength;
  
          // Function to get a range from the remote URL.
          var doXHR = (function(from, to) {
            if (from > to) throw new Error("invalid range (" + from + ", " + to + ") or no bytes requested!");
            if (to > datalength-1) throw new Error("only " + datalength + " bytes available! programmer error!");
  
            // TODO: Use mozResponseArrayBuffer, responseStream, etc. if available.
            var xhr = new XMLHttpRequest();
            xhr.open('GET', url, false);
            if (datalength !== chunkSize) xhr.setRequestHeader("Range", "bytes=" + from + "-" + to);
  
            // Some hints to the browser that we want binary data.
            if (typeof Uint8Array != 'undefined') xhr.responseType = 'arraybuffer';
            if (xhr.overrideMimeType) {
              xhr.overrideMimeType('text/plain; charset=x-user-defined');
            }
  
            xhr.send(null);
            if (!(xhr.status >= 200 && xhr.status < 300 || xhr.status === 304)) throw new Error("Couldn't load " + url + ". Status: " + xhr.status);
            if (xhr.response !== undefined) {
              return new Uint8Array(xhr.response || []);
            } else {
              return intArrayFromString(xhr.responseText || '', true);
            }
          });
          var lazyArray = this;
          lazyArray.setDataGetter(function(chunkNum) {
            var start = chunkNum * chunkSize;
            var end = (chunkNum+1) * chunkSize - 1; // including this byte
            end = Math.min(end, datalength-1); // if datalength-1 is selected, this is the last block
            if (typeof(lazyArray.chunks[chunkNum]) === "undefined") {
              lazyArray.chunks[chunkNum] = doXHR(start, end);
            }
            if (typeof(lazyArray.chunks[chunkNum]) === "undefined") throw new Error("doXHR failed!");
            return lazyArray.chunks[chunkNum];
          });
  
          this._length = datalength;
          this._chunkSize = chunkSize;
          this.lengthKnown = true;
        }
        if (typeof XMLHttpRequest !== 'undefined') {
          if (!ENVIRONMENT_IS_WORKER) throw 'Cannot do synchronous binary XHRs outside webworkers in modern browsers. Use --embed-file or --preload-file in emcc';
          var lazyArray = new LazyUint8Array();
          Object.defineProperty(lazyArray, "length", {
              get: function() {
                  if(!this.lengthKnown) {
                      this.cacheLength();
                  }
                  return this._length;
              }
          });
          Object.defineProperty(lazyArray, "chunkSize", {
              get: function() {
                  if(!this.lengthKnown) {
                      this.cacheLength();
                  }
                  return this._chunkSize;
              }
          });
  
          var properties = { isDevice: false, contents: lazyArray };
        } else {
          var properties = { isDevice: false, url: url };
        }
  
        var node = FS.createFile(parent, name, properties, canRead, canWrite);
        // This is a total hack, but I want to get this lazy file code out of the
        // core of MEMFS. If we want to keep this lazy file concept I feel it should
        // be its own thin LAZYFS proxying calls to MEMFS.
        if (properties.contents) {
          node.contents = properties.contents;
        } else if (properties.url) {
          node.contents = null;
          node.url = properties.url;
        }
        // Add a function that defers querying the file size until it is asked the first time.
        Object.defineProperty(node, "usedBytes", {
            get: function() { return this.contents.length; }
        });
        // override each stream op with one that tries to force load the lazy file first
        var stream_ops = {};
        var keys = Object.keys(node.stream_ops);
        keys.forEach(function(key) {
          var fn = node.stream_ops[key];
          stream_ops[key] = function forceLoadLazyFile() {
            if (!FS.forceLoadFile(node)) {
              throw new FS.ErrnoError(ERRNO_CODES.EIO);
            }
            return fn.apply(null, arguments);
          };
        });
        // use a custom read function
        stream_ops.read = function stream_ops_read(stream, buffer, offset, length, position) {
          if (!FS.forceLoadFile(node)) {
            throw new FS.ErrnoError(ERRNO_CODES.EIO);
          }
          var contents = stream.node.contents;
          if (position >= contents.length)
            return 0;
          var size = Math.min(contents.length - position, length);
          assert(size >= 0);
          if (contents.slice) { // normal array
            for (var i = 0; i < size; i++) {
              buffer[offset + i] = contents[position + i];
            }
          } else {
            for (var i = 0; i < size; i++) { // LazyUint8Array from sync binary XHR
              buffer[offset + i] = contents.get(position + i);
            }
          }
          return size;
        };
        node.stream_ops = stream_ops;
        return node;
      },createPreloadedFile:function (parent, name, url, canRead, canWrite, onload, onerror, dontCreateFile, canOwn) {
        Browser.init();
        // TODO we should allow people to just pass in a complete filename instead
        // of parent and name being that we just join them anyways
        var fullname = name ? PATH.resolve(PATH.join2(parent, name)) : parent;
        function processData(byteArray) {
          function finish(byteArray) {
            if (!dontCreateFile) {
              FS.createDataFile(parent, name, byteArray, canRead, canWrite, canOwn);
            }
            if (onload) onload();
            removeRunDependency('cp ' + fullname);
          }
          var handled = false;
          Module['preloadPlugins'].forEach(function(plugin) {
            if (handled) return;
            if (plugin['canHandle'](fullname)) {
              plugin['handle'](byteArray, fullname, finish, function() {
                if (onerror) onerror();
                removeRunDependency('cp ' + fullname);
              });
              handled = true;
            }
          });
          if (!handled) finish(byteArray);
        }
        addRunDependency('cp ' + fullname);
        if (typeof url == 'string') {
          Browser.asyncLoad(url, function(byteArray) {
            processData(byteArray);
          }, onerror);
        } else {
          processData(url);
        }
      },indexedDB:function () {
        return window.indexedDB || window.mozIndexedDB || window.webkitIndexedDB || window.msIndexedDB;
      },DB_NAME:function () {
        return 'EM_FS_' + window.location.pathname;
      },DB_VERSION:20,DB_STORE_NAME:"FILE_DATA",saveFilesToDB:function (paths, onload, onerror) {
        onload = onload || function(){};
        onerror = onerror || function(){};
        var indexedDB = FS.indexedDB();
        try {
          var openRequest = indexedDB.open(FS.DB_NAME(), FS.DB_VERSION);
        } catch (e) {
          return onerror(e);
        }
        openRequest.onupgradeneeded = function openRequest_onupgradeneeded() {
          console.log('creating db');
          var db = openRequest.result;
          db.createObjectStore(FS.DB_STORE_NAME);
        };
        openRequest.onsuccess = function openRequest_onsuccess() {
          var db = openRequest.result;
          var transaction = db.transaction([FS.DB_STORE_NAME], 'readwrite');
          var files = transaction.objectStore(FS.DB_STORE_NAME);
          var ok = 0, fail = 0, total = paths.length;
          function finish() {
            if (fail == 0) onload(); else onerror();
          }
          paths.forEach(function(path) {
            var putRequest = files.put(FS.analyzePath(path).object.contents, path);
            putRequest.onsuccess = function putRequest_onsuccess() { ok++; if (ok + fail == total) finish() };
            putRequest.onerror = function putRequest_onerror() { fail++; if (ok + fail == total) finish() };
          });
          transaction.onerror = onerror;
        };
        openRequest.onerror = onerror;
      },loadFilesFromDB:function (paths, onload, onerror) {
        onload = onload || function(){};
        onerror = onerror || function(){};
        var indexedDB = FS.indexedDB();
        try {
          var openRequest = indexedDB.open(FS.DB_NAME(), FS.DB_VERSION);
        } catch (e) {
          return onerror(e);
        }
        openRequest.onupgradeneeded = onerror; // no database to load from
        openRequest.onsuccess = function openRequest_onsuccess() {
          var db = openRequest.result;
          try {
            var transaction = db.transaction([FS.DB_STORE_NAME], 'readonly');
          } catch(e) {
            onerror(e);
            return;
          }
          var files = transaction.objectStore(FS.DB_STORE_NAME);
          var ok = 0, fail = 0, total = paths.length;
          function finish() {
            if (fail == 0) onload(); else onerror();
          }
          paths.forEach(function(path) {
            var getRequest = files.get(path);
            getRequest.onsuccess = function getRequest_onsuccess() {
              if (FS.analyzePath(path).exists) {
                FS.unlink(path);
              }
              FS.createDataFile(PATH.dirname(path), PATH.basename(path), getRequest.result, true, true, true);
              ok++;
              if (ok + fail == total) finish();
            };
            getRequest.onerror = function getRequest_onerror() { fail++; if (ok + fail == total) finish() };
          });
          transaction.onerror = onerror;
        };
        openRequest.onerror = onerror;
      }};function _lseek(fildes, offset, whence) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_3(56, fildes, offset, whence);
      // off_t lseek(int fildes, off_t offset, int whence);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/lseek.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      try {
        return FS.llseek(stream, offset, whence);
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }

  function _emscripten_atomic_load_u32() {
  Module['printErr']('missing function: emscripten_atomic_load_u32'); abort(-1);
  }

  function _sysconf(name) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_1(72, name);
      // long sysconf(int name);
      // http://pubs.opengroup.org/onlinepubs/009695399/functions/sysconf.html
      switch(name) {
        case 30: return PAGE_SIZE;
        case 132:
        case 133:
        case 12:
        case 137:
        case 138:
        case 15:
        case 235:
        case 16:
        case 17:
        case 18:
        case 19:
        case 20:
        case 149:
        case 13:
        case 10:
        case 236:
        case 153:
        case 9:
        case 21:
        case 22:
        case 159:
        case 154:
        case 14:
        case 77:
        case 78:
        case 139:
        case 80:
        case 81:
        case 82:
        case 68:
        case 67:
        case 164:
        case 11:
        case 29:
        case 47:
        case 48:
        case 95:
        case 52:
        case 51:
        case 46:
          return 200809;
        case 79:
          return 0;
        case 27:
        case 246:
        case 127:
        case 128:
        case 23:
        case 24:
        case 160:
        case 161:
        case 181:
        case 182:
        case 242:
        case 183:
        case 184:
        case 243:
        case 244:
        case 245:
        case 165:
        case 178:
        case 179:
        case 49:
        case 50:
        case 168:
        case 169:
        case 175:
        case 170:
        case 171:
        case 172:
        case 97:
        case 76:
        case 32:
        case 173:
        case 35:
          return -1;
        case 176:
        case 177:
        case 7:
        case 155:
        case 8:
        case 157:
        case 125:
        case 126:
        case 92:
        case 93:
        case 129:
        case 130:
        case 131:
        case 94:
        case 91:
          return 1;
        case 74:
        case 60:
        case 69:
        case 70:
        case 4:
          return 1024;
        case 31:
        case 42:
        case 72:
          return 32;
        case 87:
        case 26:
        case 33:
          return 2147483647;
        case 34:
        case 1:
          return 47839;
        case 38:
        case 36:
          return 99;
        case 43:
        case 37:
          return 2048;
        case 0: return 2097152;
        case 3: return 65536;
        case 28: return 32768;
        case 44: return 32767;
        case 75: return 16384;
        case 39: return 1000;
        case 89: return 700;
        case 71: return 256;
        case 40: return 255;
        case 2: return 100;
        case 180: return 64;
        case 25: return 20;
        case 5: return 16;
        case 6: return 6;
        case 73: return 4;
        case 84: {
          if (typeof navigator === 'object') return navigator['hardwareConcurrency'] || 1;
          return 1;
        }
      }
      ___setErrNo(ERRNO_CODES.EINVAL);
      return -1;
    }

  
  function _close(fildes) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_1(38, fildes);
      // int close(int fildes);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/close.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      try {
        FS.close(stream);
        return 0;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }
  
  function _fsync(fildes) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_1(47, fildes);
      // int fsync(int fildes);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fsync.html
      var stream = FS.getStream(fildes);
      if (stream) {
        // We write directly to the file system, so there's nothing to do here.
        return 0;
      } else {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
    }
  
  function _fileno(stream) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_1(83, stream);
      // int fileno(FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fileno.html
      stream = FS.getStreamFromPtr(stream);
      if (!stream) return -1;
      return stream.fd;
    }function _fclose(stream) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_1(4, stream);
      // int fclose(FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fclose.html
      var fd = _fileno(stream);
      _fsync(fd);
      return _close(fd);
    }

  
  function _statvfs(path, buf) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_2(24, path, buf);
      // http://pubs.opengroup.org/onlinepubs/009695399/functions/statvfs.html
      // int statvfs(const char *restrict path, struct statvfs *restrict buf);
      // NOTE: None of the constants here are true. We're just returning safe and
      //       sane values.
      HEAP32[((buf)>>2)]=4096;
      HEAP32[(((buf)+(4))>>2)]=4096;
      HEAP32[(((buf)+(8))>>2)]=1000000;
      HEAP32[(((buf)+(12))>>2)]=500000;
      HEAP32[(((buf)+(16))>>2)]=500000;
      HEAP32[(((buf)+(20))>>2)]=FS.nextInode;
      HEAP32[(((buf)+(24))>>2)]=1000000;
      HEAP32[(((buf)+(28))>>2)]=1000000;
      HEAP32[(((buf)+(32))>>2)]=42;
      HEAP32[(((buf)+(40))>>2)]=2;  // ST_NOSUID
      HEAP32[(((buf)+(44))>>2)]=255;
      return 0;
    }function _fstatvfs(fildes, buf) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_2(25, fildes, buf);
      // int fstatvfs(int fildes, struct statvfs *buf);
      // http://pubs.opengroup.org/onlinepubs/009604499/functions/statvfs.html
      return _statvfs(0, buf);
    }


  
  
  
  
  function _mkport() { throw 'TODO' }var SOCKFS={mount:function (mount) {
        // If Module['websocket'] has already been defined (e.g. for configuring
        // the subprotocol/url) use that, if not initialise it to a new object.
        Module['websocket'] = (Module['websocket'] && 
                               ('object' === typeof Module['websocket'])) ? Module['websocket'] : {};
  
        // Add the Event registration mechanism to the exported websocket configuration
        // object so we can register network callbacks from native JavaScript too.
        // For more documentation see system/include/emscripten/emscripten.h
        Module['websocket']._callbacks = {};
        Module['websocket']['on'] = function(event, callback) {
  	    if ('function' === typeof callback) {
  		  this._callbacks[event] = callback;
          }
  	    return this;
        };
  
        Module['websocket'].emit = function(event, param) {
  	    if ('function' === typeof this._callbacks[event]) {
  		  this._callbacks[event].call(this, param);
          }
        };
  
        // If debug is enabled register simple default logging callbacks for each Event.
  
        return FS.createNode(null, '/', 16384 | 511 /* 0777 */, 0);
      },createSocket:function (family, type, protocol) {
        var streaming = type == 1;
        if (protocol) {
          assert(streaming == (protocol == 6)); // if SOCK_STREAM, must be tcp
        }
  
        // create our internal socket structure
        var sock = {
          family: family,
          type: type,
          protocol: protocol,
          server: null,
          error: null, // Used in getsockopt for SOL_SOCKET/SO_ERROR test
          peers: {},
          pending: [],
          recv_queue: [],
          sock_ops: SOCKFS.websocket_sock_ops
        };
  
        // create the filesystem node to store the socket structure
        var name = SOCKFS.nextname();
        var node = FS.createNode(SOCKFS.root, name, 49152, 0);
        node.sock = sock;
  
        // and the wrapping stream that enables library functions such
        // as read and write to indirectly interact with the socket
        var stream = FS.createStream({
          path: name,
          node: node,
          flags: FS.modeStringToFlags('r+'),
          seekable: false,
          stream_ops: SOCKFS.stream_ops
        });
  
        // map the new stream to the socket structure (sockets have a 1:1
        // relationship with a stream)
        sock.stream = stream;
  
        return sock;
      },getSocket:function (fd) {
        var stream = FS.getStream(fd);
        if (!stream || !FS.isSocket(stream.node.mode)) {
          return null;
        }
        return stream.node.sock;
      },stream_ops:{poll:function (stream) {
          var sock = stream.node.sock;
          return sock.sock_ops.poll(sock);
        },ioctl:function (stream, request, varargs) {
          var sock = stream.node.sock;
          return sock.sock_ops.ioctl(sock, request, varargs);
        },read:function (stream, buffer, offset, length, position /* ignored */) {
          var sock = stream.node.sock;
          var msg = sock.sock_ops.recvmsg(sock, length);
          if (!msg) {
            // socket is closed
            return 0;
          }
          buffer.set(msg.buffer, offset);
          return msg.buffer.length;
        },write:function (stream, buffer, offset, length, position /* ignored */) {
          var sock = stream.node.sock;
          return sock.sock_ops.sendmsg(sock, buffer, offset, length);
        },close:function (stream) {
          var sock = stream.node.sock;
          sock.sock_ops.close(sock);
        }},nextname:function () {
        if (!SOCKFS.nextname.current) {
          SOCKFS.nextname.current = 0;
        }
        return 'socket[' + (SOCKFS.nextname.current++) + ']';
      },websocket_sock_ops:{createPeer:function (sock, addr, port) {
          var ws;
  
          if (typeof addr === 'object') {
            ws = addr;
            addr = null;
            port = null;
          }
  
          if (ws) {
            // for sockets that've already connected (e.g. we're the server)
            // we can inspect the _socket property for the address
            if (ws._socket) {
              addr = ws._socket.remoteAddress;
              port = ws._socket.remotePort;
            }
            // if we're just now initializing a connection to the remote,
            // inspect the url property
            else {
              var result = /ws[s]?:\/\/([^:]+):(\d+)/.exec(ws.url);
              if (!result) {
                throw new Error('WebSocket URL must be in the format ws(s)://address:port');
              }
              addr = result[1];
              port = parseInt(result[2], 10);
            }
          } else {
            // create the actual websocket object and connect
            try {
              // runtimeConfig gets set to true if WebSocket runtime configuration is available.
              var runtimeConfig = (Module['websocket'] && ('object' === typeof Module['websocket']));
  
              // The default value is 'ws://' the replace is needed because the compiler replaces '//' comments with '#'
              // comments without checking context, so we'd end up with ws:#, the replace swaps the '#' for '//' again.
              var url = 'ws:#'.replace('#', '//');
  
              if (runtimeConfig) {
                if ('string' === typeof Module['websocket']['url']) {
                  url = Module['websocket']['url']; // Fetch runtime WebSocket URL config.
                }
              }
  
              if (url === 'ws://' || url === 'wss://') { // Is the supplied URL config just a prefix, if so complete it.
                var parts = addr.split('/');
                url = url + parts[0] + ":" + port + "/" + parts.slice(1).join('/');
              }
  
              // Make the WebSocket subprotocol (Sec-WebSocket-Protocol) default to binary if no configuration is set.
              var subProtocols = 'binary'; // The default value is 'binary'
  
              if (runtimeConfig) {
                if ('string' === typeof Module['websocket']['subprotocol']) {
                  subProtocols = Module['websocket']['subprotocol']; // Fetch runtime WebSocket subprotocol config.
                }
              }
  
              // The regex trims the string (removes spaces at the beginning and end, then splits the string by
              // <any space>,<any space> into an Array. Whitespace removal is important for Websockify and ws.
              subProtocols = subProtocols.replace(/^ +| +$/g,"").split(/ *, */);
  
              // The node ws library API for specifying optional subprotocol is slightly different than the browser's.
              var opts = ENVIRONMENT_IS_NODE ? {'protocol': subProtocols.toString()} : subProtocols;
  
              // If node we use the ws library.
              var WebSocket = ENVIRONMENT_IS_NODE ? require('ws') : window['WebSocket'];
              ws = new WebSocket(url, opts);
              ws.binaryType = 'arraybuffer';
            } catch (e) {
              throw new FS.ErrnoError(ERRNO_CODES.EHOSTUNREACH);
            }
          }
  
  
          var peer = {
            addr: addr,
            port: port,
            socket: ws,
            dgram_send_queue: []
          };
  
          SOCKFS.websocket_sock_ops.addPeer(sock, peer);
          SOCKFS.websocket_sock_ops.handlePeerEvents(sock, peer);
  
          // if this is a bound dgram socket, send the port number first to allow
          // us to override the ephemeral port reported to us by remotePort on the
          // remote end.
          if (sock.type === 2 && typeof sock.sport !== 'undefined') {
            peer.dgram_send_queue.push(new Uint8Array([
                255, 255, 255, 255,
                'p'.charCodeAt(0), 'o'.charCodeAt(0), 'r'.charCodeAt(0), 't'.charCodeAt(0),
                ((sock.sport & 0xff00) >> 8) , (sock.sport & 0xff)
            ]));
          }
  
          return peer;
        },getPeer:function (sock, addr, port) {
          return sock.peers[addr + ':' + port];
        },addPeer:function (sock, peer) {
          sock.peers[peer.addr + ':' + peer.port] = peer;
        },removePeer:function (sock, peer) {
          delete sock.peers[peer.addr + ':' + peer.port];
        },handlePeerEvents:function (sock, peer) {
          var first = true;
  
          var handleOpen = function () {
  
            Module['websocket'].emit('open', sock.stream.fd);
  
            try {
              var queued = peer.dgram_send_queue.shift();
              while (queued) {
                peer.socket.send(queued);
                queued = peer.dgram_send_queue.shift();
              }
            } catch (e) {
              // not much we can do here in the way of proper error handling as we've already
              // lied and said this data was sent. shut it down.
              peer.socket.close();
            }
          };
  
          function handleMessage(data) {
            assert(typeof data !== 'string' && data.byteLength !== undefined);  // must receive an ArrayBuffer
            data = new Uint8Array(data);  // make a typed array view on the array buffer
  
  
            // if this is the port message, override the peer's port with it
            var wasfirst = first;
            first = false;
            if (wasfirst &&
                data.length === 10 &&
                data[0] === 255 && data[1] === 255 && data[2] === 255 && data[3] === 255 &&
                data[4] === 'p'.charCodeAt(0) && data[5] === 'o'.charCodeAt(0) && data[6] === 'r'.charCodeAt(0) && data[7] === 't'.charCodeAt(0)) {
              // update the peer's port and it's key in the peer map
              var newport = ((data[8] << 8) | data[9]);
              SOCKFS.websocket_sock_ops.removePeer(sock, peer);
              peer.port = newport;
              SOCKFS.websocket_sock_ops.addPeer(sock, peer);
              return;
            }
  
            sock.recv_queue.push({ addr: peer.addr, port: peer.port, data: data });
            Module['websocket'].emit('message', sock.stream.fd);
          };
  
          if (ENVIRONMENT_IS_NODE) {
            peer.socket.on('open', handleOpen);
            peer.socket.on('message', function(data, flags) {
              if (!flags.binary) {
                return;
              }
              handleMessage((new Uint8Array(data)).buffer);  // copy from node Buffer -> ArrayBuffer
            });
            peer.socket.on('close', function() {
              Module['websocket'].emit('close', sock.stream.fd);
            });
            peer.socket.on('error', function(error) {
              // Although the ws library may pass errors that may be more descriptive than
              // ECONNREFUSED they are not necessarily the expected error code e.g. 
              // ENOTFOUND on getaddrinfo seems to be node.js specific, so using ECONNREFUSED
              // is still probably the most useful thing to do.
              sock.error = ERRNO_CODES.ECONNREFUSED; // Used in getsockopt for SOL_SOCKET/SO_ERROR test.
              Module['websocket'].emit('error', [sock.stream.fd, sock.error, 'ECONNREFUSED: Connection refused']);
              // don't throw
            });
          } else {
            peer.socket.onopen = handleOpen;
            peer.socket.onclose = function() {
              Module['websocket'].emit('close', sock.stream.fd);
            };
            peer.socket.onmessage = function peer_socket_onmessage(event) {
              handleMessage(event.data);
            };
            peer.socket.onerror = function(error) {
              // The WebSocket spec only allows a 'simple event' to be thrown on error,
              // so we only really know as much as ECONNREFUSED.
              sock.error = ERRNO_CODES.ECONNREFUSED; // Used in getsockopt for SOL_SOCKET/SO_ERROR test.
              Module['websocket'].emit('error', [sock.stream.fd, sock.error, 'ECONNREFUSED: Connection refused']);
            };
          }
        },poll:function (sock) {
          if (sock.type === 1 && sock.server) {
            // listen sockets should only say they're available for reading
            // if there are pending clients.
            return sock.pending.length ? (64 | 1) : 0;
          }
  
          var mask = 0;
          var dest = sock.type === 1 ?  // we only care about the socket state for connection-based sockets
            SOCKFS.websocket_sock_ops.getPeer(sock, sock.daddr, sock.dport) :
            null;
  
          if (sock.recv_queue.length ||
              !dest ||  // connection-less sockets are always ready to read
              (dest && dest.socket.readyState === dest.socket.CLOSING) ||
              (dest && dest.socket.readyState === dest.socket.CLOSED)) {  // let recv return 0 once closed
            mask |= (64 | 1);
          }
  
          if (!dest ||  // connection-less sockets are always ready to write
              (dest && dest.socket.readyState === dest.socket.OPEN)) {
            mask |= 4;
          }
  
          if ((dest && dest.socket.readyState === dest.socket.CLOSING) ||
              (dest && dest.socket.readyState === dest.socket.CLOSED)) {
            mask |= 16;
          }
  
          return mask;
        },ioctl:function (sock, request, arg) {
          switch (request) {
            case 21531:
              var bytes = 0;
              if (sock.recv_queue.length) {
                bytes = sock.recv_queue[0].data.length;
              }
              HEAP32[((arg)>>2)]=bytes;
              return 0;
            default:
              return ERRNO_CODES.EINVAL;
          }
        },close:function (sock) {
          // if we've spawned a listen server, close it
          if (sock.server) {
            try {
              sock.server.close();
            } catch (e) {
            }
            sock.server = null;
          }
          // close any peer connections
          var peers = Object.keys(sock.peers);
          for (var i = 0; i < peers.length; i++) {
            var peer = sock.peers[peers[i]];
            try {
              peer.socket.close();
            } catch (e) {
            }
            SOCKFS.websocket_sock_ops.removePeer(sock, peer);
          }
          return 0;
        },bind:function (sock, addr, port) {
          if (typeof sock.saddr !== 'undefined' || typeof sock.sport !== 'undefined') {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);  // already bound
          }
          sock.saddr = addr;
          sock.sport = port || _mkport();
          // in order to emulate dgram sockets, we need to launch a listen server when
          // binding on a connection-less socket
          // note: this is only required on the server side
          if (sock.type === 2) {
            // close the existing server if it exists
            if (sock.server) {
              sock.server.close();
              sock.server = null;
            }
            // swallow error operation not supported error that occurs when binding in the
            // browser where this isn't supported
            try {
              sock.sock_ops.listen(sock, 0);
            } catch (e) {
              if (!(e instanceof FS.ErrnoError)) throw e;
              if (e.errno !== ERRNO_CODES.EOPNOTSUPP) throw e;
            }
          }
        },connect:function (sock, addr, port) {
          if (sock.server) {
            throw new FS.ErrnoError(ERRNO_CODES.EOPNOTSUPP);
          }
  
          // TODO autobind
          // if (!sock.addr && sock.type == 2) {
          // }
  
          // early out if we're already connected / in the middle of connecting
          if (typeof sock.daddr !== 'undefined' && typeof sock.dport !== 'undefined') {
            var dest = SOCKFS.websocket_sock_ops.getPeer(sock, sock.daddr, sock.dport);
            if (dest) {
              if (dest.socket.readyState === dest.socket.CONNECTING) {
                throw new FS.ErrnoError(ERRNO_CODES.EALREADY);
              } else {
                throw new FS.ErrnoError(ERRNO_CODES.EISCONN);
              }
            }
          }
  
          // add the socket to our peer list and set our
          // destination address / port to match
          var peer = SOCKFS.websocket_sock_ops.createPeer(sock, addr, port);
          sock.daddr = peer.addr;
          sock.dport = peer.port;
  
          // always "fail" in non-blocking mode
          throw new FS.ErrnoError(ERRNO_CODES.EINPROGRESS);
        },listen:function (sock, backlog) {
          if (!ENVIRONMENT_IS_NODE) {
            throw new FS.ErrnoError(ERRNO_CODES.EOPNOTSUPP);
          }
          if (sock.server) {
             throw new FS.ErrnoError(ERRNO_CODES.EINVAL);  // already listening
          }
          var WebSocketServer = require('ws').Server;
          var host = sock.saddr;
          sock.server = new WebSocketServer({
            host: host,
            port: sock.sport
            // TODO support backlog
          });
          Module['websocket'].emit('listen', sock.stream.fd); // Send Event with listen fd.
  
          sock.server.on('connection', function(ws) {
            if (sock.type === 1) {
              var newsock = SOCKFS.createSocket(sock.family, sock.type, sock.protocol);
  
              // create a peer on the new socket
              var peer = SOCKFS.websocket_sock_ops.createPeer(newsock, ws);
              newsock.daddr = peer.addr;
              newsock.dport = peer.port;
  
              // push to queue for accept to pick up
              sock.pending.push(newsock);
              Module['websocket'].emit('connection', newsock.stream.fd);
            } else {
              // create a peer on the listen socket so calling sendto
              // with the listen socket and an address will resolve
              // to the correct client
              SOCKFS.websocket_sock_ops.createPeer(sock, ws);
              Module['websocket'].emit('connection', sock.stream.fd);
            }
          });
          sock.server.on('closed', function() {
            Module['websocket'].emit('close', sock.stream.fd);
            sock.server = null;
          });
          sock.server.on('error', function(error) {
            // Although the ws library may pass errors that may be more descriptive than
            // ECONNREFUSED they are not necessarily the expected error code e.g. 
            // ENOTFOUND on getaddrinfo seems to be node.js specific, so using EHOSTUNREACH
            // is still probably the most useful thing to do. This error shouldn't
            // occur in a well written app as errors should get trapped in the compiled
            // app's own getaddrinfo call.
            sock.error = ERRNO_CODES.EHOSTUNREACH; // Used in getsockopt for SOL_SOCKET/SO_ERROR test.
            Module['websocket'].emit('error', [sock.stream.fd, sock.error, 'EHOSTUNREACH: Host is unreachable']);
            // don't throw
          });
        },accept:function (listensock) {
          if (!listensock.server) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
          var newsock = listensock.pending.shift();
          newsock.stream.flags = listensock.stream.flags;
          return newsock;
        },getname:function (sock, peer) {
          var addr, port;
          if (peer) {
            if (sock.daddr === undefined || sock.dport === undefined) {
              throw new FS.ErrnoError(ERRNO_CODES.ENOTCONN);
            }
            addr = sock.daddr;
            port = sock.dport;
          } else {
            // TODO saddr and sport will be set for bind()'d UDP sockets, but what
            // should we be returning for TCP sockets that've been connect()'d?
            addr = sock.saddr || 0;
            port = sock.sport || 0;
          }
          return { addr: addr, port: port };
        },sendmsg:function (sock, buffer, offset, length, addr, port) {
          if (sock.type === 2) {
            // connection-less sockets will honor the message address,
            // and otherwise fall back to the bound destination address
            if (addr === undefined || port === undefined) {
              addr = sock.daddr;
              port = sock.dport;
            }
            // if there was no address to fall back to, error out
            if (addr === undefined || port === undefined) {
              throw new FS.ErrnoError(ERRNO_CODES.EDESTADDRREQ);
            }
          } else {
            // connection-based sockets will only use the bound
            addr = sock.daddr;
            port = sock.dport;
          }
  
          // find the peer for the destination address
          var dest = SOCKFS.websocket_sock_ops.getPeer(sock, addr, port);
  
          // early out if not connected with a connection-based socket
          if (sock.type === 1) {
            if (!dest || dest.socket.readyState === dest.socket.CLOSING || dest.socket.readyState === dest.socket.CLOSED) {
              throw new FS.ErrnoError(ERRNO_CODES.ENOTCONN);
            } else if (dest.socket.readyState === dest.socket.CONNECTING) {
              throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
            }
          }
  
          // create a copy of the incoming data to send, as the WebSocket API
          // doesn't work entirely with an ArrayBufferView, it'll just send
          // the entire underlying buffer
          var data;
          if (buffer instanceof Array || buffer instanceof ArrayBuffer) {
            data = buffer.slice(offset, offset + length);
          } else {  // ArrayBufferView
            data = buffer.buffer.slice(buffer.byteOffset + offset, buffer.byteOffset + offset + length);
          }
  
          // if we're emulating a connection-less dgram socket and don't have
          // a cached connection, queue the buffer to send upon connect and
          // lie, saying the data was sent now.
          if (sock.type === 2) {
            if (!dest || dest.socket.readyState !== dest.socket.OPEN) {
              // if we're not connected, open a new connection
              if (!dest || dest.socket.readyState === dest.socket.CLOSING || dest.socket.readyState === dest.socket.CLOSED) {
                dest = SOCKFS.websocket_sock_ops.createPeer(sock, addr, port);
              }
              dest.dgram_send_queue.push(data);
              return length;
            }
          }
  
          try {
            // send the actual data
            dest.socket.send(data);
            return length;
          } catch (e) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
        },recvmsg:function (sock, length) {
          // http://pubs.opengroup.org/onlinepubs/7908799/xns/recvmsg.html
          if (sock.type === 1 && sock.server) {
            // tcp servers should not be recv()'ing on the listen socket
            throw new FS.ErrnoError(ERRNO_CODES.ENOTCONN);
          }
  
          var queued = sock.recv_queue.shift();
          if (!queued) {
            if (sock.type === 1) {
              var dest = SOCKFS.websocket_sock_ops.getPeer(sock, sock.daddr, sock.dport);
  
              if (!dest) {
                // if we have a destination address but are not connected, error out
                throw new FS.ErrnoError(ERRNO_CODES.ENOTCONN);
              }
              else if (dest.socket.readyState === dest.socket.CLOSING || dest.socket.readyState === dest.socket.CLOSED) {
                // return null if the socket has closed
                return null;
              }
              else {
                // else, our socket is in a valid state but truly has nothing available
                throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
              }
            } else {
              throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
            }
          }
  
          // queued.data will be an ArrayBuffer if it's unadulterated, but if it's
          // requeued TCP data it'll be an ArrayBufferView
          var queuedLength = queued.data.byteLength || queued.data.length;
          var queuedOffset = queued.data.byteOffset || 0;
          var queuedBuffer = queued.data.buffer || queued.data;
          var bytesRead = Math.min(length, queuedLength);
          var res = {
            buffer: new Uint8Array(queuedBuffer, queuedOffset, bytesRead),
            addr: queued.addr,
            port: queued.port
          };
  
  
          // push back any unread data for TCP connections
          if (sock.type === 1 && bytesRead < queuedLength) {
            var bytesRemaining = queuedLength - bytesRead;
            queued.data = new Uint8Array(queuedBuffer, queuedOffset + bytesRead, bytesRemaining);
            sock.recv_queue.unshift(queued);
          }
  
          return res;
        }}};function _send(fd, buf, len, flags) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_4(132, fd, buf, len, flags);
      var sock = SOCKFS.getSocket(fd);
      if (!sock) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      // TODO honor flags
      return _write(fd, buf, len);
    }
  
  function _pwrite(fildes, buf, nbyte, offset) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_4(66, fildes, buf, nbyte, offset);
      // ssize_t pwrite(int fildes, const void *buf, size_t nbyte, off_t offset);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/write.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      try {
        var slab = HEAP8;
        return FS.write(stream, slab, buf, nbyte, offset);
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _write(fildes, buf, nbyte) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_3(67, fildes, buf, nbyte);
      // ssize_t write(int fildes, const void *buf, size_t nbyte);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/write.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
  
  
      try {
        var slab = HEAP8;
        return FS.write(stream, slab, buf, nbyte);
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _fputc(c, stream) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_2(84, c, stream);
      // int fputc(int c, FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fputc.html
      var chr = unSign(c & 0xFF);
      HEAP8[((_fputc.ret)>>0)]=chr;
      var fd = _fileno(stream);
      var ret = _write(fd, _fputc.ret, 1);
      if (ret == -1) {
        var streamObj = FS.getStreamFromPtr(stream);
        if (streamObj) streamObj.error = true;
        return -1;
      } else {
        return chr;
      }
    }

  function _fwrite(ptr, size, nitems, stream) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_4(92, ptr, size, nitems, stream);
      // size_t fwrite(const void *restrict ptr, size_t size, size_t nitems, FILE *restrict stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fwrite.html
      var bytesToWrite = nitems * size;
      if (bytesToWrite == 0) return 0;
      var fd = _fileno(stream);
      var bytesWritten = _write(fd, ptr, bytesToWrite);
      if (bytesWritten == -1) {
        var streamObj = FS.getStreamFromPtr(stream);
        if (streamObj) streamObj.error = true;
        return 0;
      } else {
        return (bytesWritten / size)|0;
      }
    }

  function _umask(newMask) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_1(23, newMask);
      // mode_t umask(mode_t cmask);
      // http://pubs.opengroup.org/onlinepubs/7908799/xsh/umask.html
      // NOTE: This value isn't actually used for anything.
      if (_umask.cmask === undefined) _umask.cmask = 0x1FF;  // S_IRWXU | S_IRWXG | S_IRWXO.
      var oldMask = _umask.cmask;
      _umask.cmask = newMask;
      return oldMask;
    }


  
   
  Module["_strlen"] = _strlen;function _fputs(s, stream) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_2(3, s, stream);
      // int fputs(const char *restrict s, FILE *restrict stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fputs.html
      var fd = _fileno(stream);
      return _write(fd, s, _strlen(s));
    }

  
  function _chmod(path, mode, dontResolveLastLink) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_3(20, path, mode, dontResolveLastLink);
      // int chmod(const char *path, mode_t mode);
      // http://pubs.opengroup.org/onlinepubs/7908799/xsh/chmod.html
      // NOTE: dontResolveLastLink is a shortcut for lchmod(). It should never be
      //       used in client code.
      path = typeof path !== 'string' ? Pointer_stringify(path) : path;
      try {
        FS.chmod(path, mode);
        return 0;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _fchmod(fildes, mode) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_2(21, fildes, mode);
      // int fchmod(int fildes, mode_t mode);
      // http://pubs.opengroup.org/onlinepubs/7908799/xsh/fchmod.html
      try {
        FS.fchmod(fildes, mode);
        return 0;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }

/*
    var simulatedTime = 0;
  function _emscripten_get_now() {
    simulatedTime += 0.01;
    return simulatedTime;
  }
*/

  function _emscripten_get_now() {
    return performance.now();
    }

  
  function _chown(path, owner, group, dontResolveLastLink) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_4(36, path, owner, group, dontResolveLastLink);
      // int chown(const char *path, uid_t owner, gid_t group);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/chown.html
      // We don't support multiple users, so changing ownership makes no sense.
      // NOTE: The path argument may be a string, to simplify fchown().
      // NOTE: dontResolveLastLink is a shortcut for lchown(). It should never be
      //       used in client code.
      if (typeof path !== 'string') path = Pointer_stringify(path);
      try {
        FS.chown(path, owner, group);
        return 0;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _lchown(path, owner, group) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_3(53, path, owner, group);
      // int lchown(const char *path, uid_t owner, gid_t group);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/lchown.html
      return _chown(path, owner, group, true);
    }

  function _tmpnam(s, dir, prefix) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_3(99, s, dir, prefix);
      // char *tmpnam(char *s);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/tmpnam.html
      // NOTE: The dir and prefix arguments are for internal use only.
      dir = dir || '/tmp';
      var folder = FS.findObject(dir);
      if (!folder || !folder.isFolder) {
        dir = '/tmp';
        folder = FS.findObject(dir);
        if (!folder || !folder.isFolder) return 0;
      }
      var name = prefix || 'file';
      do {
        name += String.fromCharCode(65 + (Math.random() * 25)|0);
      } while (name in folder.contents);
      var result = dir + '/' + name;
      if (!_tmpnam.buffer) _tmpnam.buffer = _malloc(256);
      if (!s) s = _tmpnam.buffer;
      assert(result.length <= 255);
      writeAsciiToMemory(result, s);
      return s;
    }

  function _utime(path, times) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_2(12, path, times);
      // int utime(const char *path, const struct utimbuf *times);
      // http://pubs.opengroup.org/onlinepubs/009695399/basedefs/utime.h.html
      var time;
      if (times) {
        // NOTE: We don't keep track of access timestamps.
        var offset = 4;
        time = HEAP32[(((times)+(offset))>>2)];
        time *= 1000;
      } else {
        time = Date.now();
      }
      path = Pointer_stringify(path);
      try {
        FS.utime(path, time, time);
        return 0;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }

  function _emscripten_atomic_add_u32() {
  Module['printErr']('missing function: emscripten_atomic_add_u32'); abort(-1);
  }

  function _emscripten_cancel_main_loop() {
      Browser.mainLoop.pause();
      Browser.mainLoop.func = null;
    }

  
  function _fcntl(fildes, cmd, varargs, dup2) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_4(31, fildes, cmd, varargs, dup2);
      // int fcntl(int fildes, int cmd, ...);
      // http://pubs.opengroup.org/onlinepubs/009695399/functions/fcntl.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      switch (cmd) {
        case 0:
          var arg = HEAP32[((varargs)>>2)];
          if (arg < 0) {
            ___setErrNo(ERRNO_CODES.EINVAL);
            return -1;
          }
          var newStream;
          try {
            newStream = FS.open(stream.path, stream.flags, 0, arg);
          } catch (e) {
            FS.handleFSError(e);
            return -1;
          }
          return newStream.fd;
        case 1:
        case 2:
          return 0;  // FD_CLOEXEC makes no sense for a single process.
        case 3:
          return stream.flags;
        case 4:
          var arg = HEAP32[((varargs)>>2)];
          stream.flags |= arg;
          return 0;
        case 12:
        case 12:
          var arg = HEAP32[((varargs)>>2)];
          var offset = 0;
          // We're always unlocked.
          HEAP16[(((arg)+(offset))>>1)]=2;
          return 0;
        case 13:
        case 14:
        case 13:
        case 14:
          // Pretend that the locking is successful.
          return 0;
        case 8:
        case 9:
          // These are for sockets. We don't have them fully implemented yet.
          ___setErrNo(ERRNO_CODES.EINVAL);
          return -1;
        default:
          ___setErrNo(ERRNO_CODES.EINVAL);
          return -1;
      }
      // Should never be reached. Only to silence strict warnings.
      return -1;
    }function _dup2(fildes, fildes2) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_2(40, fildes, fildes2);
      // int dup2(int fildes, int fildes2);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/dup.html
      var stream = FS.getStream(fildes);
      if (fildes2 < 0) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      } else if (fildes === fildes2 && stream) {
        return fildes;
      } else {
        _close(fildes2);
        try {
          var stream2 = FS.open(stream.path, stream.flags, 0, fildes2, fildes2);
          return stream2.fd;
        } catch (e) {
          FS.handleFSError(e);
          return -1;
        }
      }
    }

  
  function _recv(fd, buf, len, flags) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_4(133, fd, buf, len, flags);
      var sock = SOCKFS.getSocket(fd);
      if (!sock) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      // TODO honor flags
      return _read(fd, buf, len);
    }
  
  function _pread(fildes, buf, nbyte, offset) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_4(58, fildes, buf, nbyte, offset);
      // ssize_t pread(int fildes, void *buf, size_t nbyte, off_t offset);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/read.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      try {
        var slab = HEAP8;
        return FS.read(stream, slab, buf, nbyte, offset);
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _read(fildes, buf, nbyte) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_3(59, fildes, buf, nbyte);
      // ssize_t read(int fildes, void *buf, size_t nbyte);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/read.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
  
  
      try {
        var slab = HEAP8;
        return FS.read(stream, slab, buf, nbyte);
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }

  
  
  var PROCINFO={ppid:1,pid:42,sid:42,pgid:42};var PThread={MAIN_THREAD_ID:1,mainThreadInfo:{schedPolicy:0,schedPrio:0},unusedWorkerPool:[],runningWorkers:[],initMainThreadBlock:function () {
        if (ENVIRONMENT_IS_PTHREAD) return undefined;
        PThread.mainThreadBlock = allocate(212, "i32*", ALLOC_STATIC);
        for(var i = 0; i < 212/4; ++i) HEAPU32[PThread.mainThreadBlock/4+i] = 0;
  
        // The pthread struct has a field that points to itself - this is used as a magic ID to detect whether the pthread_t
        // structure is 'alive'.
        HEAP32[(((PThread.mainThreadBlock)+(20))>>2)]=PThread.mainThreadBlock;
  
        // Allocate memory for thread-local storage.
        var tlsMemory = allocate(128 * 4, "i32*", ALLOC_STATIC);
        for(var i = 0; i < 128; ++i) HEAPU32[tlsMemory/4+i] = 0;
        Atomics.store(HEAPU32, (PThread.mainThreadBlock + 112 ) >> 2, tlsMemory); // Init thread-local-storage memory array.
        Atomics.store(HEAPU32, (PThread.mainThreadBlock + 44 ) >> 2, PThread.mainThreadBlock); // Main thread ID.
        Atomics.store(HEAPU32, (PThread.mainThreadBlock + 48 ) >> 2, PROCINFO.pid); // Process ID.
      },pthreads:{},pthreadIdCounter:2,exitHandlers:null,runExitHandlers:function () {
        if (PThread.exitHandlers !== null) {
          while (PThread.exitHandlers.length > 0) {
            PThread.exitHandlers.pop()();
          }
          PThread.exitHandlers = null;
        }
  
        // Call into the musl function that runs destructors of all thread-specific data.
        if (ENVIRONMENT_IS_PTHREAD && threadBlock) ___pthread_tsd_run_dtors();
      },threadExit:function (exitCode) {
        var tb = _pthread_self();
        if (tb) { // If we haven't yet exited?
          Atomics.store(HEAPU32, (tb + 4 ) >> 2, exitCode);
          // When we publish this, the main thread is free to deallocate the thread object and we are done.
          // Therefore set threadBlock = 0; above to 'release' the object in this worker thread.
          Atomics.store(HEAPU32, (tb + 0 ) >> 2, 1);
  
          // Disable all cancellation so that executing the cleanup handlers won't trigger another JS
          // canceled exception to be thrown.
          Atomics.store(HEAPU32, (tb + 68 ) >> 2, 1/*PTHREAD_CANCEL_DISABLE*/);
          Atomics.store(HEAPU32, (tb + 72 ) >> 2, 0/*PTHREAD_CANCEL_DEFERRED*/);
          PThread.runExitHandlers();
  
          _emscripten_futex_wake(tb + 0, 2147483647);
          threadBlock = 0;
          if (ENVIRONMENT_IS_PTHREAD) {
            postMessage({ cmd: 'exit' });
          }
        }
      },threadCancel:function () {
        PThread.runExitHandlers();
        Atomics.store(HEAPU32, (threadBlock + 4 ) >> 2, -1/*PTHREAD_CANCELED*/);
        Atomics.store(HEAPU32, (threadBlock + 0 ) >> 2, 1); // Mark the thread as no longer running.
        _emscripten_futex_wake(threadBlock + 0, 2147483647); // wake all threads
        threadBlock = selfThreadId = 0; // Not hosting a pthread anymore in this worker, reset the info structures to null.
        postMessage({ cmd: 'cancelDone' });
      },freeThreadData:function (pthread) {
        if (!pthread) return;
        if (pthread.threadBlock) {
          var tlsMemory = HEAP32[(((pthread.threadBlock)+(112))>>2)];
          HEAP32[(((pthread.threadBlock)+(112))>>2)]=0;
          _free(pthread.tlsMemory);
          _free(pthread.threadBlock);
        }
        pthread.threadBlock = 0;
        if (pthread.allocatedOwnStack && pthread.stackBase) _free(pthread.stackBase);
        pthread.stackBase = 0;
        if (pthread.worker) pthread.worker.pthread = null;
      },allocateUnusedWorkers:function (numWorkers, onFinishedLoading) {
        Module['print']('Preallocating ' + numWorkers + ' workers for a pthread spawn pool.');
        // Create a new one.
        // To spawn a web worker, we must give it a URL of the file to run. This means that for now, the new pthread we are spawning will
        // load the same Emscripten-compiled output .js file as the thread starts up.
        var url = window.location.pathname;
        url = url.substr(url.lastIndexOf('/')+1).replace('.html', '.js');
  
        var numWorkersLoaded = 0;
        for(var i = 0; i < numWorkers; ++i) {
          var worker = new Worker('pthread-main.js');
  
          worker.onmessage = function(e) {
            if (e.data.cmd == 'processQueuedMainThreadWork') {
              // TODO: Must post message to main Emscripten thread in PROXY_TO_WORKER mode.
              _emscripten_main_thread_process_queued_calls();
            } if (e.data.cmd == 'spawnThread') {
              __spawn_thread(e.data);
            } else if (e.data.cmd == 'cleanupThread') {
              __cleanup_thread(e.data.thread);
            } else if (e.data.cmd == 'killThread') {
              __kill_thread(e.data.thread);
            } else if (e.data.cmd == 'cancelThread') {
              __cancel_thread(e.data.thread);
            } else if (e.data.cmd == 'loaded') {
              ++numWorkersLoaded;
              if (numWorkersLoaded == numWorkers && onFinishedLoading) {
                onFinishedLoading();
              }
            } else if (e.data.cmd == 'print') {
              Module['print']('Thread ' + e.data.threadId + ': ' + e.data.text);
            } else if (e.data.cmd == 'printErr') {
              Module['printErr']('Thread ' + e.data.threadId + ': ' + e.data.text);
            } else if (e.data.cmd == 'exit') {
              // todo 
            } else if (e.data.cmd == 'cancelDone') {
                PThread.freeThreadData(worker.pthread);
                worker.pthread = undefined; // Detach the worker from the pthread object, and return it to the worker pool as an unused worker.
                PThread.unusedWorkerPool.push(worker);
                // TODO: Free if detached.
                PThread.runningWorkers.splice(PThread.runningWorkers.indexOf(worker.pthread), 1); // Not a running Worker anymore.
            } else {
              Module['printErr']("worker sent an unknown command " + e.data.cmd);
            }
          };
  
          worker.onerror = function(e) {
            Module['printErr']('pthread sent an error! ' + e.message);
          };
  
          // Ask the new worker to load up the Emscripten-compiled page. This is a heavy operation.
          worker.postMessage({ cmd: 'load', url: url, buffer: HEAPU8.buffer }, [HEAPU8.buffer]);
          PThread.unusedWorkerPool.push(worker);
        }
      },getNewWorker:function () {
        if (PThread.unusedWorkerPool.length == 0) PThread.allocateUnusedWorkers(1);
        if (PThread.unusedWorkerPool.length > 0) return PThread.unusedWorkerPool.pop();
        else return null;
      },busySpinWait:function (msecs) {
        var t = performance.now() + msecs;
        while(performance.now() < t)
          ;
      }};function _pthread_cleanup_push(routine, arg) {
      if (PThread.exitHandlers === null) {
        PThread.exitHandlers = [];
        if (!ENVIRONMENT_IS_PTHREAD) {
          __ATEXIT__.push({ func: function() { PThread.runExitHandlers(); } });
        }
      }
      PThread.exitHandlers.push(function() { Runtime.dynCall('vi', routine, [arg]) });
    }

  function _fchown(fildes, owner, group) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_3(41, fildes, owner, group);
      // int fchown(int fildes, uid_t owner, gid_t group);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fchown.html
      try {
        FS.fchown(fildes, owner, group);
        return 0;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }

  
  function _emscripten_memcpy_big(dest, src, num) {
      HEAPU8.set(HEAPU8.subarray(src, src+num), dest);
      return dest;
    } 
  Module["_memcpy"] = _memcpy;

  
  function _seekdir(dirp, loc) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_2(8, dirp, loc);
      // void seekdir(DIR *dirp, long int loc);
      // http://pubs.opengroup.org/onlinepubs/007908799/xsh/seekdir.html
      var fd = _fileno(dirp);
      _lseek(fd, loc, 0);
    }function _rewinddir(dirp) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_1(9, dirp);
      // void rewinddir(DIR *dirp);
      // http://pubs.opengroup.org/onlinepubs/007908799/xsh/rewinddir.html
      _seekdir(dirp, 0);
      var stream = FS.getStreamFromPtr(dirp);
      if (stream.currReading) stream.currReading = null;
    }

  function _lchmod(path, mode) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_2(22, path, mode);
      path = Pointer_stringify(path);
      try {
        FS.lchmod(path, mode);
        return 0;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }

  
  function _puts(s) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_1(86, s);
      // int puts(const char *s);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/puts.html
      // NOTE: puts() always writes an extra newline.
      var stdout = HEAP32[((_stdout)>>2)];
      var ret = _fputs(s, stdout);
      if (ret < 0) {
        return ret;
      } else {
        var newlineRet = _fputc(10, stdout);
        return (newlineRet < 0) ? -1 : ret + 1;
      }
    }
  
  
  function _strerror_r(errnum, strerrbuf, buflen) {
      if (errnum in ERRNO_MESSAGES) {
        if (ERRNO_MESSAGES[errnum].length > buflen - 1) {
          return ___setErrNo(ERRNO_CODES.ERANGE);
        } else {
          var msg = ERRNO_MESSAGES[errnum];
          writeAsciiToMemory(msg, strerrbuf);
          return 0;
        }
      } else {
        return ___setErrNo(ERRNO_CODES.EINVAL);
      }
    }function _strerror(errnum) {
      if (!_strerror.buffer) _strerror.buffer = _malloc(256);
      _strerror_r(errnum, _strerror.buffer, 256);
      return _strerror.buffer;
    }
  
  function ___errno_location() {
      return ___errno_state;
    }function _perror(s) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_1(95, s);
      // void perror(const char *s);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/perror.html
      var stdout = HEAP32[((_stdout)>>2)];
      if (s) {
        _fputs(s, stdout);
        _fputc(58, stdout);
        _fputc(32, stdout);
      }
      var errnum = HEAP32[((___errno_location())>>2)];
      _puts(_strerror(errnum));
    }



  function _emscripten_is_main_runtime_thread() {
      return !ENVIRONMENT_IS_PTHREAD;
    }

  function _emscripten_futex_wait(addr, val, timeout) {
      if (addr <= 0 || addr > HEAP8.length || addr&3 != 0) return -22;
      dump('futex_wait addr:' + addr + ' by thread: ' + _pthread_self() + (ENVIRONMENT_IS_PTHREAD?'(pthread)':'') + '\n');
      if (ENVIRONMENT_IS_PTHREAD) dump(new Error().stack);
      var ret = Atomics.futexWait(HEAP32, addr >> 2, val, timeout);
      dump('futex_wait by thread: ' + _pthread_self() + (ENVIRONMENT_IS_PTHREAD?'(pthread)':'') + ' done\n');
      if (ret == Atomics.TIMEDOUT) return -110;
      if (ret == Atomics.NOTEQUAL) return -11;
      if (ret == 0) return 0;
      throw 'Atomics.futexWait returned an unexpected value ' + ret;
    }

  
  
  function __inet_pton4_raw(str) {
      var b = str.split('.');
      for (var i = 0; i < 4; i++) {
        var tmp = Number(b[i]);
        if (isNaN(tmp)) return null;
        b[i] = tmp;
      }
      return (b[0] | (b[1] << 8) | (b[2] << 16) | (b[3] << 24)) >>> 0;
    }
  
  
  function _htons(value) {
      return ((value & 0xff) << 8) + ((value & 0xff00) >> 8);
    }function __inet_pton6_raw(str) {
      var words;
      var w, offset, z, i;
      /* http://home.deds.nl/~aeron/regex/ */
      var valid6regx = /^((?=.*::)(?!.*::.+::)(::)?([\dA-F]{1,4}:(:|\b)|){5}|([\dA-F]{1,4}:){6})((([\dA-F]{1,4}((?!\3)::|:\b|$))|(?!\2\3)){2}|(((2[0-4]|1\d|[1-9])?\d|25[0-5])\.?\b){4})$/i
      var parts = [];
      if (!valid6regx.test(str)) {
        return null;
      }
      if (str === "::") {
        return [0, 0, 0, 0, 0, 0, 0, 0];
      }
      // Z placeholder to keep track of zeros when splitting the string on ":"
      if (str.indexOf("::") === 0) {
        str = str.replace("::", "Z:"); // leading zeros case
      } else {
        str = str.replace("::", ":Z:");
      }
  
      if (str.indexOf(".") > 0) {
        // parse IPv4 embedded stress
        str = str.replace(new RegExp('[.]', 'g'), ":");
        words = str.split(":");
        words[words.length-4] = parseInt(words[words.length-4]) + parseInt(words[words.length-3])*256;
        words[words.length-3] = parseInt(words[words.length-2]) + parseInt(words[words.length-1])*256;
        words = words.slice(0, words.length-2);
      } else {
        words = str.split(":");
      }
  
      offset = 0; z = 0;
      for (w=0; w < words.length; w++) {
        if (typeof words[w] === 'string') {
          if (words[w] === 'Z') {
            // compressed zeros - write appropriate number of zero words
            for (z = 0; z < (8 - words.length+1); z++) {
              parts[w+z] = 0;
            }
            offset = z-1;
          } else {
            // parse hex to field to 16-bit value and write it in network byte-order
            parts[w+offset] = _htons(parseInt(words[w],16));
          }
        } else {
          // parsed IPv4 words
          parts[w+offset] = words[w];
        }
      }
      return [
        (parts[1] << 16) | parts[0],
        (parts[3] << 16) | parts[2],
        (parts[5] << 16) | parts[4],
        (parts[7] << 16) | parts[6]
      ];
    }var DNS={address_map:{id:1,addrs:{},names:{}},lookup_name:function (name) {
        // If the name is already a valid ipv4 / ipv6 address, don't generate a fake one.
        var res = __inet_pton4_raw(name);
        if (res) {
          return name;
        }
        res = __inet_pton6_raw(name);
        if (res) {
          return name;
        }
  
        // See if this name is already mapped.
        var addr;
  
        if (DNS.address_map.addrs[name]) {
          addr = DNS.address_map.addrs[name];
        } else {
          var id = DNS.address_map.id++;
          assert(id < 65535, 'exceeded max address mappings of 65535');
  
          addr = '172.29.' + (id & 0xff) + '.' + (id & 0xff00);
  
          DNS.address_map.names[addr] = name;
          DNS.address_map.addrs[name] = addr;
        }
  
        return addr;
      },lookup_addr:function (addr) {
        if (DNS.address_map.names[addr]) {
          return DNS.address_map.names[addr];
        }
  
        return null;
      }};
  
  
  var Sockets={BUFFER_SIZE:10240,MAX_BUFFER_SIZE:10485760,nextFd:1,fds:{},nextport:1,maxport:65535,peer:null,connections:{},portmap:{},localAddr:4261412874,addrPool:[33554442,50331658,67108874,83886090,100663306,117440522,134217738,150994954,167772170,184549386,201326602,218103818,234881034]};function __write_sockaddr(sa, family, addr, port) {
      switch (family) {
        case 2:
          addr = __inet_pton4_raw(addr);
          HEAP16[((sa)>>1)]=family;
          HEAP32[(((sa)+(4))>>2)]=addr;
          HEAP16[(((sa)+(2))>>1)]=_htons(port);
          break;
        case 10:
          addr = __inet_pton6_raw(addr);
          HEAP32[((sa)>>2)]=family;
          HEAP32[(((sa)+(8))>>2)]=addr[0];
          HEAP32[(((sa)+(12))>>2)]=addr[1];
          HEAP32[(((sa)+(16))>>2)]=addr[2];
          HEAP32[(((sa)+(20))>>2)]=addr[3];
          HEAP16[(((sa)+(2))>>1)]=_htons(port);
          HEAP32[(((sa)+(4))>>2)]=0;
          HEAP32[(((sa)+(24))>>2)]=0;
          break;
        default:
          return { errno: ERRNO_CODES.EAFNOSUPPORT };
      }
      // kind of lame, but let's match _read_sockaddr's interface
      return {};
    }function _recvfrom(fd, buf, len, flags, addr, addrlen) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_6(135, fd, buf, len, flags, addr, addrlen);
      var sock = SOCKFS.getSocket(fd);
      if (!sock) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
  
      // read from the socket
      var msg;
      try {
        msg = sock.sock_ops.recvmsg(sock, len);
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
  
      if (!msg) {
        // socket is closed
        return 0;
      }
  
      // write the source address out
      if (addr) {
        var res = __write_sockaddr(addr, sock.family, DNS.lookup_name(msg.addr), msg.port);
        assert(!res.errno);
      }
      // write the buffer out
      HEAPU8.set(msg.buffer, buf);
  
      return msg.buffer.byteLength;
    }

  
  function _open(path, oflag, varargs) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_3(26, path, oflag, varargs);
      // int open(const char *path, int oflag, ...);
      // http://pubs.opengroup.org/onlinepubs/009695399/functions/open.html
      var mode = HEAP32[((varargs)>>2)];
      path = Pointer_stringify(path);
      try {
        var stream = FS.open(path, oflag, mode);
        return stream.fd;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _opendir(dirname) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_1(5, dirname);
      // DIR *opendir(const char *dirname);
      // http://pubs.opengroup.org/onlinepubs/007908799/xsh/opendir.html
      // NOTE: Calculating absolute path redundantly since we need to associate it
      //       with the opened stream.
      var path = Pointer_stringify(dirname);
      if (!path) {
        ___setErrNo(ERRNO_CODES.ENOENT);
        return 0;
      }
      var node;
      try {
        var lookup = FS.lookupPath(path, { follow: true });
        node = lookup.node;
      } catch (e) {
        FS.handleFSError(e);
        return 0;
      }
      if (!FS.isDir(node.mode)) {
        ___setErrNo(ERRNO_CODES.ENOTDIR);
        return 0;
      }
      var fd = _open(dirname, 0, allocate([0, 0, 0, 0], 'i32', ALLOC_STACK));
      return fd === -1 ? 0 : FS.getPtrForStream(FS.getStream(fd));
    }

  
  function _fopen(filename, mode) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_2(1, filename, mode);
      // FILE *fopen(const char *restrict filename, const char *restrict mode);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fopen.html
      var flags;
      mode = Pointer_stringify(mode);
      if (mode[0] == 'r') {
        if (mode.indexOf('+') != -1) {
          flags = 2;
        } else {
          flags = 0;
        }
      } else if (mode[0] == 'w') {
        if (mode.indexOf('+') != -1) {
          flags = 2;
        } else {
          flags = 1;
        }
        flags |= 64;
        flags |= 512;
      } else if (mode[0] == 'a') {
        if (mode.indexOf('+') != -1) {
          flags = 2;
        } else {
          flags = 1;
        }
        flags |= 64;
        flags |= 1024;
      } else {
        ___setErrNo(ERRNO_CODES.EINVAL);
        return 0;
      }
      var fd = _open(filename, flags, allocate([0x1FF, 0, 0, 0], 'i32', ALLOC_STACK));  // All creation permissions.
      return fd === -1 ? 0 : FS.getPtrForStream(FS.getStream(fd));
    }function _freopen(filename, mode, stream) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_3(88, filename, mode, stream);
      // FILE *freopen(const char *restrict filename, const char *restrict mode, FILE *restrict stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/freopen.html
      if (!filename) {
        var streamObj = FS.getStreamFromPtr(stream);
        if (!streamObj) {
          ___setErrNo(ERRNO_CODES.EBADF);
          return 0;
        }
        if (_freopen.buffer) _free(_freopen.buffer);
        filename = intArrayFromString(streamObj.path);
        filename = allocate(filename, 'i8', ALLOC_NORMAL);
      }
      _fclose(stream);
      return _fopen(filename, mode);
    }

  
  
  function __inet_ntop4_raw(addr) {
      return (addr & 0xff) + '.' + ((addr >> 8) & 0xff) + '.' + ((addr >> 16) & 0xff) + '.' + ((addr >> 24) & 0xff)
    }
  
  
  function _ntohs() {
  return _htons.apply(null, arguments)
  }function __inet_ntop6_raw(ints) {
      //  ref:  http://www.ietf.org/rfc/rfc2373.txt - section 2.5.4
      //  Format for IPv4 compatible and mapped  128-bit IPv6 Addresses
      //  128-bits are split into eight 16-bit words
      //  stored in network byte order (big-endian)
      //  |                80 bits               | 16 |      32 bits        |
      //  +-----------------------------------------------------------------+
      //  |               10 bytes               |  2 |      4 bytes        |
      //  +--------------------------------------+--------------------------+
      //  +               5 words                |  1 |      2 words        |
      //  +--------------------------------------+--------------------------+
      //  |0000..............................0000|0000|    IPv4 ADDRESS     | (compatible)
      //  +--------------------------------------+----+---------------------+
      //  |0000..............................0000|FFFF|    IPv4 ADDRESS     | (mapped)
      //  +--------------------------------------+----+---------------------+
      var str = "";
      var word = 0;
      var longest = 0;
      var lastzero = 0;
      var zstart = 0;
      var len = 0;
      var i = 0;
      var parts = [
        ints[0] & 0xffff,
        (ints[0] >> 16),
        ints[1] & 0xffff,
        (ints[1] >> 16),
        ints[2] & 0xffff,
        (ints[2] >> 16),
        ints[3] & 0xffff,
        (ints[3] >> 16)
      ];
  
      // Handle IPv4-compatible, IPv4-mapped, loopback and any/unspecified addresses
  
      var hasipv4 = true;
      var v4part = "";
      // check if the 10 high-order bytes are all zeros (first 5 words)
      for (i = 0; i < 5; i++) {
        if (parts[i] !== 0) { hasipv4 = false; break; }
      }
  
      if (hasipv4) {
        // low-order 32-bits store an IPv4 address (bytes 13 to 16) (last 2 words)
        v4part = __inet_ntop4_raw(parts[6] | (parts[7] << 16));
        // IPv4-mapped IPv6 address if 16-bit value (bytes 11 and 12) == 0xFFFF (6th word)
        if (parts[5] === -1) {
          str = "::ffff:";
          str += v4part;
          return str;
        }
        // IPv4-compatible IPv6 address if 16-bit value (bytes 11 and 12) == 0x0000 (6th word)
        if (parts[5] === 0) {
          str = "::";
          //special case IPv6 addresses
          if(v4part === "0.0.0.0") v4part = ""; // any/unspecified address
          if(v4part === "0.0.0.1") v4part = "1";// loopback address
          str += v4part;
          return str;
        }
      }
  
      // Handle all other IPv6 addresses
  
      // first run to find the longest contiguous zero words
      for (word = 0; word < 8; word++) {
        if (parts[word] === 0) {
          if (word - lastzero > 1) {
            len = 0;
          }
          lastzero = word;
          len++;
        }
        if (len > longest) {
          longest = len;
          zstart = word - longest + 1;
        }
      }
  
      for (word = 0; word < 8; word++) {
        if (longest > 1) {
          // compress contiguous zeros - to produce "::"
          if (parts[word] === 0 && word >= zstart && word < (zstart + longest) ) {
            if (word === zstart) {
              str += ":";
              if (zstart === 0) str += ":"; //leading zeros case
            }
            continue;
          }
        }
        // converts 16-bit words from big-endian to little-endian before converting to hex string
        str += Number(_ntohs(parts[word] & 0xffff)).toString(16);
        str += word < 7 ? ":" : "";
      }
      return str;
    }function __read_sockaddr(sa, salen) {
      // family / port offsets are common to both sockaddr_in and sockaddr_in6
      var family = HEAP16[((sa)>>1)];
      var port = _ntohs(HEAP16[(((sa)+(2))>>1)]);
      var addr;
  
      switch (family) {
        case 2:
          if (salen !== 16) {
            return { errno: ERRNO_CODES.EINVAL };
          }
          addr = HEAP32[(((sa)+(4))>>2)];
          addr = __inet_ntop4_raw(addr);
          break;
        case 10:
          if (salen !== 28) {
            return { errno: ERRNO_CODES.EINVAL };
          }
          addr = [
            HEAP32[(((sa)+(8))>>2)],
            HEAP32[(((sa)+(12))>>2)],
            HEAP32[(((sa)+(16))>>2)],
            HEAP32[(((sa)+(20))>>2)]
          ];
          addr = __inet_ntop6_raw(addr);
          break;
        default:
          return { errno: ERRNO_CODES.EAFNOSUPPORT };
      }
  
      return { family: family, addr: addr, port: port };
    }function _sendto(fd, message, length, flags, dest_addr, dest_len) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_6(134, fd, message, length, flags, dest_addr, dest_len);
      var sock = SOCKFS.getSocket(fd);
      if (!sock) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
  
      // read the address and port to send to
      var info = __read_sockaddr(dest_addr, dest_len);
      if (info.errno) {
        ___setErrNo(info.errno);
        return -1;
      }
      var port = info.port;
      var addr = DNS.lookup_addr(info.addr) || info.addr;
  
      // send the message
      try {
        var slab = HEAP8;
        return sock.sock_ops.sendmsg(sock, slab, message, length, addr, port);
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }

  function _posix_fallocate(fd, offset, len) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_3(32, fd, offset, len);
      // int posix_fallocate(int fd, off_t offset, off_t len);
      // http://pubs.opengroup.org/onlinepubs/009695399/functions/posix_fallocate.html
      var stream = FS.getStream(fd);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      try {
        FS.allocate(stream, offset, len);
        return 0;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }

  
  
  function _isatty(fildes) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_1(52, fildes);
      // int isatty(int fildes);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/isatty.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return 0;
      }
      // HACK - implement tcgetattr
      if (!stream.tty) {
        ___setErrNo(ERRNO_CODES.ENOTTY);
        return 0;
      }
      return 1;
    }function _ttyname_r(fildes, name, namesize) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_3(63, fildes, name, namesize);
      // int ttyname_r(int fildes, char *name, size_t namesize);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/ttyname.html
      var stream = FS.getStream(fildes);
      var ttyname = '/dev/tty';
      if (!stream) {
        return ___setErrNo(ERRNO_CODES.EBADF);
      } else if (!_isatty(fildes)) {
         return ___setErrNo(ERRNO_CODES.ENOTTY);
      } else if (namesize < ttyname.length + 1) {
        return ___setErrNo(ERRNO_CODES.ERANGE);
      }
      writeStringToMemory(ttyname, name);
      return 0;
    }function _ttyname(fildes) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_1(62, fildes);
      // char *ttyname(int fildes);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/ttyname.html
      if (!_ttyname.ret) _ttyname.ret = _malloc(256);
      return _ttyname_r(fildes, _ttyname.ret, 256) ? 0 : _ttyname.ret;
    }

  function _tempnam(dir, pfx) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_2(100, dir, pfx);
      // char *tempnam(const char *dir, const char *pfx);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/tempnam.html
      return _tmpnam(0, Pointer_stringify(dir), Pointer_stringify(pfx));
    }

  function _getlogin_r(name, namesize) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_2(71, name, namesize);
      // int getlogin_r(char *name, size_t namesize);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/getlogin.html
      var ret = 'root';
      if (namesize < ret.length + 1) {
        return ___setErrNo(ERRNO_CODES.ERANGE);
      } else {
        writeAsciiToMemory(ret, name);
        return 0;
      }
    }

  
  function _mktemp(template) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_1(28, template);
      if (!_mktemp.counter) _mktemp.counter = 0;
      var c = (_mktemp.counter++).toString();
      var rep = 'XXXXXX';
      while (c.length < rep.length) c = '0' + c;
      writeArrayToMemory(intArrayFromString(c), template + Pointer_stringify(template).indexOf(rep));
      return template;
    }
  
  
  function _mknod(path, mode, dev) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_3(17, path, mode, dev);
      // int mknod(const char *path, mode_t mode, dev_t dev);
      // http://pubs.opengroup.org/onlinepubs/7908799/xsh/mknod.html
      path = Pointer_stringify(path);
      // we don't want this in the JS API as the JS API
      // uses mknod to create all nodes.
      switch (mode & 61440) {
        case 32768:
        case 8192:
        case 24576:
        case 4096:
        case 49152:
          break;
        default:
          ___setErrNo(ERRNO_CODES.EINVAL);
          return -1;
      }
      try {
        FS.mknod(path, mode, dev);
        return 0;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _mkdir(path, mode) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_2(18, path, mode);
      // int mkdir(const char *path, mode_t mode);
      // http://pubs.opengroup.org/onlinepubs/7908799/xsh/mkdir.html
      path = Pointer_stringify(path);
      // remove a trailing slash, if one - /a/b/ has basename of '', but
      // we want to create b in the context of this function
      path = PATH.normalize(path);
      if (path[path.length-1] === '/') path = path.substr(0, path.length-1);
      try {
        FS.mkdir(path, mode, 0);
        return 0;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _mkdtemp(template) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_1(30, template);
      template = _mktemp(template);
      return (_mkdir(template, 0700) === 0) ? template : 0;
    }

  function _mkfifo(path, mode) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_2(19, path, mode);
      // int mkfifo(const char *path, mode_t mode);
      // http://pubs.opengroup.org/onlinepubs/7908799/xsh/mkfifo.html
      // NOTE: We support running only a single process, and named pipes require
      //       blocking, which we can't provide. The error code is not very
      //       accurate, but it's the closest among those allowed in the standard
      //       and unlikely to result in retries.
      ___setErrNo(ERRNO_CODES.EROFS);
      return -1;
    }

  function _pclose(stream) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_1(94, stream);
      // int pclose(FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/pclose.html
      // We allow only one process, so no pipes.
      ___setErrNo(ERRNO_CODES.ECHILD);
      return -1;
    }

   
  Module["_memset"] = _memset;

  var _BDtoILow=true;

  
  function _readdir_r(dirp, entry, result) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_3(10, dirp, entry, result);
      // int readdir_r(DIR *dirp, struct dirent *entry, struct dirent **result);
      // http://pubs.opengroup.org/onlinepubs/007908799/xsh/readdir_r.html
      var stream = FS.getStreamFromPtr(dirp);
      if (!stream) {
        return ___setErrNo(ERRNO_CODES.EBADF);
      }
      if (!stream.currReading) {
        try {
          // load the list of entries now, then readdir will traverse that list, to ignore changes to files
          stream.currReading = FS.readdir(stream.path);
        } catch (e) {
          return FS.handleFSError(e);
        }
      }
      if (stream.position < 0 || stream.position >= stream.currReading.length) {
        HEAP32[((result)>>2)]=0;
        return 0;
      }
      var id;
      var type;
      var name = stream.currReading[stream.position++];
      if (!name.indexOf('.')) {
        id = 1;
        type = 4;
      } else {
        try {
          // child may have been removed since we started to read this directory
          var child = FS.lookupNode(stream.node, name);
        } catch (e) {
          // skip to the next entry (not infinite since position is incremented until currReading.length)
          return _readdir_r(dirp, entry, result);
        }
        id = child.id;
        type = FS.isChrdev(child.mode) ? 2 :  // DT_CHR, character device.
               FS.isDir(child.mode) ? 4 :     // DT_DIR, directory.
               FS.isLink(child.mode) ? 10 :   // DT_LNK, symbolic link.
               8;                             // DT_REG, regular file.
      }
      HEAP32[((entry)>>2)]=id;
      HEAP32[(((entry)+(4))>>2)]=stream.position;
      HEAP32[(((entry)+(8))>>2)]=268;
      for (var i = 0; i < name.length; i++) {
        HEAP8[(((entry + 11)+(i))>>0)]=name.charCodeAt(i);
      }
      HEAP8[(((entry + 11)+(i))>>0)]=0;
      HEAP8[(((entry)+(10))>>0)]=type;
      HEAP32[((result)>>2)]=entry;
      return 0;
    }function _readdir(dirp) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_1(11, dirp);
      // struct dirent *readdir(DIR *dirp);
      // http://pubs.opengroup.org/onlinepubs/007908799/xsh/readdir_r.html
      var stream = FS.getStreamFromPtr(dirp);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return 0;
      }
      // TODO Is it supposed to be safe to execute multiple readdirs?
      if (!_readdir.entry) _readdir.entry = _malloc(268);
      if (!_readdir.result) _readdir.result = _malloc(4);
      var err = _readdir_r(dirp, _readdir.entry, _readdir.result);
      if (err) {
        ___setErrNo(err);
        return 0;
      }
      return HEAP32[((_readdir.result)>>2)];
    }

  function _atexit(func, arg) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_2(110, func, arg);
      __ATEXIT__.unshift({ func: func, arg: arg });
    }


  
  function _unlink(path) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_1(61, path);
      // int unlink(const char *path);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/unlink.html
      path = Pointer_stringify(path);
      try {
        FS.unlink(path);
        return 0;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }
  
  function _rmdir(path) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_1(60, path);
      // int rmdir(const char *path);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/rmdir.html
      path = Pointer_stringify(path);
      try {
        FS.rmdir(path);
        return 0;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _remove(path) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_1(96, path);
      // int remove(const char *path);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/remove.html
      var ret = _unlink(path);
      if (ret == -1) ret = _rmdir(path);
      return ret;
    }

  function _getsockname(fd, addr, addrlen) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_3(130, fd, addr, addrlen);
      var sock = SOCKFS.getSocket(fd);
      if (!sock) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      try {
        var info = sock.sock_ops.getname(sock);
        var res = __write_sockaddr(addr, sock.family, DNS.lookup_name(info.addr), info.port);
        assert(!res.errno);
        return 0;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }

  
  function __spawn_thread(threadParams) {
      if (ENVIRONMENT_IS_WORKER || ENVIRONMENT_IS_PTHREAD) throw 'Internal Error! _spawn_thread() can only ever be called from main JS thread!';
  
      var worker = PThread.getNewWorker();
      if (worker.pthread !== undefined) throw 'Internal error!';
      if (!threadParams.pthread_ptr) throw 'Internal error, no pthread ptr!';
      PThread.runningWorkers.push(worker);
  
      // Allocate memory for thread-local storage and initialize it to zero.
      var tlsMemory = _malloc(128 * 4);
      for(var i = 0; i < 128; ++i) {
        HEAP32[(((tlsMemory)+(i*4))>>2)]=0;
      }
  
      var pthread = PThread.pthreads[threadParams.pthread_ptr] = { // Create a pthread info object to represent this thread.
        worker: worker,
        stackBase: threadParams.stackBase,
        stackSize: threadParams.stackSize,
        allocatedOwnStack: threadParams.allocatedOwnStack,
        thread: threadParams.pthread_ptr,
        threadBlock: threadParams.pthread_ptr // Info area for this thread in Emscripten HEAP (shared)
      };
      Atomics.store(HEAPU32, (pthread.threadBlock + 0 ) >> 2, 0); // threadStatus <- 0, meaning not yet exited.
      Atomics.store(HEAPU32, (pthread.threadBlock + 4 ) >> 2, 0); // threadExitCode <- 0.
      Atomics.store(HEAPU32, (pthread.threadBlock + 76 ) >> 2, threadParams.detached);
      Atomics.store(HEAPU32, (pthread.threadBlock + 112 ) >> 2, tlsMemory); // Init thread-local-storage memory array.
      Atomics.store(HEAPU32, (pthread.threadBlock + 52 ) >> 2, 0); // Mark initial status to unused.
      Atomics.store(HEAPU32, (pthread.threadBlock + 44 ) >> 2, pthread.threadBlock); // Main thread ID.
      Atomics.store(HEAPU32, (pthread.threadBlock + 48 ) >> 2, PROCINFO.pid); // Process ID.
  
      Atomics.store(HEAPU32, (pthread.threadBlock + 116) >> 2, threadParams.stackSize);
      Atomics.store(HEAPU32, (pthread.threadBlock + 116 + 8) >> 2, threadParams.stackBase);
      Atomics.store(HEAPU32, (pthread.threadBlock + 116 + 12) >> 2, threadParams.detached);
      Atomics.store(HEAPU32, (pthread.threadBlock + 116 + 20) >> 2, threadParams.schedPolicy);
      Atomics.store(HEAPU32, (pthread.threadBlock + 116 + 24) >> 2, threadParams.schedPrio);
  
      worker.pthread = pthread;
  
      // Ask the worker to start executing its pthread entry point function.
      worker.postMessage({
        cmd: 'run',
        start_routine: threadParams.startRoutine,
        arg: threadParams.arg,
        threadBlock: threadParams.pthread_ptr,
        selfThreadId: threadParams.pthread_ptr, // TODO: Remove this since thread ID is now the same as the thread address.
        stackBase: threadParams.stackBase,
        stackSize: threadParams.stackSize,
        stdin: _stdin,
        stdout: _stdout,
        stderr: _stderr
      });
    }
  
  function _pthread_getschedparam(thread, policy, schedparam) {
      if (!policy && !schedparam) return ERRNO_CODES.EINVAL;
  
      if (!thread) {
        Module['printErr']('pthread_getschedparam called with a null thread pointer!');
        return ERRNO_CODES.ESRCH;
      }
      var self = HEAP32[(((thread)+(20))>>2)];
      if (self != thread) {
        //Module['printErr']('pthread_getschedparam attempted on thread ' + thread + ', which does not point to a valid thread, or does not exist anymore!');
        return ERRNO_CODES.ESRCH;
      }
  
      var schedPolicy = Atomics.load(HEAPU32, (thread + 116 + 20 ) >> 2);
      var schedPrio = Atomics.load(HEAPU32, (thread + 116 + 24 ) >> 2);
  
      if (policy) HEAP32[((policy)>>2)]=schedPolicy;
      if (schedparam) HEAP32[((schedparam)>>2)]=schedPrio;
      return 0;
    }
  
  function _pthread_self() {
      if (ENVIRONMENT_IS_PTHREAD) return threadBlock;
      return PThread.mainThreadBlock; // Main JS thread.
    }function _pthread_create(pthread_ptr, attr, start_routine, arg) {
      if (!HEAPU8.buffer instanceof SharedArrayBuffer) {
        Module['printErr']('Current environment does not support SharedArrayBuffer, pthreads are not available!');
        return 1;
      }
      if (!pthread_ptr) {
        Module['printErr']('pthread_create called with a null thread pointer!');
        return 1;
      }
      var stackSize = 0;
      var stackBase = 0;
      var detached = 0; // Default thread attr is PTHREAD_CREATE_JOINABLE, i.e. start as not detached.
      var schedPolicy = 0; /*SCHED_OTHER*/
      var schedPrio = 0;
      if (attr) {
        stackSize = HEAP32[((attr)>>2)];
        stackBase = HEAP32[(((attr)+(8))>>2)];
        detached = HEAP32[(((attr)+(12))>>2)] != 0/*PTHREAD_CREATE_JOINABLE*/;
        var inheritSched = HEAP32[(((attr)+(16))>>2)] == 0/*PTHREAD_INHERIT_SCHED*/;
        if (inheritSched) {
          var prevSchedPolicy = HEAP32[(((attr)+(20))>>2)];
          var prevSchedPrio = HEAP32[(((attr)+(24))>>2)];
          _pthread_getschedparam(_pthread_self(), attr + 20, attr + 24);
          schedPolicy = HEAP32[(((attr)+(20))>>2)];
          schedPrio = HEAP32[(((attr)+(24))>>2)];
          HEAP32[(((attr)+(20))>>2)]=prevSchedPolicy;
          HEAP32[(((attr)+(24))>>2)]=prevSchedPrio;
        } else {
          schedPolicy = HEAP32[(((attr)+(20))>>2)];
          schedPrio = HEAP32[(((attr)+(24))>>2)];
        }
      }
      stackSize += 81920 /*DEFAULT_STACK_SIZE*/;
      var allocatedOwnStack = stackBase == 0; // If allocatedOwnStack == true, then the pthread impl maintains the stack allocation.
      if (allocatedOwnStack) {
        stackBase = _malloc(stackSize); // Allocate a stack if the user doesn't want to place the stack in a custom memory area.
      } else {
        // Musl stores the stack base address assuming stack grows downwards, so adjust it to Emscripten convention that the
        // stack grows upwards instead.
        stackBase -= stackSize;
        assert(stackBase > 0);
      }
  
      // Allocate thread block (pthread_t structure).
      var threadBlock = _malloc(212);
      for(var i = 0; i < 212 >> 2; ++i) HEAPU32[(threadBlock>>2) + i] = 0; // zero-initialize thread structure.
      HEAP32[((pthread_ptr)>>2)]=threadBlock;
  
      // The pthread struct has a field that points to itself - this is used as a magic ID to detect whether the pthread_t
      // structure is 'alive'.
      HEAP32[(((threadBlock)+(20))>>2)]=threadBlock;
  
      var threadParams = {
        stackBase: stackBase,
        stackSize: stackSize,
        allocatedOwnStack: allocatedOwnStack,
        schedPolicy: schedPolicy,
        schedPrio: schedPrio,
        detached: detached,
        startRoutine: start_routine,
        pthread_ptr: threadBlock,
        arg: arg,
      };
  
      if (ENVIRONMENT_IS_WORKER) {
        // The prepopulated pool of web workers that can host pthreads is stored in the main JS thread. Therefore if a
        // pthread is attempting to spawn a new thread, the thread creation must be deferred to the main JS thread.
        threadParams.cmd = 'spawnThread';
        postMessage(threadParams);
      } else {
        // We are the main thread, so we have the pthread warmup pool in this thread and can fire off JS thread creation
        // directly ourselves.
        __spawn_thread(threadParams);
      }
  
      return 0;
    }

  
  
  function _fread(ptr, size, nitems, stream) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_4(87, ptr, size, nitems, stream);
      // size_t fread(void *restrict ptr, size_t size, size_t nitems, FILE *restrict stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fread.html
      var bytesToRead = nitems * size;
      if (bytesToRead == 0) {
        return 0;
      }
      var bytesRead = 0;
      var streamObj = FS.getStreamFromPtr(stream);
      if (!streamObj) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return 0;
      }
      while (streamObj.ungotten.length && bytesToRead > 0) {
        HEAP8[((ptr++)>>0)]=streamObj.ungotten.pop();
        bytesToRead--;
        bytesRead++;
      }
      var err = _read(streamObj.fd, ptr, bytesToRead);
      if (err == -1) {
        if (streamObj) streamObj.error = true;
        return 0;
      }
      bytesRead += err;
      if (bytesRead < bytesToRead) streamObj.eof = true;
      return (bytesRead / size)|0;
    }function _fgetc(stream) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_1(79, stream);
      // int fgetc(FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fgetc.html
      var streamObj = FS.getStreamFromPtr(stream);
      if (!streamObj) return -1;
      if (streamObj.eof || streamObj.error) return -1;
      var ret = _fread(_fgetc.ret, 1, 1, stream);
      if (ret == 0) {
        return -1;
      } else if (ret == -1) {
        streamObj.error = true;
        return -1;
      } else {
        return HEAPU8[((_fgetc.ret)>>0)];
      }
    }function _fgets(s, n, stream) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_3(2, s, n, stream);
      // char *fgets(char *restrict s, int n, FILE *restrict stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fgets.html
      var streamObj = FS.getStreamFromPtr(stream);
      if (!streamObj) return 0;
      if (streamObj.error || streamObj.eof) return 0;
      var byte_;
      for (var i = 0; i < n - 1 && byte_ != 10; i++) {
        byte_ = _fgetc(stream);
        if (byte_ == -1) {
          if (streamObj.error || (streamObj.eof && i == 0)) return 0;
          else if (streamObj.eof) break;
        }
        HEAP8[(((s)+(i))>>0)]=byte_;
      }
      HEAP8[(((s)+(i))>>0)]=0;
      return s;
    }


  
   
  Module["_strcpy"] = _strcpy;function _ctermid(s) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_1(43, s);
      // char *ctermid(char *s);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/ctermid.html
      if (!_ctermid.ret) {
        var arr = intArrayFromString('/dev/tty');
        _ctermid.ret = allocate(arr, 'i8', ALLOC_NORMAL);
      }
      return s ? _strcpy(s, _ctermid.ret) : _ctermid.ret;
    }


  function _tcgetattr(fildes, termios_p) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_2(117, fildes, termios_p);
      // http://pubs.opengroup.org/onlinepubs/009695399/functions/tcgetattr.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      if (!stream.tty) {
        ___setErrNo(ERRNO_CODES.ENOTTY);
        return -1;
      }
      return 0;
    }


  function _fgetpos(stream, pos) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_2(81, stream, pos);
      // int fgetpos(FILE *restrict stream, fpos_t *restrict pos);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fgetpos.html
      stream = FS.getStreamFromPtr(stream);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      if (FS.isChrdev(stream.node.mode)) {
        ___setErrNo(ERRNO_CODES.ESPIPE);
        return -1;
      }
      HEAP32[((pos)>>2)]=stream.position;
      var state = (stream.eof ? 1 : 0) + (stream.error ? 2 : 0);
      HEAP32[(((pos)+(4))>>2)]=state;
      return 0;
    }

  function _fsetpos(stream, pos) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_2(90, stream, pos);
      // int fsetpos(FILE *stream, const fpos_t *pos);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fsetpos.html
      stream = FS.getStreamFromPtr(stream);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      if (FS.isChrdev(stream.node.mode)) {
        ___setErrNo(ERRNO_CODES.EPIPE);
        return -1;
      }
      stream.position = HEAP32[((pos)>>2)];
      var state = HEAP32[(((pos)+(4))>>2)];
      stream.eof = Boolean(state & 1);
      stream.error = Boolean(state & 2);
      return 0;
    }

  function _access(path, amode) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_2(34, path, amode);
      // int access(const char *path, int amode);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/access.html
      path = Pointer_stringify(path);
      if (amode & ~7) {
        // need a valid mode
        ___setErrNo(ERRNO_CODES.EINVAL);
        return -1;
      }
      var node;
      try {
        var lookup = FS.lookupPath(path, { follow: true });
        node = lookup.node;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
      var perms = '';
      if (amode & 4) perms += 'r';
      if (amode & 2) perms += 'w';
      if (amode & 1) perms += 'x';
      if (perms /* otherwise, they've just passed F_OK */ && FS.nodePermissions(node, perms)) {
        ___setErrNo(ERRNO_CODES.EACCES);
        return -1;
      }
      return 0;
    }

  function _ftell(stream) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_1(91, stream);
      // long ftell(FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/ftell.html
      stream = FS.getStreamFromPtr(stream);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      if (FS.isChrdev(stream.node.mode)) {
        ___setErrNo(ERRNO_CODES.ESPIPE);
        return -1;
      } else {
        return stream.position;
      }
    }

  
  
  function _emscripten_set_main_loop_timing(mode, value) {
      Browser.mainLoop.timingMode = mode;
      Browser.mainLoop.timingValue = value;
  
      if (!Browser.mainLoop.func) {
        console.error('emscripten_set_main_loop_timing: Cannot set timing mode for main loop since a main loop does not exist! Call emscripten_set_main_loop first to set one up.');
        return 1; // Return non-zero on failure, can't set timing mode when there is no main loop.
      }
  
      if (mode == 0 /*EM_TIMING_SETTIMEOUT*/) {
        Browser.mainLoop.scheduler = function Browser_mainLoop_scheduler() {
          setTimeout(Browser.mainLoop.runner, value); // doing this each time means that on exception, we stop
        };
        Browser.mainLoop.method = 'timeout';
      } else if (mode == 1 /*EM_TIMING_RAF*/) {
        Browser.mainLoop.scheduler = function Browser_mainLoop_scheduler() {
          Browser.requestAnimationFrame(Browser.mainLoop.runner);
        };
        Browser.mainLoop.method = 'rAF';
      }
      return 0;
    }function _emscripten_set_main_loop(func, fps, simulateInfiniteLoop, arg) {
      Module['noExitRuntime'] = true;
  
      assert(!Browser.mainLoop.func, 'emscripten_set_main_loop: there can only be one main loop function at once: call emscripten_cancel_main_loop to cancel the previous one before setting a new one with different parameters.');
  
      Browser.mainLoop.func = func;
      Browser.mainLoop.arg = arg;
  
      var thisMainLoopId = Browser.mainLoop.currentlyRunningMainloop;
  
      Browser.mainLoop.runner = function Browser_mainLoop_runner() {
        if (ABORT) return;
        if (Browser.mainLoop.queue.length > 0) {
          var start = Date.now();
          var blocker = Browser.mainLoop.queue.shift();
          blocker.func(blocker.arg);
          if (Browser.mainLoop.remainingBlockers) {
            var remaining = Browser.mainLoop.remainingBlockers;
            var next = remaining%1 == 0 ? remaining-1 : Math.floor(remaining);
            if (blocker.counted) {
              Browser.mainLoop.remainingBlockers = next;
            } else {
              // not counted, but move the progress along a tiny bit
              next = next + 0.5; // do not steal all the next one's progress
              Browser.mainLoop.remainingBlockers = (8*remaining + next)/9;
            }
          }
          console.log('main loop blocker "' + blocker.name + '" took ' + (Date.now() - start) + ' ms'); //, left: ' + Browser.mainLoop.remainingBlockers);
          Browser.mainLoop.updateStatus();
          setTimeout(Browser.mainLoop.runner, 0);
          return;
        }
  
        // catch pauses from non-main loop sources
        if (thisMainLoopId < Browser.mainLoop.currentlyRunningMainloop) return;
  
        // Implement very basic swap interval control
        Browser.mainLoop.currentFrameNumber = Browser.mainLoop.currentFrameNumber + 1 | 0;
        if (Browser.mainLoop.timingMode == 1/*EM_TIMING_RAF*/ && Browser.mainLoop.timingValue > 1 && Browser.mainLoop.currentFrameNumber % Browser.mainLoop.timingValue != 0) {
          // Not the scheduled time to render this frame - skip.
          Browser.mainLoop.scheduler();
          return;
        }
  
        // Signal GL rendering layer that processing of a new frame is about to start. This helps it optimize
        // VBO double-buffering and reduce GPU stalls.
  
        if (Browser.mainLoop.method === 'timeout' && Module.ctx) {
          Module.printErr('Looks like you are rendering without using requestAnimationFrame for the main loop. You should use 0 for the frame rate in emscripten_set_main_loop in order to use requestAnimationFrame, as that can greatly improve your frame rates!');
          Browser.mainLoop.method = ''; // just warn once per call to set main loop
        }
  
        Browser.mainLoop.runIter(function() {
          if (typeof arg !== 'undefined') {
            Runtime.dynCall('vi', func, [arg]);
          } else {
            Runtime.dynCall('v', func);
          }
        });
  
        // catch pauses from the main loop itself
        if (thisMainLoopId < Browser.mainLoop.currentlyRunningMainloop) return;
  
        // Queue new audio data. This is important to be right after the main loop invocation, so that we will immediately be able
        // to queue the newest produced audio samples.
        // TODO: Consider adding pre- and post- rAF callbacks so that GL.newRenderingFrameStarted() and SDL.audio.queueNewAudioData()
        //       do not need to be hardcoded into this function, but can be more generic.
        if (typeof SDL === 'object' && SDL.audio && SDL.audio.queueNewAudioData) SDL.audio.queueNewAudioData();
  
        Browser.mainLoop.scheduler();
      }
  
      if (fps && fps > 0) _emscripten_set_main_loop_timing(0/*EM_TIMING_SETTIMEOUT*/, 1000.0 / fps);
      else _emscripten_set_main_loop_timing(1/*EM_TIMING_RAF*/, 1); // Do rAF by rendering each frame (no decimating)
  
      Browser.mainLoop.scheduler();
  
      if (simulateInfiniteLoop) {
        throw 'SimulateInfiniteLoop';
      }
    }var Browser={mainLoop:{scheduler:null,method:"",currentlyRunningMainloop:0,func:null,arg:0,timingMode:0,timingValue:0,currentFrameNumber:0,queue:[],pause:function () {
          Browser.mainLoop.scheduler = null;
          Browser.mainLoop.currentlyRunningMainloop++; // Incrementing this signals the previous main loop that it's now become old, and it must return.
        },resume:function () {
          Browser.mainLoop.currentlyRunningMainloop++;
          var timingMode = Browser.mainLoop.timingMode;
          var timingValue = Browser.mainLoop.timingValue;
          var func = Browser.mainLoop.func;
          Browser.mainLoop.func = null;
          _emscripten_set_main_loop(func, 0, false, Browser.mainLoop.arg);
          _emscripten_set_main_loop_timing(timingMode, timingValue);
        },updateStatus:function () {
          if (Module['setStatus']) {
            var message = Module['statusMessage'] || 'Please wait...';
            var remaining = Browser.mainLoop.remainingBlockers;
            var expected = Browser.mainLoop.expectedBlockers;
            if (remaining) {
              if (remaining < expected) {
                Module['setStatus'](message + ' (' + (expected - remaining) + '/' + expected + ')');
              } else {
                Module['setStatus'](message);
              }
            } else {
              Module['setStatus']('');
            }
          }
        },runIter:function (func) {
          if (ABORT) return;
          if (Module['preMainLoop']) {
            var preRet = Module['preMainLoop']();
            if (preRet === false) {
              return; // |return false| skips a frame
            }
          }
          try {
            func();
          } catch (e) {
            if (e instanceof ExitStatus) {
              return;
            } else {
              if (e && typeof e === 'object' && e.stack) Module.printErr('exception thrown: ' + [e, e.stack]);
              throw e;
            }
          }
          if (Module['postMainLoop']) Module['postMainLoop']();
        }},isFullScreen:false,pointerLock:false,moduleContextCreatedCallbacks:[],workers:[],init:function () {
        if (!Module["preloadPlugins"]) Module["preloadPlugins"] = []; // needs to exist even in workers
  
        if (Browser.initted) return;
        Browser.initted = true;
  
        try {
          new Blob();
          Browser.hasBlobConstructor = true;
        } catch(e) {
          Browser.hasBlobConstructor = false;
          console.log("warning: no blob constructor, cannot create blobs with mimetypes");
        }
        Browser.BlobBuilder = typeof MozBlobBuilder != "undefined" ? MozBlobBuilder : (typeof WebKitBlobBuilder != "undefined" ? WebKitBlobBuilder : (!Browser.hasBlobConstructor ? console.log("warning: no BlobBuilder") : null));
        Browser.URLObject = typeof window != "undefined" ? (window.URL ? window.URL : window.webkitURL) : undefined;
        if (!Module.noImageDecoding && typeof Browser.URLObject === 'undefined') {
          console.log("warning: Browser does not support creating object URLs. Built-in browser image decoding will not be available.");
          Module.noImageDecoding = true;
        }
  
        // Support for plugins that can process preloaded files. You can add more of these to
        // your app by creating and appending to Module.preloadPlugins.
        //
        // Each plugin is asked if it can handle a file based on the file's name. If it can,
        // it is given the file's raw data. When it is done, it calls a callback with the file's
        // (possibly modified) data. For example, a plugin might decompress a file, or it
        // might create some side data structure for use later (like an Image element, etc.).
  
        var imagePlugin = {};
        imagePlugin['canHandle'] = function imagePlugin_canHandle(name) {
          return !Module.noImageDecoding && /\.(jpg|jpeg|png|bmp)$/i.test(name);
        };
        imagePlugin['handle'] = function imagePlugin_handle(byteArray, name, onload, onerror) {
          var b = null;
          if (Browser.hasBlobConstructor) {
            try {
              b = new Blob([byteArray], { type: Browser.getMimetype(name) });
              if (b.size !== byteArray.length) { // Safari bug #118630
                // Safari's Blob can only take an ArrayBuffer
                b = new Blob([(new Uint8Array(byteArray)).buffer], { type: Browser.getMimetype(name) });
              }
            } catch(e) {
              Runtime.warnOnce('Blob constructor present but fails: ' + e + '; falling back to blob builder');
            }
          }
          if (!b) {
            var bb = new Browser.BlobBuilder();
            bb.append((new Uint8Array(byteArray)).buffer); // we need to pass a buffer, and must copy the array to get the right data range
            b = bb.getBlob();
          }
          var url = Browser.URLObject.createObjectURL(b);
          assert(typeof url == 'string', 'createObjectURL must return a url as a string');
          var img = new Image();
          img.onload = function img_onload() {
            assert(img.complete, 'Image ' + name + ' could not be decoded');
            var canvas = document.createElement('canvas');
            canvas.width = img.width;
            canvas.height = img.height;
            var ctx = canvas.getContext('2d');
            ctx.drawImage(img, 0, 0);
            Module["preloadedImages"][name] = canvas;
            Browser.URLObject.revokeObjectURL(url);
            if (onload) onload(byteArray);
          };
          img.onerror = function img_onerror(event) {
            console.log('Image ' + url + ' could not be decoded');
            if (onerror) onerror();
          };
          img.src = url;
        };
        Module['preloadPlugins'].push(imagePlugin);
  
        var audioPlugin = {};
        audioPlugin['canHandle'] = function audioPlugin_canHandle(name) {
          return !Module.noAudioDecoding && name.substr(-4) in { '.ogg': 1, '.wav': 1, '.mp3': 1 };
        };
        audioPlugin['handle'] = function audioPlugin_handle(byteArray, name, onload, onerror) {
          var done = false;
          function finish(audio) {
            if (done) return;
            done = true;
            Module["preloadedAudios"][name] = audio;
            if (onload) onload(byteArray);
          }
          function fail() {
            if (done) return;
            done = true;
            Module["preloadedAudios"][name] = new Audio(); // empty shim
            if (onerror) onerror();
          }
          if (Browser.hasBlobConstructor) {
            try {
              var b = new Blob([byteArray], { type: Browser.getMimetype(name) });
            } catch(e) {
              return fail();
            }
            var url = Browser.URLObject.createObjectURL(b); // XXX we never revoke this!
            assert(typeof url == 'string', 'createObjectURL must return a url as a string');
            var audio = new Audio();
            audio.addEventListener('canplaythrough', function() { finish(audio) }, false); // use addEventListener due to chromium bug 124926
            audio.onerror = function audio_onerror(event) {
              if (done) return;
              console.log('warning: browser could not fully decode audio ' + name + ', trying slower base64 approach');
              function encode64(data) {
                var BASE = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';
                var PAD = '=';
                var ret = '';
                var leftchar = 0;
                var leftbits = 0;
                for (var i = 0; i < data.length; i++) {
                  leftchar = (leftchar << 8) | data[i];
                  leftbits += 8;
                  while (leftbits >= 6) {
                    var curr = (leftchar >> (leftbits-6)) & 0x3f;
                    leftbits -= 6;
                    ret += BASE[curr];
                  }
                }
                if (leftbits == 2) {
                  ret += BASE[(leftchar&3) << 4];
                  ret += PAD + PAD;
                } else if (leftbits == 4) {
                  ret += BASE[(leftchar&0xf) << 2];
                  ret += PAD;
                }
                return ret;
              }
              audio.src = 'data:audio/x-' + name.substr(-3) + ';base64,' + encode64(byteArray);
              finish(audio); // we don't wait for confirmation this worked - but it's worth trying
            };
            audio.src = url;
            // workaround for chrome bug 124926 - we do not always get oncanplaythrough or onerror
            Browser.safeSetTimeout(function() {
              finish(audio); // try to use it even though it is not necessarily ready to play
            }, 10000);
          } else {
            return fail();
          }
        };
        Module['preloadPlugins'].push(audioPlugin);
  
        // Canvas event setup
  
        var canvas = Module['canvas'];
        function pointerLockChange() {
          Browser.pointerLock = document['pointerLockElement'] === canvas ||
                                document['mozPointerLockElement'] === canvas ||
                                document['webkitPointerLockElement'] === canvas ||
                                document['msPointerLockElement'] === canvas;
        }
        if (canvas) {
          // forced aspect ratio can be enabled by defining 'forcedAspectRatio' on Module
          // Module['forcedAspectRatio'] = 4 / 3;
          
          canvas.requestPointerLock = canvas['requestPointerLock'] ||
                                      canvas['mozRequestPointerLock'] ||
                                      canvas['webkitRequestPointerLock'] ||
                                      canvas['msRequestPointerLock'] ||
                                      function(){};
          canvas.exitPointerLock = document['exitPointerLock'] ||
                                   document['mozExitPointerLock'] ||
                                   document['webkitExitPointerLock'] ||
                                   document['msExitPointerLock'] ||
                                   function(){}; // no-op if function does not exist
          canvas.exitPointerLock = canvas.exitPointerLock.bind(document);
  
  
          document.addEventListener('pointerlockchange', pointerLockChange, false);
          document.addEventListener('mozpointerlockchange', pointerLockChange, false);
          document.addEventListener('webkitpointerlockchange', pointerLockChange, false);
          document.addEventListener('mspointerlockchange', pointerLockChange, false);
  
          if (Module['elementPointerLock']) {
            canvas.addEventListener("click", function(ev) {
              if (!Browser.pointerLock && canvas.requestPointerLock) {
                canvas.requestPointerLock();
                ev.preventDefault();
              }
            }, false);
          }
        }
      },createContext:function (canvas, useWebGL, setInModule, webGLContextAttributes) {
        if (useWebGL && Module.ctx && canvas == Module.canvas) return Module.ctx; // no need to recreate GL context if it's already been created for this canvas.
  
        var ctx;
        var contextHandle;
        if (useWebGL) {
          // For GLES2/desktop GL compatibility, adjust a few defaults to be different to WebGL defaults, so that they align better with the desktop defaults.
          var contextAttributes = {
            antialias: false,
            alpha: false
          };
  
          if (webGLContextAttributes) {
            for (var attribute in webGLContextAttributes) {
              contextAttributes[attribute] = webGLContextAttributes[attribute];
            }
          }
  
          contextHandle = GL.createContext(canvas, contextAttributes);
          if (contextHandle) {
            ctx = GL.getContext(contextHandle).GLctx;
          }
          // Set the background of the WebGL canvas to black
          canvas.style.backgroundColor = "black";
        } else {
          ctx = canvas.getContext('2d');
        }
  
        if (!ctx) return null;
  
        if (setInModule) {
          if (!useWebGL) assert(typeof GLctx === 'undefined', 'cannot set in module if GLctx is used, but we are a non-GL context that would replace it');
  
          Module.ctx = ctx;
          if (useWebGL) GL.makeContextCurrent(contextHandle);
          Module.useWebGL = useWebGL;
          Browser.moduleContextCreatedCallbacks.forEach(function(callback) { callback() });
          Browser.init();
        }
        return ctx;
      },destroyContext:function (canvas, useWebGL, setInModule) {},fullScreenHandlersInstalled:false,lockPointer:undefined,resizeCanvas:undefined,requestFullScreen:function (lockPointer, resizeCanvas, vrDevice) {
        Browser.lockPointer = lockPointer;
        Browser.resizeCanvas = resizeCanvas;
        Browser.vrDevice = vrDevice;
        if (typeof Browser.lockPointer === 'undefined') Browser.lockPointer = true;
        if (typeof Browser.resizeCanvas === 'undefined') Browser.resizeCanvas = false;
        if (typeof Browser.vrDevice === 'undefined') Browser.vrDevice = null;
  
        var canvas = Module['canvas'];
        function fullScreenChange() {
          Browser.isFullScreen = false;
          var canvasContainer = canvas.parentNode;
          if ((document['webkitFullScreenElement'] || document['webkitFullscreenElement'] ||
               document['mozFullScreenElement'] || document['mozFullscreenElement'] ||
               document['fullScreenElement'] || document['fullscreenElement'] ||
               document['msFullScreenElement'] || document['msFullscreenElement'] ||
               document['webkitCurrentFullScreenElement']) === canvasContainer) {
            canvas.cancelFullScreen = document['cancelFullScreen'] ||
                                      document['mozCancelFullScreen'] ||
                                      document['webkitCancelFullScreen'] ||
                                      document['msExitFullscreen'] ||
                                      document['exitFullscreen'] ||
                                      function() {};
            canvas.cancelFullScreen = canvas.cancelFullScreen.bind(document);
            if (Browser.lockPointer) canvas.requestPointerLock();
            Browser.isFullScreen = true;
            if (Browser.resizeCanvas) Browser.setFullScreenCanvasSize();
          } else {
            
            // remove the full screen specific parent of the canvas again to restore the HTML structure from before going full screen
            canvasContainer.parentNode.insertBefore(canvas, canvasContainer);
            canvasContainer.parentNode.removeChild(canvasContainer);
            
            if (Browser.resizeCanvas) Browser.setWindowedCanvasSize();
          }
          if (Module['onFullScreen']) Module['onFullScreen'](Browser.isFullScreen);
          Browser.updateCanvasDimensions(canvas);
        }
  
        if (!Browser.fullScreenHandlersInstalled) {
          Browser.fullScreenHandlersInstalled = true;
          document.addEventListener('fullscreenchange', fullScreenChange, false);
          document.addEventListener('mozfullscreenchange', fullScreenChange, false);
          document.addEventListener('webkitfullscreenchange', fullScreenChange, false);
          document.addEventListener('MSFullscreenChange', fullScreenChange, false);
        }
  
        // create a new parent to ensure the canvas has no siblings. this allows browsers to optimize full screen performance when its parent is the full screen root
        var canvasContainer = document.createElement("div");
        canvas.parentNode.insertBefore(canvasContainer, canvas);
        canvasContainer.appendChild(canvas);
  
        // use parent of canvas as full screen root to allow aspect ratio correction (Firefox stretches the root to screen size)
        canvasContainer.requestFullScreen = canvasContainer['requestFullScreen'] ||
                                            canvasContainer['mozRequestFullScreen'] ||
                                            canvasContainer['msRequestFullscreen'] ||
                                           (canvasContainer['webkitRequestFullScreen'] ? function() { canvasContainer['webkitRequestFullScreen'](Element['ALLOW_KEYBOARD_INPUT']) } : null);
  
        if (vrDevice) {
          canvasContainer.requestFullScreen({ vrDisplay: vrDevice });
        } else {
          canvasContainer.requestFullScreen();
        }
      },nextRAF:0,fakeRequestAnimationFrame:function (func) {
        // try to keep 60fps between calls to here
        var now = Date.now();
        if (Browser.nextRAF === 0) {
          Browser.nextRAF = now + 1000/60;
        } else {
          while (now + 2 >= Browser.nextRAF) { // fudge a little, to avoid timer jitter causing us to do lots of delay:0
            Browser.nextRAF += 1000/60;
          }
        }
        var delay = Math.max(Browser.nextRAF - now, 0);
        setTimeout(func, delay);
      },requestAnimationFrame:function requestAnimationFrame(func) {
        if (typeof window === 'undefined') { // Provide fallback to setTimeout if window is undefined (e.g. in Node.js)
          Browser.fakeRequestAnimationFrame(func);
        } else {
          if (!window.requestAnimationFrame) {
            window.requestAnimationFrame = window['requestAnimationFrame'] ||
                                           window['mozRequestAnimationFrame'] ||
                                           window['webkitRequestAnimationFrame'] ||
                                           window['msRequestAnimationFrame'] ||
                                           window['oRequestAnimationFrame'] ||
                                           Browser.fakeRequestAnimationFrame;
          }
          window.requestAnimationFrame(func);
        }
      },safeCallback:function (func) {
        return function() {
          if (!ABORT) return func.apply(null, arguments);
        };
      },safeRequestAnimationFrame:function (func) {
        return Browser.requestAnimationFrame(function() {
          if (!ABORT) func();
        });
      },safeSetTimeout:function (func, timeout) {
        Module['noExitRuntime'] = true;
        return setTimeout(function() {
          if (!ABORT) func();
        }, timeout);
      },safeSetInterval:function (func, timeout) {
        Module['noExitRuntime'] = true;
        return setInterval(function() {
          if (!ABORT) func();
        }, timeout);
      },getMimetype:function (name) {
        return {
          'jpg': 'image/jpeg',
          'jpeg': 'image/jpeg',
          'png': 'image/png',
          'bmp': 'image/bmp',
          'ogg': 'audio/ogg',
          'wav': 'audio/wav',
          'mp3': 'audio/mpeg'
        }[name.substr(name.lastIndexOf('.')+1)];
      },getUserMedia:function (func) {
        if(!window.getUserMedia) {
          window.getUserMedia = navigator['getUserMedia'] ||
                                navigator['mozGetUserMedia'];
        }
        window.getUserMedia(func);
      },getMovementX:function (event) {
        return event['movementX'] ||
               event['mozMovementX'] ||
               event['webkitMovementX'] ||
               0;
      },getMovementY:function (event) {
        return event['movementY'] ||
               event['mozMovementY'] ||
               event['webkitMovementY'] ||
               0;
      },getMouseWheelDelta:function (event) {
        var delta = 0;
        switch (event.type) {
          case 'DOMMouseScroll': 
            delta = event.detail;
            break;
          case 'mousewheel': 
            delta = event.wheelDelta;
            break;
          case 'wheel': 
            delta = event['deltaY'];
            break;
          default:
            throw 'unrecognized mouse wheel event: ' + event.type;
        }
        return delta;
      },mouseX:0,mouseY:0,mouseMovementX:0,mouseMovementY:0,touches:{},lastTouches:{},calculateMouseEvent:function (event) { // event should be mousemove, mousedown or mouseup
        if (Browser.pointerLock) {
          // When the pointer is locked, calculate the coordinates
          // based on the movement of the mouse.
          // Workaround for Firefox bug 764498
          if (event.type != 'mousemove' &&
              ('mozMovementX' in event)) {
            Browser.mouseMovementX = Browser.mouseMovementY = 0;
          } else {
            Browser.mouseMovementX = Browser.getMovementX(event);
            Browser.mouseMovementY = Browser.getMovementY(event);
          }
          
          // check if SDL is available
          if (typeof SDL != "undefined") {
          	Browser.mouseX = SDL.mouseX + Browser.mouseMovementX;
          	Browser.mouseY = SDL.mouseY + Browser.mouseMovementY;
          } else {
          	// just add the mouse delta to the current absolut mouse position
          	// FIXME: ideally this should be clamped against the canvas size and zero
          	Browser.mouseX += Browser.mouseMovementX;
          	Browser.mouseY += Browser.mouseMovementY;
          }        
        } else {
          // Otherwise, calculate the movement based on the changes
          // in the coordinates.
          var rect = Module["canvas"].getBoundingClientRect();
          var cw = Module["canvas"].width;
          var ch = Module["canvas"].height;
  
          // Neither .scrollX or .pageXOffset are defined in a spec, but
          // we prefer .scrollX because it is currently in a spec draft.
          // (see: http://www.w3.org/TR/2013/WD-cssom-view-20131217/)
          var scrollX = ((typeof window.scrollX !== 'undefined') ? window.scrollX : window.pageXOffset);
          var scrollY = ((typeof window.scrollY !== 'undefined') ? window.scrollY : window.pageYOffset);
          // If this assert lands, it's likely because the browser doesn't support scrollX or pageXOffset
          // and we have no viable fallback.
          assert((typeof scrollX !== 'undefined') && (typeof scrollY !== 'undefined'), 'Unable to retrieve scroll position, mouse positions likely broken.');
  
          if (event.type === 'touchstart' || event.type === 'touchend' || event.type === 'touchmove') {
            var touch = event.touch;
            if (touch === undefined) {
              return; // the "touch" property is only defined in SDL
  
            }
            var adjustedX = touch.pageX - (scrollX + rect.left);
            var adjustedY = touch.pageY - (scrollY + rect.top);
  
            adjustedX = adjustedX * (cw / rect.width);
            adjustedY = adjustedY * (ch / rect.height);
  
            var coords = { x: adjustedX, y: adjustedY };
            
            if (event.type === 'touchstart') {
              Browser.lastTouches[touch.identifier] = coords;
              Browser.touches[touch.identifier] = coords;
            } else if (event.type === 'touchend' || event.type === 'touchmove') {
              Browser.lastTouches[touch.identifier] = Browser.touches[touch.identifier];
              Browser.touches[touch.identifier] = { x: adjustedX, y: adjustedY };
            } 
            return;
          }
  
          var x = event.pageX - (scrollX + rect.left);
          var y = event.pageY - (scrollY + rect.top);
  
          // the canvas might be CSS-scaled compared to its backbuffer;
          // SDL-using content will want mouse coordinates in terms
          // of backbuffer units.
          x = x * (cw / rect.width);
          y = y * (ch / rect.height);
  
          Browser.mouseMovementX = x - Browser.mouseX;
          Browser.mouseMovementY = y - Browser.mouseY;
          Browser.mouseX = x;
          Browser.mouseY = y;
        }
      },xhrLoad:function (url, onload, onerror) {
        var xhr = new XMLHttpRequest();
        xhr.open('GET', url, true);
        xhr.responseType = 'arraybuffer';
        xhr.onload = function xhr_onload() {
          if (xhr.status == 200 || (xhr.status == 0 && xhr.response)) { // file URLs can return 0
            onload(xhr.response);
          } else {
            onerror();
          }
        };
        xhr.onerror = onerror;
        xhr.send(null);
      },asyncLoad:function (url, onload, onerror, noRunDep) {
        Browser.xhrLoad(url, function(arrayBuffer) {
          assert(arrayBuffer, 'Loading data file "' + url + '" failed (no arrayBuffer).');
          onload(new Uint8Array(arrayBuffer));
          if (!noRunDep) removeRunDependency('al ' + url);
        }, function(event) {
          if (onerror) {
            onerror();
          } else {
            throw 'Loading data file "' + url + '" failed.';
          }
        });
        if (!noRunDep) addRunDependency('al ' + url);
      },resizeListeners:[],updateResizeListeners:function () {
        var canvas = Module['canvas'];
        Browser.resizeListeners.forEach(function(listener) {
          listener(canvas.width, canvas.height);
        });
      },setCanvasSize:function (width, height, noUpdates) {
        var canvas = Module['canvas'];
        Browser.updateCanvasDimensions(canvas, width, height);
        if (!noUpdates) Browser.updateResizeListeners();
      },windowedWidth:0,windowedHeight:0,setFullScreenCanvasSize:function () {
        // check if SDL is available   
        if (typeof SDL != "undefined") {
        	var flags = HEAPU32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)];
        	flags = flags | 0x00800000; // set SDL_FULLSCREEN flag
        	HEAP32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)]=flags
        }
        Browser.updateResizeListeners();
      },setWindowedCanvasSize:function () {
        // check if SDL is available       
        if (typeof SDL != "undefined") {
        	var flags = HEAPU32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)];
        	flags = flags & ~0x00800000; // clear SDL_FULLSCREEN flag
        	HEAP32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)]=flags
        }
        Browser.updateResizeListeners();
      },updateCanvasDimensions:function (canvas, wNative, hNative) {
        if (wNative && hNative) {
          canvas.widthNative = wNative;
          canvas.heightNative = hNative;
        } else {
          wNative = canvas.widthNative;
          hNative = canvas.heightNative;
        }
        var w = wNative;
        var h = hNative;
        if (Module['forcedAspectRatio'] && Module['forcedAspectRatio'] > 0) {
          if (w/h < Module['forcedAspectRatio']) {
            w = Math.round(h * Module['forcedAspectRatio']);
          } else {
            h = Math.round(w / Module['forcedAspectRatio']);
          }
        }
        if (((document['webkitFullScreenElement'] || document['webkitFullscreenElement'] ||
             document['mozFullScreenElement'] || document['mozFullscreenElement'] ||
             document['fullScreenElement'] || document['fullscreenElement'] ||
             document['msFullScreenElement'] || document['msFullscreenElement'] ||
             document['webkitCurrentFullScreenElement']) === canvas.parentNode) && (typeof screen != 'undefined')) {
           var factor = Math.min(screen.width / w, screen.height / h);
           w = Math.round(w * factor);
           h = Math.round(h * factor);
        }
        if (Browser.resizeCanvas) {
          if (canvas.width  != w) canvas.width  = w;
          if (canvas.height != h) canvas.height = h;
          if (typeof canvas.style != 'undefined') {
            canvas.style.removeProperty( "width");
            canvas.style.removeProperty("height");
          }
        } else {
          if (canvas.width  != wNative) canvas.width  = wNative;
          if (canvas.height != hNative) canvas.height = hNative;
          if (typeof canvas.style != 'undefined') {
            if (w != wNative || h != hNative) {
              canvas.style.setProperty( "width", w + "px", "important");
              canvas.style.setProperty("height", h + "px", "important");
            } else {
              canvas.style.removeProperty( "width");
              canvas.style.removeProperty("height");
            }
          }
        }
      },wgetRequests:{},nextWgetRequestHandle:0,getNextWgetRequestHandle:function () {
        var handle = Browser.nextWgetRequestHandle;
        Browser.nextWgetRequestHandle++;
        return handle;
      }};


  function _munmap(start, num) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_2(109, start, num);
      if (!_mmap.mappings) _mmap.mappings = {};
      // TODO: support unmmap'ing parts of allocations
      var info = _mmap.mappings[start];
      if (!info) return 0;
      if (num == info.num) {
        _mmap.mappings[start] = null;
        if (info.allocated) {
          _free(info.malloc);
        }
      }
      return 0;
    }


  
  
  function __exit(status) {
      // void _exit(int status);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/exit.html
      Module['exit'](status);
    }function _exit(status) {
      __exit(status);
    }function _pthread_exit(status) {
      dump('Pthread ' + _pthread_self() + ' exiting.\n');
      if (!ENVIRONMENT_IS_PTHREAD) _exit(status);
      else PThread.threadExit(status);
    }

  
  function _stat(path, buf, dontResolveLastLink) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_3(14, path, buf, dontResolveLastLink);
      // http://pubs.opengroup.org/onlinepubs/7908799/xsh/stat.html
      // int stat(const char *path, struct stat *buf);
      // NOTE: dontResolveLastLink is a shortcut for lstat(). It should never be
      //       used in client code.
      path = typeof path !== 'string' ? Pointer_stringify(path) : path;
      try {
        var stat = dontResolveLastLink ? FS.lstat(path) : FS.stat(path);
        HEAP32[((buf)>>2)]=stat.dev;
        HEAP32[(((buf)+(4))>>2)]=0;
        HEAP32[(((buf)+(8))>>2)]=stat.ino;
        HEAP32[(((buf)+(12))>>2)]=stat.mode;
        HEAP32[(((buf)+(16))>>2)]=stat.nlink;
        HEAP32[(((buf)+(20))>>2)]=stat.uid;
        HEAP32[(((buf)+(24))>>2)]=stat.gid;
        HEAP32[(((buf)+(28))>>2)]=stat.rdev;
        HEAP32[(((buf)+(32))>>2)]=0;
        HEAP32[(((buf)+(36))>>2)]=stat.size;
        HEAP32[(((buf)+(40))>>2)]=4096;
        HEAP32[(((buf)+(44))>>2)]=stat.blocks;
        HEAP32[(((buf)+(48))>>2)]=(stat.atime.getTime() / 1000)|0;
        HEAP32[(((buf)+(52))>>2)]=0;
        HEAP32[(((buf)+(56))>>2)]=(stat.mtime.getTime() / 1000)|0;
        HEAP32[(((buf)+(60))>>2)]=0;
        HEAP32[(((buf)+(64))>>2)]=(stat.ctime.getTime() / 1000)|0;
        HEAP32[(((buf)+(68))>>2)]=0;
        HEAP32[(((buf)+(72))>>2)]=stat.ino;
        return 0;
      } catch (e) {
        if (e.node && PATH.normalize(path) !== PATH.normalize(FS.getPath(e.node))) {
          // an error occurred while trying to look up the path; we should just report ENOTDIR
          e.setErrno(ERRNO_CODES.ENOTDIR);
        }
        FS.handleFSError(e);
        return -1;
      }
    }function _lstat(path, buf) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_2(15, path, buf);
      // int lstat(const char *path, struct stat *buf);
      // http://pubs.opengroup.org/onlinepubs/7908799/xsh/lstat.html
      return _stat(path, buf, true);
    }

  function _ferror(stream) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_1(77, stream);
      // int ferror(FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/ferror.html
      stream = FS.getStreamFromPtr(stream);
      return Number(stream && stream.error);
    }

  function _emscripten_atomic_sub_u32() {
  Module['printErr']('missing function: emscripten_atomic_sub_u32'); abort(-1);
  }

  function _getcwd(buf, size) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_2(50, buf, size);
      // char *getcwd(char *buf, size_t size);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/getcwd.html
      if (size == 0) {
        ___setErrNo(ERRNO_CODES.EINVAL);
        return 0;
      }
      var cwd = FS.cwd();
      if (size < cwd.length + 1) {
        ___setErrNo(ERRNO_CODES.ERANGE);
        return 0;
      } else {
        writeAsciiToMemory(cwd, buf);
        return buf;
      }
    }

  function _symlink(path1, path2) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_2(64, path1, path2);
      // int symlink(const char *path1, const char *path2);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/symlink.html
      path1 = Pointer_stringify(path1);
      path2 = Pointer_stringify(path2);
      try {
        FS.symlink(path1, path2);
        return 0;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }

  
  function _chdir(path) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_1(35, path);
      // int chdir(const char *path);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/chdir.html
      // NOTE: The path argument may be a string, to simplify fchdir().
      if (typeof path !== 'string') path = Pointer_stringify(path);
      try {
        FS.chdir(path);
        return 0;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _fchdir(fildes) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_1(42, fildes);
      // int fchdir(int fildes);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fchdir.html
      var stream = FS.getStream(fildes);
      if (stream) {
        return _chdir(stream.path);
      } else {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
    }

  function _telldir(dirp) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_1(7, dirp);
      // long int telldir(DIR *dirp);
      // http://pubs.opengroup.org/onlinepubs/007908799/xsh/telldir.html
      var stream = FS.getStreamFromPtr(dirp);
      if (!stream || !FS.isDir(stream.node.mode)) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      return stream.position;
    }

  function _getpeername(fd, addr, addrlen) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_3(131, fd, addr, addrlen);
      var sock = SOCKFS.getSocket(fd);
      if (!sock) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      try {
        var info = sock.sock_ops.getname(sock, true);
        var res = __write_sockaddr(addr, sock.family, DNS.lookup_name(info.addr), info.port);
        assert(!res.errno);
        return 0;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }


  function _putchar(c) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_1(85, c);
      // int putchar(int c);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/putchar.html
      return _fputc(c, HEAP32[((_stdout)>>2)]);
    }

  function _socket(family, type, protocol) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_3(120, family, type, protocol);
      var sock = SOCKFS.createSocket(family, type, protocol);
      assert(sock.stream.fd < 64); // select() assumes socket fd values are in 0..63
      return sock.stream.fd;
    }

  function _popen(command, mode) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_2(93, command, mode);
      // FILE *popen(const char *command, const char *mode);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/popen.html
      // We allow only one process, so no pipes.
      ___setErrNo(ERRNO_CODES.EMFILE);
      return 0;
    }

  function _mmap(start, num, prot, flags, fd, offset) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_6(108, start, num, prot, flags, fd, offset);
      /* FIXME: Since mmap is normally implemented at the kernel level,
       * this implementation simply uses malloc underneath the call to
       * mmap.
       */
      var MAP_PRIVATE = 2;
      var ptr;
      var allocated = false;
  
      if (!_mmap.mappings) _mmap.mappings = {};
  
      if (fd == -1) {
        ptr = _malloc(num);
        if (!ptr) return -1;
        _memset(ptr, 0, num);
        allocated = true;
      } else {
        var info = FS.getStream(fd);
        if (!info) return -1;
        try {
          var res = FS.mmap(info, HEAPU8, start, num, offset, prot, flags);
          ptr = res.ptr;
          allocated = res.allocated;
        } catch (e) {
          FS.handleFSError(e);
          return -1;
        }
      }
  
      _mmap.mappings[ptr] = { malloc: ptr, num: num, allocated: allocated };
      return ptr;
    }

  function _sendmsg(fd, message, flags) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_3(122, fd, message, flags);
      var sock = SOCKFS.getSocket(fd);
      if (!sock) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
  
      var iov = HEAP32[(((message)+(8))>>2)];
      var num = HEAP32[(((message)+(12))>>2)];
  
      // read the address and port to send to
      var addr;
      var port;
      var name = HEAP32[((message)>>2)];
      var namelen = HEAP32[(((message)+(4))>>2)];
      if (name) {
        var info = __read_sockaddr(name, namelen);
        if (info.errno) {
          ___setErrNo(info.errno);
          return -1;
        }
        port = info.port;
        addr = DNS.lookup_addr(info.addr) || info.addr;
      }
  
      // concatenate scatter-gather arrays into one message buffer
      var total = 0;
      for (var i = 0; i < num; i++) {
        total += HEAP32[(((iov)+((8 * i) + 4))>>2)];
      }
      var view = new Uint8Array(total);
      var offset = 0;
      for (var i = 0; i < num; i++) {
        var iovbase = HEAP32[(((iov)+((8 * i) + 0))>>2)];
        var iovlen = HEAP32[(((iov)+((8 * i) + 4))>>2)];
        for (var j = 0; j < iovlen; j++) {  
          view[offset++] = HEAP8[(((iovbase)+(j))>>0)];
        }
      }
  
      // write the buffer
      try {
        return sock.sock_ops.sendmsg(sock, view, 0, total, addr, port);
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }

  
  var _tzname=allocate(8, "i32*", ALLOC_STATIC);
  
  var _daylight=allocate(1, "i32*", ALLOC_STATIC);
  
  var _timezone=allocate(1, "i32*", ALLOC_STATIC);function _tzset() {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_0(119);
      // TODO: Use (malleable) environment variables instead of system settings.
      if (_tzset.called) return;
      _tzset.called = true;
  
      HEAP32[((_timezone)>>2)]=-(new Date()).getTimezoneOffset() * 60;
  
      var winter = new Date(2000, 0, 1);
      var summer = new Date(2000, 6, 1);
      HEAP32[((_daylight)>>2)]=Number(winter.getTimezoneOffset() != summer.getTimezoneOffset());
  
      function extractZone(date) {
        var match = date.toTimeString().match(/\(([A-Za-z ]+)\)$/);
        return match ? match[1] : "GMT";
      };
      var winterName = extractZone(winter);
      var summerName = extractZone(summer);
      var winterNamePtr = allocate(intArrayFromString(winterName), 'i8', ALLOC_NORMAL);
      var summerNamePtr = allocate(intArrayFromString(summerName), 'i8', ALLOC_NORMAL);
      if (summer.getTimezoneOffset() < winter.getTimezoneOffset()) {
        // Northern hemisphere
        HEAP32[((_tzname)>>2)]=winterNamePtr;
        HEAP32[(((_tzname)+(4))>>2)]=summerNamePtr;
      } else {
        HEAP32[((_tzname)>>2)]=summerNamePtr;
        HEAP32[(((_tzname)+(4))>>2)]=winterNamePtr;
      }
    }

  
  function _emscripten_get_now_is_monotonic() {
      // return whether emscripten_get_now is guaranteed monotonic; the Date.now
      // implementation is not :(
      return ENVIRONMENT_IS_NODE || (typeof dateNow !== 'undefined') ||
          ((ENVIRONMENT_IS_WEB || ENVIRONMENT_IS_WORKER) && self['performance'] && self['performance']['now']);
    }function _clock_gettime(clk_id, tp) {
      // int clock_gettime(clockid_t clk_id, struct timespec *tp);
      var now;
      if (clk_id === 0) {
        now = Date.now();
      } else if (clk_id === 1 && _emscripten_get_now_is_monotonic()) {
        now = _emscripten_get_now();
      } else {
        ___setErrNo(ERRNO_CODES.EINVAL);
        return -1;
      }
      HEAP32[((tp)>>2)]=(now/1000)|0; // seconds
      HEAP32[(((tp)+(4))>>2)]=((now % 1000)*1000*1000)|0; // nanoseconds
      return 0;
    }

  function _tmpfile() {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_0(101);
      // FILE *tmpfile(void);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/tmpfile.html
      // TODO: Delete the created file on closing.
      if (!_tmpfile.mode) {
        _tmpfile.mode = allocate(intArrayFromString('w+'), 'i8', ALLOC_NORMAL);
      }
      return _fopen(_tmpnam(0), _tmpfile.mode);
    }


  function _shutdown(fd, how) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_2(124, fd, how);
      var sock = SOCKFS.getSocket(fd);
      if (!sock) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      _close(fd);
    }

  
  
  
  function __reallyNegative(x) {
      return x < 0 || (x === 0 && (1/x) === -Infinity);
    }function __formatString(format, varargs) {
      var textIndex = format;
      var argIndex = 0;
      function getNextArg(type) {
        // NOTE: Explicitly ignoring type safety. Otherwise this fails:
        //       int x = 4; printf("%c\n", (char)x);
        var ret;
        if (type === 'double') {
          ret = (HEAP32[((tempDoublePtr)>>2)]=HEAP32[(((varargs)+(argIndex))>>2)],HEAP32[(((tempDoublePtr)+(4))>>2)]=HEAP32[(((varargs)+((argIndex)+(4)))>>2)],(+(HEAPF64[(tempDoublePtr)>>3])));
        } else if (type == 'i64') {
          ret = [HEAP32[(((varargs)+(argIndex))>>2)],
                 HEAP32[(((varargs)+(argIndex+4))>>2)]];
  
        } else {
          type = 'i32'; // varargs are always i32, i64, or double
          ret = HEAP32[(((varargs)+(argIndex))>>2)];
        }
        argIndex += Runtime.getNativeFieldSize(type);
        return ret;
      }
  
      var ret = [];
      var curr, next, currArg;
      while(1) {
        var startTextIndex = textIndex;
        curr = HEAP8[((textIndex)>>0)];
        if (curr === 0) break;
        next = HEAP8[((textIndex+1)>>0)];
        if (curr == 37) {
          // Handle flags.
          var flagAlwaysSigned = false;
          var flagLeftAlign = false;
          var flagAlternative = false;
          var flagZeroPad = false;
          var flagPadSign = false;
          flagsLoop: while (1) {
            switch (next) {
              case 43:
                flagAlwaysSigned = true;
                break;
              case 45:
                flagLeftAlign = true;
                break;
              case 35:
                flagAlternative = true;
                break;
              case 48:
                if (flagZeroPad) {
                  break flagsLoop;
                } else {
                  flagZeroPad = true;
                  break;
                }
              case 32:
                flagPadSign = true;
                break;
              default:
                break flagsLoop;
            }
            textIndex++;
            next = HEAP8[((textIndex+1)>>0)];
          }
  
          // Handle width.
          var width = 0;
          if (next == 42) {
            width = getNextArg('i32');
            textIndex++;
            next = HEAP8[((textIndex+1)>>0)];
          } else {
            while (next >= 48 && next <= 57) {
              width = width * 10 + (next - 48);
              textIndex++;
              next = HEAP8[((textIndex+1)>>0)];
            }
          }
  
          // Handle precision.
          var precisionSet = false, precision = -1;
          if (next == 46) {
            precision = 0;
            precisionSet = true;
            textIndex++;
            next = HEAP8[((textIndex+1)>>0)];
            if (next == 42) {
              precision = getNextArg('i32');
              textIndex++;
            } else {
              while(1) {
                var precisionChr = HEAP8[((textIndex+1)>>0)];
                if (precisionChr < 48 ||
                    precisionChr > 57) break;
                precision = precision * 10 + (precisionChr - 48);
                textIndex++;
              }
            }
            next = HEAP8[((textIndex+1)>>0)];
          }
          if (precision < 0) {
            precision = 6; // Standard default.
            precisionSet = false;
          }
  
          // Handle integer sizes. WARNING: These assume a 32-bit architecture!
          var argSize;
          switch (String.fromCharCode(next)) {
            case 'h':
              var nextNext = HEAP8[((textIndex+2)>>0)];
              if (nextNext == 104) {
                textIndex++;
                argSize = 1; // char (actually i32 in varargs)
              } else {
                argSize = 2; // short (actually i32 in varargs)
              }
              break;
            case 'l':
              var nextNext = HEAP8[((textIndex+2)>>0)];
              if (nextNext == 108) {
                textIndex++;
                argSize = 8; // long long
              } else {
                argSize = 4; // long
              }
              break;
            case 'L': // long long
            case 'q': // int64_t
            case 'j': // intmax_t
              argSize = 8;
              break;
            case 'z': // size_t
            case 't': // ptrdiff_t
            case 'I': // signed ptrdiff_t or unsigned size_t
              argSize = 4;
              break;
            default:
              argSize = null;
          }
          if (argSize) textIndex++;
          next = HEAP8[((textIndex+1)>>0)];
  
          // Handle type specifier.
          switch (String.fromCharCode(next)) {
            case 'd': case 'i': case 'u': case 'o': case 'x': case 'X': case 'p': {
              // Integer.
              var signed = next == 100 || next == 105;
              argSize = argSize || 4;
              var currArg = getNextArg('i' + (argSize * 8));
              var origArg = currArg;
              var argText;
              // Flatten i64-1 [low, high] into a (slightly rounded) double
              if (argSize == 8) {
                currArg = Runtime.makeBigInt(currArg[0], currArg[1], next == 117);
              }
              // Truncate to requested size.
              if (argSize <= 4) {
                var limit = Math.pow(256, argSize) - 1;
                currArg = (signed ? reSign : unSign)(currArg & limit, argSize * 8);
              }
              // Format the number.
              var currAbsArg = Math.abs(currArg);
              var prefix = '';
              if (next == 100 || next == 105) {
                if (argSize == 8 && i64Math) argText = i64Math.stringify(origArg[0], origArg[1], null); else
                argText = reSign(currArg, 8 * argSize, 1).toString(10);
              } else if (next == 117) {
                if (argSize == 8 && i64Math) argText = i64Math.stringify(origArg[0], origArg[1], true); else
                argText = unSign(currArg, 8 * argSize, 1).toString(10);
                currArg = Math.abs(currArg);
              } else if (next == 111) {
                argText = (flagAlternative ? '0' : '') + currAbsArg.toString(8);
              } else if (next == 120 || next == 88) {
                prefix = (flagAlternative && currArg != 0) ? '0x' : '';
                if (argSize == 8 && i64Math) {
                  if (origArg[1]) {
                    argText = (origArg[1]>>>0).toString(16);
                    var lower = (origArg[0]>>>0).toString(16);
                    while (lower.length < 8) lower = '0' + lower;
                    argText += lower;
                  } else {
                    argText = (origArg[0]>>>0).toString(16);
                  }
                } else
                if (currArg < 0) {
                  // Represent negative numbers in hex as 2's complement.
                  currArg = -currArg;
                  argText = (currAbsArg - 1).toString(16);
                  var buffer = [];
                  for (var i = 0; i < argText.length; i++) {
                    buffer.push((0xF - parseInt(argText[i], 16)).toString(16));
                  }
                  argText = buffer.join('');
                  while (argText.length < argSize * 2) argText = 'f' + argText;
                } else {
                  argText = currAbsArg.toString(16);
                }
                if (next == 88) {
                  prefix = prefix.toUpperCase();
                  argText = argText.toUpperCase();
                }
              } else if (next == 112) {
                if (currAbsArg === 0) {
                  argText = '(nil)';
                } else {
                  prefix = '0x';
                  argText = currAbsArg.toString(16);
                }
              }
              if (precisionSet) {
                while (argText.length < precision) {
                  argText = '0' + argText;
                }
              }
  
              // Add sign if needed
              if (currArg >= 0) {
                if (flagAlwaysSigned) {
                  prefix = '+' + prefix;
                } else if (flagPadSign) {
                  prefix = ' ' + prefix;
                }
              }
  
              // Move sign to prefix so we zero-pad after the sign
              if (argText.charAt(0) == '-') {
                prefix = '-' + prefix;
                argText = argText.substr(1);
              }
  
              // Add padding.
              while (prefix.length + argText.length < width) {
                if (flagLeftAlign) {
                  argText += ' ';
                } else {
                  if (flagZeroPad) {
                    argText = '0' + argText;
                  } else {
                    prefix = ' ' + prefix;
                  }
                }
              }
  
              // Insert the result into the buffer.
              argText = prefix + argText;
              argText.split('').forEach(function(chr) {
                ret.push(chr.charCodeAt(0));
              });
              break;
            }
            case 'f': case 'F': case 'e': case 'E': case 'g': case 'G': {
              // Float.
              var currArg = getNextArg('double');
              var argText;
              if (isNaN(currArg)) {
                argText = 'nan';
                flagZeroPad = false;
              } else if (!isFinite(currArg)) {
                argText = (currArg < 0 ? '-' : '') + 'inf';
                flagZeroPad = false;
              } else {
                var isGeneral = false;
                var effectivePrecision = Math.min(precision, 20);
  
                // Convert g/G to f/F or e/E, as per:
                // http://pubs.opengroup.org/onlinepubs/9699919799/functions/printf.html
                if (next == 103 || next == 71) {
                  isGeneral = true;
                  precision = precision || 1;
                  var exponent = parseInt(currArg.toExponential(effectivePrecision).split('e')[1], 10);
                  if (precision > exponent && exponent >= -4) {
                    next = ((next == 103) ? 'f' : 'F').charCodeAt(0);
                    precision -= exponent + 1;
                  } else {
                    next = ((next == 103) ? 'e' : 'E').charCodeAt(0);
                    precision--;
                  }
                  effectivePrecision = Math.min(precision, 20);
                }
  
                if (next == 101 || next == 69) {
                  argText = currArg.toExponential(effectivePrecision);
                  // Make sure the exponent has at least 2 digits.
                  if (/[eE][-+]\d$/.test(argText)) {
                    argText = argText.slice(0, -1) + '0' + argText.slice(-1);
                  }
                } else if (next == 102 || next == 70) {
                  argText = currArg.toFixed(effectivePrecision);
                  if (currArg === 0 && __reallyNegative(currArg)) {
                    argText = '-' + argText;
                  }
                }
  
                var parts = argText.split('e');
                if (isGeneral && !flagAlternative) {
                  // Discard trailing zeros and periods.
                  while (parts[0].length > 1 && parts[0].indexOf('.') != -1 &&
                         (parts[0].slice(-1) == '0' || parts[0].slice(-1) == '.')) {
                    parts[0] = parts[0].slice(0, -1);
                  }
                } else {
                  // Make sure we have a period in alternative mode.
                  if (flagAlternative && argText.indexOf('.') == -1) parts[0] += '.';
                  // Zero pad until required precision.
                  while (precision > effectivePrecision++) parts[0] += '0';
                }
                argText = parts[0] + (parts.length > 1 ? 'e' + parts[1] : '');
  
                // Capitalize 'E' if needed.
                if (next == 69) argText = argText.toUpperCase();
  
                // Add sign.
                if (currArg >= 0) {
                  if (flagAlwaysSigned) {
                    argText = '+' + argText;
                  } else if (flagPadSign) {
                    argText = ' ' + argText;
                  }
                }
              }
  
              // Add padding.
              while (argText.length < width) {
                if (flagLeftAlign) {
                  argText += ' ';
                } else {
                  if (flagZeroPad && (argText[0] == '-' || argText[0] == '+')) {
                    argText = argText[0] + '0' + argText.slice(1);
                  } else {
                    argText = (flagZeroPad ? '0' : ' ') + argText;
                  }
                }
              }
  
              // Adjust case.
              if (next < 97) argText = argText.toUpperCase();
  
              // Insert the result into the buffer.
              argText.split('').forEach(function(chr) {
                ret.push(chr.charCodeAt(0));
              });
              break;
            }
            case 's': {
              // String.
              var arg = getNextArg('i8*');
              var argLength = arg ? _strlen(arg) : '(null)'.length;
              if (precisionSet) argLength = Math.min(argLength, precision);
              if (!flagLeftAlign) {
                while (argLength < width--) {
                  ret.push(32);
                }
              }
              if (arg) {
                for (var i = 0; i < argLength; i++) {
                  ret.push(HEAPU8[((arg++)>>0)]);
                }
              } else {
                ret = ret.concat(intArrayFromString('(null)'.substr(0, argLength), true));
              }
              if (flagLeftAlign) {
                while (argLength < width--) {
                  ret.push(32);
                }
              }
              break;
            }
            case 'c': {
              // Character.
              if (flagLeftAlign) ret.push(getNextArg('i8'));
              while (--width > 0) {
                ret.push(32);
              }
              if (!flagLeftAlign) ret.push(getNextArg('i8'));
              break;
            }
            case 'n': {
              // Write the length written so far to the next parameter.
              var ptr = getNextArg('i32*');
              HEAP32[((ptr)>>2)]=ret.length;
              break;
            }
            case '%': {
              // Literal percent sign.
              ret.push(curr);
              break;
            }
            default: {
              // Unknown specifiers remain untouched.
              for (var i = startTextIndex; i < textIndex + 2; i++) {
                ret.push(HEAP8[((i)>>0)]);
              }
            }
          }
          textIndex += 2;
          // TODO: Support a/A (hex float) and m (last error) specifiers.
          // TODO: Support %1${specifier} for arg selection.
        } else {
          ret.push(curr);
          textIndex += 1;
        }
      }
      return ret;
    }function _fprintf(stream, format, varargs) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_3(105, stream, format, varargs);
      // int fprintf(FILE *restrict stream, const char *restrict format, ...);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/printf.html
      var result = __formatString(format, varargs);
      var stack = Runtime.stackSave();
      var ret = _fwrite(allocate(result, 'i8', ALLOC_STACK), 1, result.length, stream);
      Runtime.stackRestore(stack);
      return ret;
    }function _printf(format, varargs) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_2(106, format, varargs);
      // int printf(const char *restrict format, ...);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/printf.html
      var stdout = HEAP32[((_stdout)>>2)];
      return _fprintf(stdout, format, varargs);
    }


  
  var ___DEFAULT_POLLMASK=5;function _poll(fds, nfds, timeout) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_3(33, fds, nfds, timeout);
      // int poll(struct pollfd fds[], nfds_t nfds, int timeout);
      // http://pubs.opengroup.org/onlinepubs/009695399/functions/poll.html
      var nonzero = 0;
      for (var i = 0; i < nfds; i++) {
        var pollfd = fds + 8 * i;
        var fd = HEAP32[((pollfd)>>2)];
        var events = HEAP16[(((pollfd)+(4))>>1)];
        var mask = 32;
        var stream = FS.getStream(fd);
        if (stream) {
          mask = ___DEFAULT_POLLMASK;
          if (stream.stream_ops.poll) {
            mask = stream.stream_ops.poll(stream);
          }
        }
        mask &= events | 8 | 16;
        if (mask) nonzero++;
        HEAP16[(((pollfd)+(6))>>1)]=mask;
      }
      return nonzero;
    }


  function _select(nfds, readfds, writefds, exceptfds, timeout) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_5(127, nfds, readfds, writefds, exceptfds, timeout);
      // readfds are supported,
      // writefds checks socket open status
      // exceptfds not supported
      // timeout is always 0 - fully async
      assert(nfds <= 64, 'nfds must be less than or equal to 64');  // fd sets have 64 bits
      assert(!exceptfds, 'exceptfds not supported');
  
      var total = 0;
      
      var srcReadLow = (readfds ? HEAP32[((readfds)>>2)] : 0),
          srcReadHigh = (readfds ? HEAP32[(((readfds)+(4))>>2)] : 0);
      var srcWriteLow = (writefds ? HEAP32[((writefds)>>2)] : 0),
          srcWriteHigh = (writefds ? HEAP32[(((writefds)+(4))>>2)] : 0);
      var srcExceptLow = (exceptfds ? HEAP32[((exceptfds)>>2)] : 0),
          srcExceptHigh = (exceptfds ? HEAP32[(((exceptfds)+(4))>>2)] : 0);
  
      var dstReadLow = 0,
          dstReadHigh = 0;
      var dstWriteLow = 0,
          dstWriteHigh = 0;
      var dstExceptLow = 0,
          dstExceptHigh = 0;
  
      var allLow = (readfds ? HEAP32[((readfds)>>2)] : 0) |
                   (writefds ? HEAP32[((writefds)>>2)] : 0) |
                   (exceptfds ? HEAP32[((exceptfds)>>2)] : 0);
      var allHigh = (readfds ? HEAP32[(((readfds)+(4))>>2)] : 0) |
                    (writefds ? HEAP32[(((writefds)+(4))>>2)] : 0) |
                    (exceptfds ? HEAP32[(((exceptfds)+(4))>>2)] : 0);
  
      function get(fd, low, high, val) {
        return (fd < 32 ? (low & val) : (high & val));
      }
  
      for (var fd = 0; fd < nfds; fd++) {
        var mask = 1 << (fd % 32);
        if (!(get(fd, allLow, allHigh, mask))) {
          continue;  // index isn't in the set
        }
  
        var stream = FS.getStream(fd);
        if (!stream) {
          ___setErrNo(ERRNO_CODES.EBADF);
          return -1;
        }
  
        var flags = ___DEFAULT_POLLMASK;
  
        if (stream.stream_ops.poll) {
          flags = stream.stream_ops.poll(stream);
        }
  
        if ((flags & 1) && get(fd, srcReadLow, srcReadHigh, mask)) {
          fd < 32 ? (dstReadLow = dstReadLow | mask) : (dstReadHigh = dstReadHigh | mask);
          total++;
        }
        if ((flags & 4) && get(fd, srcWriteLow, srcWriteHigh, mask)) {
          fd < 32 ? (dstWriteLow = dstWriteLow | mask) : (dstWriteHigh = dstWriteHigh | mask);
          total++;
        }
        if ((flags & 2) && get(fd, srcExceptLow, srcExceptHigh, mask)) {
          fd < 32 ? (dstExceptLow = dstExceptLow | mask) : (dstExceptHigh = dstExceptHigh | mask);
          total++;
        }
      }
  
      if (readfds) {
        HEAP32[((readfds)>>2)]=dstReadLow;
        HEAP32[(((readfds)+(4))>>2)]=dstReadHigh;
      }
      if (writefds) {
        HEAP32[((writefds)>>2)]=dstWriteLow;
        HEAP32[(((writefds)+(4))>>2)]=dstWriteHigh;
      }
      if (exceptfds) {
        HEAP32[((exceptfds)>>2)]=dstExceptLow;
        HEAP32[(((exceptfds)+(4))>>2)]=dstExceptHigh;
      }
      
      return total;
    }

  function _getchar() {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_0(80);
      // int getchar(void);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/getchar.html
      return _fgetc(HEAP32[((_stdin)>>2)]);
    }

  function _gethostname(name, namelen) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_2(69, name, namelen);
      // int gethostname(char *name, size_t namelen);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/gethostname.html
      var host = 'emscripten';
      if (typeof window !== 'undefined' && window.location.host) {
        host = window.location.host;
      }
      var length = Math.min(namelen, host.length);
      for (var i = 0; i < length; i++) {
        HEAP8[(((name)+(i))>>0)]=host.charCodeAt(i);
      }
      if (namelen > length) {
        HEAP8[(((name)+(i))>>0)]=0;
        return 0;
      } else {
        ___setErrNo(ERRNO_CODES.ENAMETOOLONG);
        return -1;
      }
    }

  function _getsockopt(fd, level, optname, optval, optlen) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_5(136, fd, level, optname, optval, optlen);
      // int getsockopt(int sockfd, int level, int optname, void *optval, socklen_t *optlen);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/getsockopt.html
      // Minimal getsockopt aimed at resolving https://github.com/kripken/emscripten/issues/2211
      // so only supports SOL_SOCKET with SO_ERROR.
      var sock = SOCKFS.getSocket(fd);
      if (!sock) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
  
      if (level === 1) {
        if (optname === 4) {
          HEAP32[((optval)>>2)]=sock.error;
          HEAP32[((optlen)>>2)]=4;
          sock.error = null; // Clear the error (The SO_ERROR option obtains and then clears this field).
          return 0;
        } else {
          ___setErrNo(ERRNO_CODES.ENOPROTOOPT); // The option is unknown at the level indicated.
          Runtime.warnOnce('getsockopt() returning an error as we currently only support optname SO_ERROR');
          return -1;
        }
      } else {
        ___setErrNo(ERRNO_CODES.ENOPROTOOPT); //The option is unknown at the level indicated.
        Runtime.warnOnce('getsockopt() returning an error as we only support level SOL_SOCKET');
        return -1;
      }
    }

  
  
  
  
  var _environ=ENVIRONMENT_IS_PTHREAD?0:allocate(1, "i32*", ALLOC_STATIC);var ___environ=_environ;function ___buildEnvironment(env) {
      // WARNING: Arbitrary limit!
      var MAX_ENV_VALUES = 64;
      var TOTAL_ENV_SIZE = 1024;
  
      // Statically allocate memory for the environment.
      var poolPtr;
      var envPtr;
      if (!___buildEnvironment.called) {
        ___buildEnvironment.called = true;
        // Set default values. Use string keys for Closure Compiler compatibility.
        ENV['USER'] = 'web_user';
        ENV['PATH'] = '/';
        ENV['PWD'] = '/';
        ENV['HOME'] = '/home/web_user';
        ENV['LANG'] = 'C';
        ENV['_'] = Module['thisProgram'];
        // Allocate memory.
        poolPtr = allocate(TOTAL_ENV_SIZE, 'i8', ALLOC_STATIC);
        envPtr = allocate(MAX_ENV_VALUES * 4,
                          'i8*', ALLOC_STATIC);
        HEAP32[((envPtr)>>2)]=poolPtr;
        HEAP32[((_environ)>>2)]=envPtr;
      } else {
        envPtr = HEAP32[((_environ)>>2)];
        poolPtr = HEAP32[((envPtr)>>2)];
      }
  
      // Collect key=value lines.
      var strings = [];
      var totalSize = 0;
      for (var key in env) {
        if (typeof env[key] === 'string') {
          var line = key + '=' + env[key];
          strings.push(line);
          totalSize += line.length;
        }
      }
      if (totalSize > TOTAL_ENV_SIZE) {
        throw new Error('Environment size exceeded TOTAL_ENV_SIZE!');
      }
  
      // Make new.
      var ptrSize = 4;
      for (var i = 0; i < strings.length; i++) {
        var line = strings[i];
        writeAsciiToMemory(line, poolPtr);
        HEAP32[(((envPtr)+(i * ptrSize))>>2)]=poolPtr;
        poolPtr += line.length + 1;
      }
      HEAP32[(((envPtr)+(strings.length * ptrSize))>>2)]=0;
    }var ENV={};function _putenv(string) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_1(115, string);
      // int putenv(char *string);
      // http://pubs.opengroup.org/onlinepubs/009695399/functions/putenv.html
      // WARNING: According to the standard (and the glibc implementation), the
      //          string is taken by reference so future changes are reflected.
      //          We copy it instead, possibly breaking some uses.
      if (string === 0) {
        ___setErrNo(ERRNO_CODES.EINVAL);
        return -1;
      }
      string = Pointer_stringify(string);
      var splitPoint = string.indexOf('=')
      if (string === '' || string.indexOf('=') === -1) {
        ___setErrNo(ERRNO_CODES.EINVAL);
        return -1;
      }
      var name = string.slice(0, splitPoint);
      var value = string.slice(splitPoint + 1);
      if (!(name in ENV) || ENV[name] !== value) {
        ENV[name] = value;
        ___buildEnvironment(ENV);
      }
      return 0;
    }

  function _fdopen(fildes, mode) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_2(75, fildes, mode);
      // FILE *fdopen(int fildes, const char *mode);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fdopen.html
      mode = Pointer_stringify(mode);
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return 0;
      }
      if ((mode.indexOf('w') != -1 && !stream.isWrite) ||
          (mode.indexOf('r') != -1 && !stream.isRead) ||
          (mode.indexOf('a') != -1 && !stream.isAppend) ||
          (mode.indexOf('+') != -1 && (!stream.isRead || !stream.isWrite))) {
        ___setErrNo(ERRNO_CODES.EINVAL);
        return 0;
      } else {
        stream.error = false;
        stream.eof = false;
        return FS.getPtrForStream(stream);
      }
    }

  
  
  
  function __getFloat(text) {
      return /^[+-]?[0-9]*\.?[0-9]+([eE][+-]?[0-9]+)?/.exec(text);
    }function __scanString(format, get, unget, varargs) {
      if (!__scanString.whiteSpace) {
        __scanString.whiteSpace = {};
        __scanString.whiteSpace[32] = 1;
        __scanString.whiteSpace[9] = 1;
        __scanString.whiteSpace[10] = 1;
        __scanString.whiteSpace[11] = 1;
        __scanString.whiteSpace[12] = 1;
        __scanString.whiteSpace[13] = 1;
      }
      // Supports %x, %4x, %d.%d, %lld, %s, %f, %lf.
      // TODO: Support all format specifiers.
      format = Pointer_stringify(format);
      var soFar = 0;
      if (format.indexOf('%n') >= 0) {
        // need to track soFar
        var _get = get;
        get = function get() {
          soFar++;
          return _get();
        }
        var _unget = unget;
        unget = function unget() {
          soFar--;
          return _unget();
        }
      }
      var formatIndex = 0;
      var argsi = 0;
      var fields = 0;
      var argIndex = 0;
      var next;
  
      mainLoop:
      for (var formatIndex = 0; formatIndex < format.length;) {
        if (format[formatIndex] === '%' && format[formatIndex+1] == 'n') {
          var argPtr = HEAP32[(((varargs)+(argIndex))>>2)];
          argIndex += Runtime.getAlignSize('void*', null, true);
          HEAP32[((argPtr)>>2)]=soFar;
          formatIndex += 2;
          continue;
        }
  
        if (format[formatIndex] === '%') {
          var nextC = format.indexOf('c', formatIndex+1);
          if (nextC > 0) {
            var maxx = 1;
            if (nextC > formatIndex+1) {
              var sub = format.substring(formatIndex+1, nextC);
              maxx = parseInt(sub);
              if (maxx != sub) maxx = 0;
            }
            if (maxx) {
              var argPtr = HEAP32[(((varargs)+(argIndex))>>2)];
              argIndex += Runtime.getAlignSize('void*', null, true);
              fields++;
              for (var i = 0; i < maxx; i++) {
                next = get();
                HEAP8[((argPtr++)>>0)]=next;
                if (next === 0) return i > 0 ? fields : fields-1; // we failed to read the full length of this field
              }
              formatIndex += nextC - formatIndex + 1;
              continue;
            }
          }
        }
  
        // handle %[...]
        if (format[formatIndex] === '%' && format.indexOf('[', formatIndex+1) > 0) {
          var match = /\%([0-9]*)\[(\^)?(\]?[^\]]*)\]/.exec(format.substring(formatIndex));
          if (match) {
            var maxNumCharacters = parseInt(match[1]) || Infinity;
            var negateScanList = (match[2] === '^');
            var scanList = match[3];
  
            // expand "middle" dashs into character sets
            var middleDashMatch;
            while ((middleDashMatch = /([^\-])\-([^\-])/.exec(scanList))) {
              var rangeStartCharCode = middleDashMatch[1].charCodeAt(0);
              var rangeEndCharCode = middleDashMatch[2].charCodeAt(0);
              for (var expanded = ''; rangeStartCharCode <= rangeEndCharCode; expanded += String.fromCharCode(rangeStartCharCode++));
              scanList = scanList.replace(middleDashMatch[1] + '-' + middleDashMatch[2], expanded);
            }
  
            var argPtr = HEAP32[(((varargs)+(argIndex))>>2)];
            argIndex += Runtime.getAlignSize('void*', null, true);
            fields++;
  
            for (var i = 0; i < maxNumCharacters; i++) {
              next = get();
              if (negateScanList) {
                if (scanList.indexOf(String.fromCharCode(next)) < 0) {
                  HEAP8[((argPtr++)>>0)]=next;
                } else {
                  unget();
                  break;
                }
              } else {
                if (scanList.indexOf(String.fromCharCode(next)) >= 0) {
                  HEAP8[((argPtr++)>>0)]=next;
                } else {
                  unget();
                  break;
                }
              }
            }
  
            // write out null-terminating character
            HEAP8[((argPtr++)>>0)]=0;
            formatIndex += match[0].length;
            
            continue;
          }
        }      
        // remove whitespace
        while (1) {
          next = get();
          if (next == 0) return fields;
          if (!(next in __scanString.whiteSpace)) break;
        }
        unget();
  
        if (format[formatIndex] === '%') {
          formatIndex++;
          var suppressAssignment = false;
          if (format[formatIndex] == '*') {
            suppressAssignment = true;
            formatIndex++;
          }
          var maxSpecifierStart = formatIndex;
          while (format[formatIndex].charCodeAt(0) >= 48 &&
                 format[formatIndex].charCodeAt(0) <= 57) {
            formatIndex++;
          }
          var max_;
          if (formatIndex != maxSpecifierStart) {
            max_ = parseInt(format.slice(maxSpecifierStart, formatIndex), 10);
          }
          var long_ = false;
          var half = false;
          var quarter = false;
          var longLong = false;
          if (format[formatIndex] == 'l') {
            long_ = true;
            formatIndex++;
            if (format[formatIndex] == 'l') {
              longLong = true;
              formatIndex++;
            }
          } else if (format[formatIndex] == 'h') {
            half = true;
            formatIndex++;
            if (format[formatIndex] == 'h') {
              quarter = true;
              formatIndex++;
            }
          }
          var type = format[formatIndex];
          formatIndex++;
          var curr = 0;
          var buffer = [];
          // Read characters according to the format. floats are trickier, they may be in an unfloat state in the middle, then be a valid float later
          if (type == 'f' || type == 'e' || type == 'g' ||
              type == 'F' || type == 'E' || type == 'G') {
            next = get();
            while (next > 0 && (!(next in __scanString.whiteSpace)))  {
              buffer.push(String.fromCharCode(next));
              next = get();
            }
            var m = __getFloat(buffer.join(''));
            var last = m ? m[0].length : 0;
            for (var i = 0; i < buffer.length - last + 1; i++) {
              unget();
            }
            buffer.length = last;
          } else {
            next = get();
            var first = true;
            
            // Strip the optional 0x prefix for %x.
            if ((type == 'x' || type == 'X') && (next == 48)) {
              var peek = get();
              if (peek == 120 || peek == 88) {
                next = get();
              } else {
                unget();
              }
            }
            
            while ((curr < max_ || isNaN(max_)) && next > 0) {
              if (!(next in __scanString.whiteSpace) && // stop on whitespace
                  (type == 's' ||
                   ((type === 'd' || type == 'u' || type == 'i') && ((next >= 48 && next <= 57) ||
                                                                     (first && next == 45))) ||
                   ((type === 'x' || type === 'X') && (next >= 48 && next <= 57 ||
                                     next >= 97 && next <= 102 ||
                                     next >= 65 && next <= 70))) &&
                  (formatIndex >= format.length || next !== format[formatIndex].charCodeAt(0))) { // Stop when we read something that is coming up
                buffer.push(String.fromCharCode(next));
                next = get();
                curr++;
                first = false;
              } else {
                break;
              }
            }
            unget();
          }
          if (buffer.length === 0) return fields; // Stop here.
          if (suppressAssignment) continue;
  
          var text = buffer.join('');
          var argPtr = HEAP32[(((varargs)+(argIndex))>>2)];
          argIndex += Runtime.getAlignSize('void*', null, true);
          var base = 10;
          switch (type) {
            case 'X': case 'x':
              base = 16;
            case 'd': case 'u': case 'i':
              if (quarter) {
                HEAP8[((argPtr)>>0)]=parseInt(text, base);
              } else if (half) {
                HEAP16[((argPtr)>>1)]=parseInt(text, base);
              } else if (longLong) {
                (tempI64 = [parseInt(text, base)>>>0,(tempDouble=parseInt(text, base),(+(Math_abs(tempDouble))) >= 1.0 ? (tempDouble > 0.0 ? ((Math_min((+(Math_floor((tempDouble)/4294967296.0))), 4294967295.0))|0)>>>0 : (~~((+(Math_ceil((tempDouble - +(((~~(tempDouble)))>>>0))/4294967296.0)))))>>>0) : 0)],HEAP32[((argPtr)>>2)]=tempI64[0],HEAP32[(((argPtr)+(4))>>2)]=tempI64[1]);
              } else {
                HEAP32[((argPtr)>>2)]=parseInt(text, base);
              }
              break;
            case 'F':
            case 'f':
            case 'E':
            case 'e':
            case 'G':
            case 'g':
            case 'E':
              // fallthrough intended
              if (long_) {
                HEAPF64[((argPtr)>>3)]=parseFloat(text);
              } else {
                HEAPF32[((argPtr)>>2)]=parseFloat(text);
              }
              break;
            case 's':
              var array = intArrayFromString(text);
              for (var j = 0; j < array.length; j++) {
                HEAP8[(((argPtr)+(j))>>0)]=array[j];
              }
              break;
          }
          fields++;
        } else if (format[formatIndex].charCodeAt(0) in __scanString.whiteSpace) {
          next = get();
          while (next in __scanString.whiteSpace) {
            if (next <= 0) break mainLoop;  // End of input.
            next = get();
          }
          unget(next);
          formatIndex++;
        } else {
          // Not a specifier.
          next = get();
          if (format[formatIndex].charCodeAt(0) !== next) {
            unget(next);
            break mainLoop;
          }
          formatIndex++;
        }
      }
      return fields;
    }
  
  function _ungetc(c, stream) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_2(102, c, stream);
      // int ungetc(int c, FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/ungetc.html
      stream = FS.getStreamFromPtr(stream);
      if (!stream) {
        return -1;
      }
      if (c === -1) {
        // do nothing for EOF character
        return c;
      }
      c = unSign(c & 0xFF);
      stream.ungotten.push(c);
      stream.eof = false;
      return c;
    }function _fscanf(stream, format, varargs) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_3(103, stream, format, varargs);
      // int fscanf(FILE *restrict stream, const char *restrict format, ... );
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/scanf.html
      var streamObj = FS.getStreamFromPtr(stream);
      if (!streamObj) {
        return -1;
      }
      var buffer = [];
      function get() {
        var c = _fgetc(stream);
        buffer.push(c);
        return c;
      };
      function unget() {
        _ungetc(buffer.pop(), stream);
      };
      return __scanString(format, get, unget, varargs);
    }function _scanf(format, varargs) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_2(104, format, varargs);
      // int scanf(const char *restrict format, ... );
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/scanf.html
      var stdin = HEAP32[((_stdin)>>2)];
      return _fscanf(stdin, format, varargs);
    }




  function _putc() {
  return _fputc.apply(null, arguments)
  }


  function _dup(fildes) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_1(39, fildes);
      // int dup(int fildes);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/dup.html
      return _fcntl(fildes, 0, allocate([0, 0, 0, 0], 'i32', ALLOC_STACK));  // F_DUPFD.
    }

  function _dprintf(fd, format, varargs) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_3(107, fd, format, varargs);
      var result = __formatString(format, varargs);
      var stack = Runtime.stackSave();
      var ret = _write(fd, allocate(result, 'i8', ALLOC_STACK), result.length);
      Runtime.stackRestore(stack);
    }

  function _listen(fd, backlog) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_2(129, fd, backlog);
      var sock = SOCKFS.getSocket(fd);
      if (!sock) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      try {
        sock.sock_ops.listen(sock, backlog);
        return 0;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }


  var _BDtoIHigh=true;

  function _getenv(name) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_1(111, name);
      // char *getenv(const char *name);
      // http://pubs.opengroup.org/onlinepubs/009695399/functions/getenv.html
      if (name === 0) return 0;
      name = Pointer_stringify(name);
      if (!ENV.hasOwnProperty(name)) return 0;
  
      if (_getenv.ret) _free(_getenv.ret);
      _getenv.ret = allocate(intArrayFromString(ENV[name]), 'i8', ALLOC_NORMAL);
      return _getenv.ret;
    }

  function _gettimeofday(ptr) {
      var now = Date.now();
      HEAP32[((ptr)>>2)]=(now/1000)|0; // seconds
      HEAP32[(((ptr)+(4))>>2)]=((now % 1000)*1000)|0; // microseconds
      return 0;
    }

  function _link(path1, path2) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_2(54, path1, path2);
      // int link(const char *path1, const char *path2);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/link.html
      // We don't support hard links.
      ___setErrNo(ERRNO_CODES.EMLINK);
      return -1;
    }

  function _emscripten_futex_wake(addr, count) {
      if (addr <= 0 || addr > HEAP8.length || addr&3 != 0 || count < 0) return -22;
      dump('futex_wake addr:' + addr + ' by thread: ' + _pthread_self() + (ENVIRONMENT_IS_PTHREAD?'(pthread)':'') + '\n');
      var ret = Atomics.futexWake(HEAP32, addr >> 2, count);
      if (ret >= 0) return ret;
      throw 'Atomics.futexWake returned an unexpected value ' + ret;
    }


  function _sbrk(bytes) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_1(73, bytes);
      // Implement a Linux-like 'memory area' for our 'process'.
      // Changes the size of the memory area by |bytes|; returns the
      // address of the previous top ('break') of the memory area
      // We control the "dynamic" memory - DYNAMIC_BASE to DYNAMICTOP
      var self = _sbrk;
      if (!self.called) {
        DYNAMICTOP = alignMemoryPage(DYNAMICTOP); // make sure we start out aligned
        self.called = true;
        assert(Runtime.dynamicAlloc);
        self.alloc = Runtime.dynamicAlloc;
        Runtime.dynamicAlloc = function() { abort('cannot dynamically allocate, sbrk now has control') };
      }
      var ret = DYNAMICTOP;
      if (bytes != 0) self.alloc(bytes);
      return ret;  // Previous break location.
    }



  var _BItoD=true;

  function _fstat(fildes, buf) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_2(16, fildes, buf);
      // int fstat(int fildes, struct stat *buf);
      // http://pubs.opengroup.org/onlinepubs/7908799/xsh/fstat.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      return _stat(stream.path, buf);
    }


  
  function _truncate(path, length) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_2(48, path, length);
      // int truncate(const char *path, off_t length);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/truncate.html
      // NOTE: The path argument may be a string, to simplify ftruncate().
      if (typeof path !== 'string') path = Pointer_stringify(path);
      try {
        FS.truncate(path, length);
        return 0;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _ftruncate(fildes, length) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_2(49, fildes, length);
      // int ftruncate(int fildes, off_t length);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/ftruncate.html
      try {
        FS.ftruncate(fildes, length);
        return 0;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }

  function _clearenv(name) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_1(112, name);
      // int clearenv (void);
      // http://www.gnu.org/s/hello/manual/libc/Environment-Access.html#index-clearenv-3107
      ENV = {};
      ___buildEnvironment(ENV);
      return 0;
    }

  function _connect(fd, addrp, addrlen) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_3(128, fd, addrp, addrlen);
      var sock = SOCKFS.getSocket(fd);
      if (!sock) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
  
      var info = __read_sockaddr(addrp, addrlen);
      if (info.errno) {
        ___setErrNo(info.errno);
        return -1;
      }
      var port = info.port;
      var addr = DNS.lookup_addr(info.addr) || info.addr;
  
      try {
        sock.sock_ops.connect(sock, addr, port);
        return 0;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }

  function _rename(old_path, new_path) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_2(97, old_path, new_path);
      // int rename(const char *old, const char *new);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/rename.html
      old_path = Pointer_stringify(old_path);
      new_path = Pointer_stringify(new_path);
      try {
        FS.rename(old_path, new_path);
        return 0;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }

  function _lockf(fildes, func, size) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_3(55, fildes, func, size);
      // int lockf(int fildes, int function, off_t size);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/lockf.html
      var stream = FS.getStream(fildes);
      if (stream) {
        // Pretend whatever locking or unlocking operation succeeded. Locking does
        // not make much sense, since we have a single process/thread.
        return 0;
      } else {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
    }

  function ___assert_fail(condition, filename, line, func) {
      ABORT = true;
      throw 'Assertion failed: ' + Pointer_stringify(condition) + ', at: ' + [filename ? Pointer_stringify(filename) : 'unknown filename', line, func ? Pointer_stringify(func) : 'unknown function'] + ' at ' + stackTrace();
    }

  function _creat(path, mode) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_2(27, path, mode);
      // int creat(const char *path, mode_t mode);
      // http://pubs.opengroup.org/onlinepubs/009695399/functions/creat.html
      return _open(path, 1 | 64 | 512, allocate([mode, 0, 0, 0], 'i32', ALLOC_STACK));
    }


  function _accept(fd, addr, addrlen) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_3(126, fd, addr, addrlen);
      var sock = SOCKFS.getSocket(fd);
      if (!sock) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      try {
        var newsock = sock.sock_ops.accept(sock);
        if (addr) {
          var res = __write_sockaddr(addr, newsock.family, DNS.lookup_name(newsock.daddr), newsock.dport);
          assert(!res.errno);
        }
        return newsock.stream.fd;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }

  function _getwd(path_name) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_1(51, path_name);
      // char *getwd(char *path_name);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/getwd.html
      return _getcwd(path_name, 4096);  // PATH_MAX.
    }

  function _abort() {
      Module['abort']();
    }


  function _confstr(name, buf, len) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_3(68, name, buf, len);
      // size_t confstr(int name, char *buf, size_t len);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/confstr.html
      var value;
      switch (name) {
        case 0:
          value = ENV['PATH'] || '/';
          break;
        case 1:
          // Mimicking glibc.
          value = 'POSIX_V6_ILP32_OFF32\nPOSIX_V6_ILP32_OFFBIG';
          break;
        case 2:
          // This JS implementation was tested against this glibc version.
          value = 'glibc 2.14';
          break;
        case 3:
          // We don't support pthreads.
          value = '';
          break;
        case 1118:
        case 1122:
        case 1124:
        case 1125:
        case 1126:
        case 1128:
        case 1129:
        case 1130:
          value = '';
          break;
        case 1116:
        case 1117:
        case 1121:
          value = '-m32';
          break;
        case 1120:
          value = '-m32 -D_LARGEFILE_SOURCE -D_FILE_OFFSET_BITS=64';
          break;
        default:
          ___setErrNo(ERRNO_CODES.EINVAL);
          return 0;
      }
      if (len == 0 || buf == 0) {
        return value.length + 1;
      } else {
        var length = Math.min(len, value.length);
        for (var i = 0; i < length; i++) {
          HEAP8[(((buf)+(i))>>0)]=value.charCodeAt(i);
        }
        if (len > length) HEAP8[(((buf)+(i++))>>0)]=0;
        return i;
      }
    }

  function _feof(stream) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_1(76, stream);
      // int feof(FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/feof.html
      stream = FS.getStreamFromPtr(stream);
      return Number(stream && stream.eof);
    }

  function _pipe(fildes) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_1(57, fildes);
      // int pipe(int fildes[2]);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/pipe.html
      // It is possible to implement this using two device streams, but pipes make
      // little sense in a single-threaded environment, so we do not support them.
      ___setErrNo(ERRNO_CODES.ENOSYS);
      Runtime.warnOnce('pipe() returning an error as we do not support them');
      return -1;
    }

  function _emscripten_asm_const(code) {
      Runtime.getAsmConst(code, 0)();
    }

  function _chroot(path) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_1(37, path);
      // int chroot(const char *path);
      // http://pubs.opengroup.org/onlinepubs/7908799/xsh/chroot.html
      ___setErrNo(ERRNO_CODES.EACCES);
      return -1;
    }

  function _encrypt(block, edflag) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_2(45, block, edflag);
      // void encrypt(char block[64], int edflag);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/encrypt.html
      // TODO: Implement (probably compile from C).
      Runtime.warnOnce('encrypt() returning an error as we do not support it');
      ___setErrNo(ERRNO_CODES.ENOSYS);
    }



  function _fseek(stream, offset, whence) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_3(89, stream, offset, whence);
      // int fseek(FILE *stream, long offset, int whence);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fseek.html
      var fd = _fileno(stream);
      var ret = _lseek(fd, offset, whence);
      if (ret == -1) {
        return -1;
      }
      stream = FS.getStreamFromPtr(stream);
      stream.eof = false;
      return 0;
    }


  function _emscripten_asm_const_int(code) {
      var args = Array.prototype.slice.call(arguments, 1);
      return Runtime.getAsmConst(code, args.length).apply(null, args) | 0;
    }

  function _setenv(envname, envval, overwrite) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_3(113, envname, envval, overwrite);
      // int setenv(const char *envname, const char *envval, int overwrite);
      // http://pubs.opengroup.org/onlinepubs/009695399/functions/setenv.html
      if (envname === 0) {
        ___setErrNo(ERRNO_CODES.EINVAL);
        return -1;
      }
      var name = Pointer_stringify(envname);
      var val = Pointer_stringify(envval);
      if (name === '' || name.indexOf('=') !== -1) {
        ___setErrNo(ERRNO_CODES.EINVAL);
        return -1;
      }
      if (ENV.hasOwnProperty(name) && !overwrite) return 0;
      ENV[name] = val;
      ___buildEnvironment(ENV);
      return 0;
    }

  function _recvmsg(fd, message, flags) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_3(123, fd, message, flags);
      var sock = SOCKFS.getSocket(fd);
      if (!sock) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
  
      var iov = HEAP32[(((message)+(8))>>2)];
      var num = HEAP32[(((message)+(12))>>2)];
  
      // get the total amount of data we can read across all arrays
      var total = 0;
      for (var i = 0; i < num; i++) {
        total += HEAP32[(((iov)+((8 * i) + 4))>>2)];
      }
  
      // try to read total data
      var msg;
      try {
        msg = sock.sock_ops.recvmsg(sock, total);
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
  
      if (!msg) {
        // socket is closed
        return 0;
      }
  
      // TODO honor flags:
      // MSG_OOB
      // Requests out-of-band data. The significance and semantics of out-of-band data are protocol-specific.
      // MSG_PEEK
      // Peeks at the incoming message.
      // MSG_WAITALL
      // Requests that the function block until the full amount of data requested can be returned. The function may return a smaller amount of data if a signal is caught, if the connection is terminated, if MSG_PEEK was specified, or if an error is pending for the socket.
  
      // write the source address out
      var name = HEAP32[((message)>>2)];
      if (name) {
        var res = __write_sockaddr(name, sock.family, DNS.lookup_name(msg.addr), msg.port);
        assert(!res.errno);
      }
      // write the buffer out to the scatter-gather arrays
      var bytesRead = 0;
      var bytesRemaining = msg.buffer.byteLength;
  
      for (var i = 0; bytesRemaining > 0 && i < num; i++) {
        var iovbase = HEAP32[(((iov)+((8 * i) + 0))>>2)];
        var iovlen = HEAP32[(((iov)+((8 * i) + 4))>>2)];
        if (!iovlen) {
          continue;
        }
        var length = Math.min(iovlen, bytesRemaining);
        var buf = msg.buffer.subarray(bytesRead, bytesRead + length);
        HEAPU8.set(buf, iovbase + bytesRead);
        bytesRead += length;
        bytesRemaining -= length;
      }
  
      // TODO set msghdr.msg_flags
      // MSG_EOR
      // End of record was received (if supported by the protocol).
      // MSG_OOB
      // Out-of-band data was received.
      // MSG_TRUNC
      // Normal data was truncated.
      // MSG_CTRUNC
  
      return bytesRead;
    }

  function _emscripten_atomic_cas_u32() {
  Module['printErr']('missing function: emscripten_atomic_cas_u32'); abort(-1);
  }

  function _rewind(stream) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_1(98, stream);
      // void rewind(FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/rewind.html
      _fseek(stream, 0, 0);  // SEEK_SET.
      var streamObj = FS.getStreamFromPtr(stream);
      if (streamObj) streamObj.error = false;
    }

  function _mkstemp(template) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_1(29, template);
      return _creat(_mktemp(template), 0600);
    }

  function _gets(s) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_1(82, s);
      // char *gets(char *s);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/gets.html
      return _fgets(s, 1e6, HEAP32[((_stdin)>>2)]);
    }

  function _realpath(file_name, resolved_name) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_2(116, file_name, resolved_name);
      // char *realpath(const char *restrict file_name, char *restrict resolved_name);
      // http://pubs.opengroup.org/onlinepubs/009604499/functions/realpath.html
      var absolute = FS.analyzePath(Pointer_stringify(file_name));
      if (absolute.error) {
        ___setErrNo(absolute.error);
        return 0;
      } else {
        var size = Math.min(4095, absolute.path.length);  // PATH_MAX - 1.
        if (resolved_name === 0) resolved_name = _malloc(size+1);
        for (var i = 0; i < size; i++) {
          HEAP8[(((resolved_name)+(i))>>0)]=absolute.path.charCodeAt(i);
        }
        HEAP8[(((resolved_name)+(size))>>0)]=0;
        return resolved_name;
      }
    }


  function _getlogin() {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_0(70);
      // char *getlogin(void);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/getlogin.html
      if (!_getlogin.ret) _getlogin.ret = _malloc(8);
      return _getlogin_r(_getlogin.ret, 8) ? 0 : _getlogin.ret;
    }

  function _tcsetattr(fildes, optional_actions, termios_p) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_3(118, fildes, optional_actions, termios_p);
      // http://pubs.opengroup.org/onlinepubs/7908799/xsh/tcsetattr.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      if (!stream.tty) {
        ___setErrNo(ERRNO_CODES.ENOTTY);
        return -1;
      }
      return 0;
    }


  function _unsetenv(name) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_1(114, name);
      // int unsetenv(const char *name);
      // http://pubs.opengroup.org/onlinepubs/009695399/functions/unsetenv.html
      if (name === 0) {
        ___setErrNo(ERRNO_CODES.EINVAL);
        return -1;
      }
      name = Pointer_stringify(name);
      if (name === '' || name.indexOf('=') !== -1) {
        ___setErrNo(ERRNO_CODES.EINVAL);
        return -1;
      }
      if (ENV.hasOwnProperty(name)) {
        delete ENV[name];
        ___buildEnvironment(ENV);
      }
      return 0;
    }


  function _time(ptr) {
      var ret = (Date.now()/1000)|0;
      if (ptr) {
        HEAP32[((ptr)>>2)]=ret;
      }
      return ret;
    }

  function _closedir(dirp) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_1(6, dirp);
      // int closedir(DIR *dirp);
      // http://pubs.opengroup.org/onlinepubs/007908799/xsh/closedir.html
      var fd = _fileno(dirp);
      var stream = FS.getStream(fd);
      if (stream.currReading) stream.currReading = null;
      return _close(fd);
    }

  function _ioctl(fd, request, varargs) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_3(125, fd, request, varargs);
      var stream = FS.getStream(fd);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      var arg = HEAP32[((varargs)>>2)];
  
      try {
        return FS.ioctl(stream, request, arg);
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }

  function _readlink(path, buf, bufsize) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_3(65, path, buf, bufsize);
      // ssize_t readlink(const char *restrict path, char *restrict buf, size_t bufsize);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/readlink.html
      path = Pointer_stringify(path);
      var str;
      try {
        str = FS.readlink(path);
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
      str = str.slice(0, Math.max(0, bufsize - 1));
      writeStringToMemory(str, buf, true);
      return str.length;
    }


  
  function __cleanup_thread(pthread_ptr) {
      if (ENVIRONMENT_IS_WORKER || ENVIRONMENT_IS_PTHREAD) throw 'Internal Error! _cleanup_thread() can only ever be called from main JS thread!';
      if (!pthread_ptr) throw 'Internal Error! Null pthread_ptr in _cleanup_thread!';
      HEAP32[(((pthread_ptr)+(20))>>2)]=0;
      var pthread = PThread.pthreads[pthread_ptr];
      var worker = pthread.worker;
      PThread.freeThreadData(pthread);
      worker.pthread = undefined; // Detach the worker from the pthread object, and return it to the worker pool as an unused worker.
      PThread.unusedWorkerPool.push(worker);
      PThread.runningWorkers.splice(PThread.runningWorkers.indexOf(worker.pthread), 1); // Not a running Worker anymore.
    }
  
  function __pthread_testcancel_js() {
      if (!ENVIRONMENT_IS_PTHREAD) return;
      if (!threadBlock) return;
      var cancelDisabled = Atomics.load(HEAPU32, (threadBlock + 68 ) >> 2);
      if (cancelDisabled) return;
      var canceled = Atomics.load(HEAPU32, (threadBlock + 0 ) >> 2);
      if (canceled == 2) throw 'Canceled!';
    }function _pthread_join(thread, status) {
      if (!thread) {
        Module['printErr']('pthread_join attempted on a null thread pointer!');
        return ERRNO_CODES.ESRCH;
      }
      if (ENVIRONMENT_IS_PTHREAD && selfThreadId == thread) {
        Module['printErr']('PThread ' + thread + ' is attempting to join to itself!');
        return ERRNO_CODES.EDEADLK;
      }
      else if (!ENVIRONMENT_IS_PTHREAD && PThread.mainThreadBlock == thread) {
        Module['printErr']('Main thread ' + thread + ' is attempting to join to itself!');
        return ERRNO_CODES.EDEADLK;
      }
      var self = HEAP32[(((thread)+(20))>>2)];
      if (self != thread) {
        Module['printErr']('pthread_join attempted on thread ' + thread + ', which does not point to a valid thread, or does not exist anymore!');
        return ERRNO_CODES.ESRCH;
      }
  
      var detached = Atomics.load(HEAPU32, (thread + 76 ) >> 2);
      if (detached) {
        Module['printErr']('Attempted to join thread ' + thread + ', which was already detached!');
        return ERRNO_CODES.EINVAL; // The thread is already detached, can no longer join it!
      }
      for(;;) {
        var threadStatus = Atomics.load(HEAPU32, (thread + 0 ) >> 2);
        if (threadStatus == 1) { // Exited?
          var threadExitCode = Atomics.load(HEAPU32, (thread + 4 ) >> 2);
          if (status) HEAP32[((status)>>2)]=threadExitCode;
          Atomics.store(HEAPU32, (thread + 76 ) >> 2, 1); // Mark the thread as detached.
  
          if (!ENVIRONMENT_IS_WORKER) __cleanup_thread(thread);
          else postMessage({ cmd: 'cleanupThread', thread: thread});
          return 0;
        }
        // TODO HACK! Replace the _js variant with just _pthread_testcancel:
        //_pthread_testcancel();
        __pthread_testcancel_js();
        // In Main runtime thread, assist pthreads in performing operations that they need to access the Emscripten main runtime for.
        if (!ENVIRONMENT_IS_PTHREAD) _emscripten_main_thread_process_queued_calls();
        _emscripten_futex_wait(thread + 0, threadStatus, 100 * 1000 * 1000);
      }
    }

  function _clearerr(stream) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_1(74, stream);
      // void clearerr(FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/clearerr.html
      stream = FS.getStreamFromPtr(stream);
      if (!stream) {
        return;
      }
      stream.eof = false;
      stream.error = false;
    }



  function _bind(fd, addrp, addrlen) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_3(121, fd, addrp, addrlen);
      var sock = SOCKFS.getSocket(fd);
      if (!sock) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
  
      var info = __read_sockaddr(addrp, addrlen);
      if (info.errno) {
        ___setErrNo(info.errno);
        return -1;
      }
      var port = info.port;
      var addr = DNS.lookup_addr(info.addr) || info.addr;
  
      try {
        sock.sock_ops.bind(sock, addr, port);
        return 0;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }

  function _pthread_cleanup_pop(execute) {
      var routine = PThread.exitHandlers.pop();
      if (execute) routine();
    }

  function _fpathconf(fildes, name) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_2(46, fildes, name);
      // long fpathconf(int fildes, int name);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/encrypt.html
      // NOTE: The first parameter is ignored, so pathconf == fpathconf.
      // The constants here aren't real values. Just mimicking glibc.
      switch (name) {
        case 0:
          return 32000;
        case 1:
        case 2:
        case 3:
          return 255;
        case 4:
        case 5:
        case 16:
        case 17:
        case 18:
          return 4096;
        case 6:
        case 7:
        case 20:
          return 1;
        case 8:
          return 0;
        case 9:
        case 10:
        case 11:
        case 12:
        case 14:
        case 15:
        case 19:
          return -1;
        case 13:
          return 64;
      }
      ___setErrNo(ERRNO_CODES.EINVAL);
      return -1;
    }


  function _emscripten_atomic_store_u32() {
  Module['printErr']('missing function: emscripten_atomic_store_u32'); abort(-1);
  }

  function _crypt(key, salt) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_2(44, key, salt);
      // char *(const char *, const char *);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/crypt.html
      // TODO: Implement (probably compile from C).
      ___setErrNo(ERRNO_CODES.ENOSYS);
      Runtime.warnOnce('crypt() returning an error as we do not support it');
      return 0;
    }


  function _utimes(path, times) {
      if (ENVIRONMENT_IS_PTHREAD) return _emscripten_sync_run_in_main_thread_2(13, path, times);
      var time;
      if (times) {
        var offset = 8 + 0;
        time = HEAP32[(((times)+(offset))>>2)] * 1000;
        offset = 8 + 4;
        time += HEAP32[(((times)+(offset))>>2)] / 1000;
      } else {
        time = Date.now();
      }
      path = Pointer_stringify(path);
      try {
        FS.utime(path, time, time);
        return 0;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }
FS.staticInit();__ATINIT__.unshift({ func: function() { if (!Module["noFSInit"] && !FS.init.initialized) FS.init() } });__ATMAIN__.push({ func: function() { FS.ignorePermissions = false } });__ATEXIT__.push({ func: function() { FS.quit() } });Module["FS_createFolder"] = FS.createFolder;Module["FS_createPath"] = FS.createPath;Module["FS_createDataFile"] = FS.createDataFile;Module["FS_createPreloadedFile"] = FS.createPreloadedFile;Module["FS_createLazyFile"] = FS.createLazyFile;Module["FS_createLink"] = FS.createLink;Module["FS_createDevice"] = FS.createDevice;
___errno_state = Runtime.staticAlloc(4); HEAP32[((___errno_state)>>2)]=0;
__ATINIT__.unshift({ func: function() { TTY.init() } });__ATEXIT__.push({ func: function() { TTY.shutdown() } });
if (ENVIRONMENT_IS_NODE) { var fs = require("fs"); NODEFS.staticInit(); }
_fputc.ret = ENVIRONMENT_IS_PTHREAD?0:allocate([0], "i8", ALLOC_STATIC);
__ATINIT__.push({ func: function() { SOCKFS.root = FS.mount(SOCKFS, {}, null); } });
if (!ENVIRONMENT_IS_PTHREAD) PThread.initMainThreadBlock();
_fgetc.ret = ENVIRONMENT_IS_PTHREAD?0:allocate([0], "i8", ALLOC_STATIC);
Module["requestFullScreen"] = function Module_requestFullScreen(lockPointer, resizeCanvas, vrDevice) { Browser.requestFullScreen(lockPointer, resizeCanvas, vrDevice) };
  Module["requestAnimationFrame"] = function Module_requestAnimationFrame(func) { Browser.requestAnimationFrame(func) };
  Module["setCanvasSize"] = function Module_setCanvasSize(width, height, noUpdates) { Browser.setCanvasSize(width, height, noUpdates) };
  Module["pauseMainLoop"] = function Module_pauseMainLoop() { Browser.mainLoop.pause() };
  Module["resumeMainLoop"] = function Module_resumeMainLoop() { Browser.mainLoop.resume() };
  Module["getUserMedia"] = function Module_getUserMedia() { Browser.getUserMedia() }
___buildEnvironment(ENV);
STACK_BASE = STACKTOP = Runtime.alignMemory(STATICTOP);

staticSealed = true; // seal the static portion of memory

STACK_MAX = STACK_BASE + TOTAL_STACK;

DYNAMIC_BASE = DYNAMICTOP = Runtime.alignMemory(STACK_MAX);

assert(DYNAMIC_BASE < TOTAL_MEMORY, "TOTAL_MEMORY not big enough for stack");



function nullFunc_ii(x) { Module["printErr"]("Invalid function pointer called with signature 'ii'. Perhaps this is an invalid value (e.g. caused by calling a virtual method on a NULL pointer)? Or calling a function with an incorrect type, which will fail? (it is worth building your source files with -Werror (warnings are errors), as warnings can indicate undefined behavior which can cause this)");  Module["printErr"]("Build with ASSERTIONS=2 for more info."); abort(x) }

function nullFunc_vi(x) { Module["printErr"]("Invalid function pointer called with signature 'vi'. Perhaps this is an invalid value (e.g. caused by calling a virtual method on a NULL pointer)? Or calling a function with an incorrect type, which will fail? (it is worth building your source files with -Werror (warnings are errors), as warnings can indicate undefined behavior which can cause this)");  Module["printErr"]("Build with ASSERTIONS=2 for more info."); abort(x) }

function nullFunc_v(x) { Module["printErr"]("Invalid function pointer called with signature 'v'. Perhaps this is an invalid value (e.g. caused by calling a virtual method on a NULL pointer)? Or calling a function with an incorrect type, which will fail? (it is worth building your source files with -Werror (warnings are errors), as warnings can indicate undefined behavior which can cause this)");  Module["printErr"]("Build with ASSERTIONS=2 for more info."); abort(x) }

function invoke_ii(index,a1) {
  try {
    return Module["dynCall_ii"](index,a1);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_vi(index,a1) {
  try {
    Module["dynCall_vi"](index,a1);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_v(index) {
  try {
    Module["dynCall_v"](index);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

Module.asmGlobalArg = { "Atomics": Atomics, "Math": Math, "SharedInt8Array": SharedInt8Array, "SharedInt16Array": SharedInt16Array, "SharedInt32Array": SharedInt32Array, "SharedUint8Array": SharedUint8Array, "SharedUint16Array": SharedUint16Array, "SharedUint32Array": SharedUint32Array, "SharedFloat32Array": SharedFloat32Array, "SharedFloat64Array": SharedFloat64Array, "Int8Array": Int8Array, "Int16Array": Int16Array, "Int32Array": Int32Array, "Uint8Array": Uint8Array, "Uint16Array": Uint16Array, "Uint32Array": Uint32Array, "Float32Array": Float32Array, "Float64Array": Float64Array, "NaN": NaN, "Infinity": Infinity };
Module.asmLibraryArg = { "abort": abort, "assert": assert, "nullFunc_ii": nullFunc_ii, "nullFunc_vi": nullFunc_vi, "nullFunc_v": nullFunc_v, "invoke_ii": invoke_ii, "invoke_vi": invoke_vi, "invoke_v": invoke_v, "_emscripten_atomic_store_u32": _emscripten_atomic_store_u32, "_lchmod": _lchmod, "__inet_ntop6_raw": __inet_ntop6_raw, "_ctermid": _ctermid, "_fread": _fread, "_lchown": _lchown, "_sendto": _sendto, "_rewinddir": _rewinddir, "_bind": _bind, "_pclose": _pclose, "_fgetpos": _fgetpos, "___assert_fail": ___assert_fail, "_pthread_join": _pthread_join, "__write_sockaddr": __write_sockaddr, "_clock_gettime": _clock_gettime, "_ftell": _ftell, "_emscripten_set_main_loop_timing": _emscripten_set_main_loop_timing, "_sbrk": _sbrk, "_emscripten_memcpy_big": _emscripten_memcpy_big, "_tcgetattr": _tcgetattr, "_sysconf": _sysconf, "_utime": _utime, "_close": _close, "_ferror": _ferror, "_rewind": _rewind, "_readdir": _readdir, "_recvfrom": _recvfrom, "_listen": _listen, "_puts": _puts, "_freopen": _freopen, "_isatty": _isatty, "_umask": _umask, "_write": _write, "_mkdtemp": _mkdtemp, "_fsync": _fsync, "_strerror": _strerror, "_select": _select, "_chroot": _chroot, "__exit": __exit, "_gethostname": _gethostname, "_readdir_r": _readdir_r, "_unsetenv": _unsetenv, "_tmpnam": _tmpnam, "_lseek": _lseek, "_mknod": _mknod, "_getlogin": _getlogin, "_mkdir": _mkdir, "_closedir": _closedir, "_tmpfile": _tmpfile, "_scanf": _scanf, "_emscripten_get_now_is_monotonic": _emscripten_get_now_is_monotonic, "_encrypt": _encrypt, "_send": _send, "_symlink": _symlink, "_fstat": _fstat, "__inet_ntop4_raw": __inet_ntop4_raw, "_emscripten_atomic_cas_u32": _emscripten_atomic_cas_u32, "_pthread_cleanup_push": _pthread_cleanup_push, "_chmod": _chmod, "_chown": _chown, "_fcntl": _fcntl, "_lstat": _lstat, "_opendir": _opendir, "_truncate": _truncate, "_strerror_r": _strerror_r, "_fscanf": _fscanf, "___setErrNo": ___setErrNo, "_creat": _creat, "_dup2": _dup2, "_unlink": _unlink, "_pthread_self": _pthread_self, "_ntohs": _ntohs, "_putchar": _putchar, "_fsetpos": _fsetpos, "_mktemp": _mktemp, "_printf": _printf, "_dprintf": _dprintf, "_poll": _poll, "_emscripten_get_now": _emscripten_get_now, "_stat": _stat, "_connect": _connect, "_mkport": _mkport, "_sendmsg": _sendmsg, "_read": _read, "_getsockopt": _getsockopt, "_fwrite": _fwrite, "_time": _time, "_fprintf": _fprintf, "_gettimeofday": _gettimeofday, "_munmap": _munmap, "_mmap": _mmap, "_exit": _exit, "_rename": _rename, "__spawn_thread": __spawn_thread, "_pthread_cleanup_pop": _pthread_cleanup_pop, "__inet_pton4_raw": __inet_pton4_raw, "_putenv": _putenv, "_utimes": _utimes, "_getcwd": _getcwd, "_fchdir": _fchdir, "_pipe": _pipe, "_tempnam": _tempnam, "_link": _link, "_access": _access, "_htons": _htons, "_rmdir": _rmdir, "_feof": _feof, "___buildEnvironment": ___buildEnvironment, "_chdir": _chdir, "_socket": _socket, "_tzset": _tzset, "_dup": _dup, "_remove": _remove, "_readlink": _readlink, "___errno_location": ___errno_location, "_getlogin_r": _getlogin_r, "_perror": _perror, "_emscripten_futex_wake": _emscripten_futex_wake, "_fchmod": _fchmod, "_getsockname": _getsockname, "_ftruncate": _ftruncate, "_clearenv": _clearenv, "_fdopen": _fdopen, "_fseek": _fseek, "_posix_fallocate": _posix_fallocate, "_mkfifo": _mkfifo, "_getenv": _getenv, "_fclose": _fclose, "_getwd": _getwd, "_popen": _popen, "_recv": _recv, "_fgetc": _fgetc, "_shutdown": _shutdown, "__getFloat": __getFloat, "_accept": _accept, "_abort": _abort, "_emscripten_atomic_add_u32": _emscripten_atomic_add_u32, "__cleanup_thread": __cleanup_thread, "_fopen": _fopen, "_lockf": _lockf, "_ttyname": _ttyname, "_putc": _putc, "_getchar": _getchar, "_emscripten_asm_const": _emscripten_asm_const, "_atexit": _atexit, "__inet_pton6_raw": __inet_pton6_raw, "_ungetc": _ungetc, "__read_sockaddr": __read_sockaddr, "_emscripten_asm_const_int": _emscripten_asm_const_int, "_emscripten_atomic_sub_u32": _emscripten_atomic_sub_u32, "_ttyname_r": _ttyname_r, "_open": _open, "_mkstemp": _mkstemp, "_getpeername": _getpeername, "_emscripten_futex_wait": _emscripten_futex_wait, "__reallyNegative": __reallyNegative, "_pthread_getschedparam": _pthread_getschedparam, "_telldir": _telldir, "_recvmsg": _recvmsg, "_fileno": _fileno, "__pthread_testcancel_js": __pthread_testcancel_js, "_seekdir": _seekdir, "_fstatvfs": _fstatvfs, "_tcsetattr": _tcsetattr, "_fgets": _fgets, "_gets": _gets, "_confstr": _confstr, "_crypt": _crypt, "_pwrite": _pwrite, "_pread": _pread, "_realpath": _realpath, "_fchown": _fchown, "_pthread_exit": _pthread_exit, "_ioctl": _ioctl, "_emscripten_is_main_runtime_thread": _emscripten_is_main_runtime_thread, "_pthread_create": _pthread_create, "_fflush": _fflush, "__scanString": __scanString, "_emscripten_atomic_load_u32": _emscripten_atomic_load_u32, "_clearerr": _clearerr, "_fputc": _fputc, "_emscripten_set_main_loop": _emscripten_set_main_loop, "_emscripten_cancel_main_loop": _emscripten_cancel_main_loop, "__formatString": __formatString, "_fputs": _fputs, "_fpathconf": _fpathconf, "_statvfs": _statvfs, "_setenv": _setenv, "STACKTOP": STACKTOP, "STACK_MAX": STACK_MAX, "tempDoublePtr": tempDoublePtr, "ABORT": ABORT };
// EMSCRIPTEN_START_ASM
var asm = (function(global, env, buffer) {
  'almost asm';
  
  // TODO: Enable the following if() conditional to a linker flag. (-lpthread?)
  //if (typeof SharedArrayBuffer != 'undefined') {
    var HEAP8 = new global.SharedInt8Array(buffer);
    var HEAP16 = new global.SharedInt16Array(buffer);
    var HEAP32 = new global.SharedInt32Array(buffer);
    var HEAPU8 = new global.SharedUint8Array(buffer);
    var HEAPU16 = new global.SharedUint16Array(buffer);
    var HEAPU32 = new global.SharedUint32Array(buffer);
    var HEAPF32 = new global.SharedFloat32Array(buffer);
    var HEAPF64 = new global.SharedFloat64Array(buffer);
  /*} else {
    var HEAP8 = new global.Int8Array(buffer);
    var HEAP16 = new global.Int16Array(buffer);
    var HEAP32 = new global.Int32Array(buffer);
    var HEAPU8 = new global.Uint8Array(buffer);
    var HEAPU16 = new global.Uint16Array(buffer);
    var HEAPU32 = new global.Uint32Array(buffer);
    var HEAPF32 = new global.Float32Array(buffer);
    var HEAPF64 = new global.Float64Array(buffer);
  }*/


  var STACKTOP=env.STACKTOP|0;
  var STACK_MAX=env.STACK_MAX|0;
  var tempDoublePtr=env.tempDoublePtr|0;
  var ABORT=env.ABORT|0;

  var __THREW__ = 0;
  var threwValue = 0;
  var setjmpId = 0;
  var undef = 0;
  var nan = global.NaN, inf = global.Infinity;
  var tempInt = 0, tempBigInt = 0, tempBigIntP = 0, tempBigIntS = 0, tempBigIntR = 0.0, tempBigIntI = 0, tempBigIntD = 0, tempValue = 0, tempDouble = 0.0;

  var tempRet0 = 0;
  var tempRet1 = 0;
  var tempRet2 = 0;
  var tempRet3 = 0;
  var tempRet4 = 0;
  var tempRet5 = 0;
  var tempRet6 = 0;
  var tempRet7 = 0;
  var tempRet8 = 0;
  var tempRet9 = 0;
  var Math_floor=global.Math.floor;
  var Math_abs=global.Math.abs;
  var Math_sqrt=global.Math.sqrt;
  var Math_pow=global.Math.pow;
  var Math_cos=global.Math.cos;
  var Math_sin=global.Math.sin;
  var Math_tan=global.Math.tan;
  var Math_acos=global.Math.acos;
  var Math_asin=global.Math.asin;
  var Math_atan=global.Math.atan;
  var Math_atan2=global.Math.atan2;
  var Math_exp=global.Math.exp;
  var Math_log=global.Math.log;
  var Math_ceil=global.Math.ceil;
  var Math_imul=global.Math.imul;
  var Math_min=global.Math.min;
  var Math_clz32=global.Math.clz32;
  var abort=env.abort;
  var assert=env.assert;
  var nullFunc_ii=env.nullFunc_ii;
  var nullFunc_vi=env.nullFunc_vi;
  var nullFunc_v=env.nullFunc_v;
  var invoke_ii=env.invoke_ii;
  var invoke_vi=env.invoke_vi;
  var invoke_v=env.invoke_v;
  var _emscripten_atomic_store_u32=env._emscripten_atomic_store_u32;
  var _lchmod=env._lchmod;
  var __inet_ntop6_raw=env.__inet_ntop6_raw;
  var _ctermid=env._ctermid;
  var _fread=env._fread;
  var _lchown=env._lchown;
  var _sendto=env._sendto;
  var _rewinddir=env._rewinddir;
  var _bind=env._bind;
  var _pclose=env._pclose;
  var _fgetpos=env._fgetpos;
  var ___assert_fail=env.___assert_fail;
  var _pthread_join=env._pthread_join;
  var __write_sockaddr=env.__write_sockaddr;
  var _clock_gettime=env._clock_gettime;
  var _ftell=env._ftell;
  var _emscripten_set_main_loop_timing=env._emscripten_set_main_loop_timing;
  var _sbrk=env._sbrk;
  var _emscripten_memcpy_big=env._emscripten_memcpy_big;
  var _tcgetattr=env._tcgetattr;
  var _sysconf=env._sysconf;
  var _utime=env._utime;
  var _close=env._close;
  var _ferror=env._ferror;
  var _rewind=env._rewind;
  var _readdir=env._readdir;
  var _recvfrom=env._recvfrom;
  var _listen=env._listen;
  var _puts=env._puts;
  var _freopen=env._freopen;
  var _isatty=env._isatty;
  var _umask=env._umask;
  var _write=env._write;
  var _mkdtemp=env._mkdtemp;
  var _fsync=env._fsync;
  var _strerror=env._strerror;
  var _select=env._select;
  var _chroot=env._chroot;
  var __exit=env.__exit;
  var _gethostname=env._gethostname;
  var _readdir_r=env._readdir_r;
  var _unsetenv=env._unsetenv;
  var _tmpnam=env._tmpnam;
  var _lseek=env._lseek;
  var _mknod=env._mknod;
  var _getlogin=env._getlogin;
  var _mkdir=env._mkdir;
  var _closedir=env._closedir;
  var _tmpfile=env._tmpfile;
  var _scanf=env._scanf;
  var _emscripten_get_now_is_monotonic=env._emscripten_get_now_is_monotonic;
  var _encrypt=env._encrypt;
  var _send=env._send;
  var _symlink=env._symlink;
  var _fstat=env._fstat;
  var __inet_ntop4_raw=env.__inet_ntop4_raw;
  var _emscripten_atomic_cas_u32=env._emscripten_atomic_cas_u32;
  var _pthread_cleanup_push=env._pthread_cleanup_push;
  var _chmod=env._chmod;
  var _chown=env._chown;
  var _fcntl=env._fcntl;
  var _lstat=env._lstat;
  var _opendir=env._opendir;
  var _truncate=env._truncate;
  var _strerror_r=env._strerror_r;
  var _fscanf=env._fscanf;
  var ___setErrNo=env.___setErrNo;
  var _creat=env._creat;
  var _dup2=env._dup2;
  var _unlink=env._unlink;
  var _pthread_self=env._pthread_self;
  var _ntohs=env._ntohs;
  var _putchar=env._putchar;
  var _fsetpos=env._fsetpos;
  var _mktemp=env._mktemp;
  var _printf=env._printf;
  var _dprintf=env._dprintf;
  var _poll=env._poll;
  var _emscripten_get_now=env._emscripten_get_now;
  var _stat=env._stat;
  var _connect=env._connect;
  var _mkport=env._mkport;
  var _sendmsg=env._sendmsg;
  var _read=env._read;
  var _getsockopt=env._getsockopt;
  var _fwrite=env._fwrite;
  var _time=env._time;
  var _fprintf=env._fprintf;
  var _gettimeofday=env._gettimeofday;
  var _munmap=env._munmap;
  var _mmap=env._mmap;
  var _exit=env._exit;
  var _rename=env._rename;
  var __spawn_thread=env.__spawn_thread;
  var _pthread_cleanup_pop=env._pthread_cleanup_pop;
  var __inet_pton4_raw=env.__inet_pton4_raw;
  var _putenv=env._putenv;
  var _utimes=env._utimes;
  var _getcwd=env._getcwd;
  var _fchdir=env._fchdir;
  var _pipe=env._pipe;
  var _tempnam=env._tempnam;
  var _link=env._link;
  var _access=env._access;
  var _htons=env._htons;
  var _rmdir=env._rmdir;
  var _feof=env._feof;
  var ___buildEnvironment=env.___buildEnvironment;
  var _chdir=env._chdir;
  var _socket=env._socket;
  var _tzset=env._tzset;
  var _dup=env._dup;
  var _remove=env._remove;
  var _readlink=env._readlink;
  var ___errno_location=env.___errno_location;
  var _getlogin_r=env._getlogin_r;
  var _perror=env._perror;
  var _emscripten_futex_wake=env._emscripten_futex_wake;
  var _fchmod=env._fchmod;
  var _getsockname=env._getsockname;
  var _ftruncate=env._ftruncate;
  var _clearenv=env._clearenv;
  var _fdopen=env._fdopen;
  var _fseek=env._fseek;
  var _posix_fallocate=env._posix_fallocate;
  var _mkfifo=env._mkfifo;
  var _getenv=env._getenv;
  var _fclose=env._fclose;
  var _getwd=env._getwd;
  var _popen=env._popen;
  var _recv=env._recv;
  var _fgetc=env._fgetc;
  var _shutdown=env._shutdown;
  var __getFloat=env.__getFloat;
  var _accept=env._accept;
  var _abort=env._abort;
  var _emscripten_atomic_add_u32=env._emscripten_atomic_add_u32;
  var __cleanup_thread=env.__cleanup_thread;
  var _fopen=env._fopen;
  var _lockf=env._lockf;
  var _ttyname=env._ttyname;
  var _putc=env._putc;
  var _getchar=env._getchar;
  var _emscripten_asm_const=env._emscripten_asm_const;
  var _atexit=env._atexit;
  var __inet_pton6_raw=env.__inet_pton6_raw;
  var _ungetc=env._ungetc;
  var __read_sockaddr=env.__read_sockaddr;
  var _emscripten_asm_const_int=env._emscripten_asm_const_int;
  var _emscripten_atomic_sub_u32=env._emscripten_atomic_sub_u32;
  var _ttyname_r=env._ttyname_r;
  var _open=env._open;
  var _mkstemp=env._mkstemp;
  var _getpeername=env._getpeername;
  var _emscripten_futex_wait=env._emscripten_futex_wait;
  var __reallyNegative=env.__reallyNegative;
  var _pthread_getschedparam=env._pthread_getschedparam;
  var _telldir=env._telldir;
  var _recvmsg=env._recvmsg;
  var _fileno=env._fileno;
  var __pthread_testcancel_js=env.__pthread_testcancel_js;
  var _seekdir=env._seekdir;
  var _fstatvfs=env._fstatvfs;
  var _tcsetattr=env._tcsetattr;
  var _fgets=env._fgets;
  var _gets=env._gets;
  var _confstr=env._confstr;
  var _crypt=env._crypt;
  var _pwrite=env._pwrite;
  var _pread=env._pread;
  var _realpath=env._realpath;
  var _fchown=env._fchown;
  var _pthread_exit=env._pthread_exit;
  var _ioctl=env._ioctl;
  var _emscripten_is_main_runtime_thread=env._emscripten_is_main_runtime_thread;
  var _pthread_create=env._pthread_create;
  var _fflush=env._fflush;
  var __scanString=env.__scanString;
  var _emscripten_atomic_load_u32=env._emscripten_atomic_load_u32;
  var _clearerr=env._clearerr;
  var _fputc=env._fputc;
  var _emscripten_set_main_loop=env._emscripten_set_main_loop;
  var _emscripten_cancel_main_loop=env._emscripten_cancel_main_loop;
  var __formatString=env.__formatString;
  var _fputs=env._fputs;
  var _fpathconf=env._fpathconf;
  var _statvfs=env._statvfs;
  var _setenv=env._setenv;
  var Atomics_load=global.Atomics.load;
  var Atomics_store=global.Atomics.store;
  var Atomics_compareExchange=global.Atomics.compareExchange;
  var Atomics_add=global.Atomics.add;
  var Atomics_sub=global.Atomics.sub;
  var Atomics_and=global.Atomics.and;
  var Atomics_or=global.Atomics.or;
  var Atomics_xor=global.Atomics.xor;
  var Atomics_fence=global.Atomics.fence;
  var tempFloat = 0.0;

// EMSCRIPTEN_START_FUNCS
function stackAlloc(size) {
  size = size|0;
  var ret = 0;
  ret = STACKTOP;
  STACKTOP = (STACKTOP + size)|0;
STACKTOP = (STACKTOP + 15)&-16;
if ((STACKTOP|0) >= (STACK_MAX|0)) abort();

  return ret|0;
}
function stackSave() {
  return STACKTOP|0;
}
function stackRestore(top) {
  top = top|0;
  STACKTOP = top;
}
function establishStackSpace(stackBase, stackMax) {
  stackBase = stackBase|0;
  stackMax = stackMax|0;
  STACKTOP = stackBase;
  STACK_MAX = stackMax;
}

function setThrew(threw, value) {
  threw = threw|0;
  value = value|0;
  if ((__THREW__|0) == 0) {
    __THREW__ = threw;
    threwValue = value;
  }
}
function copyTempFloat(ptr) {
  ptr = ptr|0;
  HEAP8[tempDoublePtr>>0] = HEAP8[ptr>>0];
  HEAP8[tempDoublePtr+1>>0] = HEAP8[ptr+1>>0];
  HEAP8[tempDoublePtr+2>>0] = HEAP8[ptr+2>>0];
  HEAP8[tempDoublePtr+3>>0] = HEAP8[ptr+3>>0];
}
function copyTempDouble(ptr) {
  ptr = ptr|0;
  HEAP8[tempDoublePtr>>0] = HEAP8[ptr>>0];
  HEAP8[tempDoublePtr+1>>0] = HEAP8[ptr+1>>0];
  HEAP8[tempDoublePtr+2>>0] = HEAP8[ptr+2>>0];
  HEAP8[tempDoublePtr+3>>0] = HEAP8[ptr+3>>0];
  HEAP8[tempDoublePtr+4>>0] = HEAP8[ptr+4>>0];
  HEAP8[tempDoublePtr+5>>0] = HEAP8[ptr+5>>0];
  HEAP8[tempDoublePtr+6>>0] = HEAP8[ptr+6>>0];
  HEAP8[tempDoublePtr+7>>0] = HEAP8[ptr+7>>0];
}
function setTempRet0(value) {
  value = value|0;
  tempRet0 = value;
}
function getTempRet0() {
  return tempRet0|0;
}

function __Z5sleepi($msecs) {
 $msecs = $msecs|0;
 var $0 = 0, $1 = 0.0, $2 = 0.0, $3 = 0, $4 = 0.0, $5 = 0.0, $6 = 0.0, $7 = 0.0, $8 = 0, $t0 = 0.0, $t1 = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $msecs;
 $1 = (+_emscripten_get_now());
 $t0 = $1;
 $2 = $t0;
 $3 = $0;
 $4 = (+($3|0));
 $5 = $2 + $4;
 $t1 = $5;
// dump('about to while1\n');
 while(1) {
  $6 = (+_emscripten_get_now());
  $7 = $t1;
  $8 = $6 < $7;
  if (!($8)) {
   break;
  }
 }
// dump('done while1\n');
 STACKTOP = sp;return;
}
function __Z10ThreadMainPv($arg) {
 $arg = $arg|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $c = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $1 = $arg;
 dump('Thread ' + _pthread_self() + ' attempting to lock mutex\n');
 (_pthread_mutex_lock(32)|0);
 dump('Thread ' + _pthread_self() + ' got lock\n');
 $2 = Atomics_load(HEAP32,24>>2);
 $c = $2;
 dump('Thread ' + _pthread_self() + ' going to sleep\n');
 __Z5sleepi(100);
 dump('Thread ' + _pthread_self() + ' back from sleep\n');
 $3 = $c;
 $4 = (($3) + 1)|0;
 $c = $4;
 $5 = $c;
 Atomics_store(HEAP32,24>>2,$5);
 dump('Thread ' + _pthread_self() + ' unlocking mutex\n');
 (_pthread_mutex_unlock(32)|0);
 dump('Thread ' + _pthread_self() + ' exiting\n');
 _pthread_exit((0|0));
 // unreachable;
 return (0)|0;
}
function __Z12CreateThreadii($i,$n) {
 $i = $i|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $attr = 0, $rc = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer = sp;
 $attr = sp + 8|0;
 $0 = $i;
 $1 = $n;
 (_pthread_attr_init($attr)|0);
 (_pthread_attr_setdetachstate($attr,0)|0);
 (_pthread_attr_setstacksize($attr,4096)|0);
 $2 = $0;
 $3 = (16 + ($2<<2)|0);
 $4 = (_pthread_create(($3|0),($attr|0),(1|0),(0|0))|0);
 $rc = $4;
 $5 = $rc;
 $6 = ($5|0)!=(0);
 if (!($6)) {
  $7 = $0;
  $8 = (16 + ($7<<2)|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($9|0)==(0);
  if (!($10)) {
   STACKTOP = sp;return;
  }
 }
 (_printf((64|0),($vararg_buffer|0))|0);
 STACKTOP = sp;return;
}
function __Z10WaitToJoinv() {
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $rc = 0, $status = 0, $threadsRunning = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $status = sp + 4|0;
 $threadsRunning = 0;
 $i = 0;
 while(1) {
  $0 = $i;
  $1 = ($0|0)<(2);
  if (!($1)) {
   break;
  }
  $2 = $i;
  $3 = (16 + ($2<<2)|0);
  $4 = HEAP32[$3>>2]|0;
  $5 = ($4|0)!=(0);
  if ($5) {
   $6 = $i;
   $7 = (16 + ($6<<2)|0);
   $8 = HEAP32[$7>>2]|0;
   $9 = (_pthread_join(($8|0),($status|0))|0);
   $rc = $9;
   $10 = $rc;
   $11 = ($10|0)==(0);
   if ($11) {
    $12 = $i;
    $13 = (16 + ($12<<2)|0);
    HEAP32[$13>>2] = 0;
    $14 = HEAP32[96>>2]|0;
    $15 = HEAP32[8>>2]|0;
    $16 = ($14|0)<($15|0);
    if ($16) {
     $17 = $i;
     $18 = HEAP32[96>>2]|0;
     $19 = (($18) + 1)|0;
     HEAP32[96>>2] = $19;
     __Z12CreateThreadii($17,$18);
     $20 = $threadsRunning;
     $21 = (($20) + 1)|0;
     $threadsRunning = $21;
    }
   } else {
    $22 = $threadsRunning;
    $23 = (($22) + 1)|0;
    $threadsRunning = $23;
   }
  }
  $24 = $i;
  $25 = (($24) + 1)|0;
  $i = $25;
 }
 $26 = $threadsRunning;
 $27 = ($26|0)!=(0);
 if ($27) {
  STACKTOP = sp;return;
 }
 $28 = Atomics_load(HEAP32,24>>2);
 $29 = HEAP32[8>>2]|0;
 $30 = ($28|0)==($29|0);
 if ($30) {
  $31 = Atomics_load(HEAP32,24>>2);
  (_emscripten_asm_const_int((104|0),($31|0))|0);
 } else {
  $32 = Atomics_load(HEAP32,24>>2);
  $33 = HEAP32[8>>2]|0;
  (_emscripten_asm_const_int((184|0),($32|0),($33|0))|0);
 }
 _emscripten_cancel_main_loop();
 STACKTOP = sp;return;
}
function _main() {
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = 0;
 (_pthread_mutex_init(32,0)|0);
 $i = 0;
 while(1) {
  $1 = $i;
  $2 = ($1|0)<(2);
  if (!($2)) {
   break;
  }
  $3 = $i;
  $4 = HEAP32[96>>2]|0;
  $5 = (($4) + 1)|0;
  HEAP32[96>>2] = $5;
  __Z12CreateThreadii($3,$4);
  $6 = $i;
  $7 = (($6) + 1)|0;
  $i = $7;
 }
 _emscripten_set_main_loop((2|0),0,0);
 $8 = $0;
 STACKTOP = sp;return ($8|0);
}
function _malloc($bytes) {
 $bytes = $bytes|0;
 var $$0 = 0, $$lcssa = 0, $$lcssa110 = 0, $$lcssa112 = 0, $$lcssa115 = 0, $$lcssa116 = 0, $$lcssa117 = 0, $$lcssa118 = 0, $$lcssa120 = 0, $$lcssa123 = 0, $$lcssa125 = 0, $$lcssa127 = 0, $$lcssa130 = 0, $$lcssa132 = 0, $$lcssa134 = 0, $$lcssa137 = 0, $$pre = 0, $$pre$i = 0, $$pre$i$i = 0, $$pre$i23$i = 0;
 var $$pre$i25 = 0, $$pre$phi$i$iZ2D = 0, $$pre$phi$i24$iZ2D = 0, $$pre$phi$i26Z2D = 0, $$pre$phi$iZ2D = 0, $$pre$phi59$i$iZ2D = 0, $$pre$phiZ2D = 0, $$pre105 = 0, $$pre58$i$i = 0, $$rsize$0$i = 0, $$rsize$3$i = 0, $$rsize$3$i$lcssa = 0, $$sum = 0, $$sum$i$i = 0, $$sum$i$i$i = 0, $$sum$i12$i = 0, $$sum$i14$i = 0, $$sum$i16$i = 0, $$sum$i19$i = 0, $$sum$i2338 = 0;
 var $$sum$i32 = 0, $$sum$i39 = 0, $$sum1 = 0, $$sum1$i = 0, $$sum1$i$i = 0, $$sum1$i13$i = 0, $$sum1$i20$i = 0, $$sum1$i24 = 0, $$sum10 = 0, $$sum10$i = 0, $$sum10$i$i = 0, $$sum10$pre$i$i = 0, $$sum100$i = 0, $$sum101$i = 0, $$sum102$i = 0, $$sum103$i = 0, $$sum104$i = 0, $$sum105$i = 0, $$sum106$i = 0, $$sum107$i = 0;
 var $$sum108$i = 0, $$sum109$i = 0, $$sum11$i = 0, $$sum11$i$i = 0, $$sum11$i22$i = 0, $$sum1112 = 0, $$sum12$i = 0, $$sum12$i$i = 0, $$sum13$i = 0, $$sum13$i$i = 0, $$sum14$i$i = 0, $$sum14$pre$i = 0, $$sum15$i = 0, $$sum15$i$i = 0, $$sum16$i = 0, $$sum16$i$i = 0, $$sum17$i = 0, $$sum17$i$i = 0, $$sum18$i = 0, $$sum1819$i$i = 0;
 var $$sum2 = 0, $$sum2$i = 0, $$sum2$i$i = 0, $$sum2$i$i$i = 0, $$sum2$i15$i = 0, $$sum2$i17$i = 0, $$sum2$i21$i = 0, $$sum2$pre$i = 0, $$sum20$i$i = 0, $$sum21$i$i = 0, $$sum22$i$i = 0, $$sum23$i$i = 0, $$sum24$i$i = 0, $$sum25$i$i = 0, $$sum26$pre$i$i = 0, $$sum27$i$i = 0, $$sum28$i$i = 0, $$sum29$i$i = 0, $$sum3$i = 0, $$sum3$i$i = 0;
 var $$sum3$i27 = 0, $$sum30$i$i = 0, $$sum3132$i$i = 0, $$sum34$i$i = 0, $$sum3536$i$i = 0, $$sum3738$i$i = 0, $$sum39$i$i = 0, $$sum4 = 0, $$sum4$i = 0, $$sum4$i28 = 0, $$sum40$i$i = 0, $$sum41$i$i = 0, $$sum42$i$i = 0, $$sum5$i = 0, $$sum5$i$i = 0, $$sum56 = 0, $$sum6$i = 0, $$sum67$i$i = 0, $$sum7$i = 0, $$sum8$i = 0;
 var $$sum8$pre = 0, $$sum9 = 0, $$sum9$i = 0, $$sum9$i$i = 0, $$sum96$i = 0, $$sum97$i = 0, $$sum98$i = 0, $$sum99$i = 0, $$tsize$2$i = 0, $$v$0$i = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $1000 = 0, $1001 = 0, $1002 = 0, $1003 = 0, $1004 = 0, $1005 = 0;
 var $1006 = 0, $1007 = 0, $1008 = 0, $1009 = 0, $101 = 0, $1010 = 0, $1011 = 0, $1012 = 0, $1013 = 0, $1014 = 0, $1015 = 0, $1016 = 0, $1017 = 0, $1018 = 0, $1019 = 0, $102 = 0, $1020 = 0, $1021 = 0, $1022 = 0, $1023 = 0;
 var $1024 = 0, $1025 = 0, $1026 = 0, $1027 = 0, $1028 = 0, $1029 = 0, $103 = 0, $1030 = 0, $1031 = 0, $1032 = 0, $1033 = 0, $1034 = 0, $1035 = 0, $1036 = 0, $1037 = 0, $1038 = 0, $1039 = 0, $104 = 0, $1040 = 0, $1041 = 0;
 var $1042 = 0, $1043 = 0, $1044 = 0, $1045 = 0, $1046 = 0, $1047 = 0, $1048 = 0, $1049 = 0, $105 = 0, $1050 = 0, $1051 = 0, $1052 = 0, $1053 = 0, $1054 = 0, $1055 = 0, $1056 = 0, $1057 = 0, $1058 = 0, $1059 = 0, $106 = 0;
 var $1060 = 0, $1061 = 0, $1062 = 0, $1063 = 0, $1064 = 0, $1065 = 0, $1066 = 0, $1067 = 0, $1068 = 0, $1069 = 0, $107 = 0, $1070 = 0, $1071 = 0, $1072 = 0, $1073 = 0, $1074 = 0, $1075 = 0, $1076 = 0, $1077 = 0, $1078 = 0;
 var $1079 = 0, $108 = 0, $1080 = 0, $1081 = 0, $1082 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0;
 var $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0;
 var $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0;
 var $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0;
 var $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0;
 var $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0;
 var $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0;
 var $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0;
 var $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0;
 var $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0;
 var $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0;
 var $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0;
 var $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0;
 var $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0;
 var $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0;
 var $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0;
 var $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0;
 var $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0;
 var $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0;
 var $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0;
 var $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0;
 var $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0;
 var $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0, $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0;
 var $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0, $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0;
 var $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0, $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0;
 var $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0, $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0;
 var $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0, $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0;
 var $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0, $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0;
 var $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0, $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0;
 var $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0, $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0;
 var $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0, $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0;
 var $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0, $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0;
 var $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0, $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0;
 var $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0, $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0;
 var $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0, $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0;
 var $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0, $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0;
 var $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0, $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0;
 var $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0, $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0;
 var $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0, $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0;
 var $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0, $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0;
 var $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0, $836 = 0, $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0;
 var $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0, $853 = 0, $854 = 0, $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0;
 var $861 = 0, $862 = 0, $863 = 0, $864 = 0, $865 = 0, $866 = 0, $867 = 0, $868 = 0, $869 = 0, $87 = 0, $870 = 0, $871 = 0, $872 = 0, $873 = 0, $874 = 0, $875 = 0, $876 = 0, $877 = 0, $878 = 0, $879 = 0;
 var $88 = 0, $880 = 0, $881 = 0, $882 = 0, $883 = 0, $884 = 0, $885 = 0, $886 = 0, $887 = 0, $888 = 0, $889 = 0, $89 = 0, $890 = 0, $891 = 0, $892 = 0, $893 = 0, $894 = 0, $895 = 0, $896 = 0, $897 = 0;
 var $898 = 0, $899 = 0, $9 = 0, $90 = 0, $900 = 0, $901 = 0, $902 = 0, $903 = 0, $904 = 0, $905 = 0, $906 = 0, $907 = 0, $908 = 0, $909 = 0, $91 = 0, $910 = 0, $911 = 0, $912 = 0, $913 = 0, $914 = 0;
 var $915 = 0, $916 = 0, $917 = 0, $918 = 0, $919 = 0, $92 = 0, $920 = 0, $921 = 0, $922 = 0, $923 = 0, $924 = 0, $925 = 0, $926 = 0, $927 = 0, $928 = 0, $929 = 0, $93 = 0, $930 = 0, $931 = 0, $932 = 0;
 var $933 = 0, $934 = 0, $935 = 0, $936 = 0, $937 = 0, $938 = 0, $939 = 0, $94 = 0, $940 = 0, $941 = 0, $942 = 0, $943 = 0, $944 = 0, $945 = 0, $946 = 0, $947 = 0, $948 = 0, $949 = 0, $95 = 0, $950 = 0;
 var $951 = 0, $952 = 0, $953 = 0, $954 = 0, $955 = 0, $956 = 0, $957 = 0, $958 = 0, $959 = 0, $96 = 0, $960 = 0, $961 = 0, $962 = 0, $963 = 0, $964 = 0, $965 = 0, $966 = 0, $967 = 0, $968 = 0, $969 = 0;
 var $97 = 0, $970 = 0, $971 = 0, $972 = 0, $973 = 0, $974 = 0, $975 = 0, $976 = 0, $977 = 0, $978 = 0, $979 = 0, $98 = 0, $980 = 0, $981 = 0, $982 = 0, $983 = 0, $984 = 0, $985 = 0, $986 = 0, $987 = 0;
 var $988 = 0, $989 = 0, $99 = 0, $990 = 0, $991 = 0, $992 = 0, $993 = 0, $994 = 0, $995 = 0, $996 = 0, $997 = 0, $998 = 0, $999 = 0, $F$0$i$i = 0, $F1$0$i = 0, $F4$0 = 0, $F4$0$i$i = 0, $F5$0$i = 0, $I1$0$c$i$i = 0, $I1$0$i$i = 0;
 var $I7$0$i = 0, $I7$0$i$i = 0, $K12$029$i = 0, $K2$015$i$i = 0, $K8$053$i$i = 0, $R$0$i = 0, $R$0$i$be = 0, $R$0$i$i = 0, $R$0$i$i$be = 0, $R$0$i$i$lcssa = 0, $R$0$i$i$ph = 0, $R$0$i$lcssa = 0, $R$0$i$ph = 0, $R$0$i18 = 0, $R$0$i18$be = 0, $R$0$i18$lcssa = 0, $R$0$i18$ph = 0, $R$1$i = 0, $R$1$i$i = 0, $R$1$i20 = 0;
 var $RP$0$i = 0, $RP$0$i$be = 0, $RP$0$i$i = 0, $RP$0$i$i$be = 0, $RP$0$i$i$lcssa = 0, $RP$0$i$i$ph = 0, $RP$0$i$lcssa = 0, $RP$0$i$ph = 0, $RP$0$i17 = 0, $RP$0$i17$be = 0, $RP$0$i17$lcssa = 0, $RP$0$i17$ph = 0, $T$0$lcssa$i = 0, $T$0$lcssa$i$i = 0, $T$0$lcssa$i26$i = 0, $T$014$i$i = 0, $T$014$i$i$lcssa = 0, $T$028$i = 0, $T$028$i$lcssa = 0, $T$052$i$i = 0;
 var $T$052$i$i$lcssa = 0, $br$0$i = 0, $br$030$i = 0, $cond$i = 0, $cond$i$i = 0, $cond$i21 = 0, $exitcond$i$i = 0, $i$02$i$i = 0, $idx$0$i = 0, $mem$0 = 0, $nb$0 = 0, $oldfirst$0$i$i = 0, $or$cond$i = 0, $or$cond$i$i = 0, $or$cond$i27$i = 0, $or$cond$i29 = 0, $or$cond1$i = 0, $or$cond19$i = 0, $or$cond2$i = 0, $or$cond24$i = 0;
 var $or$cond3$i = 0, $or$cond4$i = 0, $or$cond41$i = 0, $or$cond5$i = 0, $or$cond6$i = 0, $or$cond7$i = 0, $or$cond8$i = 0, $qsize$0$i$i = 0, $rsize$0$i = 0, $rsize$0$i$lcssa = 0, $rsize$0$i15 = 0, $rsize$1$i = 0, $rsize$2$i = 0, $rsize$2$i$ph = 0, $rsize$3$lcssa$i = 0, $rsize$331$i = 0, $rst$0$i = 0, $rst$1$i = 0, $sizebits$0$i = 0, $sp$0$i$i = 0;
 var $sp$0$i$i$i = 0, $sp$0$i$i$lcssa = 0, $sp$068$i = 0, $sp$068$i$lcssa = 0, $sp$167$i = 0, $sp$167$i$lcssa = 0, $ssize$0$i = 0, $ssize$1$i = 0, $ssize$129$i = 0, $ssize$2$i = 0, $t$0$i = 0, $t$0$i14 = 0, $t$1$i = 0, $t$1$i$ph = 0, $t$2$ph$i = 0, $t$2$v$3$i = 0, $t$2$v$3$i$lcssa = 0, $t$230$i = 0, $t$230$i$be = 0, $tbase$1$i = 0;
 var $tbase$2$i = 0, $tbase$3$i = 0, $tsize$03138$i = 0, $tsize$1$i = 0, $tsize$2$i = 0, $tsize$3$i = 0, $v$0$i = 0, $v$0$i$lcssa = 0, $v$0$i16 = 0, $v$1$i = 0, $v$2$i = 0, $v$2$i$ph = 0, $v$3$lcssa$i = 0, $v$332$i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = HEAP32[272>>2]|0;
 $1 = ($0|0)==(0);
 if ($1) {
  _init_mparams();
 }
 $2 = HEAP32[((296 + 444|0))>>2]|0;
 $3 = $2 & 2;
 $4 = ($3|0)==(0);
 if (!($4)) {
  $5 = (_pthread_mutex_lock(((296 + 448|0)))|0);
  $6 = ($5|0)==(0);
  if (!($6)) {
   $$0 = 0;
   STACKTOP = sp;return ($$0|0);
  }
 }
 $7 = ($bytes>>>0)<(245);
 do {
  if ($7) {
   $8 = ($bytes>>>0)<(11);
   if ($8) {
    $12 = 16;
   } else {
    $9 = (($bytes) + 11)|0;
    $10 = $9 & -8;
    $12 = $10;
   }
   $11 = $12 >>> 3;
   $13 = HEAP32[296>>2]|0;
   $14 = $13 >>> $11;
   $15 = $14 & 3;
   $16 = ($15|0)==(0);
   if (!($16)) {
    $17 = $14 & 1;
    $18 = $17 ^ 1;
    $19 = (($18) + ($11))|0;
    $20 = $19 << 1;
    $21 = ((296 + ($20<<2)|0) + 40|0);
    $$sum10 = (($20) + 2)|0;
    $22 = ((296 + ($$sum10<<2)|0) + 40|0);
    $23 = HEAP32[$22>>2]|0;
    $24 = (($23) + 8|0);
    $25 = HEAP32[$24>>2]|0;
    $26 = ($21|0)==($25|0);
    do {
     if ($26) {
      $27 = 1 << $19;
      $28 = $27 ^ -1;
      $29 = $13 & $28;
      HEAP32[296>>2] = $29;
     } else {
      $30 = HEAP32[((296 + 16|0))>>2]|0;
      $31 = ($25>>>0)<($30>>>0);
      if ($31) {
       _abort();
       // unreachable;
      }
      $32 = (($25) + 12|0);
      $33 = HEAP32[$32>>2]|0;
      $34 = ($33|0)==($23|0);
      if ($34) {
       HEAP32[$32>>2] = $21;
       HEAP32[$22>>2] = $25;
       break;
      } else {
       _abort();
       // unreachable;
      }
     }
    } while(0);
    $35 = $19 << 3;
    $36 = $35 | 3;
    $37 = (($23) + 4|0);
    HEAP32[$37>>2] = $36;
    $$sum1112 = $35 | 4;
    $38 = (($23) + ($$sum1112)|0);
    $39 = HEAP32[$38>>2]|0;
    $40 = $39 | 1;
    HEAP32[$38>>2] = $40;
    $mem$0 = $24;
    break;
   }
   $41 = HEAP32[((296 + 8|0))>>2]|0;
   $42 = ($12>>>0)>($41>>>0);
   if ($42) {
    $43 = ($14|0)==(0);
    if (!($43)) {
     $44 = $14 << $11;
     $45 = 2 << $11;
     $46 = (0 - ($45))|0;
     $47 = $45 | $46;
     $48 = $44 & $47;
     $49 = (0 - ($48))|0;
     $50 = $48 & $49;
     $51 = (($50) + -1)|0;
     $52 = $51 >>> 12;
     $53 = $52 & 16;
     $54 = $51 >>> $53;
     $55 = $54 >>> 5;
     $56 = $55 & 8;
     $57 = $56 | $53;
     $58 = $54 >>> $56;
     $59 = $58 >>> 2;
     $60 = $59 & 4;
     $61 = $57 | $60;
     $62 = $58 >>> $60;
     $63 = $62 >>> 1;
     $64 = $63 & 2;
     $65 = $61 | $64;
     $66 = $62 >>> $64;
     $67 = $66 >>> 1;
     $68 = $67 & 1;
     $69 = $65 | $68;
     $70 = $66 >>> $68;
     $71 = (($69) + ($70))|0;
     $72 = $71 << 1;
     $73 = ((296 + ($72<<2)|0) + 40|0);
     $$sum4 = (($72) + 2)|0;
     $74 = ((296 + ($$sum4<<2)|0) + 40|0);
     $75 = HEAP32[$74>>2]|0;
     $76 = (($75) + 8|0);
     $77 = HEAP32[$76>>2]|0;
     $78 = ($73|0)==($77|0);
     do {
      if ($78) {
       $79 = 1 << $71;
       $80 = $79 ^ -1;
       $81 = $13 & $80;
       HEAP32[296>>2] = $81;
       $96 = $41;
      } else {
       $82 = HEAP32[((296 + 16|0))>>2]|0;
       $83 = ($77>>>0)<($82>>>0);
       if ($83) {
        _abort();
        // unreachable;
       }
       $84 = (($77) + 12|0);
       $85 = HEAP32[$84>>2]|0;
       $86 = ($85|0)==($75|0);
       if ($86) {
        HEAP32[$84>>2] = $73;
        HEAP32[$74>>2] = $77;
        $$pre = HEAP32[((296 + 8|0))>>2]|0;
        $96 = $$pre;
        break;
       } else {
        _abort();
        // unreachable;
       }
      }
     } while(0);
     $87 = $71 << 3;
     $88 = (($87) - ($12))|0;
     $89 = $12 | 3;
     $90 = (($75) + 4|0);
     HEAP32[$90>>2] = $89;
     $91 = (($75) + ($12)|0);
     $92 = $88 | 1;
     $$sum56 = $12 | 4;
     $93 = (($75) + ($$sum56)|0);
     HEAP32[$93>>2] = $92;
     $94 = (($75) + ($87)|0);
     HEAP32[$94>>2] = $88;
     $95 = ($96|0)==(0);
     if (!($95)) {
      $97 = HEAP32[((296 + 20|0))>>2]|0;
      $98 = $96 >>> 3;
      $99 = $98 << 1;
      $100 = ((296 + ($99<<2)|0) + 40|0);
      $101 = HEAP32[296>>2]|0;
      $102 = 1 << $98;
      $103 = $101 & $102;
      $104 = ($103|0)==(0);
      if ($104) {
       $105 = $101 | $102;
       HEAP32[296>>2] = $105;
       $$sum8$pre = (($99) + 2)|0;
       $$pre105 = ((296 + ($$sum8$pre<<2)|0) + 40|0);
       $$pre$phiZ2D = $$pre105;$F4$0 = $100;
      } else {
       $$sum9 = (($99) + 2)|0;
       $106 = ((296 + ($$sum9<<2)|0) + 40|0);
       $107 = HEAP32[$106>>2]|0;
       $108 = HEAP32[((296 + 16|0))>>2]|0;
       $109 = ($107>>>0)<($108>>>0);
       if ($109) {
        _abort();
        // unreachable;
       } else {
        $$pre$phiZ2D = $106;$F4$0 = $107;
       }
      }
      HEAP32[$$pre$phiZ2D>>2] = $97;
      $110 = (($F4$0) + 12|0);
      HEAP32[$110>>2] = $97;
      $111 = (($97) + 8|0);
      HEAP32[$111>>2] = $F4$0;
      $112 = (($97) + 12|0);
      HEAP32[$112>>2] = $100;
     }
     HEAP32[((296 + 8|0))>>2] = $88;
     HEAP32[((296 + 20|0))>>2] = $91;
     $mem$0 = $76;
     break;
    }
    $113 = HEAP32[((296 + 4|0))>>2]|0;
    $114 = ($113|0)==(0);
    if ($114) {
     $nb$0 = $12;
     label = 173;
    } else {
     $115 = (0 - ($113))|0;
     $116 = $113 & $115;
     $117 = (($116) + -1)|0;
     $118 = $117 >>> 12;
     $119 = $118 & 16;
     $120 = $117 >>> $119;
     $121 = $120 >>> 5;
     $122 = $121 & 8;
     $123 = $122 | $119;
     $124 = $120 >>> $122;
     $125 = $124 >>> 2;
     $126 = $125 & 4;
     $127 = $123 | $126;
     $128 = $124 >>> $126;
     $129 = $128 >>> 1;
     $130 = $129 & 2;
     $131 = $127 | $130;
     $132 = $128 >>> $130;
     $133 = $132 >>> 1;
     $134 = $133 & 1;
     $135 = $131 | $134;
     $136 = $132 >>> $134;
     $137 = (($135) + ($136))|0;
     $138 = ((296 + ($137<<2)|0) + 304|0);
     $139 = HEAP32[$138>>2]|0;
     $140 = (($139) + 4|0);
     $141 = HEAP32[$140>>2]|0;
     $142 = $141 & -8;
     $143 = (($142) - ($12))|0;
     $rsize$0$i = $143;$t$0$i = $139;$v$0$i = $139;
     while(1) {
      $144 = (($t$0$i) + 16|0);
      $145 = HEAP32[$144>>2]|0;
      $146 = ($145|0)==(0|0);
      if ($146) {
       $147 = (($t$0$i) + 20|0);
       $148 = HEAP32[$147>>2]|0;
       $149 = ($148|0)==(0|0);
       if ($149) {
        $rsize$0$i$lcssa = $rsize$0$i;$v$0$i$lcssa = $v$0$i;
        break;
       } else {
        $151 = $148;
       }
      } else {
       $151 = $145;
      }
      $150 = (($151) + 4|0);
      $152 = HEAP32[$150>>2]|0;
      $153 = $152 & -8;
      $154 = (($153) - ($12))|0;
      $155 = ($154>>>0)<($rsize$0$i>>>0);
      $$rsize$0$i = $155 ? $154 : $rsize$0$i;
      $$v$0$i = $155 ? $151 : $v$0$i;
      $rsize$0$i = $$rsize$0$i;$t$0$i = $151;$v$0$i = $$v$0$i;
     }
     $156 = HEAP32[((296 + 16|0))>>2]|0;
     $157 = ($v$0$i$lcssa>>>0)<($156>>>0);
     if ($157) {
      _abort();
      // unreachable;
     }
     $158 = (($v$0$i$lcssa) + ($12)|0);
     $159 = ($v$0$i$lcssa>>>0)<($158>>>0);
     if (!($159)) {
      _abort();
      // unreachable;
     }
     $160 = (($v$0$i$lcssa) + 24|0);
     $161 = HEAP32[$160>>2]|0;
     $162 = (($v$0$i$lcssa) + 12|0);
     $163 = HEAP32[$162>>2]|0;
     $164 = ($163|0)==($v$0$i$lcssa|0);
     do {
      if ($164) {
       $174 = (($v$0$i$lcssa) + 20|0);
       $175 = HEAP32[$174>>2]|0;
       $176 = ($175|0)==(0|0);
       if ($176) {
        $177 = (($v$0$i$lcssa) + 16|0);
        $178 = HEAP32[$177>>2]|0;
        $179 = ($178|0)==(0|0);
        if ($179) {
         $R$1$i = 0;
         break;
        } else {
         $R$0$i$ph = $178;$RP$0$i$ph = $177;
        }
       } else {
        $R$0$i$ph = $175;$RP$0$i$ph = $174;
       }
       $R$0$i = $R$0$i$ph;$RP$0$i = $RP$0$i$ph;
       while(1) {
        $180 = (($R$0$i) + 20|0);
        $181 = HEAP32[$180>>2]|0;
        $182 = ($181|0)==(0|0);
        if ($182) {
         $183 = (($R$0$i) + 16|0);
         $184 = HEAP32[$183>>2]|0;
         $185 = ($184|0)==(0|0);
         if ($185) {
          $R$0$i$lcssa = $R$0$i;$RP$0$i$lcssa = $RP$0$i;
          break;
         } else {
          $R$0$i$be = $184;$RP$0$i$be = $183;
         }
        } else {
         $R$0$i$be = $181;$RP$0$i$be = $180;
        }
        $R$0$i = $R$0$i$be;$RP$0$i = $RP$0$i$be;
       }
       $186 = ($RP$0$i$lcssa>>>0)<($156>>>0);
       if ($186) {
        _abort();
        // unreachable;
       } else {
        HEAP32[$RP$0$i$lcssa>>2] = 0;
        $R$1$i = $R$0$i$lcssa;
        break;
       }
      } else {
       $165 = (($v$0$i$lcssa) + 8|0);
       $166 = HEAP32[$165>>2]|0;
       $167 = ($166>>>0)<($156>>>0);
       if ($167) {
        _abort();
        // unreachable;
       }
       $168 = (($166) + 12|0);
       $169 = HEAP32[$168>>2]|0;
       $170 = ($169|0)==($v$0$i$lcssa|0);
       if (!($170)) {
        _abort();
        // unreachable;
       }
       $171 = (($163) + 8|0);
       $172 = HEAP32[$171>>2]|0;
       $173 = ($172|0)==($v$0$i$lcssa|0);
       if ($173) {
        HEAP32[$168>>2] = $163;
        HEAP32[$171>>2] = $166;
        $R$1$i = $163;
        break;
       } else {
        _abort();
        // unreachable;
       }
      }
     } while(0);
     $187 = ($161|0)==(0|0);
     do {
      if (!($187)) {
       $188 = (($v$0$i$lcssa) + 28|0);
       $189 = HEAP32[$188>>2]|0;
       $190 = ((296 + ($189<<2)|0) + 304|0);
       $191 = HEAP32[$190>>2]|0;
       $192 = ($v$0$i$lcssa|0)==($191|0);
       if ($192) {
        HEAP32[$190>>2] = $R$1$i;
        $cond$i = ($R$1$i|0)==(0|0);
        if ($cond$i) {
         $193 = 1 << $189;
         $194 = $193 ^ -1;
         $195 = HEAP32[((296 + 4|0))>>2]|0;
         $196 = $195 & $194;
         HEAP32[((296 + 4|0))>>2] = $196;
         break;
        }
       } else {
        $197 = HEAP32[((296 + 16|0))>>2]|0;
        $198 = ($161>>>0)<($197>>>0);
        if ($198) {
         _abort();
         // unreachable;
        }
        $199 = (($161) + 16|0);
        $200 = HEAP32[$199>>2]|0;
        $201 = ($200|0)==($v$0$i$lcssa|0);
        if ($201) {
         HEAP32[$199>>2] = $R$1$i;
        } else {
         $202 = (($161) + 20|0);
         HEAP32[$202>>2] = $R$1$i;
        }
        $203 = ($R$1$i|0)==(0|0);
        if ($203) {
         break;
        }
       }
       $204 = HEAP32[((296 + 16|0))>>2]|0;
       $205 = ($R$1$i>>>0)<($204>>>0);
       if ($205) {
        _abort();
        // unreachable;
       }
       $206 = (($R$1$i) + 24|0);
       HEAP32[$206>>2] = $161;
       $207 = (($v$0$i$lcssa) + 16|0);
       $208 = HEAP32[$207>>2]|0;
       $209 = ($208|0)==(0|0);
       do {
        if (!($209)) {
         $210 = ($208>>>0)<($204>>>0);
         if ($210) {
          _abort();
          // unreachable;
         } else {
          $211 = (($R$1$i) + 16|0);
          HEAP32[$211>>2] = $208;
          $212 = (($208) + 24|0);
          HEAP32[$212>>2] = $R$1$i;
          break;
         }
        }
       } while(0);
       $213 = (($v$0$i$lcssa) + 20|0);
       $214 = HEAP32[$213>>2]|0;
       $215 = ($214|0)==(0|0);
       if (!($215)) {
        $216 = HEAP32[((296 + 16|0))>>2]|0;
        $217 = ($214>>>0)<($216>>>0);
        if ($217) {
         _abort();
         // unreachable;
        } else {
         $218 = (($R$1$i) + 20|0);
         HEAP32[$218>>2] = $214;
         $219 = (($214) + 24|0);
         HEAP32[$219>>2] = $R$1$i;
         break;
        }
       }
      }
     } while(0);
     $220 = ($rsize$0$i$lcssa>>>0)<(16);
     if ($220) {
      $221 = (($rsize$0$i$lcssa) + ($12))|0;
      $222 = $221 | 3;
      $223 = (($v$0$i$lcssa) + 4|0);
      HEAP32[$223>>2] = $222;
      $$sum4$i = (($221) + 4)|0;
      $224 = (($v$0$i$lcssa) + ($$sum4$i)|0);
      $225 = HEAP32[$224>>2]|0;
      $226 = $225 | 1;
      HEAP32[$224>>2] = $226;
     } else {
      $227 = $12 | 3;
      $228 = (($v$0$i$lcssa) + 4|0);
      HEAP32[$228>>2] = $227;
      $229 = $rsize$0$i$lcssa | 1;
      $$sum$i39 = $12 | 4;
      $230 = (($v$0$i$lcssa) + ($$sum$i39)|0);
      HEAP32[$230>>2] = $229;
      $$sum1$i = (($rsize$0$i$lcssa) + ($12))|0;
      $231 = (($v$0$i$lcssa) + ($$sum1$i)|0);
      HEAP32[$231>>2] = $rsize$0$i$lcssa;
      $232 = HEAP32[((296 + 8|0))>>2]|0;
      $233 = ($232|0)==(0);
      if (!($233)) {
       $234 = HEAP32[((296 + 20|0))>>2]|0;
       $235 = $232 >>> 3;
       $236 = $235 << 1;
       $237 = ((296 + ($236<<2)|0) + 40|0);
       $238 = HEAP32[296>>2]|0;
       $239 = 1 << $235;
       $240 = $238 & $239;
       $241 = ($240|0)==(0);
       if ($241) {
        $242 = $238 | $239;
        HEAP32[296>>2] = $242;
        $$sum2$pre$i = (($236) + 2)|0;
        $$pre$i = ((296 + ($$sum2$pre$i<<2)|0) + 40|0);
        $$pre$phi$iZ2D = $$pre$i;$F1$0$i = $237;
       } else {
        $$sum3$i = (($236) + 2)|0;
        $243 = ((296 + ($$sum3$i<<2)|0) + 40|0);
        $244 = HEAP32[$243>>2]|0;
        $245 = HEAP32[((296 + 16|0))>>2]|0;
        $246 = ($244>>>0)<($245>>>0);
        if ($246) {
         _abort();
         // unreachable;
        } else {
         $$pre$phi$iZ2D = $243;$F1$0$i = $244;
        }
       }
       HEAP32[$$pre$phi$iZ2D>>2] = $234;
       $247 = (($F1$0$i) + 12|0);
       HEAP32[$247>>2] = $234;
       $248 = (($234) + 8|0);
       HEAP32[$248>>2] = $F1$0$i;
       $249 = (($234) + 12|0);
       HEAP32[$249>>2] = $237;
      }
      HEAP32[((296 + 8|0))>>2] = $rsize$0$i$lcssa;
      HEAP32[((296 + 20|0))>>2] = $158;
     }
     $250 = (($v$0$i$lcssa) + 8|0);
     $mem$0 = $250;
    }
   } else {
    $nb$0 = $12;
    label = 173;
   }
  } else {
   $251 = ($bytes>>>0)>(4294967231);
   if ($251) {
    $nb$0 = -1;
    label = 173;
   } else {
    $252 = (($bytes) + 11)|0;
    $253 = $252 & -8;
    $254 = HEAP32[((296 + 4|0))>>2]|0;
    $255 = ($254|0)==(0);
    if ($255) {
     $nb$0 = $253;
     label = 173;
    } else {
     $256 = (0 - ($253))|0;
     $257 = $252 >>> 8;
     $258 = ($257|0)==(0);
     if ($258) {
      $idx$0$i = 0;
     } else {
      $259 = ($253>>>0)>(16777215);
      if ($259) {
       $idx$0$i = 31;
      } else {
       $260 = (($257) + 1048320)|0;
       $261 = $260 >>> 16;
       $262 = $261 & 8;
       $263 = $257 << $262;
       $264 = (($263) + 520192)|0;
       $265 = $264 >>> 16;
       $266 = $265 & 4;
       $267 = $266 | $262;
       $268 = $263 << $266;
       $269 = (($268) + 245760)|0;
       $270 = $269 >>> 16;
       $271 = $270 & 2;
       $272 = $267 | $271;
       $273 = (14 - ($272))|0;
       $274 = $268 << $271;
       $275 = $274 >>> 15;
       $276 = (($273) + ($275))|0;
       $277 = $276 << 1;
       $278 = (($276) + 7)|0;
       $279 = $253 >>> $278;
       $280 = $279 & 1;
       $281 = $280 | $277;
       $idx$0$i = $281;
      }
     }
     $282 = ((296 + ($idx$0$i<<2)|0) + 304|0);
     $283 = HEAP32[$282>>2]|0;
     $284 = ($283|0)==(0|0);
     if ($284) {
      $rsize$2$i = $256;$t$1$i = 0;$v$2$i = 0;
     } else {
      $285 = ($idx$0$i|0)==(31);
      if ($285) {
       $289 = 0;
      } else {
       $286 = $idx$0$i >>> 1;
       $287 = (25 - ($286))|0;
       $289 = $287;
      }
      $288 = $253 << $289;
      $rsize$0$i15 = $256;$rst$0$i = 0;$sizebits$0$i = $288;$t$0$i14 = $283;$v$0$i16 = 0;
      while(1) {
       $290 = (($t$0$i14) + 4|0);
       $291 = HEAP32[$290>>2]|0;
       $292 = $291 & -8;
       $293 = (($292) - ($253))|0;
       $294 = ($293>>>0)<($rsize$0$i15>>>0);
       if ($294) {
        $295 = ($292|0)==($253|0);
        if ($295) {
         $rsize$2$i$ph = $293;$t$1$i$ph = $t$0$i14;$v$2$i$ph = $t$0$i14;
         break;
        } else {
         $rsize$1$i = $293;$v$1$i = $t$0$i14;
        }
       } else {
        $rsize$1$i = $rsize$0$i15;$v$1$i = $v$0$i16;
       }
       $296 = (($t$0$i14) + 20|0);
       $297 = HEAP32[$296>>2]|0;
       $298 = $sizebits$0$i >>> 31;
       $299 = ((($t$0$i14) + ($298<<2)|0) + 16|0);
       $300 = HEAP32[$299>>2]|0;
       $301 = ($297|0)==(0|0);
       $302 = ($297|0)==($300|0);
       $or$cond19$i = $301 | $302;
       $rst$1$i = $or$cond19$i ? $rst$0$i : $297;
       $303 = ($300|0)==(0|0);
       $304 = $sizebits$0$i << 1;
       if ($303) {
        $rsize$2$i$ph = $rsize$1$i;$t$1$i$ph = $rst$1$i;$v$2$i$ph = $v$1$i;
        break;
       } else {
        $rsize$0$i15 = $rsize$1$i;$rst$0$i = $rst$1$i;$sizebits$0$i = $304;$t$0$i14 = $300;$v$0$i16 = $v$1$i;
       }
      }
      $rsize$2$i = $rsize$2$i$ph;$t$1$i = $t$1$i$ph;$v$2$i = $v$2$i$ph;
     }
     $305 = ($t$1$i|0)==(0|0);
     $306 = ($v$2$i|0)==(0|0);
     $or$cond$i = $305 & $306;
     if ($or$cond$i) {
      $307 = 2 << $idx$0$i;
      $308 = (0 - ($307))|0;
      $309 = $307 | $308;
      $310 = $254 & $309;
      $311 = ($310|0)==(0);
      if ($311) {
       $nb$0 = $253;
       label = 173;
       break;
      }
      $312 = (0 - ($310))|0;
      $313 = $310 & $312;
      $314 = (($313) + -1)|0;
      $315 = $314 >>> 12;
      $316 = $315 & 16;
      $317 = $314 >>> $316;
      $318 = $317 >>> 5;
      $319 = $318 & 8;
      $320 = $319 | $316;
      $321 = $317 >>> $319;
      $322 = $321 >>> 2;
      $323 = $322 & 4;
      $324 = $320 | $323;
      $325 = $321 >>> $323;
      $326 = $325 >>> 1;
      $327 = $326 & 2;
      $328 = $324 | $327;
      $329 = $325 >>> $327;
      $330 = $329 >>> 1;
      $331 = $330 & 1;
      $332 = $328 | $331;
      $333 = $329 >>> $331;
      $334 = (($332) + ($333))|0;
      $335 = ((296 + ($334<<2)|0) + 304|0);
      $336 = HEAP32[$335>>2]|0;
      $t$2$ph$i = $336;
     } else {
      $t$2$ph$i = $t$1$i;
     }
     $337 = ($t$2$ph$i|0)==(0|0);
     if ($337) {
      $rsize$3$lcssa$i = $rsize$2$i;$v$3$lcssa$i = $v$2$i;
     } else {
      $rsize$331$i = $rsize$2$i;$t$230$i = $t$2$ph$i;$v$332$i = $v$2$i;
      while(1) {
       $338 = (($t$230$i) + 4|0);
       $339 = HEAP32[$338>>2]|0;
       $340 = $339 & -8;
       $341 = (($340) - ($253))|0;
       $342 = ($341>>>0)<($rsize$331$i>>>0);
       $$rsize$3$i = $342 ? $341 : $rsize$331$i;
       $t$2$v$3$i = $342 ? $t$230$i : $v$332$i;
       $343 = (($t$230$i) + 16|0);
       $344 = HEAP32[$343>>2]|0;
       $345 = ($344|0)==(0|0);
       if ($345) {
        $346 = (($t$230$i) + 20|0);
        $347 = HEAP32[$346>>2]|0;
        $348 = ($347|0)==(0|0);
        if ($348) {
         $$rsize$3$i$lcssa = $$rsize$3$i;$t$2$v$3$i$lcssa = $t$2$v$3$i;
         break;
        } else {
         $t$230$i$be = $347;
        }
       } else {
        $t$230$i$be = $344;
       }
       $rsize$331$i = $$rsize$3$i;$t$230$i = $t$230$i$be;$v$332$i = $t$2$v$3$i;
      }
      $rsize$3$lcssa$i = $$rsize$3$i$lcssa;$v$3$lcssa$i = $t$2$v$3$i$lcssa;
     }
     $349 = ($v$3$lcssa$i|0)==(0|0);
     if ($349) {
      $nb$0 = $253;
      label = 173;
     } else {
      $350 = HEAP32[((296 + 8|0))>>2]|0;
      $351 = (($350) - ($253))|0;
      $352 = ($rsize$3$lcssa$i>>>0)<($351>>>0);
      if ($352) {
       $353 = HEAP32[((296 + 16|0))>>2]|0;
       $354 = ($v$3$lcssa$i>>>0)<($353>>>0);
       if ($354) {
        _abort();
        // unreachable;
       }
       $355 = (($v$3$lcssa$i) + ($253)|0);
       $356 = ($v$3$lcssa$i>>>0)<($355>>>0);
       if (!($356)) {
        _abort();
        // unreachable;
       }
       $357 = (($v$3$lcssa$i) + 24|0);
       $358 = HEAP32[$357>>2]|0;
       $359 = (($v$3$lcssa$i) + 12|0);
       $360 = HEAP32[$359>>2]|0;
       $361 = ($360|0)==($v$3$lcssa$i|0);
       do {
        if ($361) {
         $371 = (($v$3$lcssa$i) + 20|0);
         $372 = HEAP32[$371>>2]|0;
         $373 = ($372|0)==(0|0);
         if ($373) {
          $374 = (($v$3$lcssa$i) + 16|0);
          $375 = HEAP32[$374>>2]|0;
          $376 = ($375|0)==(0|0);
          if ($376) {
           $R$1$i20 = 0;
           break;
          } else {
           $R$0$i18$ph = $375;$RP$0$i17$ph = $374;
          }
         } else {
          $R$0$i18$ph = $372;$RP$0$i17$ph = $371;
         }
         $R$0$i18 = $R$0$i18$ph;$RP$0$i17 = $RP$0$i17$ph;
         while(1) {
          $377 = (($R$0$i18) + 20|0);
          $378 = HEAP32[$377>>2]|0;
          $379 = ($378|0)==(0|0);
          if ($379) {
           $380 = (($R$0$i18) + 16|0);
           $381 = HEAP32[$380>>2]|0;
           $382 = ($381|0)==(0|0);
           if ($382) {
            $R$0$i18$lcssa = $R$0$i18;$RP$0$i17$lcssa = $RP$0$i17;
            break;
           } else {
            $R$0$i18$be = $381;$RP$0$i17$be = $380;
           }
          } else {
           $R$0$i18$be = $378;$RP$0$i17$be = $377;
          }
          $R$0$i18 = $R$0$i18$be;$RP$0$i17 = $RP$0$i17$be;
         }
         $383 = ($RP$0$i17$lcssa>>>0)<($353>>>0);
         if ($383) {
          _abort();
          // unreachable;
         } else {
          HEAP32[$RP$0$i17$lcssa>>2] = 0;
          $R$1$i20 = $R$0$i18$lcssa;
          break;
         }
        } else {
         $362 = (($v$3$lcssa$i) + 8|0);
         $363 = HEAP32[$362>>2]|0;
         $364 = ($363>>>0)<($353>>>0);
         if ($364) {
          _abort();
          // unreachable;
         }
         $365 = (($363) + 12|0);
         $366 = HEAP32[$365>>2]|0;
         $367 = ($366|0)==($v$3$lcssa$i|0);
         if (!($367)) {
          _abort();
          // unreachable;
         }
         $368 = (($360) + 8|0);
         $369 = HEAP32[$368>>2]|0;
         $370 = ($369|0)==($v$3$lcssa$i|0);
         if ($370) {
          HEAP32[$365>>2] = $360;
          HEAP32[$368>>2] = $363;
          $R$1$i20 = $360;
          break;
         } else {
          _abort();
          // unreachable;
         }
        }
       } while(0);
       $384 = ($358|0)==(0|0);
       do {
        if (!($384)) {
         $385 = (($v$3$lcssa$i) + 28|0);
         $386 = HEAP32[$385>>2]|0;
         $387 = ((296 + ($386<<2)|0) + 304|0);
         $388 = HEAP32[$387>>2]|0;
         $389 = ($v$3$lcssa$i|0)==($388|0);
         if ($389) {
          HEAP32[$387>>2] = $R$1$i20;
          $cond$i21 = ($R$1$i20|0)==(0|0);
          if ($cond$i21) {
           $390 = 1 << $386;
           $391 = $390 ^ -1;
           $392 = HEAP32[((296 + 4|0))>>2]|0;
           $393 = $392 & $391;
           HEAP32[((296 + 4|0))>>2] = $393;
           break;
          }
         } else {
          $394 = HEAP32[((296 + 16|0))>>2]|0;
          $395 = ($358>>>0)<($394>>>0);
          if ($395) {
           _abort();
           // unreachable;
          }
          $396 = (($358) + 16|0);
          $397 = HEAP32[$396>>2]|0;
          $398 = ($397|0)==($v$3$lcssa$i|0);
          if ($398) {
           HEAP32[$396>>2] = $R$1$i20;
          } else {
           $399 = (($358) + 20|0);
           HEAP32[$399>>2] = $R$1$i20;
          }
          $400 = ($R$1$i20|0)==(0|0);
          if ($400) {
           break;
          }
         }
         $401 = HEAP32[((296 + 16|0))>>2]|0;
         $402 = ($R$1$i20>>>0)<($401>>>0);
         if ($402) {
          _abort();
          // unreachable;
         }
         $403 = (($R$1$i20) + 24|0);
         HEAP32[$403>>2] = $358;
         $404 = (($v$3$lcssa$i) + 16|0);
         $405 = HEAP32[$404>>2]|0;
         $406 = ($405|0)==(0|0);
         do {
          if (!($406)) {
           $407 = ($405>>>0)<($401>>>0);
           if ($407) {
            _abort();
            // unreachable;
           } else {
            $408 = (($R$1$i20) + 16|0);
            HEAP32[$408>>2] = $405;
            $409 = (($405) + 24|0);
            HEAP32[$409>>2] = $R$1$i20;
            break;
           }
          }
         } while(0);
         $410 = (($v$3$lcssa$i) + 20|0);
         $411 = HEAP32[$410>>2]|0;
         $412 = ($411|0)==(0|0);
         if ($412) {
          break;
         }
         $413 = HEAP32[((296 + 16|0))>>2]|0;
         $414 = ($411>>>0)<($413>>>0);
         if ($414) {
          _abort();
          // unreachable;
         } else {
          $415 = (($R$1$i20) + 20|0);
          HEAP32[$415>>2] = $411;
          $416 = (($411) + 24|0);
          HEAP32[$416>>2] = $R$1$i20;
          break;
         }
        }
       } while(0);
       $417 = ($rsize$3$lcssa$i>>>0)<(16);
       L219: do {
        if ($417) {
         $418 = (($rsize$3$lcssa$i) + ($253))|0;
         $419 = $418 | 3;
         $420 = (($v$3$lcssa$i) + 4|0);
         HEAP32[$420>>2] = $419;
         $$sum18$i = (($418) + 4)|0;
         $421 = (($v$3$lcssa$i) + ($$sum18$i)|0);
         $422 = HEAP32[$421>>2]|0;
         $423 = $422 | 1;
         HEAP32[$421>>2] = $423;
        } else {
         $424 = $253 | 3;
         $425 = (($v$3$lcssa$i) + 4|0);
         HEAP32[$425>>2] = $424;
         $426 = $rsize$3$lcssa$i | 1;
         $$sum$i2338 = $253 | 4;
         $427 = (($v$3$lcssa$i) + ($$sum$i2338)|0);
         HEAP32[$427>>2] = $426;
         $$sum1$i24 = (($rsize$3$lcssa$i) + ($253))|0;
         $428 = (($v$3$lcssa$i) + ($$sum1$i24)|0);
         HEAP32[$428>>2] = $rsize$3$lcssa$i;
         $429 = $rsize$3$lcssa$i >>> 3;
         $430 = ($rsize$3$lcssa$i>>>0)<(256);
         if ($430) {
          $431 = $429 << 1;
          $432 = ((296 + ($431<<2)|0) + 40|0);
          $433 = HEAP32[296>>2]|0;
          $434 = 1 << $429;
          $435 = $433 & $434;
          $436 = ($435|0)==(0);
          do {
           if ($436) {
            $437 = $433 | $434;
            HEAP32[296>>2] = $437;
            $$sum14$pre$i = (($431) + 2)|0;
            $$pre$i25 = ((296 + ($$sum14$pre$i<<2)|0) + 40|0);
            $$pre$phi$i26Z2D = $$pre$i25;$F5$0$i = $432;
           } else {
            $$sum17$i = (($431) + 2)|0;
            $438 = ((296 + ($$sum17$i<<2)|0) + 40|0);
            $439 = HEAP32[$438>>2]|0;
            $440 = HEAP32[((296 + 16|0))>>2]|0;
            $441 = ($439>>>0)<($440>>>0);
            if (!($441)) {
             $$pre$phi$i26Z2D = $438;$F5$0$i = $439;
             break;
            }
            _abort();
            // unreachable;
           }
          } while(0);
          HEAP32[$$pre$phi$i26Z2D>>2] = $355;
          $442 = (($F5$0$i) + 12|0);
          HEAP32[$442>>2] = $355;
          $$sum15$i = (($253) + 8)|0;
          $443 = (($v$3$lcssa$i) + ($$sum15$i)|0);
          HEAP32[$443>>2] = $F5$0$i;
          $$sum16$i = (($253) + 12)|0;
          $444 = (($v$3$lcssa$i) + ($$sum16$i)|0);
          HEAP32[$444>>2] = $432;
          break;
         }
         $445 = $rsize$3$lcssa$i >>> 8;
         $446 = ($445|0)==(0);
         do {
          if ($446) {
           $I7$0$i = 0;
          } else {
           $447 = ($rsize$3$lcssa$i>>>0)>(16777215);
           if ($447) {
            $I7$0$i = 31;
            break;
           }
           $448 = (($445) + 1048320)|0;
           $449 = $448 >>> 16;
           $450 = $449 & 8;
           $451 = $445 << $450;
           $452 = (($451) + 520192)|0;
           $453 = $452 >>> 16;
           $454 = $453 & 4;
           $455 = $454 | $450;
           $456 = $451 << $454;
           $457 = (($456) + 245760)|0;
           $458 = $457 >>> 16;
           $459 = $458 & 2;
           $460 = $455 | $459;
           $461 = (14 - ($460))|0;
           $462 = $456 << $459;
           $463 = $462 >>> 15;
           $464 = (($461) + ($463))|0;
           $465 = $464 << 1;
           $466 = (($464) + 7)|0;
           $467 = $rsize$3$lcssa$i >>> $466;
           $468 = $467 & 1;
           $469 = $468 | $465;
           $I7$0$i = $469;
          }
         } while(0);
         $470 = ((296 + ($I7$0$i<<2)|0) + 304|0);
         $$sum2$i = (($253) + 28)|0;
         $471 = (($v$3$lcssa$i) + ($$sum2$i)|0);
         HEAP32[$471>>2] = $I7$0$i;
         $$sum3$i27 = (($253) + 16)|0;
         $472 = (($v$3$lcssa$i) + ($$sum3$i27)|0);
         $$sum4$i28 = (($253) + 20)|0;
         $473 = (($v$3$lcssa$i) + ($$sum4$i28)|0);
         HEAP32[$473>>2] = 0;
         HEAP32[$472>>2] = 0;
         $474 = HEAP32[((296 + 4|0))>>2]|0;
         $475 = 1 << $I7$0$i;
         $476 = $474 & $475;
         $477 = ($476|0)==(0);
         if ($477) {
          $478 = $474 | $475;
          HEAP32[((296 + 4|0))>>2] = $478;
          HEAP32[$470>>2] = $355;
          $$sum5$i = (($253) + 24)|0;
          $479 = (($v$3$lcssa$i) + ($$sum5$i)|0);
          HEAP32[$479>>2] = $470;
          $$sum6$i = (($253) + 12)|0;
          $480 = (($v$3$lcssa$i) + ($$sum6$i)|0);
          HEAP32[$480>>2] = $355;
          $$sum7$i = (($253) + 8)|0;
          $481 = (($v$3$lcssa$i) + ($$sum7$i)|0);
          HEAP32[$481>>2] = $355;
          break;
         }
         $482 = HEAP32[$470>>2]|0;
         $483 = ($I7$0$i|0)==(31);
         if ($483) {
          $491 = 0;
         } else {
          $484 = $I7$0$i >>> 1;
          $485 = (25 - ($484))|0;
          $491 = $485;
         }
         $486 = (($482) + 4|0);
         $487 = HEAP32[$486>>2]|0;
         $488 = $487 & -8;
         $489 = ($488|0)==($rsize$3$lcssa$i|0);
         do {
          if ($489) {
           $T$0$lcssa$i = $482;
          } else {
           $490 = $rsize$3$lcssa$i << $491;
           $K12$029$i = $490;$T$028$i = $482;
           while(1) {
            $498 = $K12$029$i >>> 31;
            $499 = ((($T$028$i) + ($498<<2)|0) + 16|0);
            $494 = HEAP32[$499>>2]|0;
            $500 = ($494|0)==(0|0);
            if ($500) {
             $$lcssa134 = $499;$T$028$i$lcssa = $T$028$i;
             break;
            }
            $492 = $K12$029$i << 1;
            $493 = (($494) + 4|0);
            $495 = HEAP32[$493>>2]|0;
            $496 = $495 & -8;
            $497 = ($496|0)==($rsize$3$lcssa$i|0);
            if ($497) {
             $$lcssa137 = $494;
             label = 167;
             break;
            } else {
             $K12$029$i = $492;$T$028$i = $494;
            }
           }
           if ((label|0) == 167) {
            $T$0$lcssa$i = $$lcssa137;
            break;
           }
           $501 = HEAP32[((296 + 16|0))>>2]|0;
           $502 = ($$lcssa134>>>0)<($501>>>0);
           if ($502) {
            _abort();
            // unreachable;
           } else {
            HEAP32[$$lcssa134>>2] = $355;
            $$sum11$i = (($253) + 24)|0;
            $503 = (($v$3$lcssa$i) + ($$sum11$i)|0);
            HEAP32[$503>>2] = $T$028$i$lcssa;
            $$sum12$i = (($253) + 12)|0;
            $504 = (($v$3$lcssa$i) + ($$sum12$i)|0);
            HEAP32[$504>>2] = $355;
            $$sum13$i = (($253) + 8)|0;
            $505 = (($v$3$lcssa$i) + ($$sum13$i)|0);
            HEAP32[$505>>2] = $355;
            break L219;
           }
          }
         } while(0);
         $506 = (($T$0$lcssa$i) + 8|0);
         $507 = HEAP32[$506>>2]|0;
         $508 = HEAP32[((296 + 16|0))>>2]|0;
         $509 = ($T$0$lcssa$i>>>0)>=($508>>>0);
         $510 = ($507>>>0)>=($508>>>0);
         $or$cond24$i = $509 & $510;
         if ($or$cond24$i) {
          $511 = (($507) + 12|0);
          HEAP32[$511>>2] = $355;
          HEAP32[$506>>2] = $355;
          $$sum8$i = (($253) + 8)|0;
          $512 = (($v$3$lcssa$i) + ($$sum8$i)|0);
          HEAP32[$512>>2] = $507;
          $$sum9$i = (($253) + 12)|0;
          $513 = (($v$3$lcssa$i) + ($$sum9$i)|0);
          HEAP32[$513>>2] = $T$0$lcssa$i;
          $$sum10$i = (($253) + 24)|0;
          $514 = (($v$3$lcssa$i) + ($$sum10$i)|0);
          HEAP32[$514>>2] = 0;
          break;
         } else {
          _abort();
          // unreachable;
         }
        }
       } while(0);
       $515 = (($v$3$lcssa$i) + 8|0);
       $mem$0 = $515;
      } else {
       $nb$0 = $253;
       label = 173;
      }
     }
    }
   }
  }
 } while(0);
 L256: do {
  if ((label|0) == 173) {
   $516 = HEAP32[((296 + 8|0))>>2]|0;
   $517 = ($516>>>0)<($nb$0>>>0);
   if (!($517)) {
    $518 = (($516) - ($nb$0))|0;
    $519 = HEAP32[((296 + 20|0))>>2]|0;
    $520 = ($518>>>0)>(15);
    if ($520) {
     $521 = (($519) + ($nb$0)|0);
     HEAP32[((296 + 20|0))>>2] = $521;
     HEAP32[((296 + 8|0))>>2] = $518;
     $522 = $518 | 1;
     $$sum2 = (($nb$0) + 4)|0;
     $523 = (($519) + ($$sum2)|0);
     HEAP32[$523>>2] = $522;
     $524 = (($519) + ($516)|0);
     HEAP32[$524>>2] = $518;
     $525 = $nb$0 | 3;
     $526 = (($519) + 4|0);
     HEAP32[$526>>2] = $525;
    } else {
     HEAP32[((296 + 8|0))>>2] = 0;
     HEAP32[((296 + 20|0))>>2] = 0;
     $527 = $516 | 3;
     $528 = (($519) + 4|0);
     HEAP32[$528>>2] = $527;
     $$sum1 = (($516) + 4)|0;
     $529 = (($519) + ($$sum1)|0);
     $530 = HEAP32[$529>>2]|0;
     $531 = $530 | 1;
     HEAP32[$529>>2] = $531;
    }
    $532 = (($519) + 8|0);
    $mem$0 = $532;
    break;
   }
   $533 = HEAP32[((296 + 12|0))>>2]|0;
   $534 = ($533>>>0)>($nb$0>>>0);
   if ($534) {
    $535 = (($533) - ($nb$0))|0;
    HEAP32[((296 + 12|0))>>2] = $535;
    $536 = HEAP32[((296 + 24|0))>>2]|0;
    $537 = (($536) + ($nb$0)|0);
    HEAP32[((296 + 24|0))>>2] = $537;
    $538 = $535 | 1;
    $$sum = (($nb$0) + 4)|0;
    $539 = (($536) + ($$sum)|0);
    HEAP32[$539>>2] = $538;
    $540 = $nb$0 | 3;
    $541 = (($536) + 4|0);
    HEAP32[$541>>2] = $540;
    $542 = (($536) + 8|0);
    $mem$0 = $542;
    break;
   }
   $543 = HEAP32[272>>2]|0;
   $544 = ($543|0)==(0);
   if ($544) {
    _init_mparams();
   }
   $545 = (($nb$0) + 48)|0;
   $546 = HEAP32[((272 + 8|0))>>2]|0;
   $547 = (($nb$0) + 47)|0;
   $548 = (($546) + ($547))|0;
   $549 = (0 - ($546))|0;
   $550 = $548 & $549;
   $551 = ($550>>>0)>($nb$0>>>0);
   if ($551) {
    $552 = HEAP32[((296 + 440|0))>>2]|0;
    $553 = ($552|0)==(0);
    if (!($553)) {
     $554 = HEAP32[((296 + 432|0))>>2]|0;
     $555 = (($554) + ($550))|0;
     $556 = ($555>>>0)<=($554>>>0);
     $557 = ($555>>>0)>($552>>>0);
     $or$cond1$i = $556 | $557;
     if ($or$cond1$i) {
      $mem$0 = 0;
      break;
     }
    }
    $558 = HEAP32[((296 + 444|0))>>2]|0;
    $559 = $558 & 4;
    $560 = ($559|0)==(0);
    if ($560) {
     $561 = HEAP32[((296 + 24|0))>>2]|0;
     $562 = ($561|0)==(0|0);
     do {
      if ($562) {
       label = 192;
      } else {
       $sp$0$i$i = ((296 + 476|0));
       while(1) {
        $563 = HEAP32[$sp$0$i$i>>2]|0;
        $564 = ($563>>>0)>($561>>>0);
        if (!($564)) {
         $565 = (($sp$0$i$i) + 4|0);
         $566 = HEAP32[$565>>2]|0;
         $567 = (($563) + ($566)|0);
         $568 = ($567>>>0)>($561>>>0);
         if ($568) {
          $$lcssa130 = $sp$0$i$i;$$lcssa132 = $565;$sp$0$i$i$lcssa = $sp$0$i$i;
          break;
         }
        }
        $569 = (($sp$0$i$i) + 8|0);
        $570 = HEAP32[$569>>2]|0;
        $571 = ($570|0)==(0|0);
        if ($571) {
         label = 191;
         break;
        } else {
         $sp$0$i$i = $570;
        }
       }
       if ((label|0) == 191) {
        label = 192;
        break;
       }
       (_pthread_mutex_lock(800)|0);
       $572 = ($sp$0$i$i$lcssa|0)==(0|0);
       if ($572) {
        label = 194;
       } else {
        $595 = HEAP32[((296 + 12|0))>>2]|0;
        $596 = HEAP32[((272 + 8|0))>>2]|0;
        $597 = (($547) - ($595))|0;
        $598 = (($597) + ($596))|0;
        $599 = (0 - ($596))|0;
        $600 = $598 & $599;
        $601 = ($600>>>0)<(2147483647);
        if ($601) {
         $602 = (_sbrk(($600|0))|0);
         $603 = HEAP32[$$lcssa130>>2]|0;
         $604 = HEAP32[$$lcssa132>>2]|0;
         $605 = (($603) + ($604)|0);
         $606 = ($602|0)==($605|0);
         if ($606) {
          $br$0$i = $602;$ssize$1$i = $600;
          label = 203;
         } else {
          $br$030$i = $602;$ssize$129$i = $600;
          label = 204;
         }
        } else {
         $tsize$03138$i = 0;
         label = 210;
        }
       }
      }
     } while(0);
     if ((label|0) == 192) {
      (_pthread_mutex_lock(800)|0);
      label = 194;
     }
     do {
      if ((label|0) == 194) {
       $573 = (_sbrk(0)|0);
       $574 = ($573|0)==((-1)|0);
       if ($574) {
        $tsize$03138$i = 0;
        label = 210;
       } else {
        $575 = $573;
        $576 = HEAP32[((272 + 4|0))>>2]|0;
        $577 = (($576) + -1)|0;
        $578 = $577 & $575;
        $579 = ($578|0)==(0);
        if ($579) {
         $ssize$0$i = $550;
        } else {
         $580 = (($577) + ($575))|0;
         $581 = (0 - ($576))|0;
         $582 = $580 & $581;
         $583 = (($550) - ($575))|0;
         $584 = (($583) + ($582))|0;
         $ssize$0$i = $584;
        }
        $585 = HEAP32[((296 + 432|0))>>2]|0;
        $586 = (($585) + ($ssize$0$i))|0;
        $587 = ($ssize$0$i>>>0)>($nb$0>>>0);
        $588 = ($ssize$0$i>>>0)<(2147483647);
        $or$cond$i29 = $587 & $588;
        if ($or$cond$i29) {
         $589 = HEAP32[((296 + 440|0))>>2]|0;
         $590 = ($589|0)==(0);
         if (!($590)) {
          $591 = ($586>>>0)<=($585>>>0);
          $592 = ($586>>>0)>($589>>>0);
          $or$cond2$i = $591 | $592;
          if ($or$cond2$i) {
           $tsize$03138$i = 0;
           label = 210;
           break;
          }
         }
         $593 = (_sbrk(($ssize$0$i|0))|0);
         $594 = ($593|0)==($573|0);
         if ($594) {
          $br$0$i = $573;$ssize$1$i = $ssize$0$i;
          label = 203;
         } else {
          $br$030$i = $593;$ssize$129$i = $ssize$0$i;
          label = 204;
         }
        } else {
         $tsize$03138$i = 0;
         label = 210;
        }
       }
      }
     } while(0);
     L301: do {
      if ((label|0) == 203) {
       $607 = ($br$0$i|0)==((-1)|0);
       if ($607) {
        $tsize$03138$i = $ssize$1$i;
        label = 210;
       } else {
        $tbase$1$i = $br$0$i;$tsize$1$i = $ssize$1$i;
       }
      }
      else if ((label|0) == 204) {
       $608 = (0 - ($ssize$129$i))|0;
       $609 = ($br$030$i|0)!=((-1)|0);
       $610 = ($ssize$129$i>>>0)<(2147483647);
       $or$cond5$i = $609 & $610;
       $611 = ($545>>>0)>($ssize$129$i>>>0);
       $or$cond4$i = $or$cond5$i & $611;
       do {
        if ($or$cond4$i) {
         $612 = HEAP32[((272 + 8|0))>>2]|0;
         $613 = (($547) - ($ssize$129$i))|0;
         $614 = (($613) + ($612))|0;
         $615 = (0 - ($612))|0;
         $616 = $614 & $615;
         $617 = ($616>>>0)<(2147483647);
         if ($617) {
          $618 = (_sbrk(($616|0))|0);
          $619 = ($618|0)==((-1)|0);
          if ($619) {
           (_sbrk(($608|0))|0);
           $tsize$03138$i = 0;
           label = 210;
           break L301;
          } else {
           $620 = (($616) + ($ssize$129$i))|0;
           $ssize$2$i = $620;
           break;
          }
         } else {
          $ssize$2$i = $ssize$129$i;
         }
        } else {
         $ssize$2$i = $ssize$129$i;
        }
       } while(0);
       $621 = ($br$030$i|0)==((-1)|0);
       if ($621) {
        $tsize$03138$i = 0;
        label = 210;
       } else {
        $tbase$1$i = $br$030$i;$tsize$1$i = $ssize$2$i;
       }
      }
     } while(0);
     if ((label|0) == 210) {
      $622 = HEAP32[((296 + 444|0))>>2]|0;
      $623 = $622 | 4;
      HEAP32[((296 + 444|0))>>2] = $623;
      $tbase$1$i = (-1);$tsize$1$i = $tsize$03138$i;
     }
     (_pthread_mutex_unlock(800)|0);
     $tbase$2$i = $tbase$1$i;$tsize$2$i = $tsize$1$i;
    } else {
     $tbase$2$i = (-1);$tsize$2$i = 0;
    }
    $624 = ($tbase$2$i|0)==((-1)|0);
    $625 = ($550>>>0)<(2147483647);
    $or$cond7$i = $624 & $625;
    if ($or$cond7$i) {
     (_pthread_mutex_lock(800)|0);
     $626 = (_sbrk(($550|0))|0);
     $627 = (_sbrk(0)|0);
     (_pthread_mutex_unlock(800)|0);
     $628 = ($626|0)!=((-1)|0);
     $629 = ($627|0)!=((-1)|0);
     $or$cond3$i = $628 & $629;
     $630 = ($626>>>0)<($627>>>0);
     $or$cond6$i = $or$cond3$i & $630;
     if ($or$cond6$i) {
      $631 = $627;
      $632 = $626;
      $633 = (($631) - ($632))|0;
      $634 = (($nb$0) + 40)|0;
      $635 = ($633>>>0)>($634>>>0);
      $$tsize$2$i = $635 ? $633 : $tsize$2$i;
      if ($635) {
       $tbase$3$i = $626;$tsize$3$i = $$tsize$2$i;
       label = 215;
      }
     }
    } else {
     $tbase$3$i = $tbase$2$i;$tsize$3$i = $tsize$2$i;
     label = 215;
    }
    if ((label|0) == 215) {
     $636 = ($tbase$3$i|0)==((-1)|0);
     if (!($636)) {
      $637 = HEAP32[((296 + 432|0))>>2]|0;
      $638 = (($637) + ($tsize$3$i))|0;
      HEAP32[((296 + 432|0))>>2] = $638;
      $639 = HEAP32[((296 + 436|0))>>2]|0;
      $640 = ($638>>>0)>($639>>>0);
      if ($640) {
       HEAP32[((296 + 436|0))>>2] = $638;
      }
      $641 = HEAP32[((296 + 24|0))>>2]|0;
      $642 = ($641|0)==(0|0);
      L324: do {
       if ($642) {
        $643 = HEAP32[((296 + 16|0))>>2]|0;
        $644 = ($643|0)==(0|0);
        $645 = ($tbase$3$i>>>0)<($643>>>0);
        $or$cond8$i = $644 | $645;
        if ($or$cond8$i) {
         HEAP32[((296 + 16|0))>>2] = $tbase$3$i;
        }
        HEAP32[((296 + 476|0))>>2] = $tbase$3$i;
        HEAP32[((296 + 480|0))>>2] = $tsize$3$i;
        HEAP32[((296 + 488|0))>>2] = 0;
        $646 = HEAP32[272>>2]|0;
        HEAP32[((296 + 36|0))>>2] = $646;
        HEAP32[((296 + 32|0))>>2] = -1;
        $i$02$i$i = 0;
        while(1) {
         $647 = $i$02$i$i << 1;
         $648 = ((296 + ($647<<2)|0) + 40|0);
         $$sum$i12$i = (($647) + 3)|0;
         $649 = ((296 + ($$sum$i12$i<<2)|0) + 40|0);
         HEAP32[$649>>2] = $648;
         $$sum1$i13$i = (($647) + 2)|0;
         $650 = ((296 + ($$sum1$i13$i<<2)|0) + 40|0);
         HEAP32[$650>>2] = $648;
         $651 = (($i$02$i$i) + 1)|0;
         $exitcond$i$i = ($651|0)==(32);
         if ($exitcond$i$i) {
          break;
         } else {
          $i$02$i$i = $651;
         }
        }
        $652 = (($tsize$3$i) + -40)|0;
        $653 = (($tbase$3$i) + 8|0);
        $654 = $653;
        $655 = $654 & 7;
        $656 = ($655|0)==(0);
        if ($656) {
         $660 = 0;
        } else {
         $657 = (0 - ($654))|0;
         $658 = $657 & 7;
         $660 = $658;
        }
        $659 = (($tbase$3$i) + ($660)|0);
        $661 = (($652) - ($660))|0;
        HEAP32[((296 + 24|0))>>2] = $659;
        HEAP32[((296 + 12|0))>>2] = $661;
        $662 = $661 | 1;
        $$sum$i14$i = (($660) + 4)|0;
        $663 = (($tbase$3$i) + ($$sum$i14$i)|0);
        HEAP32[$663>>2] = $662;
        $$sum2$i15$i = (($tsize$3$i) + -36)|0;
        $664 = (($tbase$3$i) + ($$sum2$i15$i)|0);
        HEAP32[$664>>2] = 40;
        $665 = HEAP32[((272 + 16|0))>>2]|0;
        HEAP32[((296 + 28|0))>>2] = $665;
       } else {
        $sp$068$i = ((296 + 476|0));
        while(1) {
         $666 = HEAP32[$sp$068$i>>2]|0;
         $667 = (($sp$068$i) + 4|0);
         $668 = HEAP32[$667>>2]|0;
         $669 = (($666) + ($668)|0);
         $670 = ($tbase$3$i|0)==($669|0);
         if ($670) {
          $$lcssa123 = $666;$$lcssa125 = $667;$$lcssa127 = $668;$sp$068$i$lcssa = $sp$068$i;
          label = 229;
          break;
         }
         $671 = (($sp$068$i) + 8|0);
         $672 = HEAP32[$671>>2]|0;
         $673 = ($672|0)==(0|0);
         if ($673) {
          label = 234;
          break;
         } else {
          $sp$068$i = $672;
         }
        }
        if ((label|0) == 229) {
         $674 = (($sp$068$i$lcssa) + 12|0);
         $675 = HEAP32[$674>>2]|0;
         $676 = $675 & 8;
         $677 = ($676|0)==(0);
         if ($677) {
          $678 = ($641>>>0)>=($$lcssa123>>>0);
          $679 = ($641>>>0)<($tbase$3$i>>>0);
          $or$cond41$i = $678 & $679;
          if ($or$cond41$i) {
           $680 = (($$lcssa127) + ($tsize$3$i))|0;
           HEAP32[$$lcssa125>>2] = $680;
           $681 = HEAP32[((296 + 12|0))>>2]|0;
           $682 = (($681) + ($tsize$3$i))|0;
           $683 = (($641) + 8|0);
           $684 = $683;
           $685 = $684 & 7;
           $686 = ($685|0)==(0);
           if ($686) {
            $690 = 0;
           } else {
            $687 = (0 - ($684))|0;
            $688 = $687 & 7;
            $690 = $688;
           }
           $689 = (($641) + ($690)|0);
           $691 = (($682) - ($690))|0;
           HEAP32[((296 + 24|0))>>2] = $689;
           HEAP32[((296 + 12|0))>>2] = $691;
           $692 = $691 | 1;
           $$sum$i16$i = (($690) + 4)|0;
           $693 = (($641) + ($$sum$i16$i)|0);
           HEAP32[$693>>2] = $692;
           $$sum2$i17$i = (($682) + 4)|0;
           $694 = (($641) + ($$sum2$i17$i)|0);
           HEAP32[$694>>2] = 40;
           $695 = HEAP32[((272 + 16|0))>>2]|0;
           HEAP32[((296 + 28|0))>>2] = $695;
           break;
          }
         }
        }
        else if ((label|0) == 234) {
        }
        $696 = HEAP32[((296 + 16|0))>>2]|0;
        $697 = ($tbase$3$i>>>0)<($696>>>0);
        if ($697) {
         HEAP32[((296 + 16|0))>>2] = $tbase$3$i;
         $761 = $tbase$3$i;
        } else {
         $761 = $696;
        }
        $698 = (($tbase$3$i) + ($tsize$3$i)|0);
        $sp$167$i = ((296 + 476|0));
        while(1) {
         $699 = HEAP32[$sp$167$i>>2]|0;
         $700 = ($699|0)==($698|0);
         if ($700) {
          $$lcssa120 = $sp$167$i;$sp$167$i$lcssa = $sp$167$i;
          label = 240;
          break;
         }
         $701 = (($sp$167$i) + 8|0);
         $702 = HEAP32[$701>>2]|0;
         $703 = ($702|0)==(0|0);
         if ($703) {
          label = 324;
          break;
         } else {
          $sp$167$i = $702;
         }
        }
        if ((label|0) == 240) {
         $704 = (($sp$167$i$lcssa) + 12|0);
         $705 = HEAP32[$704>>2]|0;
         $706 = $705 & 8;
         $707 = ($706|0)==(0);
         if ($707) {
          HEAP32[$$lcssa120>>2] = $tbase$3$i;
          $708 = (($sp$167$i$lcssa) + 4|0);
          $709 = HEAP32[$708>>2]|0;
          $710 = (($709) + ($tsize$3$i))|0;
          HEAP32[$708>>2] = $710;
          $711 = (($tbase$3$i) + 8|0);
          $712 = $711;
          $713 = $712 & 7;
          $714 = ($713|0)==(0);
          if ($714) {
           $718 = 0;
          } else {
           $715 = (0 - ($712))|0;
           $716 = $715 & 7;
           $718 = $716;
          }
          $717 = (($tbase$3$i) + ($718)|0);
          $$sum96$i = (($tsize$3$i) + 8)|0;
          $719 = (($tbase$3$i) + ($$sum96$i)|0);
          $720 = $719;
          $721 = $720 & 7;
          $722 = ($721|0)==(0);
          if ($722) {
           $725 = 0;
          } else {
           $723 = (0 - ($720))|0;
           $724 = $723 & 7;
           $725 = $724;
          }
          $$sum97$i = (($725) + ($tsize$3$i))|0;
          $726 = (($tbase$3$i) + ($$sum97$i)|0);
          $727 = $726;
          $728 = $717;
          $729 = (($727) - ($728))|0;
          $$sum$i19$i = (($718) + ($nb$0))|0;
          $730 = (($tbase$3$i) + ($$sum$i19$i)|0);
          $731 = (($729) - ($nb$0))|0;
          $732 = $nb$0 | 3;
          $$sum1$i20$i = (($718) + 4)|0;
          $733 = (($tbase$3$i) + ($$sum1$i20$i)|0);
          HEAP32[$733>>2] = $732;
          $734 = ($726|0)==($641|0);
          L353: do {
           if ($734) {
            $735 = HEAP32[((296 + 12|0))>>2]|0;
            $736 = (($735) + ($731))|0;
            HEAP32[((296 + 12|0))>>2] = $736;
            HEAP32[((296 + 24|0))>>2] = $730;
            $737 = $736 | 1;
            $$sum42$i$i = (($$sum$i19$i) + 4)|0;
            $738 = (($tbase$3$i) + ($$sum42$i$i)|0);
            HEAP32[$738>>2] = $737;
           } else {
            $739 = HEAP32[((296 + 20|0))>>2]|0;
            $740 = ($726|0)==($739|0);
            if ($740) {
             $741 = HEAP32[((296 + 8|0))>>2]|0;
             $742 = (($741) + ($731))|0;
             HEAP32[((296 + 8|0))>>2] = $742;
             HEAP32[((296 + 20|0))>>2] = $730;
             $743 = $742 | 1;
             $$sum40$i$i = (($$sum$i19$i) + 4)|0;
             $744 = (($tbase$3$i) + ($$sum40$i$i)|0);
             HEAP32[$744>>2] = $743;
             $$sum41$i$i = (($742) + ($$sum$i19$i))|0;
             $745 = (($tbase$3$i) + ($$sum41$i$i)|0);
             HEAP32[$745>>2] = $742;
             break;
            }
            $$sum2$i21$i = (($tsize$3$i) + 4)|0;
            $$sum98$i = (($$sum2$i21$i) + ($725))|0;
            $746 = (($tbase$3$i) + ($$sum98$i)|0);
            $747 = HEAP32[$746>>2]|0;
            $748 = $747 & 3;
            $749 = ($748|0)==(1);
            if ($749) {
             $750 = $747 & -8;
             $751 = $747 >>> 3;
             $752 = ($747>>>0)<(256);
             L361: do {
              if ($752) {
               $$sum3738$i$i = $725 | 8;
               $$sum108$i = (($$sum3738$i$i) + ($tsize$3$i))|0;
               $753 = (($tbase$3$i) + ($$sum108$i)|0);
               $754 = HEAP32[$753>>2]|0;
               $$sum39$i$i = (($tsize$3$i) + 12)|0;
               $$sum109$i = (($$sum39$i$i) + ($725))|0;
               $755 = (($tbase$3$i) + ($$sum109$i)|0);
               $756 = HEAP32[$755>>2]|0;
               $757 = $751 << 1;
               $758 = ((296 + ($757<<2)|0) + 40|0);
               $759 = ($754|0)==($758|0);
               do {
                if (!($759)) {
                 $760 = ($754>>>0)<($761>>>0);
                 if ($760) {
                  _abort();
                  // unreachable;
                 }
                 $762 = (($754) + 12|0);
                 $763 = HEAP32[$762>>2]|0;
                 $764 = ($763|0)==($726|0);
                 if ($764) {
                  break;
                 }
                 _abort();
                 // unreachable;
                }
               } while(0);
               $765 = ($756|0)==($754|0);
               if ($765) {
                $766 = 1 << $751;
                $767 = $766 ^ -1;
                $768 = HEAP32[296>>2]|0;
                $769 = $768 & $767;
                HEAP32[296>>2] = $769;
                break;
               }
               $770 = ($756|0)==($758|0);
               do {
                if ($770) {
                 $$pre58$i$i = (($756) + 8|0);
                 $$pre$phi59$i$iZ2D = $$pre58$i$i;
                } else {
                 $771 = ($756>>>0)<($761>>>0);
                 if ($771) {
                  _abort();
                  // unreachable;
                 }
                 $772 = (($756) + 8|0);
                 $773 = HEAP32[$772>>2]|0;
                 $774 = ($773|0)==($726|0);
                 if ($774) {
                  $$pre$phi59$i$iZ2D = $772;
                  break;
                 }
                 _abort();
                 // unreachable;
                }
               } while(0);
               $775 = (($754) + 12|0);
               HEAP32[$775>>2] = $756;
               HEAP32[$$pre$phi59$i$iZ2D>>2] = $754;
              } else {
               $$sum34$i$i = $725 | 24;
               $$sum99$i = (($$sum34$i$i) + ($tsize$3$i))|0;
               $776 = (($tbase$3$i) + ($$sum99$i)|0);
               $777 = HEAP32[$776>>2]|0;
               $$sum5$i$i = (($tsize$3$i) + 12)|0;
               $$sum100$i = (($$sum5$i$i) + ($725))|0;
               $778 = (($tbase$3$i) + ($$sum100$i)|0);
               $779 = HEAP32[$778>>2]|0;
               $780 = ($779|0)==($726|0);
               do {
                if ($780) {
                 $$sum67$i$i = $725 | 16;
                 $$sum106$i = (($$sum2$i21$i) + ($$sum67$i$i))|0;
                 $790 = (($tbase$3$i) + ($$sum106$i)|0);
                 $791 = HEAP32[$790>>2]|0;
                 $792 = ($791|0)==(0|0);
                 if ($792) {
                  $$sum107$i = (($$sum67$i$i) + ($tsize$3$i))|0;
                  $793 = (($tbase$3$i) + ($$sum107$i)|0);
                  $794 = HEAP32[$793>>2]|0;
                  $795 = ($794|0)==(0|0);
                  if ($795) {
                   $R$1$i$i = 0;
                   break;
                  } else {
                   $R$0$i$i$ph = $794;$RP$0$i$i$ph = $793;
                  }
                 } else {
                  $R$0$i$i$ph = $791;$RP$0$i$i$ph = $790;
                 }
                 $R$0$i$i = $R$0$i$i$ph;$RP$0$i$i = $RP$0$i$i$ph;
                 while(1) {
                  $796 = (($R$0$i$i) + 20|0);
                  $797 = HEAP32[$796>>2]|0;
                  $798 = ($797|0)==(0|0);
                  if ($798) {
                   $799 = (($R$0$i$i) + 16|0);
                   $800 = HEAP32[$799>>2]|0;
                   $801 = ($800|0)==(0|0);
                   if ($801) {
                    $R$0$i$i$lcssa = $R$0$i$i;$RP$0$i$i$lcssa = $RP$0$i$i;
                    break;
                   } else {
                    $R$0$i$i$be = $800;$RP$0$i$i$be = $799;
                   }
                  } else {
                   $R$0$i$i$be = $797;$RP$0$i$i$be = $796;
                  }
                  $R$0$i$i = $R$0$i$i$be;$RP$0$i$i = $RP$0$i$i$be;
                 }
                 $802 = ($RP$0$i$i$lcssa>>>0)<($761>>>0);
                 if ($802) {
                  _abort();
                  // unreachable;
                 } else {
                  HEAP32[$RP$0$i$i$lcssa>>2] = 0;
                  $R$1$i$i = $R$0$i$i$lcssa;
                  break;
                 }
                } else {
                 $$sum3536$i$i = $725 | 8;
                 $$sum101$i = (($$sum3536$i$i) + ($tsize$3$i))|0;
                 $781 = (($tbase$3$i) + ($$sum101$i)|0);
                 $782 = HEAP32[$781>>2]|0;
                 $783 = ($782>>>0)<($761>>>0);
                 if ($783) {
                  _abort();
                  // unreachable;
                 }
                 $784 = (($782) + 12|0);
                 $785 = HEAP32[$784>>2]|0;
                 $786 = ($785|0)==($726|0);
                 if (!($786)) {
                  _abort();
                  // unreachable;
                 }
                 $787 = (($779) + 8|0);
                 $788 = HEAP32[$787>>2]|0;
                 $789 = ($788|0)==($726|0);
                 if ($789) {
                  HEAP32[$784>>2] = $779;
                  HEAP32[$787>>2] = $782;
                  $R$1$i$i = $779;
                  break;
                 } else {
                  _abort();
                  // unreachable;
                 }
                }
               } while(0);
               $803 = ($777|0)==(0|0);
               if ($803) {
                break;
               }
               $$sum30$i$i = (($tsize$3$i) + 28)|0;
               $$sum102$i = (($$sum30$i$i) + ($725))|0;
               $804 = (($tbase$3$i) + ($$sum102$i)|0);
               $805 = HEAP32[$804>>2]|0;
               $806 = ((296 + ($805<<2)|0) + 304|0);
               $807 = HEAP32[$806>>2]|0;
               $808 = ($726|0)==($807|0);
               do {
                if ($808) {
                 HEAP32[$806>>2] = $R$1$i$i;
                 $cond$i$i = ($R$1$i$i|0)==(0|0);
                 if (!($cond$i$i)) {
                  break;
                 }
                 $809 = 1 << $805;
                 $810 = $809 ^ -1;
                 $811 = HEAP32[((296 + 4|0))>>2]|0;
                 $812 = $811 & $810;
                 HEAP32[((296 + 4|0))>>2] = $812;
                 break L361;
                } else {
                 $813 = HEAP32[((296 + 16|0))>>2]|0;
                 $814 = ($777>>>0)<($813>>>0);
                 if ($814) {
                  _abort();
                  // unreachable;
                 }
                 $815 = (($777) + 16|0);
                 $816 = HEAP32[$815>>2]|0;
                 $817 = ($816|0)==($726|0);
                 if ($817) {
                  HEAP32[$815>>2] = $R$1$i$i;
                 } else {
                  $818 = (($777) + 20|0);
                  HEAP32[$818>>2] = $R$1$i$i;
                 }
                 $819 = ($R$1$i$i|0)==(0|0);
                 if ($819) {
                  break L361;
                 }
                }
               } while(0);
               $820 = HEAP32[((296 + 16|0))>>2]|0;
               $821 = ($R$1$i$i>>>0)<($820>>>0);
               if ($821) {
                _abort();
                // unreachable;
               }
               $822 = (($R$1$i$i) + 24|0);
               HEAP32[$822>>2] = $777;
               $$sum3132$i$i = $725 | 16;
               $$sum103$i = (($$sum3132$i$i) + ($tsize$3$i))|0;
               $823 = (($tbase$3$i) + ($$sum103$i)|0);
               $824 = HEAP32[$823>>2]|0;
               $825 = ($824|0)==(0|0);
               do {
                if (!($825)) {
                 $826 = ($824>>>0)<($820>>>0);
                 if ($826) {
                  _abort();
                  // unreachable;
                 } else {
                  $827 = (($R$1$i$i) + 16|0);
                  HEAP32[$827>>2] = $824;
                  $828 = (($824) + 24|0);
                  HEAP32[$828>>2] = $R$1$i$i;
                  break;
                 }
                }
               } while(0);
               $$sum104$i = (($$sum2$i21$i) + ($$sum3132$i$i))|0;
               $829 = (($tbase$3$i) + ($$sum104$i)|0);
               $830 = HEAP32[$829>>2]|0;
               $831 = ($830|0)==(0|0);
               if ($831) {
                break;
               }
               $832 = HEAP32[((296 + 16|0))>>2]|0;
               $833 = ($830>>>0)<($832>>>0);
               if ($833) {
                _abort();
                // unreachable;
               } else {
                $834 = (($R$1$i$i) + 20|0);
                HEAP32[$834>>2] = $830;
                $835 = (($830) + 24|0);
                HEAP32[$835>>2] = $R$1$i$i;
                break;
               }
              }
             } while(0);
             $$sum9$i$i = $750 | $725;
             $$sum105$i = (($$sum9$i$i) + ($tsize$3$i))|0;
             $836 = (($tbase$3$i) + ($$sum105$i)|0);
             $837 = (($750) + ($731))|0;
             $oldfirst$0$i$i = $836;$qsize$0$i$i = $837;
            } else {
             $oldfirst$0$i$i = $726;$qsize$0$i$i = $731;
            }
            $838 = (($oldfirst$0$i$i) + 4|0);
            $839 = HEAP32[$838>>2]|0;
            $840 = $839 & -2;
            HEAP32[$838>>2] = $840;
            $841 = $qsize$0$i$i | 1;
            $$sum10$i$i = (($$sum$i19$i) + 4)|0;
            $842 = (($tbase$3$i) + ($$sum10$i$i)|0);
            HEAP32[$842>>2] = $841;
            $$sum11$i22$i = (($qsize$0$i$i) + ($$sum$i19$i))|0;
            $843 = (($tbase$3$i) + ($$sum11$i22$i)|0);
            HEAP32[$843>>2] = $qsize$0$i$i;
            $844 = $qsize$0$i$i >>> 3;
            $845 = ($qsize$0$i$i>>>0)<(256);
            if ($845) {
             $846 = $844 << 1;
             $847 = ((296 + ($846<<2)|0) + 40|0);
             $848 = HEAP32[296>>2]|0;
             $849 = 1 << $844;
             $850 = $848 & $849;
             $851 = ($850|0)==(0);
             do {
              if ($851) {
               $852 = $848 | $849;
               HEAP32[296>>2] = $852;
               $$sum26$pre$i$i = (($846) + 2)|0;
               $$pre$i23$i = ((296 + ($$sum26$pre$i$i<<2)|0) + 40|0);
               $$pre$phi$i24$iZ2D = $$pre$i23$i;$F4$0$i$i = $847;
              } else {
               $$sum29$i$i = (($846) + 2)|0;
               $853 = ((296 + ($$sum29$i$i<<2)|0) + 40|0);
               $854 = HEAP32[$853>>2]|0;
               $855 = HEAP32[((296 + 16|0))>>2]|0;
               $856 = ($854>>>0)<($855>>>0);
               if (!($856)) {
                $$pre$phi$i24$iZ2D = $853;$F4$0$i$i = $854;
                break;
               }
               _abort();
               // unreachable;
              }
             } while(0);
             HEAP32[$$pre$phi$i24$iZ2D>>2] = $730;
             $857 = (($F4$0$i$i) + 12|0);
             HEAP32[$857>>2] = $730;
             $$sum27$i$i = (($$sum$i19$i) + 8)|0;
             $858 = (($tbase$3$i) + ($$sum27$i$i)|0);
             HEAP32[$858>>2] = $F4$0$i$i;
             $$sum28$i$i = (($$sum$i19$i) + 12)|0;
             $859 = (($tbase$3$i) + ($$sum28$i$i)|0);
             HEAP32[$859>>2] = $847;
             break;
            }
            $860 = $qsize$0$i$i >>> 8;
            $861 = ($860|0)==(0);
            do {
             if ($861) {
              $I7$0$i$i = 0;
             } else {
              $862 = ($qsize$0$i$i>>>0)>(16777215);
              if ($862) {
               $I7$0$i$i = 31;
               break;
              }
              $863 = (($860) + 1048320)|0;
              $864 = $863 >>> 16;
              $865 = $864 & 8;
              $866 = $860 << $865;
              $867 = (($866) + 520192)|0;
              $868 = $867 >>> 16;
              $869 = $868 & 4;
              $870 = $869 | $865;
              $871 = $866 << $869;
              $872 = (($871) + 245760)|0;
              $873 = $872 >>> 16;
              $874 = $873 & 2;
              $875 = $870 | $874;
              $876 = (14 - ($875))|0;
              $877 = $871 << $874;
              $878 = $877 >>> 15;
              $879 = (($876) + ($878))|0;
              $880 = $879 << 1;
              $881 = (($879) + 7)|0;
              $882 = $qsize$0$i$i >>> $881;
              $883 = $882 & 1;
              $884 = $883 | $880;
              $I7$0$i$i = $884;
             }
            } while(0);
            $885 = ((296 + ($I7$0$i$i<<2)|0) + 304|0);
            $$sum12$i$i = (($$sum$i19$i) + 28)|0;
            $886 = (($tbase$3$i) + ($$sum12$i$i)|0);
            HEAP32[$886>>2] = $I7$0$i$i;
            $$sum13$i$i = (($$sum$i19$i) + 16)|0;
            $887 = (($tbase$3$i) + ($$sum13$i$i)|0);
            $$sum14$i$i = (($$sum$i19$i) + 20)|0;
            $888 = (($tbase$3$i) + ($$sum14$i$i)|0);
            HEAP32[$888>>2] = 0;
            HEAP32[$887>>2] = 0;
            $889 = HEAP32[((296 + 4|0))>>2]|0;
            $890 = 1 << $I7$0$i$i;
            $891 = $889 & $890;
            $892 = ($891|0)==(0);
            if ($892) {
             $893 = $889 | $890;
             HEAP32[((296 + 4|0))>>2] = $893;
             HEAP32[$885>>2] = $730;
             $$sum15$i$i = (($$sum$i19$i) + 24)|0;
             $894 = (($tbase$3$i) + ($$sum15$i$i)|0);
             HEAP32[$894>>2] = $885;
             $$sum16$i$i = (($$sum$i19$i) + 12)|0;
             $895 = (($tbase$3$i) + ($$sum16$i$i)|0);
             HEAP32[$895>>2] = $730;
             $$sum17$i$i = (($$sum$i19$i) + 8)|0;
             $896 = (($tbase$3$i) + ($$sum17$i$i)|0);
             HEAP32[$896>>2] = $730;
             break;
            }
            $897 = HEAP32[$885>>2]|0;
            $898 = ($I7$0$i$i|0)==(31);
            if ($898) {
             $906 = 0;
            } else {
             $899 = $I7$0$i$i >>> 1;
             $900 = (25 - ($899))|0;
             $906 = $900;
            }
            $901 = (($897) + 4|0);
            $902 = HEAP32[$901>>2]|0;
            $903 = $902 & -8;
            $904 = ($903|0)==($qsize$0$i$i|0);
            do {
             if ($904) {
              $T$0$lcssa$i26$i = $897;
             } else {
              $905 = $qsize$0$i$i << $906;
              $K8$053$i$i = $905;$T$052$i$i = $897;
              while(1) {
               $913 = $K8$053$i$i >>> 31;
               $914 = ((($T$052$i$i) + ($913<<2)|0) + 16|0);
               $909 = HEAP32[$914>>2]|0;
               $915 = ($909|0)==(0|0);
               if ($915) {
                $$lcssa = $914;$T$052$i$i$lcssa = $T$052$i$i;
                break;
               }
               $907 = $K8$053$i$i << 1;
               $908 = (($909) + 4|0);
               $910 = HEAP32[$908>>2]|0;
               $911 = $910 & -8;
               $912 = ($911|0)==($qsize$0$i$i|0);
               if ($912) {
                $$lcssa110 = $909;
                label = 319;
                break;
               } else {
                $K8$053$i$i = $907;$T$052$i$i = $909;
               }
              }
              if ((label|0) == 319) {
               $T$0$lcssa$i26$i = $$lcssa110;
               break;
              }
              $916 = HEAP32[((296 + 16|0))>>2]|0;
              $917 = ($$lcssa>>>0)<($916>>>0);
              if ($917) {
               _abort();
               // unreachable;
              } else {
               HEAP32[$$lcssa>>2] = $730;
               $$sum23$i$i = (($$sum$i19$i) + 24)|0;
               $918 = (($tbase$3$i) + ($$sum23$i$i)|0);
               HEAP32[$918>>2] = $T$052$i$i$lcssa;
               $$sum24$i$i = (($$sum$i19$i) + 12)|0;
               $919 = (($tbase$3$i) + ($$sum24$i$i)|0);
               HEAP32[$919>>2] = $730;
               $$sum25$i$i = (($$sum$i19$i) + 8)|0;
               $920 = (($tbase$3$i) + ($$sum25$i$i)|0);
               HEAP32[$920>>2] = $730;
               break L353;
              }
             }
            } while(0);
            $921 = (($T$0$lcssa$i26$i) + 8|0);
            $922 = HEAP32[$921>>2]|0;
            $923 = HEAP32[((296 + 16|0))>>2]|0;
            $924 = ($T$0$lcssa$i26$i>>>0)>=($923>>>0);
            $925 = ($922>>>0)>=($923>>>0);
            $or$cond$i27$i = $924 & $925;
            if ($or$cond$i27$i) {
             $926 = (($922) + 12|0);
             HEAP32[$926>>2] = $730;
             HEAP32[$921>>2] = $730;
             $$sum20$i$i = (($$sum$i19$i) + 8)|0;
             $927 = (($tbase$3$i) + ($$sum20$i$i)|0);
             HEAP32[$927>>2] = $922;
             $$sum21$i$i = (($$sum$i19$i) + 12)|0;
             $928 = (($tbase$3$i) + ($$sum21$i$i)|0);
             HEAP32[$928>>2] = $T$0$lcssa$i26$i;
             $$sum22$i$i = (($$sum$i19$i) + 24)|0;
             $929 = (($tbase$3$i) + ($$sum22$i$i)|0);
             HEAP32[$929>>2] = 0;
             break;
            } else {
             _abort();
             // unreachable;
            }
           }
          } while(0);
          $$sum1819$i$i = $718 | 8;
          $930 = (($tbase$3$i) + ($$sum1819$i$i)|0);
          $mem$0 = $930;
          break L256;
         }
        }
        else if ((label|0) == 324) {
        }
        $sp$0$i$i$i = ((296 + 476|0));
        while(1) {
         $931 = HEAP32[$sp$0$i$i$i>>2]|0;
         $932 = ($931>>>0)>($641>>>0);
         if (!($932)) {
          $933 = (($sp$0$i$i$i) + 4|0);
          $934 = HEAP32[$933>>2]|0;
          $935 = (($931) + ($934)|0);
          $936 = ($935>>>0)>($641>>>0);
          if ($936) {
           $$lcssa116 = $931;$$lcssa117 = $934;$$lcssa118 = $935;
           break;
          }
         }
         $937 = (($sp$0$i$i$i) + 8|0);
         $938 = HEAP32[$937>>2]|0;
         $sp$0$i$i$i = $938;
        }
        $$sum$i$i = (($$lcssa117) + -47)|0;
        $$sum1$i$i = (($$lcssa117) + -39)|0;
        $939 = (($$lcssa116) + ($$sum1$i$i)|0);
        $940 = $939;
        $941 = $940 & 7;
        $942 = ($941|0)==(0);
        if ($942) {
         $945 = 0;
        } else {
         $943 = (0 - ($940))|0;
         $944 = $943 & 7;
         $945 = $944;
        }
        $$sum2$i$i = (($$sum$i$i) + ($945))|0;
        $946 = (($$lcssa116) + ($$sum2$i$i)|0);
        $947 = (($641) + 16|0);
        $948 = ($946>>>0)<($947>>>0);
        $949 = $948 ? $641 : $946;
        $950 = (($949) + 8|0);
        $951 = (($tsize$3$i) + -40)|0;
        $952 = (($tbase$3$i) + 8|0);
        $953 = $952;
        $954 = $953 & 7;
        $955 = ($954|0)==(0);
        if ($955) {
         $959 = 0;
        } else {
         $956 = (0 - ($953))|0;
         $957 = $956 & 7;
         $959 = $957;
        }
        $958 = (($tbase$3$i) + ($959)|0);
        $960 = (($951) - ($959))|0;
        HEAP32[((296 + 24|0))>>2] = $958;
        HEAP32[((296 + 12|0))>>2] = $960;
        $961 = $960 | 1;
        $$sum$i$i$i = (($959) + 4)|0;
        $962 = (($tbase$3$i) + ($$sum$i$i$i)|0);
        HEAP32[$962>>2] = $961;
        $$sum2$i$i$i = (($tsize$3$i) + -36)|0;
        $963 = (($tbase$3$i) + ($$sum2$i$i$i)|0);
        HEAP32[$963>>2] = 40;
        $964 = HEAP32[((272 + 16|0))>>2]|0;
        HEAP32[((296 + 28|0))>>2] = $964;
        $965 = (($949) + 4|0);
        HEAP32[$965>>2] = 27;
        ;HEAP32[$950+0>>2]=HEAP32[((296 + 476|0))+0>>2]|0;HEAP32[$950+4>>2]=HEAP32[((296 + 476|0))+4>>2]|0;HEAP32[$950+8>>2]=HEAP32[((296 + 476|0))+8>>2]|0;HEAP32[$950+12>>2]=HEAP32[((296 + 476|0))+12>>2]|0;
        HEAP32[((296 + 476|0))>>2] = $tbase$3$i;
        HEAP32[((296 + 480|0))>>2] = $tsize$3$i;
        HEAP32[((296 + 488|0))>>2] = 0;
        HEAP32[((296 + 484|0))>>2] = $950;
        $966 = (($949) + 28|0);
        HEAP32[$966>>2] = 7;
        $967 = (($949) + 32|0);
        $968 = ($967>>>0)<($$lcssa118>>>0);
        if ($968) {
         $970 = $966;
         while(1) {
          $969 = (($970) + 4|0);
          HEAP32[$969>>2] = 7;
          $971 = (($970) + 8|0);
          $972 = ($971>>>0)<($$lcssa118>>>0);
          if ($972) {
           $970 = $969;
          } else {
           break;
          }
         }
        }
        $973 = ($949|0)==($641|0);
        if ($973) {
         break;
        }
        $974 = $949;
        $975 = $641;
        $976 = (($974) - ($975))|0;
        $977 = (($641) + ($976)|0);
        $$sum3$i$i = (($976) + 4)|0;
        $978 = (($641) + ($$sum3$i$i)|0);
        $979 = HEAP32[$978>>2]|0;
        $980 = $979 & -2;
        HEAP32[$978>>2] = $980;
        $981 = $976 | 1;
        $982 = (($641) + 4|0);
        HEAP32[$982>>2] = $981;
        HEAP32[$977>>2] = $976;
        $983 = $976 >>> 3;
        $984 = ($976>>>0)<(256);
        if ($984) {
         $985 = $983 << 1;
         $986 = ((296 + ($985<<2)|0) + 40|0);
         $987 = HEAP32[296>>2]|0;
         $988 = 1 << $983;
         $989 = $987 & $988;
         $990 = ($989|0)==(0);
         do {
          if ($990) {
           $991 = $987 | $988;
           HEAP32[296>>2] = $991;
           $$sum10$pre$i$i = (($985) + 2)|0;
           $$pre$i$i = ((296 + ($$sum10$pre$i$i<<2)|0) + 40|0);
           $$pre$phi$i$iZ2D = $$pre$i$i;$F$0$i$i = $986;
          } else {
           $$sum11$i$i = (($985) + 2)|0;
           $992 = ((296 + ($$sum11$i$i<<2)|0) + 40|0);
           $993 = HEAP32[$992>>2]|0;
           $994 = HEAP32[((296 + 16|0))>>2]|0;
           $995 = ($993>>>0)<($994>>>0);
           if (!($995)) {
            $$pre$phi$i$iZ2D = $992;$F$0$i$i = $993;
            break;
           }
           _abort();
           // unreachable;
          }
         } while(0);
         HEAP32[$$pre$phi$i$iZ2D>>2] = $641;
         $996 = (($F$0$i$i) + 12|0);
         HEAP32[$996>>2] = $641;
         $997 = (($641) + 8|0);
         HEAP32[$997>>2] = $F$0$i$i;
         $998 = (($641) + 12|0);
         HEAP32[$998>>2] = $986;
         break;
        }
        $999 = $976 >>> 8;
        $1000 = ($999|0)==(0);
        do {
         if ($1000) {
          $I1$0$i$i = 0;
         } else {
          $1001 = ($976>>>0)>(16777215);
          if ($1001) {
           $I1$0$i$i = 31;
           break;
          }
          $1002 = (($999) + 1048320)|0;
          $1003 = $1002 >>> 16;
          $1004 = $1003 & 8;
          $1005 = $999 << $1004;
          $1006 = (($1005) + 520192)|0;
          $1007 = $1006 >>> 16;
          $1008 = $1007 & 4;
          $1009 = $1008 | $1004;
          $1010 = $1005 << $1008;
          $1011 = (($1010) + 245760)|0;
          $1012 = $1011 >>> 16;
          $1013 = $1012 & 2;
          $1014 = $1009 | $1013;
          $1015 = (14 - ($1014))|0;
          $1016 = $1010 << $1013;
          $1017 = $1016 >>> 15;
          $1018 = (($1015) + ($1017))|0;
          $1019 = $1018 << 1;
          $1020 = (($1018) + 7)|0;
          $1021 = $976 >>> $1020;
          $1022 = $1021 & 1;
          $1023 = $1022 | $1019;
          $I1$0$i$i = $1023;
         }
        } while(0);
        $1024 = ((296 + ($I1$0$i$i<<2)|0) + 304|0);
        $1025 = (($641) + 28|0);
        $I1$0$c$i$i = $I1$0$i$i;
        HEAP32[$1025>>2] = $I1$0$c$i$i;
        $1026 = (($641) + 20|0);
        HEAP32[$1026>>2] = 0;
        $1027 = (($641) + 16|0);
        HEAP32[$1027>>2] = 0;
        $1028 = HEAP32[((296 + 4|0))>>2]|0;
        $1029 = 1 << $I1$0$i$i;
        $1030 = $1028 & $1029;
        $1031 = ($1030|0)==(0);
        if ($1031) {
         $1032 = $1028 | $1029;
         HEAP32[((296 + 4|0))>>2] = $1032;
         HEAP32[$1024>>2] = $641;
         $1033 = (($641) + 24|0);
         HEAP32[$1033>>2] = $1024;
         $1034 = (($641) + 12|0);
         HEAP32[$1034>>2] = $641;
         $1035 = (($641) + 8|0);
         HEAP32[$1035>>2] = $641;
         break;
        }
        $1036 = HEAP32[$1024>>2]|0;
        $1037 = ($I1$0$i$i|0)==(31);
        if ($1037) {
         $1045 = 0;
        } else {
         $1038 = $I1$0$i$i >>> 1;
         $1039 = (25 - ($1038))|0;
         $1045 = $1039;
        }
        $1040 = (($1036) + 4|0);
        $1041 = HEAP32[$1040>>2]|0;
        $1042 = $1041 & -8;
        $1043 = ($1042|0)==($976|0);
        do {
         if ($1043) {
          $T$0$lcssa$i$i = $1036;
         } else {
          $1044 = $976 << $1045;
          $K2$015$i$i = $1044;$T$014$i$i = $1036;
          while(1) {
           $1052 = $K2$015$i$i >>> 31;
           $1053 = ((($T$014$i$i) + ($1052<<2)|0) + 16|0);
           $1048 = HEAP32[$1053>>2]|0;
           $1054 = ($1048|0)==(0|0);
           if ($1054) {
            $$lcssa112 = $1053;$T$014$i$i$lcssa = $T$014$i$i;
            break;
           }
           $1046 = $K2$015$i$i << 1;
           $1047 = (($1048) + 4|0);
           $1049 = HEAP32[$1047>>2]|0;
           $1050 = $1049 & -8;
           $1051 = ($1050|0)==($976|0);
           if ($1051) {
            $$lcssa115 = $1048;
            label = 358;
            break;
           } else {
            $K2$015$i$i = $1046;$T$014$i$i = $1048;
           }
          }
          if ((label|0) == 358) {
           $T$0$lcssa$i$i = $$lcssa115;
           break;
          }
          $1055 = HEAP32[((296 + 16|0))>>2]|0;
          $1056 = ($$lcssa112>>>0)<($1055>>>0);
          if ($1056) {
           _abort();
           // unreachable;
          } else {
           HEAP32[$$lcssa112>>2] = $641;
           $1057 = (($641) + 24|0);
           HEAP32[$1057>>2] = $T$014$i$i$lcssa;
           $1058 = (($641) + 12|0);
           HEAP32[$1058>>2] = $641;
           $1059 = (($641) + 8|0);
           HEAP32[$1059>>2] = $641;
           break L324;
          }
         }
        } while(0);
        $1060 = (($T$0$lcssa$i$i) + 8|0);
        $1061 = HEAP32[$1060>>2]|0;
        $1062 = HEAP32[((296 + 16|0))>>2]|0;
        $1063 = ($T$0$lcssa$i$i>>>0)>=($1062>>>0);
        $1064 = ($1061>>>0)>=($1062>>>0);
        $or$cond$i$i = $1063 & $1064;
        if ($or$cond$i$i) {
         $1065 = (($1061) + 12|0);
         HEAP32[$1065>>2] = $641;
         HEAP32[$1060>>2] = $641;
         $1066 = (($641) + 8|0);
         HEAP32[$1066>>2] = $1061;
         $1067 = (($641) + 12|0);
         HEAP32[$1067>>2] = $T$0$lcssa$i$i;
         $1068 = (($641) + 24|0);
         HEAP32[$1068>>2] = 0;
         break;
        } else {
         _abort();
         // unreachable;
        }
       }
      } while(0);
      $1069 = HEAP32[((296 + 12|0))>>2]|0;
      $1070 = ($1069>>>0)>($nb$0>>>0);
      if ($1070) {
       $1071 = (($1069) - ($nb$0))|0;
       HEAP32[((296 + 12|0))>>2] = $1071;
       $1072 = HEAP32[((296 + 24|0))>>2]|0;
       $1073 = (($1072) + ($nb$0)|0);
       HEAP32[((296 + 24|0))>>2] = $1073;
       $1074 = $1071 | 1;
       $$sum$i32 = (($nb$0) + 4)|0;
       $1075 = (($1072) + ($$sum$i32)|0);
       HEAP32[$1075>>2] = $1074;
       $1076 = $nb$0 | 3;
       $1077 = (($1072) + 4|0);
       HEAP32[$1077>>2] = $1076;
       $1078 = (($1072) + 8|0);
       $mem$0 = $1078;
       break;
      }
     }
    }
    $1079 = (___errno_location()|0);
    HEAP32[$1079>>2] = 12;
    $mem$0 = 0;
   } else {
    $mem$0 = 0;
   }
  }
 } while(0);
 $1080 = HEAP32[((296 + 444|0))>>2]|0;
 $1081 = $1080 & 2;
 $1082 = ($1081|0)==(0);
 if ($1082) {
  $$0 = $mem$0;
  STACKTOP = sp;return ($$0|0);
 }
 (_pthread_mutex_unlock(((296 + 448|0)))|0);
 $$0 = $mem$0;
 STACKTOP = sp;return ($$0|0);
}
function _free($mem) {
 $mem = $mem|0;
 var $$lcssa = 0, $$lcssa73 = 0, $$pre = 0, $$pre$phi66Z2D = 0, $$pre$phi68Z2D = 0, $$pre$phiZ2D = 0, $$pre65 = 0, $$pre67 = 0, $$sum = 0, $$sum16$pre = 0, $$sum17 = 0, $$sum18 = 0, $$sum19 = 0, $$sum2 = 0, $$sum20 = 0, $$sum2324 = 0, $$sum25 = 0, $$sum26 = 0, $$sum28 = 0, $$sum29 = 0;
 var $$sum3 = 0, $$sum30 = 0, $$sum31 = 0, $$sum32 = 0, $$sum33 = 0, $$sum34 = 0, $$sum35 = 0, $$sum36 = 0, $$sum37 = 0, $$sum5 = 0, $$sum67 = 0, $$sum8 = 0, $$sum9 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0;
 var $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0;
 var $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0;
 var $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0;
 var $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0;
 var $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0;
 var $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0;
 var $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0;
 var $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0;
 var $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0;
 var $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0;
 var $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0;
 var $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0;
 var $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $F16$0 = 0, $I18$0 = 0, $I18$0$c = 0, $K19$060 = 0, $R$0 = 0, $R$0$be = 0, $R$0$lcssa = 0, $R$0$ph = 0, $R$1 = 0, $R7$0 = 0, $R7$0$be = 0, $R7$0$lcssa = 0, $R7$0$ph = 0, $R7$1 = 0, $RP$0 = 0, $RP$0$be = 0, $RP$0$lcssa = 0;
 var $RP$0$ph = 0, $RP9$0 = 0, $RP9$0$be = 0, $RP9$0$lcssa = 0, $RP9$0$ph = 0, $T$0$lcssa = 0, $T$059 = 0, $T$059$lcssa = 0, $cond = 0, $cond54 = 0, $or$cond = 0, $p$0 = 0, $psize$0 = 0, $psize$1 = 0, $sp$0$i = 0, $sp$0$in$i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($mem|0)==(0|0);
 if ($0) {
  STACKTOP = sp;return;
 }
 $1 = (($mem) + -8|0);
 $2 = HEAP32[((296 + 444|0))>>2]|0;
 $3 = $2 & 2;
 $4 = ($3|0)==(0);
 if (!($4)) {
  $5 = (_pthread_mutex_lock(((296 + 448|0)))|0);
  $6 = ($5|0)==(0);
  if (!($6)) {
   STACKTOP = sp;return;
  }
 }
 $7 = HEAP32[((296 + 16|0))>>2]|0;
 $8 = ($1>>>0)<($7>>>0);
 if ($8) {
  _abort();
  // unreachable;
 }
 $9 = (($mem) + -4|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = $10 & 3;
 $12 = ($11|0)==(1);
 if ($12) {
  _abort();
  // unreachable;
 }
 $13 = $10 & -8;
 $$sum = (($13) + -8)|0;
 $14 = (($mem) + ($$sum)|0);
 $15 = $10 & 1;
 $16 = ($15|0)==(0);
 do {
  if ($16) {
   $17 = HEAP32[$1>>2]|0;
   $18 = ($11|0)==(0);
   if (!($18)) {
    $$sum2 = (-8 - ($17))|0;
    $19 = (($mem) + ($$sum2)|0);
    $20 = (($17) + ($13))|0;
    $21 = ($19>>>0)<($7>>>0);
    if ($21) {
     _abort();
     // unreachable;
    }
    $22 = HEAP32[((296 + 20|0))>>2]|0;
    $23 = ($19|0)==($22|0);
    if ($23) {
     $$sum3 = (($13) + -4)|0;
     $108 = (($mem) + ($$sum3)|0);
     $109 = HEAP32[$108>>2]|0;
     $110 = $109 & 3;
     $111 = ($110|0)==(3);
     if (!($111)) {
      $p$0 = $19;$psize$0 = $20;
      label = 60;
      break;
     }
     HEAP32[((296 + 8|0))>>2] = $20;
     $112 = $109 & -2;
     HEAP32[$108>>2] = $112;
     $113 = $20 | 1;
     $$sum26 = (($$sum2) + 4)|0;
     $114 = (($mem) + ($$sum26)|0);
     HEAP32[$114>>2] = $113;
     HEAP32[$14>>2] = $20;
     break;
    }
    $24 = $17 >>> 3;
    $25 = ($17>>>0)<(256);
    if ($25) {
     $$sum36 = (($$sum2) + 8)|0;
     $26 = (($mem) + ($$sum36)|0);
     $27 = HEAP32[$26>>2]|0;
     $$sum37 = (($$sum2) + 12)|0;
     $28 = (($mem) + ($$sum37)|0);
     $29 = HEAP32[$28>>2]|0;
     $30 = $24 << 1;
     $31 = ((296 + ($30<<2)|0) + 40|0);
     $32 = ($27|0)==($31|0);
     if (!($32)) {
      $33 = ($27>>>0)<($7>>>0);
      if ($33) {
       _abort();
       // unreachable;
      }
      $34 = (($27) + 12|0);
      $35 = HEAP32[$34>>2]|0;
      $36 = ($35|0)==($19|0);
      if (!($36)) {
       _abort();
       // unreachable;
      }
     }
     $37 = ($29|0)==($27|0);
     if ($37) {
      $38 = 1 << $24;
      $39 = $38 ^ -1;
      $40 = HEAP32[296>>2]|0;
      $41 = $40 & $39;
      HEAP32[296>>2] = $41;
      $p$0 = $19;$psize$0 = $20;
      label = 60;
      break;
     }
     $42 = ($29|0)==($31|0);
     if ($42) {
      $$pre67 = (($29) + 8|0);
      $$pre$phi68Z2D = $$pre67;
     } else {
      $43 = ($29>>>0)<($7>>>0);
      if ($43) {
       _abort();
       // unreachable;
      }
      $44 = (($29) + 8|0);
      $45 = HEAP32[$44>>2]|0;
      $46 = ($45|0)==($19|0);
      if ($46) {
       $$pre$phi68Z2D = $44;
      } else {
       _abort();
       // unreachable;
      }
     }
     $47 = (($27) + 12|0);
     HEAP32[$47>>2] = $29;
     HEAP32[$$pre$phi68Z2D>>2] = $27;
     $p$0 = $19;$psize$0 = $20;
     label = 60;
     break;
    }
    $$sum28 = (($$sum2) + 24)|0;
    $48 = (($mem) + ($$sum28)|0);
    $49 = HEAP32[$48>>2]|0;
    $$sum29 = (($$sum2) + 12)|0;
    $50 = (($mem) + ($$sum29)|0);
    $51 = HEAP32[$50>>2]|0;
    $52 = ($51|0)==($19|0);
    do {
     if ($52) {
      $$sum31 = (($$sum2) + 20)|0;
      $62 = (($mem) + ($$sum31)|0);
      $63 = HEAP32[$62>>2]|0;
      $64 = ($63|0)==(0|0);
      if ($64) {
       $$sum30 = (($$sum2) + 16)|0;
       $65 = (($mem) + ($$sum30)|0);
       $66 = HEAP32[$65>>2]|0;
       $67 = ($66|0)==(0|0);
       if ($67) {
        $R$1 = 0;
        break;
       } else {
        $R$0$ph = $66;$RP$0$ph = $65;
       }
      } else {
       $R$0$ph = $63;$RP$0$ph = $62;
      }
      $R$0 = $R$0$ph;$RP$0 = $RP$0$ph;
      while(1) {
       $68 = (($R$0) + 20|0);
       $69 = HEAP32[$68>>2]|0;
       $70 = ($69|0)==(0|0);
       if ($70) {
        $71 = (($R$0) + 16|0);
        $72 = HEAP32[$71>>2]|0;
        $73 = ($72|0)==(0|0);
        if ($73) {
         $R$0$lcssa = $R$0;$RP$0$lcssa = $RP$0;
         break;
        } else {
         $R$0$be = $72;$RP$0$be = $71;
        }
       } else {
        $R$0$be = $69;$RP$0$be = $68;
       }
       $R$0 = $R$0$be;$RP$0 = $RP$0$be;
      }
      $74 = ($RP$0$lcssa>>>0)<($7>>>0);
      if ($74) {
       _abort();
       // unreachable;
      } else {
       HEAP32[$RP$0$lcssa>>2] = 0;
       $R$1 = $R$0$lcssa;
       break;
      }
     } else {
      $$sum35 = (($$sum2) + 8)|0;
      $53 = (($mem) + ($$sum35)|0);
      $54 = HEAP32[$53>>2]|0;
      $55 = ($54>>>0)<($7>>>0);
      if ($55) {
       _abort();
       // unreachable;
      }
      $56 = (($54) + 12|0);
      $57 = HEAP32[$56>>2]|0;
      $58 = ($57|0)==($19|0);
      if (!($58)) {
       _abort();
       // unreachable;
      }
      $59 = (($51) + 8|0);
      $60 = HEAP32[$59>>2]|0;
      $61 = ($60|0)==($19|0);
      if ($61) {
       HEAP32[$56>>2] = $51;
       HEAP32[$59>>2] = $54;
       $R$1 = $51;
       break;
      } else {
       _abort();
       // unreachable;
      }
     }
    } while(0);
    $75 = ($49|0)==(0|0);
    if ($75) {
     $p$0 = $19;$psize$0 = $20;
     label = 60;
    } else {
     $$sum32 = (($$sum2) + 28)|0;
     $76 = (($mem) + ($$sum32)|0);
     $77 = HEAP32[$76>>2]|0;
     $78 = ((296 + ($77<<2)|0) + 304|0);
     $79 = HEAP32[$78>>2]|0;
     $80 = ($19|0)==($79|0);
     if ($80) {
      HEAP32[$78>>2] = $R$1;
      $cond = ($R$1|0)==(0|0);
      if ($cond) {
       $81 = 1 << $77;
       $82 = $81 ^ -1;
       $83 = HEAP32[((296 + 4|0))>>2]|0;
       $84 = $83 & $82;
       HEAP32[((296 + 4|0))>>2] = $84;
       $p$0 = $19;$psize$0 = $20;
       label = 60;
       break;
      }
     } else {
      $85 = HEAP32[((296 + 16|0))>>2]|0;
      $86 = ($49>>>0)<($85>>>0);
      if ($86) {
       _abort();
       // unreachable;
      }
      $87 = (($49) + 16|0);
      $88 = HEAP32[$87>>2]|0;
      $89 = ($88|0)==($19|0);
      if ($89) {
       HEAP32[$87>>2] = $R$1;
      } else {
       $90 = (($49) + 20|0);
       HEAP32[$90>>2] = $R$1;
      }
      $91 = ($R$1|0)==(0|0);
      if ($91) {
       $p$0 = $19;$psize$0 = $20;
       label = 60;
       break;
      }
     }
     $92 = HEAP32[((296 + 16|0))>>2]|0;
     $93 = ($R$1>>>0)<($92>>>0);
     if ($93) {
      _abort();
      // unreachable;
     }
     $94 = (($R$1) + 24|0);
     HEAP32[$94>>2] = $49;
     $$sum33 = (($$sum2) + 16)|0;
     $95 = (($mem) + ($$sum33)|0);
     $96 = HEAP32[$95>>2]|0;
     $97 = ($96|0)==(0|0);
     do {
      if (!($97)) {
       $98 = ($96>>>0)<($92>>>0);
       if ($98) {
        _abort();
        // unreachable;
       } else {
        $99 = (($R$1) + 16|0);
        HEAP32[$99>>2] = $96;
        $100 = (($96) + 24|0);
        HEAP32[$100>>2] = $R$1;
        break;
       }
      }
     } while(0);
     $$sum34 = (($$sum2) + 20)|0;
     $101 = (($mem) + ($$sum34)|0);
     $102 = HEAP32[$101>>2]|0;
     $103 = ($102|0)==(0|0);
     if ($103) {
      $p$0 = $19;$psize$0 = $20;
      label = 60;
     } else {
      $104 = HEAP32[((296 + 16|0))>>2]|0;
      $105 = ($102>>>0)<($104>>>0);
      if ($105) {
       _abort();
       // unreachable;
      } else {
       $106 = (($R$1) + 20|0);
       HEAP32[$106>>2] = $102;
       $107 = (($102) + 24|0);
       HEAP32[$107>>2] = $R$1;
       $p$0 = $19;$psize$0 = $20;
       label = 60;
       break;
      }
     }
    }
   }
  } else {
   $p$0 = $1;$psize$0 = $13;
   label = 60;
  }
 } while(0);
 do {
  if ((label|0) == 60) {
   $115 = ($p$0>>>0)<($14>>>0);
   if (!($115)) {
    _abort();
    // unreachable;
   }
   $$sum25 = (($13) + -4)|0;
   $116 = (($mem) + ($$sum25)|0);
   $117 = HEAP32[$116>>2]|0;
   $118 = $117 & 1;
   $119 = ($118|0)==(0);
   if ($119) {
    _abort();
    // unreachable;
   }
   $120 = $117 & 2;
   $121 = ($120|0)==(0);
   if ($121) {
    $122 = HEAP32[((296 + 24|0))>>2]|0;
    $123 = ($14|0)==($122|0);
    if ($123) {
     $124 = HEAP32[((296 + 12|0))>>2]|0;
     $125 = (($124) + ($psize$0))|0;
     HEAP32[((296 + 12|0))>>2] = $125;
     HEAP32[((296 + 24|0))>>2] = $p$0;
     $126 = $125 | 1;
     $127 = (($p$0) + 4|0);
     HEAP32[$127>>2] = $126;
     $128 = HEAP32[((296 + 20|0))>>2]|0;
     $129 = ($p$0|0)==($128|0);
     if (!($129)) {
      break;
     }
     HEAP32[((296 + 20|0))>>2] = 0;
     HEAP32[((296 + 8|0))>>2] = 0;
     break;
    }
    $130 = HEAP32[((296 + 20|0))>>2]|0;
    $131 = ($14|0)==($130|0);
    if ($131) {
     $132 = HEAP32[((296 + 8|0))>>2]|0;
     $133 = (($132) + ($psize$0))|0;
     HEAP32[((296 + 8|0))>>2] = $133;
     HEAP32[((296 + 20|0))>>2] = $p$0;
     $134 = $133 | 1;
     $135 = (($p$0) + 4|0);
     HEAP32[$135>>2] = $134;
     $136 = (($p$0) + ($133)|0);
     HEAP32[$136>>2] = $133;
     break;
    }
    $137 = $117 & -8;
    $138 = (($137) + ($psize$0))|0;
    $139 = $117 >>> 3;
    $140 = ($117>>>0)<(256);
    do {
     if ($140) {
      $141 = (($mem) + ($13)|0);
      $142 = HEAP32[$141>>2]|0;
      $$sum2324 = $13 | 4;
      $143 = (($mem) + ($$sum2324)|0);
      $144 = HEAP32[$143>>2]|0;
      $145 = $139 << 1;
      $146 = ((296 + ($145<<2)|0) + 40|0);
      $147 = ($142|0)==($146|0);
      if (!($147)) {
       $148 = HEAP32[((296 + 16|0))>>2]|0;
       $149 = ($142>>>0)<($148>>>0);
       if ($149) {
        _abort();
        // unreachable;
       }
       $150 = (($142) + 12|0);
       $151 = HEAP32[$150>>2]|0;
       $152 = ($151|0)==($14|0);
       if (!($152)) {
        _abort();
        // unreachable;
       }
      }
      $153 = ($144|0)==($142|0);
      if ($153) {
       $154 = 1 << $139;
       $155 = $154 ^ -1;
       $156 = HEAP32[296>>2]|0;
       $157 = $156 & $155;
       HEAP32[296>>2] = $157;
       break;
      }
      $158 = ($144|0)==($146|0);
      if ($158) {
       $$pre65 = (($144) + 8|0);
       $$pre$phi66Z2D = $$pre65;
      } else {
       $159 = HEAP32[((296 + 16|0))>>2]|0;
       $160 = ($144>>>0)<($159>>>0);
       if ($160) {
        _abort();
        // unreachable;
       }
       $161 = (($144) + 8|0);
       $162 = HEAP32[$161>>2]|0;
       $163 = ($162|0)==($14|0);
       if ($163) {
        $$pre$phi66Z2D = $161;
       } else {
        _abort();
        // unreachable;
       }
      }
      $164 = (($142) + 12|0);
      HEAP32[$164>>2] = $144;
      HEAP32[$$pre$phi66Z2D>>2] = $142;
     } else {
      $$sum5 = (($13) + 16)|0;
      $165 = (($mem) + ($$sum5)|0);
      $166 = HEAP32[$165>>2]|0;
      $$sum67 = $13 | 4;
      $167 = (($mem) + ($$sum67)|0);
      $168 = HEAP32[$167>>2]|0;
      $169 = ($168|0)==($14|0);
      do {
       if ($169) {
        $$sum9 = (($13) + 12)|0;
        $180 = (($mem) + ($$sum9)|0);
        $181 = HEAP32[$180>>2]|0;
        $182 = ($181|0)==(0|0);
        if ($182) {
         $$sum8 = (($13) + 8)|0;
         $183 = (($mem) + ($$sum8)|0);
         $184 = HEAP32[$183>>2]|0;
         $185 = ($184|0)==(0|0);
         if ($185) {
          $R7$1 = 0;
          break;
         } else {
          $R7$0$ph = $184;$RP9$0$ph = $183;
         }
        } else {
         $R7$0$ph = $181;$RP9$0$ph = $180;
        }
        $R7$0 = $R7$0$ph;$RP9$0 = $RP9$0$ph;
        while(1) {
         $186 = (($R7$0) + 20|0);
         $187 = HEAP32[$186>>2]|0;
         $188 = ($187|0)==(0|0);
         if ($188) {
          $189 = (($R7$0) + 16|0);
          $190 = HEAP32[$189>>2]|0;
          $191 = ($190|0)==(0|0);
          if ($191) {
           $R7$0$lcssa = $R7$0;$RP9$0$lcssa = $RP9$0;
           break;
          } else {
           $R7$0$be = $190;$RP9$0$be = $189;
          }
         } else {
          $R7$0$be = $187;$RP9$0$be = $186;
         }
         $R7$0 = $R7$0$be;$RP9$0 = $RP9$0$be;
        }
        $192 = HEAP32[((296 + 16|0))>>2]|0;
        $193 = ($RP9$0$lcssa>>>0)<($192>>>0);
        if ($193) {
         _abort();
         // unreachable;
        } else {
         HEAP32[$RP9$0$lcssa>>2] = 0;
         $R7$1 = $R7$0$lcssa;
         break;
        }
       } else {
        $170 = (($mem) + ($13)|0);
        $171 = HEAP32[$170>>2]|0;
        $172 = HEAP32[((296 + 16|0))>>2]|0;
        $173 = ($171>>>0)<($172>>>0);
        if ($173) {
         _abort();
         // unreachable;
        }
        $174 = (($171) + 12|0);
        $175 = HEAP32[$174>>2]|0;
        $176 = ($175|0)==($14|0);
        if (!($176)) {
         _abort();
         // unreachable;
        }
        $177 = (($168) + 8|0);
        $178 = HEAP32[$177>>2]|0;
        $179 = ($178|0)==($14|0);
        if ($179) {
         HEAP32[$174>>2] = $168;
         HEAP32[$177>>2] = $171;
         $R7$1 = $168;
         break;
        } else {
         _abort();
         // unreachable;
        }
       }
      } while(0);
      $194 = ($166|0)==(0|0);
      if (!($194)) {
       $$sum18 = (($13) + 20)|0;
       $195 = (($mem) + ($$sum18)|0);
       $196 = HEAP32[$195>>2]|0;
       $197 = ((296 + ($196<<2)|0) + 304|0);
       $198 = HEAP32[$197>>2]|0;
       $199 = ($14|0)==($198|0);
       if ($199) {
        HEAP32[$197>>2] = $R7$1;
        $cond54 = ($R7$1|0)==(0|0);
        if ($cond54) {
         $200 = 1 << $196;
         $201 = $200 ^ -1;
         $202 = HEAP32[((296 + 4|0))>>2]|0;
         $203 = $202 & $201;
         HEAP32[((296 + 4|0))>>2] = $203;
         break;
        }
       } else {
        $204 = HEAP32[((296 + 16|0))>>2]|0;
        $205 = ($166>>>0)<($204>>>0);
        if ($205) {
         _abort();
         // unreachable;
        }
        $206 = (($166) + 16|0);
        $207 = HEAP32[$206>>2]|0;
        $208 = ($207|0)==($14|0);
        if ($208) {
         HEAP32[$206>>2] = $R7$1;
        } else {
         $209 = (($166) + 20|0);
         HEAP32[$209>>2] = $R7$1;
        }
        $210 = ($R7$1|0)==(0|0);
        if ($210) {
         break;
        }
       }
       $211 = HEAP32[((296 + 16|0))>>2]|0;
       $212 = ($R7$1>>>0)<($211>>>0);
       if ($212) {
        _abort();
        // unreachable;
       }
       $213 = (($R7$1) + 24|0);
       HEAP32[$213>>2] = $166;
       $$sum19 = (($13) + 8)|0;
       $214 = (($mem) + ($$sum19)|0);
       $215 = HEAP32[$214>>2]|0;
       $216 = ($215|0)==(0|0);
       do {
        if (!($216)) {
         $217 = ($215>>>0)<($211>>>0);
         if ($217) {
          _abort();
          // unreachable;
         } else {
          $218 = (($R7$1) + 16|0);
          HEAP32[$218>>2] = $215;
          $219 = (($215) + 24|0);
          HEAP32[$219>>2] = $R7$1;
          break;
         }
        }
       } while(0);
       $$sum20 = (($13) + 12)|0;
       $220 = (($mem) + ($$sum20)|0);
       $221 = HEAP32[$220>>2]|0;
       $222 = ($221|0)==(0|0);
       if (!($222)) {
        $223 = HEAP32[((296 + 16|0))>>2]|0;
        $224 = ($221>>>0)<($223>>>0);
        if ($224) {
         _abort();
         // unreachable;
        } else {
         $225 = (($R7$1) + 20|0);
         HEAP32[$225>>2] = $221;
         $226 = (($221) + 24|0);
         HEAP32[$226>>2] = $R7$1;
         break;
        }
       }
      }
     }
    } while(0);
    $227 = $138 | 1;
    $228 = (($p$0) + 4|0);
    HEAP32[$228>>2] = $227;
    $229 = (($p$0) + ($138)|0);
    HEAP32[$229>>2] = $138;
    $230 = HEAP32[((296 + 20|0))>>2]|0;
    $231 = ($p$0|0)==($230|0);
    if ($231) {
     HEAP32[((296 + 8|0))>>2] = $138;
     break;
    } else {
     $psize$1 = $138;
    }
   } else {
    $232 = $117 & -2;
    HEAP32[$116>>2] = $232;
    $233 = $psize$0 | 1;
    $234 = (($p$0) + 4|0);
    HEAP32[$234>>2] = $233;
    $235 = (($p$0) + ($psize$0)|0);
    HEAP32[$235>>2] = $psize$0;
    $psize$1 = $psize$0;
   }
   $236 = $psize$1 >>> 3;
   $237 = ($psize$1>>>0)<(256);
   if ($237) {
    $238 = $236 << 1;
    $239 = ((296 + ($238<<2)|0) + 40|0);
    $240 = HEAP32[296>>2]|0;
    $241 = 1 << $236;
    $242 = $240 & $241;
    $243 = ($242|0)==(0);
    if ($243) {
     $244 = $240 | $241;
     HEAP32[296>>2] = $244;
     $$sum16$pre = (($238) + 2)|0;
     $$pre = ((296 + ($$sum16$pre<<2)|0) + 40|0);
     $$pre$phiZ2D = $$pre;$F16$0 = $239;
    } else {
     $$sum17 = (($238) + 2)|0;
     $245 = ((296 + ($$sum17<<2)|0) + 40|0);
     $246 = HEAP32[$245>>2]|0;
     $247 = HEAP32[((296 + 16|0))>>2]|0;
     $248 = ($246>>>0)<($247>>>0);
     if ($248) {
      _abort();
      // unreachable;
     } else {
      $$pre$phiZ2D = $245;$F16$0 = $246;
     }
    }
    HEAP32[$$pre$phiZ2D>>2] = $p$0;
    $249 = (($F16$0) + 12|0);
    HEAP32[$249>>2] = $p$0;
    $250 = (($p$0) + 8|0);
    HEAP32[$250>>2] = $F16$0;
    $251 = (($p$0) + 12|0);
    HEAP32[$251>>2] = $239;
    break;
   }
   $252 = $psize$1 >>> 8;
   $253 = ($252|0)==(0);
   if ($253) {
    $I18$0 = 0;
   } else {
    $254 = ($psize$1>>>0)>(16777215);
    if ($254) {
     $I18$0 = 31;
    } else {
     $255 = (($252) + 1048320)|0;
     $256 = $255 >>> 16;
     $257 = $256 & 8;
     $258 = $252 << $257;
     $259 = (($258) + 520192)|0;
     $260 = $259 >>> 16;
     $261 = $260 & 4;
     $262 = $261 | $257;
     $263 = $258 << $261;
     $264 = (($263) + 245760)|0;
     $265 = $264 >>> 16;
     $266 = $265 & 2;
     $267 = $262 | $266;
     $268 = (14 - ($267))|0;
     $269 = $263 << $266;
     $270 = $269 >>> 15;
     $271 = (($268) + ($270))|0;
     $272 = $271 << 1;
     $273 = (($271) + 7)|0;
     $274 = $psize$1 >>> $273;
     $275 = $274 & 1;
     $276 = $275 | $272;
     $I18$0 = $276;
    }
   }
   $277 = ((296 + ($I18$0<<2)|0) + 304|0);
   $278 = (($p$0) + 28|0);
   $I18$0$c = $I18$0;
   HEAP32[$278>>2] = $I18$0$c;
   $279 = (($p$0) + 20|0);
   HEAP32[$279>>2] = 0;
   $280 = (($p$0) + 16|0);
   HEAP32[$280>>2] = 0;
   $281 = HEAP32[((296 + 4|0))>>2]|0;
   $282 = 1 << $I18$0;
   $283 = $281 & $282;
   $284 = ($283|0)==(0);
   L201: do {
    if ($284) {
     $285 = $281 | $282;
     HEAP32[((296 + 4|0))>>2] = $285;
     HEAP32[$277>>2] = $p$0;
     $286 = (($p$0) + 24|0);
     HEAP32[$286>>2] = $277;
     $287 = (($p$0) + 12|0);
     HEAP32[$287>>2] = $p$0;
     $288 = (($p$0) + 8|0);
     HEAP32[$288>>2] = $p$0;
    } else {
     $289 = HEAP32[$277>>2]|0;
     $290 = ($I18$0|0)==(31);
     if ($290) {
      $298 = 0;
     } else {
      $291 = $I18$0 >>> 1;
      $292 = (25 - ($291))|0;
      $298 = $292;
     }
     $293 = (($289) + 4|0);
     $294 = HEAP32[$293>>2]|0;
     $295 = $294 & -8;
     $296 = ($295|0)==($psize$1|0);
     do {
      if ($296) {
       $T$0$lcssa = $289;
      } else {
       $297 = $psize$1 << $298;
       $K19$060 = $297;$T$059 = $289;
       while(1) {
        $305 = $K19$060 >>> 31;
        $306 = ((($T$059) + ($305<<2)|0) + 16|0);
        $301 = HEAP32[$306>>2]|0;
        $307 = ($301|0)==(0|0);
        if ($307) {
         $$lcssa = $306;$T$059$lcssa = $T$059;
         break;
        }
        $299 = $K19$060 << 1;
        $300 = (($301) + 4|0);
        $302 = HEAP32[$300>>2]|0;
        $303 = $302 & -8;
        $304 = ($303|0)==($psize$1|0);
        if ($304) {
         $$lcssa73 = $301;
         label = 139;
         break;
        } else {
         $K19$060 = $299;$T$059 = $301;
        }
       }
       if ((label|0) == 139) {
        $T$0$lcssa = $$lcssa73;
        break;
       }
       $308 = HEAP32[((296 + 16|0))>>2]|0;
       $309 = ($$lcssa>>>0)<($308>>>0);
       if ($309) {
        _abort();
        // unreachable;
       } else {
        HEAP32[$$lcssa>>2] = $p$0;
        $310 = (($p$0) + 24|0);
        HEAP32[$310>>2] = $T$059$lcssa;
        $311 = (($p$0) + 12|0);
        HEAP32[$311>>2] = $p$0;
        $312 = (($p$0) + 8|0);
        HEAP32[$312>>2] = $p$0;
        break L201;
       }
      }
     } while(0);
     $313 = (($T$0$lcssa) + 8|0);
     $314 = HEAP32[$313>>2]|0;
     $315 = HEAP32[((296 + 16|0))>>2]|0;
     $316 = ($T$0$lcssa>>>0)>=($315>>>0);
     $317 = ($314>>>0)>=($315>>>0);
     $or$cond = $316 & $317;
     if ($or$cond) {
      $318 = (($314) + 12|0);
      HEAP32[$318>>2] = $p$0;
      HEAP32[$313>>2] = $p$0;
      $319 = (($p$0) + 8|0);
      HEAP32[$319>>2] = $314;
      $320 = (($p$0) + 12|0);
      HEAP32[$320>>2] = $T$0$lcssa;
      $321 = (($p$0) + 24|0);
      HEAP32[$321>>2] = 0;
      break;
     } else {
      _abort();
      // unreachable;
     }
    }
   } while(0);
   $322 = HEAP32[((296 + 32|0))>>2]|0;
   $323 = (($322) + -1)|0;
   HEAP32[((296 + 32|0))>>2] = $323;
   $324 = ($323|0)==(0);
   if ($324) {
    $sp$0$in$i = ((296 + 484|0));
    while(1) {
     $sp$0$i = HEAP32[$sp$0$in$i>>2]|0;
     $325 = ($sp$0$i|0)==(0|0);
     $326 = (($sp$0$i) + 8|0);
     if ($325) {
      break;
     } else {
      $sp$0$in$i = $326;
     }
    }
    HEAP32[((296 + 32|0))>>2] = -1;
   }
  }
 } while(0);
 $327 = HEAP32[((296 + 444|0))>>2]|0;
 $328 = $327 & 2;
 $329 = ($328|0)==(0);
 if ($329) {
  STACKTOP = sp;return;
 }
 (_pthread_mutex_unlock(((296 + 448|0)))|0);
 STACKTOP = sp;return;
}
function __pthread_nsecs_until($at) {
 $at = $at|0;
 var $0 = 0, $1 = 0.0, $10 = 0.0, $11 = 0, $12 = 0, $13 = 0.0, $14 = 0.0, $15 = 0.0, $2 = 0.0, $3 = 0, $4 = 0, $5 = 0.0, $6 = 0.0, $7 = 0.0, $8 = 0, $9 = 0.0, $t = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $t = sp;
 (_gettimeofday(($t|0),(0|0))|0);
 $0 = HEAP32[$t>>2]|0;
 $1 = (+($0|0));
 $2 = $1 * 1.0E+9;
 $3 = (($t) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = (+($4|0));
 $6 = $5 * 1000.0;
 $7 = $2 + $6;
 $8 = HEAP32[$at>>2]|0;
 $9 = (+($8|0));
 $10 = $9 * 1.0E+9;
 $11 = (($at) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (+($12|0));
 $14 = $10 + $13;
 $15 = $14 - $7;
 STACKTOP = sp;return (+$15);
}
function _pthread_setcancelstate($new,$old) {
 $new = $new|0;
 $old = $old|0;
 var $$0 = 0, $$pre = 0, $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($new>>>0)>(1);
 if ($0) {
  $$0 = 22;
  STACKTOP = sp;return ($$0|0);
 }
 $1 = (_pthread_self()|0);
 $2 = ($old|0)==(0|0);
 $$pre = (($1) + 68|0);
 if (!($2)) {
  $3 = Atomics_load(HEAP32,$$pre>>2);
  HEAP32[$old>>2] = $3;
 }
 Atomics_store(HEAP32,$$pre>>2,$new);
 $$0 = 0;
 STACKTOP = sp;return ($$0|0);
}
function __pthread_isduecanceled($pthread_ptr) {
 $pthread_ptr = $pthread_ptr|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = HEAP32[$pthread_ptr>>2]|0;
 $1 = ($0|0)==(2);
 $2 = $1&1;
 STACKTOP = sp;return ($2|0);
}
function _emscripten_sync_run_in_main_thread($call) {
 $call = $call|0;
 var $$expand_i1_val = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $call_queue$init$val = 0, $call_queue$init$val$pre_trunc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($call|0)==(0|0);
 if ($0) {
  ___assert_fail((832|0),(840|0),387,(928|0));
  // unreachable;
 }
 $1 = (_emscripten_is_main_runtime_thread()|0);
 $2 = ($1|0)==(0);
 if (!($2)) {
  __do_call($call);
  STACKTOP = sp;return;
 }
 (_pthread_mutex_lock(968)|0);
 $call_queue$init$val$pre_trunc = HEAP8[1512>>0]|0;
 $call_queue$init$val = $call_queue$init$val$pre_trunc&1;
 if (!($call_queue$init$val)) {
  $$expand_i1_val = 1;
  HEAP8[1512>>0] = $$expand_i1_val;
 }
 $3 = HEAP32[1520>>2]|0;
 $4 = ($3|0)<(128);
 if (!($4)) {
  ___assert_fail((1528|0),(840|0),396,(928|0));
  // unreachable;
 }
 $5 = (1000 + ($3<<2)|0);
 HEAP32[$5>>2] = $call;
 $6 = (($3) + 1)|0;
 HEAP32[1520>>2] = $6;
 $7 = ($3|0)==(0);
 if ($7) {
  _emscripten_asm_const((1568|0));
 }
 (_pthread_mutex_unlock(968)|0);
 $8 = (($call) + 4|0);
 while(1) {
  $9 = (_emscripten_futex_wait(($8|0),0,inf)|0);
  $10 = ($9|0)==(0);
  if ($10) {
   break;
  }
  $11 = HEAP32[$8>>2]|0;
  $12 = ($11|0)==(0);
  if (!($12)) {
   break;
  }
 }
 STACKTOP = sp;return;
}
function _emscripten_sync_run_in_main_thread_1($function,$arg1) {
 $function = $function|0;
 $arg1 = $arg1|0;
 var $0 = 0, $1 = 0, $2 = 0, $q = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $q = sp;
 dest=$q+0|0; stop=dest+80|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 HEAP32[$q>>2] = $function;
 $0 = (($q) + 8|0);
 HEAP32[$0>>2] = $arg1;
 $1 = (($q) + 72|0);
 HEAP32[$1>>2] = 0;
 _emscripten_sync_run_in_main_thread($q);
 $2 = HEAP32[$1>>2]|0;
 STACKTOP = sp;return ($2|0);
}
function _emscripten_sync_run_in_main_thread_2($function,$arg1,$arg2) {
 $function = $function|0;
 $arg1 = $arg1|0;
 $arg2 = $arg2|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $q = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $q = sp;
 dest=$q+0|0; stop=dest+80|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 HEAP32[$q>>2] = $function;
 $0 = (($q) + 8|0);
 HEAP32[$0>>2] = $arg1;
 $1 = (($q) + 16|0);
 HEAP32[$1>>2] = $arg2;
 $2 = (($q) + 72|0);
 HEAP32[$2>>2] = 0;
 _emscripten_sync_run_in_main_thread($q);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _emscripten_sync_run_in_main_thread_3($function,$arg1,$arg2,$arg3) {
 $function = $function|0;
 $arg1 = $arg1|0;
 $arg2 = $arg2|0;
 $arg3 = $arg3|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $q = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $q = sp;
 dest=$q+0|0; stop=dest+80|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 HEAP32[$q>>2] = $function;
 $0 = (($q) + 8|0);
 HEAP32[$0>>2] = $arg1;
 $1 = (($q) + 16|0);
 HEAP32[$1>>2] = $arg2;
 $2 = (($q) + 24|0);
 HEAP32[$2>>2] = $arg3;
 $3 = (($q) + 72|0);
 HEAP32[$3>>2] = 0;
 _emscripten_sync_run_in_main_thread($q);
 $4 = HEAP32[$3>>2]|0;
 STACKTOP = sp;return ($4|0);
}
function _emscripten_sync_run_in_main_thread_4($function,$arg1,$arg2,$arg3,$arg4) {
 $function = $function|0;
 $arg1 = $arg1|0;
 $arg2 = $arg2|0;
 $arg3 = $arg3|0;
 $arg4 = $arg4|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $q = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $q = sp;
 dest=$q+0|0; stop=dest+80|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 HEAP32[$q>>2] = $function;
 $0 = (($q) + 8|0);
 HEAP32[$0>>2] = $arg1;
 $1 = (($q) + 16|0);
 HEAP32[$1>>2] = $arg2;
 $2 = (($q) + 24|0);
 HEAP32[$2>>2] = $arg3;
 $3 = (($q) + 32|0);
 HEAP32[$3>>2] = $arg4;
 $4 = (($q) + 72|0);
 HEAP32[$4>>2] = 0;
 _emscripten_sync_run_in_main_thread($q);
 $5 = HEAP32[$4>>2]|0;
 STACKTOP = sp;return ($5|0);
}
function _emscripten_sync_run_in_main_thread_5($function,$arg1,$arg2,$arg3,$arg4,$arg5) {
 $function = $function|0;
 $arg1 = $arg1|0;
 $arg2 = $arg2|0;
 $arg3 = $arg3|0;
 $arg4 = $arg4|0;
 $arg5 = $arg5|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $q = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $q = sp;
 dest=$q+0|0; stop=dest+80|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 HEAP32[$q>>2] = $function;
 $0 = (($q) + 8|0);
 HEAP32[$0>>2] = $arg1;
 $1 = (($q) + 16|0);
 HEAP32[$1>>2] = $arg2;
 $2 = (($q) + 24|0);
 HEAP32[$2>>2] = $arg3;
 $3 = (($q) + 32|0);
 HEAP32[$3>>2] = $arg4;
 $4 = (($q) + 40|0);
 HEAP32[$4>>2] = $arg5;
 $5 = (($q) + 72|0);
 HEAP32[$5>>2] = 0;
 _emscripten_sync_run_in_main_thread($q);
 $6 = HEAP32[$5>>2]|0;
 STACKTOP = sp;return ($6|0);
}
function _emscripten_sync_run_in_main_thread_6($function,$arg1,$arg2,$arg3,$arg4,$arg5,$arg6) {
 $function = $function|0;
 $arg1 = $arg1|0;
 $arg2 = $arg2|0;
 $arg3 = $arg3|0;
 $arg4 = $arg4|0;
 $arg5 = $arg5|0;
 $arg6 = $arg6|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $q = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $q = sp;
 dest=$q+0|0; stop=dest+80|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 HEAP32[$q>>2] = $function;
 $0 = (($q) + 8|0);
 HEAP32[$0>>2] = $arg1;
 $1 = (($q) + 16|0);
 HEAP32[$1>>2] = $arg2;
 $2 = (($q) + 24|0);
 HEAP32[$2>>2] = $arg3;
 $3 = (($q) + 32|0);
 HEAP32[$3>>2] = $arg4;
 $4 = (($q) + 40|0);
 HEAP32[$4>>2] = $arg5;
 $5 = (($q) + 48|0);
 HEAP32[$5>>2] = $arg6;
 $6 = (($q) + 72|0);
 HEAP32[$6>>2] = 0;
 _emscripten_sync_run_in_main_thread($q);
 $7 = HEAP32[$6>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _emscripten_main_thread_process_queued_calls() {
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i$01 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (_emscripten_is_main_runtime_thread()|0);
 $1 = ($0|0)==(0);
 if ($1) {
  ___assert_fail((1624|0),(840|0),480,(1752|0));
  // unreachable;
 }
 $2 = (_emscripten_is_main_runtime_thread()|0);
 $3 = ($2|0)==(0);
 if ($3) {
  STACKTOP = sp;return;
 }
 (_pthread_mutex_lock(968)|0);
 $4 = HEAP32[1520>>2]|0;
 $5 = ($4|0)>(0);
 if ($5) {
  $i$01 = 0;
  while(1) {
   $6 = (1000 + ($i$01<<2)|0);
   $7 = HEAP32[$6>>2]|0;
   __do_call($7);
   $8 = (($i$01) + 1)|0;
   $9 = HEAP32[1520>>2]|0;
   $10 = ($8|0)<($9|0);
   if ($10) {
    $i$01 = $8;
   } else {
    break;
   }
  }
 }
 HEAP32[1520>>2] = 0;
 (_pthread_mutex_unlock(968)|0);
 STACKTOP = sp;return;
}
function __Atomics_load_HEAPF32_emulated($addr) {
 $addr = $addr|0;
 var $0 = 0, $1 = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = Atomics_load(HEAP32, $addr>>2);
 $1 = (HEAP32[tempDoublePtr>>2]=$0,+HEAPF32[tempDoublePtr>>2]);
 STACKTOP = sp;return (+$1);
}
function __Atomics_load_HEAPF64_emulated($addr) {
 $addr = $addr|0;
 var $$lcssa = 0, $$lcssa3 = 0, $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = $addr;
 $1 = (($0) + 4)|0;
 $2 = $1;
 while(1) {
  $3 = Atomics_load(HEAP32, $addr>>2);
  $4 = Atomics_load(HEAP32, $2>>2);
  $5 = Atomics_load(HEAP32, $addr>>2);
  $6 = ($3|0)==($5|0);
  if ($6) {
   $$lcssa = $3;$$lcssa3 = $4;
   break;
  }
 }
 HEAP32[tempDoublePtr>>2] = $$lcssa;HEAP32[tempDoublePtr+4>>2] = $$lcssa3;$7 = +HEAPF64[tempDoublePtr>>3];
 STACKTOP = sp;return (+$7);
}
function __Atomics_store_HEAPF32_emulated($addr,$val) {
 $addr = $addr|0;
 $val = +$val;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (HEAPF32[tempDoublePtr>>2]=$val,HEAP32[tempDoublePtr>>2]|0);
 $1 = Atomics_store(HEAP32, $addr>>2, $0);
 STACKTOP = sp;return;
}
function __Atomics_store_HEAPF64_emulated($addr,$val) {
 $addr = $addr|0;
 $val = +$val;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 HEAPF64[tempDoublePtr>>3] = $val;$0 = HEAP32[tempDoublePtr>>2]|0;
 $1 = HEAP32[tempDoublePtr+4>>2]|0;
 $2 = $addr;
 $3 = (($2) + 4)|0;
 $4 = $3;
 while(1) {
  $5 = Atomics_store(HEAP32, $addr>>2, $0);
  $6 = Atomics_store(HEAP32, $4>>2, $1);
  $7 = Atomics_load(HEAP32, $addr>>2);
  $8 = ($0|0)==($7|0);
  if ($8) {
   break;
  }
 }
 STACKTOP = sp;return;
}
function ___timedwait($addr,$val,$clk,$at,$cleanup,$arg,$priv) {
 $addr = $addr|0;
 $val = $val|0;
 $clk = $clk|0;
 $at = $at|0;
 $cleanup = $cleanup|0;
 $arg = $arg|0;
 $priv = $priv|0;
 var $$0$i = 0, $$0$i$ph = 0, $$0$i$ph6 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0.0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cs = 0, $to$i = 0, $waitNsecs$0$us$i = 0.0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $to$i = sp + 8|0;
 $cs = sp;
 $0 = ($cleanup|0)!=(0|0);
 if (!($0)) {
  (_pthread_setcancelstate(1,$cs)|0);
 }
 _pthread_cleanup_push(($cleanup|0),($arg|0));
 $1 = ($at|0)==(0|0);
 do {
  if ($1) {
   $20 = (_pthread_self()|0);
   while(1) {
    $28 = (__pthread_isduecanceled($20)|0);
    $29 = ($28|0)==(0);
    if (!($29)) {
     $$0$i$ph = 125;
     label = 18;
     break;
    }
    $30 = (_emscripten_futex_wait(($addr|0),($val|0),1.0E+8)|0);
    $31 = (0 - ($30))|0;
    if ((($31|0) == 22) | (($31|0) == 4)) {
     $$0$i$ph = $31;
     label = 18;
     break;
    } else if (!((($31|0) == 110))) {
     label = 17;
     break;
    }
   }
   if ((label|0) == 17) {
    $$0$i = 0;
    break;
   }
   else if ((label|0) == 18) {
    $$0$i = $$0$i$ph;
    break;
   }
  } else {
   $2 = (($at) + 4|0);
   $3 = HEAP32[$2>>2]|0;
   $4 = ($3>>>0)>(999999999);
   if ($4) {
    $$0$i = 22;
   } else {
    $5 = (_clock_gettime(($clk|0),($to$i|0))|0);
    $6 = ($5|0)==(0);
    if ($6) {
     $7 = HEAP32[$at>>2]|0;
     $8 = HEAP32[$to$i>>2]|0;
     $9 = (($7) - ($8))|0;
     HEAP32[$to$i>>2] = $9;
     $10 = HEAP32[$2>>2]|0;
     $11 = (($to$i) + 4|0);
     $12 = HEAP32[$11>>2]|0;
     $13 = (($10) - ($12))|0;
     HEAP32[$11>>2] = $13;
     $14 = ($13|0)<(0);
     if ($14) {
      $15 = (($9) + -1)|0;
      HEAP32[$to$i>>2] = $15;
      $16 = (($13) + 1000000000)|0;
      HEAP32[$11>>2] = $16;
      $18 = $15;
     } else {
      $18 = $9;
     }
     $17 = ($18|0)<(0);
     if ($17) {
      $$0$i = 110;
     } else {
      $19 = (_pthread_self()|0);
      while(1) {
       $21 = (__pthread_isduecanceled($19)|0);
       $22 = ($21|0)==(0);
       if (!($22)) {
        $$0$i$ph6 = 125;
        label = 19;
        break;
       }
       $23 = (+__pthread_nsecs_until($at));
       $24 = !($23 <= 0.0);
       if (!($24)) {
        $$0$i$ph6 = 110;
        label = 19;
        break;
       }
       $25 = $23 > 1.0E+8;
       $waitNsecs$0$us$i = $25 ? 1.0E+8 : $23;
       $26 = (_emscripten_futex_wait(($addr|0),($val|0),(+$waitNsecs$0$us$i))|0);
       $27 = (0 - ($26))|0;
       if ((($27|0) == 22) | (($27|0) == 4)) {
        $$0$i$ph6 = $27;
        label = 19;
        break;
       } else if (!((($27|0) == 110))) {
        label = 16;
        break;
       }
      }
      if ((label|0) == 16) {
       $$0$i = 0;
       break;
      }
      else if ((label|0) == 19) {
       $$0$i = $$0$i$ph6;
       break;
      }
     }
    } else {
     $$0$i = 22;
    }
   }
  }
 } while(0);
 _pthread_cleanup_pop(0);
 if ($0) {
  STACKTOP = sp;return ($$0$i|0);
 }
 $32 = HEAP32[$cs>>2]|0;
 (_pthread_setcancelstate($32,0)|0);
 STACKTOP = sp;return ($$0$i|0);
}
function ___wait($addr,$waiters,$val,$priv) {
 $addr = $addr|0;
 $waiters = $waiters|0;
 $val = $val|0;
 $priv = $priv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $spins$0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $spins$0 = 10000;
 while(1) {
  $0 = ($spins$0|0)==(0);
  if ($0) {
   break;
  }
  $1 = (($spins$0) + -1)|0;
  $2 = Atomics_load(HEAP32,$addr>>2);
  $3 = ($2|0)==($val|0);
  if ($3) {
   $spins$0 = $1;
  } else {
   label = 20;
   break;
  }
 }
 if ((label|0) == 20) {
  STACKTOP = sp;return;
 }
 $4 = ($waiters|0)!=(0|0);
 if ($4) {
  $5 = Atomics_add(HEAP32, $waiters>>2, 1);
 }
 $6 = Atomics_load(HEAP32,$addr>>2);
 $7 = ($6|0)==($val|0);
 do {
  if ($7) {
   $8 = (_pthread_self()|0);
   $9 = (($8) + 72|0);
   L13: while(1) {
    $10 = Atomics_load(HEAP32,$9>>2);
    $11 = ($10|0)==(1);
    if ($11) {
     while(1) {
      $12 = (__pthread_isduecanceled($8)|0);
      $13 = ($12|0)==(0);
      if (!($13)) {
       break L13;
      }
      $15 = (_emscripten_futex_wait(($addr|0),($val|0),1.0E+8)|0);
      $16 = ($15|0)==(-110);
      if (!($16)) {
       break;
      }
     }
    } else {
     (_emscripten_futex_wait(($addr|0),($val|0),inf)|0);
    }
    $17 = Atomics_load(HEAP32,$addr>>2);
    $18 = ($17|0)==($val|0);
    if (!($18)) {
     label = 17;
     break;
    }
   }
   if ((label|0) == 17) {
    break;
   }
   if (!($4)) {
    STACKTOP = sp;return;
   }
   $14 = Atomics_sub(HEAP32, $waiters>>2, 1);
   STACKTOP = sp;return;
  }
 } while(0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $19 = Atomics_sub(HEAP32, $waiters>>2, 1);
 STACKTOP = sp;return;
}
function _pthread_attr_destroy($a) {
 $a = $a|0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = sp;return 0;
}
function _pthread_attr_init($a) {
 $a = $a|0;
 var $$sroa$0 = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $$sroa$0 = sp;
 dest=$$sroa$0+0|0; stop=dest+36|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 dest=$a+0|0; src=$$sroa$0+0|0; stop=dest+36|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
 STACKTOP = sp;return 0;
}
function _pthread_attr_setdetachstate($a,$state) {
 $a = $a|0;
 $state = $state|0;
 var $$0 = 0, $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($state>>>0)>(1);
 if ($0) {
  $$0 = 22;
 } else {
  $1 = (($a) + 12|0);
  HEAP32[$1>>2] = $state;
  $$0 = 0;
 }
 STACKTOP = sp;return ($$0|0);
}
function _pthread_attr_setstacksize($a,$size) {
 $a = $a|0;
 $size = $size|0;
 var $$0 = 0, $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (($size) + -2048)|0;
 $1 = ($0>>>0)>(1073741823);
 if ($1) {
  $$0 = 22;
  STACKTOP = sp;return ($$0|0);
 }
 $2 = (($a) + 8|0);
 HEAP32[$2>>2] = 0;
 $3 = (($size) + -81920)|0;
 HEAP32[$a>>2] = $3;
 $$0 = 0;
 STACKTOP = sp;return ($$0|0);
}
function ___pthread_tsd_run_dtors() {
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $exitcond = 0, $i$03 = 0, $j$04 = 0;
 var $not_finished$12 = 0, $not_finished$2 = 0, $not_finished$2$lcssa = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (_pthread_self()|0);
 $1 = (($0) + 52|0);
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0);
 if ($3) {
  STACKTOP = sp;return;
 }
 $4 = (($0) + 112|0);
 $j$04 = 0;
 while(1) {
  $i$03 = 0;$not_finished$12 = 0;
  while(1) {
   $5 = HEAP32[$4>>2]|0;
   $6 = (($5) + ($i$03<<2)|0);
   $7 = HEAP32[$6>>2]|0;
   $8 = ($7|0)==(0|0);
   if ($8) {
    $not_finished$2 = $not_finished$12;
   } else {
    $9 = (1872 + ($i$03<<2)|0);
    $10 = HEAP32[$9>>2]|0;
    $11 = ($10|0)==(0|0);
    if ($11) {
     $not_finished$2 = $not_finished$12;
    } else {
     HEAP32[$6>>2] = 0;
     $12 = HEAP32[$9>>2]|0;
     FUNCTION_TABLE_vi[$12 & 0]($7);
     $not_finished$2 = 1;
    }
   }
   $13 = (($i$03) + 1)|0;
   $exitcond = ($13|0)==(128);
   if ($exitcond) {
    $not_finished$2$lcssa = $not_finished$2;
    break;
   } else {
    $i$03 = $13;$not_finished$12 = $not_finished$2;
   }
  }
  $14 = (($j$04) + 1)|0;
  $15 = ($not_finished$2$lcssa|0)!=(0);
  $16 = ($14|0)<(4);
  $or$cond = $15 & $16;
  if ($or$cond) {
   $j$04 = $14;
  } else {
   break;
  }
 }
 STACKTOP = sp;return;
}
function _pthread_mutex_init($m,$a) {
 $m = $m|0;
 $a = $a|0;
 var $$sroa$0 = 0, $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $$sroa$0 = sp;
 ;HEAP32[$$sroa$0+0>>2]=0|0;HEAP32[$$sroa$0+4>>2]=0|0;HEAP32[$$sroa$0+8>>2]=0|0;HEAP32[$$sroa$0+12>>2]=0|0;HEAP32[$$sroa$0+16>>2]=0|0;HEAP32[$$sroa$0+20>>2]=0|0;HEAP32[$$sroa$0+24>>2]=0|0;
 ;HEAP32[$m+0>>2]=HEAP32[$$sroa$0+0>>2]|0;HEAP32[$m+4>>2]=HEAP32[$$sroa$0+4>>2]|0;HEAP32[$m+8>>2]=HEAP32[$$sroa$0+8>>2]|0;HEAP32[$m+12>>2]=HEAP32[$$sroa$0+12>>2]|0;HEAP32[$m+16>>2]=HEAP32[$$sroa$0+16>>2]|0;HEAP32[$m+20>>2]=HEAP32[$$sroa$0+20>>2]|0;HEAP32[$m+24>>2]=HEAP32[$$sroa$0+24>>2]|0;
 $0 = ($a|0)==(0|0);
 if ($0) {
  STACKTOP = sp;return 0;
 }
 $1 = HEAP32[$a>>2]|0;
 $2 = $1 & 7;
 HEAP32[$m>>2] = $2;
 STACKTOP = sp;return 0;
}
function _pthread_mutex_lock($m) {
 $m = $m|0;
 var $$0 = 0, $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = HEAP32[$m>>2]|0;
 $1 = ($0|0)==(0);
 if ($1) {
  $2 = (($m) + 4|0);
  $3 = Atomics_compareExchange(HEAP32, $2>>2, 0, 16);
  $4 = ($3|0)==(0);
  if ($4) {
   $$0 = 0;
   STACKTOP = sp;return ($$0|0);
  }
 }
 $5 = (_pthread_mutex_timedlock($m,0)|0);
 $$0 = $5;
 STACKTOP = sp;return ($$0|0);
}
function _pthread_mutex_timedlock($m,$at) {
 $m = $m|0;
 $at = $at|0;
 var $$0 = 0, $$0$ph = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = HEAP32[$m>>2]|0;
 $1 = ($0|0)==(0);
 if ($1) {
  $2 = (($m) + 4|0);
  $3 = Atomics_compareExchange(HEAP32, $2>>2, 0, 16);
  $4 = ($3|0)==(0);
  if ($4) {
   $$0 = 0;
   STACKTOP = sp;return ($$0|0);
  }
 }
 $5 = (_pthread_mutex_trylock($m)|0);
 $6 = ($5|0)==(16);
 if (!($6)) {
  $$0 = $5;
  STACKTOP = sp;return ($$0|0);
 }
 $7 = (($m) + 4|0);
 $8 = (($m) + 8|0);
 while(1) {
  $9 = HEAP32[$7>>2]|0;
  $10 = ($9|0)!=(0);
  $11 = $9 & 1073741824;
  $12 = ($11|0)==(0);
  $or$cond3 = $10 & $12;
  if ($or$cond3) {
   $15 = HEAP32[$m>>2]|0;
   $16 = $15 & 3;
   $17 = ($16|0)==(2);
   if ($17) {
    $18 = $9 & 536870911;
    $19 = (_pthread_self()|0);
    $20 = (($19) + 44|0);
    $21 = HEAP32[$20>>2]|0;
    $22 = ($18|0)==($21|0);
    if ($22) {
     $$0$ph = 35;
     break;
    }
   }
   $23 = Atomics_add(HEAP32, $8>>2, 1);
   $24 = $9 | -2147483648;
   $25 = Atomics_compareExchange(HEAP32, $7>>2, $9, $24);
   $26 = (___timedwait($7,$24,0,$at,0,0,0)|0);
   $27 = Atomics_sub(HEAP32, $8>>2, 1);
   if (!((($26|0) == 0) | (($26|0) == 4))) {
    $$0$ph = $26;
    break;
   }
  }
  $13 = (_pthread_mutex_trylock($m)|0);
  $14 = ($13|0)==(16);
  if (!($14)) {
   $$0$ph = $13;
   break;
  }
 }
 $$0 = $$0$ph;
 STACKTOP = sp;return ($$0|0);
}
function _pthread_mutex_trylock($m) {
 $m = $m|0;
 var $$0 = 0, $$c = 0, $$c1 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $or$cond4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = HEAP32[$m>>2]|0;
 $1 = ($0|0)==(0);
 if ($1) {
  $2 = (($m) + 4|0);
  $3 = Atomics_compareExchange(HEAP32, $2>>2, 0, 16);
  $4 = $3 & 16;
  $$0 = $4;
  STACKTOP = sp;return ($$0|0);
 }
 $5 = (_pthread_self()|0);
 $6 = (($5) + 44|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($0|0)>(3);
 if ($8) {
  $9 = (($m) + 16|0);
  $10 = (($5) + 160|0);
  HEAP32[$10>>2] = -12;
  $11 = (($5) + 164|0);
  HEAP32[$11>>2] = $9;
 }
 $12 = (($m) + 4|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = $13 & 2147483647;
 $15 = ($14|0)==($7|0);
 $16 = $0 & 3;
 $17 = ($16|0)==(1);
 $or$cond4 = $15 & $17;
 if ($or$cond4) {
  $18 = (($m) + 20|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = ($19>>>0)>(2147483646);
  if ($20) {
   $$0 = 11;
   STACKTOP = sp;return ($$0|0);
  }
  $21 = (($19) + 1)|0;
  HEAP32[$18>>2] = $21;
  $$0 = 0;
  STACKTOP = sp;return ($$0|0);
 }
 $22 = ($14|0)!=(0);
 $23 = $13 & 1073741824;
 $24 = ($23|0)==(0);
 $or$cond = $22 & $24;
 if ($or$cond) {
  $$0 = 16;
  STACKTOP = sp;return ($$0|0);
 }
 $25 = Atomics_compareExchange(HEAP32, $12>>2, $13, $7);
 $26 = ($25|0)==($13|0);
 if (!($26)) {
  $$0 = 16;
  STACKTOP = sp;return ($$0|0);
 }
 $27 = HEAP32[$m>>2]|0;
 $28 = ($27|0)<(4);
 if ($28) {
  $$0 = 0;
  STACKTOP = sp;return ($$0|0);
 }
 $29 = ($27|0)>(7);
 if ($29) {
  HEAP32[$12>>2] = 0;
  $$0 = 131;
  STACKTOP = sp;return ($$0|0);
 }
 $30 = (($5) + 156|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = (($m) + 16|0);
 $$c = $31;
 HEAP32[$32>>2] = $$c;
 $33 = (($m) + 12|0);
 $$c1 = $30;
 HEAP32[$33>>2] = $$c1;
 $34 = HEAP32[$30>>2]|0;
 $35 = ($34|0)==(0|0);
 if (!($35)) {
  $36 = (($34) + -4|0);
  HEAP32[$36>>2] = $32;
 }
 HEAP32[$30>>2] = $32;
 $37 = (($5) + 164|0);
 HEAP32[$37>>2] = 0;
 if (!($22)) {
  $$0 = 0;
  STACKTOP = sp;return ($$0|0);
 }
 $38 = (($m) + 20|0);
 HEAP32[$38>>2] = 0;
 $39 = (($27) + 8)|0;
 HEAP32[$m>>2] = $39;
 $$0 = 130;
 STACKTOP = sp;return ($$0|0);
}
function _pthread_mutex_unlock($m) {
 $m = $m|0;
 var $$0 = 0, $$lcssa = 0, $$pre = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $robust$0 = 0, $self$0 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 $0 = (($m) + 8|0);
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$m>>2]|0;
 $3 = ($2|0)==(0);
 $$pre = (($m) + 4|0);
 if ($3) {
  $robust$0 = 0;$self$0 = 0;
 } else {
  $4 = HEAP32[$$pre>>2]|0;
  $5 = ($4|0)==(0);
  if ($5) {
   $$0 = 1;
   STACKTOP = sp;return ($$0|0);
  }
  $6 = (_pthread_self()|0);
  $7 = $4 & 536870911;
  $8 = (($6) + 44|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($7|0)==($9|0);
  if (!($10)) {
   $$0 = 1;
   STACKTOP = sp;return ($$0|0);
  }
  $11 = $2 & 3;
  $12 = ($11|0)==(1);
  if ($12) {
   $13 = (($m) + 20|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = ($14|0)==(0);
   if (!($15)) {
    $16 = (($14) + -1)|0;
    HEAP32[$13>>2] = $16;
    $$0 = 0;
    STACKTOP = sp;return ($$0|0);
   }
  }
  $17 = ($2|0)>(3);
  if ($17) {
   $18 = (($m) + 16|0);
   $19 = (($6) + 164|0);
   HEAP32[$19>>2] = $18;
   $20 = HEAP32[$18>>2]|0;
   $21 = (($m) + 12|0);
   $22 = HEAP32[$21>>2]|0;
   HEAP32[$22>>2] = $20;
   $23 = HEAP32[$18>>2]|0;
   $24 = ($23|0)==(0|0);
   if (!($24)) {
    $25 = HEAP32[$21>>2]|0;
    $26 = (($23) + -4|0);
    HEAP32[$26>>2] = $25;
   }
   ___vm_lock(1);
   $robust$0 = 1;$self$0 = $6;
  } else {
   $robust$0 = 0;$self$0 = $6;
  }
 }
 while(1) {
  $27 = Atomics_load(HEAP32, $$pre>>2);
  $28 = Atomics_compareExchange(HEAP32, $$pre>>2, $27, 0);
  $29 = ($28|0)==($27|0);
  if ($29) {
   $$lcssa = $27;
   break;
  }
 }
 if ($robust$0) {
  $30 = (($self$0) + 164|0);
  HEAP32[$30>>2] = 0;
  ___vm_unlock();
 }
 $31 = ($1|0)!=(0);
 $32 = ($$lcssa|0)<(0);
 $or$cond = $31 | $32;
 if (!($or$cond)) {
  $$0 = 0;
  STACKTOP = sp;return ($$0|0);
 }
 (_emscripten_futex_wake(($$pre|0),1)|0);
 $$0 = 0;
 STACKTOP = sp;return ($$0|0);
}
function _pthread_mutexattr_destroy($a) {
 $a = $a|0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = sp;return 0;
}
function _pthread_mutexattr_init($a) {
 $a = $a|0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 HEAP32[$a>>2] = 0;
 STACKTOP = sp;return 0;
}
function ___vm_lock($inc) {
 $inc = $inc|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 while(1) {
  $0 = HEAP32[2384>>2]|0;
  $1 = Math_imul($0, $inc)|0;
  $2 = ($1|0)<(0);
  if ($2) {
   ___wait(2384,((2384 + 4|0)),$0,1);
  } else {
   $3 = (($0) + ($inc))|0;
   $4 = Atomics_compareExchange(HEAP32, 2384>>2, $0, $3);
   $5 = ($4|0)==($0|0);
   if ($5) {
    break;
   }
  }
 }
 STACKTOP = sp;return;
}
function ___vm_unlock() {
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = HEAP32[2384>>2]|0;
 $1 = ($0|0)>(0);
 $2 = $1 ? -1 : 1;
 $3 = Atomics_add(HEAP32, 2384>>2, $2);
 $4 = (0 - ($2))|0;
 $5 = ($3|0)==($4|0);
 $6 = HEAP32[((2384 + 4|0))>>2]|0;
 $7 = ($6|0)!=(0);
 $or$cond = $5 & $7;
 if (!($or$cond)) {
  STACKTOP = sp;return;
 }
 (_emscripten_futex_wake((2384|0),2147483647)|0);
 STACKTOP = sp;return;
}
function _init_mparams() {
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $attr$i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $attr$i = sp;
 (_pthread_mutex_lock(800)|0);
 $0 = HEAP32[272>>2]|0;
 $1 = ($0|0)==(0);
 if (!($1)) {
  (_pthread_mutex_unlock(800)|0);
  STACKTOP = sp;return;
 }
 $2 = (_sysconf(30)|0);
 $3 = (($2) + -1)|0;
 $4 = $3 & $2;
 $5 = ($4|0)==(0);
 if (!($5)) {
  _abort();
  // unreachable;
 }
 HEAP32[((272 + 8|0))>>2] = $2;
 HEAP32[((272 + 4|0))>>2] = $2;
 HEAP32[((272 + 12|0))>>2] = -1;
 HEAP32[((272 + 16|0))>>2] = -1;
 HEAP32[((272 + 20|0))>>2] = 2;
 HEAP32[((296 + 444|0))>>2] = 2;
 $6 = (_pthread_mutexattr_init($attr$i)|0);
 $7 = ($6|0)==(0);
 if ($7) {
  $8 = (_pthread_mutex_init(((296 + 448|0)),$attr$i)|0);
  $9 = ($8|0)==(0);
  if ($9) {
  }
 }
 $10 = (_time((0|0))|0);
 $11 = $10 & -16;
 $12 = $11 ^ 1431655768;
 Atomics_store(HEAP32,272>>2,$12);
 (_pthread_mutex_unlock(800)|0);
 STACKTOP = sp;return;
}
function __do_call($q) {
 $q = $q|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0;
 var $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0;
 var $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0;
 var $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0;
 var $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0;
 var $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0;
 var $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0;
 var $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0;
 var $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0;
 var $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0;
 var $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0;
 var $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0;
 var $819 = 0, $82 = 0, $820 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer10 = 0, $vararg_buffer13 = 0, $vararg_buffer16 = 0, $vararg_buffer19 = 0, $vararg_buffer4 = 0, $vararg_buffer7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer19 = sp + 8|0;
 $vararg_buffer16 = sp + 32|0;
 $vararg_buffer13 = sp + 16|0;
 $vararg_buffer10 = sp + 40|0;
 $vararg_buffer7 = sp + 56|0;
 $vararg_buffer4 = sp;
 $vararg_buffer1 = sp + 24|0;
 $vararg_buffer = sp + 48|0;
 $0 = HEAP32[$q>>2]|0;
 do {
  switch ($0|0) {
  case 105:  {
   $577 = (($q) + 8|0);
   $578 = HEAP32[$577>>2]|0;
   $579 = (($q) + 16|0);
   $580 = HEAP32[$579>>2]|0;
   $581 = (($q) + 24|0);
   $582 = HEAP32[$581>>2]|0;
   HEAP32[$vararg_buffer10>>2] = $582;
   $583 = (_fprintf(($578|0),($580|0),($vararg_buffer10|0))|0);
   $584 = (($q) + 72|0);
   HEAP32[$584>>2] = $583;
   break;
  }
  case 106:  {
   $585 = (($q) + 8|0);
   $586 = HEAP32[$585>>2]|0;
   $587 = (($q) + 16|0);
   $588 = HEAP32[$587>>2]|0;
   HEAP32[$vararg_buffer13>>2] = $588;
   $589 = (_printf(($586|0),($vararg_buffer13|0))|0);
   $590 = (($q) + 72|0);
   HEAP32[$590>>2] = $589;
   break;
  }
  case 107:  {
   $591 = (($q) + 8|0);
   $592 = HEAP32[$591>>2]|0;
   $593 = (($q) + 16|0);
   $594 = HEAP32[$593>>2]|0;
   $595 = (($q) + 24|0);
   $596 = HEAP32[$595>>2]|0;
   HEAP32[$vararg_buffer16>>2] = $596;
   $597 = (_dprintf(($592|0),($594|0),($vararg_buffer16|0))|0);
   $598 = (($q) + 72|0);
   HEAP32[$598>>2] = $597;
   break;
  }
  case 108:  {
   $599 = (($q) + 8|0);
   $600 = HEAP32[$599>>2]|0;
   $601 = (($q) + 16|0);
   $602 = HEAP32[$601>>2]|0;
   $603 = (($q) + 24|0);
   $604 = HEAP32[$603>>2]|0;
   $605 = (($q) + 32|0);
   $606 = HEAP32[$605>>2]|0;
   $607 = (($q) + 40|0);
   $608 = HEAP32[$607>>2]|0;
   $609 = (($q) + 48|0);
   $610 = HEAP32[$609>>2]|0;
   $611 = (_mmap(($600|0),($602|0),($604|0),($606|0),($608|0),($610|0))|0);
   $612 = (($q) + 72|0);
   HEAP32[$612>>2] = $611;
   break;
  }
  case 109:  {
   $613 = (($q) + 8|0);
   $614 = HEAP32[$613>>2]|0;
   $615 = (($q) + 16|0);
   $616 = HEAP32[$615>>2]|0;
   $617 = (_munmap(($614|0),($616|0))|0);
   $618 = (($q) + 72|0);
   HEAP32[$618>>2] = $617;
   break;
  }
  case 110:  {
   $619 = (($q) + 8|0);
   $620 = HEAP32[$619>>2]|0;
   $621 = (_atexit(($620|0))|0);
   $622 = (($q) + 72|0);
   HEAP32[$622>>2] = $621;
   break;
  }
  case 111:  {
   $623 = (($q) + 8|0);
   $624 = HEAP32[$623>>2]|0;
   $625 = (_getenv(($624|0))|0);
   $626 = (($q) + 72|0);
   HEAP32[$626>>2] = $625;
   break;
  }
  case 112:  {
   $627 = (_clearenv()|0);
   $628 = (($q) + 72|0);
   HEAP32[$628>>2] = $627;
   break;
  }
  case 113:  {
   $629 = (($q) + 8|0);
   $630 = HEAP32[$629>>2]|0;
   $631 = (($q) + 16|0);
   $632 = HEAP32[$631>>2]|0;
   $633 = (($q) + 24|0);
   $634 = HEAP32[$633>>2]|0;
   $635 = (_setenv(($630|0),($632|0),($634|0))|0);
   $636 = (($q) + 72|0);
   HEAP32[$636>>2] = $635;
   break;
  }
  case 114:  {
   $637 = (($q) + 8|0);
   $638 = HEAP32[$637>>2]|0;
   $639 = (_unsetenv(($638|0))|0);
   $640 = (($q) + 72|0);
   HEAP32[$640>>2] = $639;
   break;
  }
  case 115:  {
   $641 = (($q) + 8|0);
   $642 = HEAP32[$641>>2]|0;
   $643 = (_putenv(($642|0))|0);
   $644 = (($q) + 72|0);
   HEAP32[$644>>2] = $643;
   break;
  }
  case 116:  {
   $645 = (($q) + 8|0);
   $646 = HEAP32[$645>>2]|0;
   $647 = (($q) + 16|0);
   $648 = HEAP32[$647>>2]|0;
   $649 = (_realpath(($646|0),($648|0))|0);
   $650 = (($q) + 72|0);
   HEAP32[$650>>2] = $649;
   break;
  }
  case 117:  {
   $651 = (($q) + 8|0);
   $652 = HEAP32[$651>>2]|0;
   $653 = (($q) + 16|0);
   $654 = HEAP32[$653>>2]|0;
   $655 = (_tcgetattr(($652|0),($654|0))|0);
   $656 = (($q) + 72|0);
   HEAP32[$656>>2] = $655;
   break;
  }
  case 119:  {
   _tzset();
   break;
  }
  case 118:  {
   $657 = (($q) + 8|0);
   $658 = HEAP32[$657>>2]|0;
   $659 = (($q) + 16|0);
   $660 = HEAP32[$659>>2]|0;
   $661 = (($q) + 24|0);
   $662 = HEAP32[$661>>2]|0;
   $663 = (_tcsetattr(($658|0),($660|0),($662|0))|0);
   $664 = (($q) + 72|0);
   HEAP32[$664>>2] = $663;
   break;
  }
  case 120:  {
   $665 = (($q) + 8|0);
   $666 = HEAP32[$665>>2]|0;
   $667 = (($q) + 16|0);
   $668 = HEAP32[$667>>2]|0;
   $669 = (($q) + 24|0);
   $670 = HEAP32[$669>>2]|0;
   $671 = (_socket(($666|0),($668|0),($670|0))|0);
   $672 = (($q) + 72|0);
   HEAP32[$672>>2] = $671;
   break;
  }
  case 121:  {
   $673 = (($q) + 8|0);
   $674 = HEAP32[$673>>2]|0;
   $675 = (($q) + 16|0);
   $676 = HEAP32[$675>>2]|0;
   $677 = (($q) + 24|0);
   $678 = HEAP32[$677>>2]|0;
   $679 = (_bind(($674|0),($676|0),($678|0))|0);
   $680 = (($q) + 72|0);
   HEAP32[$680>>2] = $679;
   break;
  }
  case 122:  {
   $681 = (($q) + 8|0);
   $682 = HEAP32[$681>>2]|0;
   $683 = (($q) + 16|0);
   $684 = HEAP32[$683>>2]|0;
   $685 = (($q) + 24|0);
   $686 = HEAP32[$685>>2]|0;
   $687 = (_sendmsg(($682|0),($684|0),($686|0))|0);
   $688 = (($q) + 72|0);
   HEAP32[$688>>2] = $687;
   break;
  }
  case 123:  {
   $689 = (($q) + 8|0);
   $690 = HEAP32[$689>>2]|0;
   $691 = (($q) + 16|0);
   $692 = HEAP32[$691>>2]|0;
   $693 = (($q) + 24|0);
   $694 = HEAP32[$693>>2]|0;
   $695 = (_recvmsg(($690|0),($692|0),($694|0))|0);
   $696 = (($q) + 72|0);
   HEAP32[$696>>2] = $695;
   break;
  }
  case 9:  {
   $41 = (($q) + 8|0);
   $42 = HEAP32[$41>>2]|0;
   _rewinddir(($42|0));
   break;
  }
  case 10:  {
   $43 = (($q) + 8|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = (($q) + 16|0);
   $46 = HEAP32[$45>>2]|0;
   $47 = (($q) + 24|0);
   $48 = HEAP32[$47>>2]|0;
   $49 = (_readdir_r(($44|0),($46|0),($48|0))|0);
   $50 = (($q) + 72|0);
   HEAP32[$50>>2] = $49;
   break;
  }
  case 11:  {
   $51 = (($q) + 8|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = (_readdir(($52|0))|0);
   $54 = (($q) + 72|0);
   HEAP32[$54>>2] = $53;
   break;
  }
  case 12:  {
   $55 = (($q) + 8|0);
   $56 = HEAP32[$55>>2]|0;
   $57 = (($q) + 16|0);
   $58 = HEAP32[$57>>2]|0;
   $59 = (_utime(($56|0),($58|0))|0);
   $60 = (($q) + 72|0);
   HEAP32[$60>>2] = $59;
   break;
  }
  case 13:  {
   $61 = (($q) + 8|0);
   $62 = HEAP32[$61>>2]|0;
   $63 = (($q) + 16|0);
   $64 = HEAP32[$63>>2]|0;
   $65 = (_utimes(($62|0),($64|0))|0);
   $66 = (($q) + 72|0);
   HEAP32[$66>>2] = $65;
   break;
  }
  case 14:  {
   $67 = (($q) + 8|0);
   $68 = HEAP32[$67>>2]|0;
   $69 = (($q) + 16|0);
   $70 = HEAP32[$69>>2]|0;
   $71 = (_stat(($68|0),($70|0))|0);
   $72 = (($q) + 72|0);
   HEAP32[$72>>2] = $71;
   break;
  }
  case 15:  {
   $73 = (($q) + 8|0);
   $74 = HEAP32[$73>>2]|0;
   $75 = (($q) + 16|0);
   $76 = HEAP32[$75>>2]|0;
   $77 = (_lstat(($74|0),($76|0))|0);
   $78 = (($q) + 72|0);
   HEAP32[$78>>2] = $77;
   break;
  }
  case 16:  {
   $79 = (($q) + 8|0);
   $80 = HEAP32[$79>>2]|0;
   $81 = (($q) + 16|0);
   $82 = HEAP32[$81>>2]|0;
   $83 = (_fstat(($80|0),($82|0))|0);
   $84 = (($q) + 72|0);
   HEAP32[$84>>2] = $83;
   break;
  }
  case 17:  {
   $85 = (($q) + 8|0);
   $86 = HEAP32[$85>>2]|0;
   $87 = (($q) + 16|0);
   $88 = HEAP32[$87>>2]|0;
   $89 = (($q) + 24|0);
   $90 = HEAP32[$89>>2]|0;
   $91 = (_mknod(($86|0),($88|0),($90|0))|0);
   $92 = (($q) + 72|0);
   HEAP32[$92>>2] = $91;
   break;
  }
  case 18:  {
   $93 = (($q) + 8|0);
   $94 = HEAP32[$93>>2]|0;
   $95 = (($q) + 16|0);
   $96 = HEAP32[$95>>2]|0;
   $97 = (_mkdir(($94|0),($96|0))|0);
   $98 = (($q) + 72|0);
   HEAP32[$98>>2] = $97;
   break;
  }
  case 19:  {
   $99 = (($q) + 8|0);
   $100 = HEAP32[$99>>2]|0;
   $101 = (($q) + 16|0);
   $102 = HEAP32[$101>>2]|0;
   $103 = (_mkfifo(($100|0),($102|0))|0);
   $104 = (($q) + 72|0);
   HEAP32[$104>>2] = $103;
   break;
  }
  case 20:  {
   $105 = (($q) + 8|0);
   $106 = HEAP32[$105>>2]|0;
   $107 = (($q) + 16|0);
   $108 = HEAP32[$107>>2]|0;
   $109 = (_chmod(($106|0),($108|0))|0);
   $110 = (($q) + 72|0);
   HEAP32[$110>>2] = $109;
   break;
  }
  case 21:  {
   $111 = (($q) + 8|0);
   $112 = HEAP32[$111>>2]|0;
   $113 = (($q) + 16|0);
   $114 = HEAP32[$113>>2]|0;
   $115 = (_fchmod(($112|0),($114|0))|0);
   $116 = (($q) + 72|0);
   HEAP32[$116>>2] = $115;
   break;
  }
  case 22:  {
   $117 = (($q) + 8|0);
   $118 = HEAP32[$117>>2]|0;
   $119 = (($q) + 16|0);
   $120 = HEAP32[$119>>2]|0;
   $121 = (_lchmod(($118|0),($120|0))|0);
   $122 = (($q) + 72|0);
   HEAP32[$122>>2] = $121;
   break;
  }
  case 23:  {
   $123 = (($q) + 8|0);
   $124 = HEAP32[$123>>2]|0;
   $125 = (_umask(($124|0))|0);
   $126 = (($q) + 72|0);
   HEAP32[$126>>2] = $125;
   break;
  }
  case 24:  {
   $127 = (($q) + 8|0);
   $128 = HEAP32[$127>>2]|0;
   $129 = (($q) + 16|0);
   $130 = HEAP32[$129>>2]|0;
   $131 = (_statvfs(($128|0),($130|0))|0);
   $132 = (($q) + 72|0);
   HEAP32[$132>>2] = $131;
   break;
  }
  case 25:  {
   $133 = (($q) + 8|0);
   $134 = HEAP32[$133>>2]|0;
   $135 = (($q) + 16|0);
   $136 = HEAP32[$135>>2]|0;
   $137 = (_fstatvfs(($134|0),($136|0))|0);
   $138 = (($q) + 72|0);
   HEAP32[$138>>2] = $137;
   break;
  }
  case 26:  {
   $139 = (($q) + 8|0);
   $140 = HEAP32[$139>>2]|0;
   $141 = (($q) + 16|0);
   $142 = HEAP32[$141>>2]|0;
   $143 = (($q) + 24|0);
   $144 = HEAP32[$143>>2]|0;
   HEAP32[$vararg_buffer>>2] = $144;
   $145 = (_open(($140|0),($142|0),($vararg_buffer|0))|0);
   $146 = (($q) + 72|0);
   HEAP32[$146>>2] = $145;
   break;
  }
  case 27:  {
   $147 = (($q) + 8|0);
   $148 = HEAP32[$147>>2]|0;
   $149 = (($q) + 16|0);
   $150 = HEAP32[$149>>2]|0;
   $151 = (_creat(($148|0),($150|0))|0);
   $152 = (($q) + 72|0);
   HEAP32[$152>>2] = $151;
   break;
  }
  case 28:  {
   $153 = (($q) + 8|0);
   $154 = HEAP32[$153>>2]|0;
   $155 = (_mktemp(($154|0))|0);
   $156 = (($q) + 72|0);
   HEAP32[$156>>2] = $155;
   break;
  }
  case 29:  {
   $157 = (($q) + 8|0);
   $158 = HEAP32[$157>>2]|0;
   $159 = (_mkstemp(($158|0))|0);
   $160 = (($q) + 72|0);
   HEAP32[$160>>2] = $159;
   break;
  }
  case 30:  {
   $161 = (($q) + 8|0);
   $162 = HEAP32[$161>>2]|0;
   $163 = (_mkdtemp(($162|0))|0);
   $164 = (($q) + 72|0);
   HEAP32[$164>>2] = $163;
   break;
  }
  case 31:  {
   $165 = (($q) + 8|0);
   $166 = HEAP32[$165>>2]|0;
   $167 = (($q) + 16|0);
   $168 = HEAP32[$167>>2]|0;
   $169 = (($q) + 24|0);
   $170 = HEAP32[$169>>2]|0;
   HEAP32[$vararg_buffer1>>2] = $170;
   $171 = (_fcntl(($166|0),($168|0),($vararg_buffer1|0))|0);
   $172 = (($q) + 72|0);
   HEAP32[$172>>2] = $171;
   break;
  }
  case 32:  {
   $173 = (($q) + 8|0);
   $174 = HEAP32[$173>>2]|0;
   $175 = (($q) + 16|0);
   $176 = HEAP32[$175>>2]|0;
   $177 = (($q) + 24|0);
   $178 = HEAP32[$177>>2]|0;
   $179 = (_posix_fallocate(($174|0),($176|0),($178|0))|0);
   $180 = (($q) + 72|0);
   HEAP32[$180>>2] = $179;
   break;
  }
  case 87:  {
   $475 = (($q) + 8|0);
   $476 = HEAP32[$475>>2]|0;
   $477 = (($q) + 16|0);
   $478 = HEAP32[$477>>2]|0;
   $479 = (($q) + 24|0);
   $480 = HEAP32[$479>>2]|0;
   $481 = (($q) + 32|0);
   $482 = HEAP32[$481>>2]|0;
   $483 = (_fread(($476|0),($478|0),($480|0),($482|0))|0);
   $484 = (($q) + 72|0);
   HEAP32[$484>>2] = $483;
   break;
  }
  case 88:  {
   $485 = (($q) + 8|0);
   $486 = HEAP32[$485>>2]|0;
   $487 = (($q) + 16|0);
   $488 = HEAP32[$487>>2]|0;
   $489 = (($q) + 24|0);
   $490 = HEAP32[$489>>2]|0;
   $491 = (_freopen(($486|0),($488|0),($490|0))|0);
   $492 = (($q) + 72|0);
   HEAP32[$492>>2] = $491;
   break;
  }
  case 89:  {
   $493 = (($q) + 8|0);
   $494 = HEAP32[$493>>2]|0;
   $495 = (($q) + 16|0);
   $496 = HEAP32[$495>>2]|0;
   $497 = (($q) + 24|0);
   $498 = HEAP32[$497>>2]|0;
   $499 = (_fseek(($494|0),($496|0),($498|0))|0);
   $500 = (($q) + 72|0);
   HEAP32[$500>>2] = $499;
   break;
  }
  case 90:  {
   $501 = (($q) + 8|0);
   $502 = HEAP32[$501>>2]|0;
   $503 = (($q) + 16|0);
   $504 = HEAP32[$503>>2]|0;
   $505 = (_fsetpos(($502|0),($504|0))|0);
   $506 = (($q) + 72|0);
   HEAP32[$506>>2] = $505;
   break;
  }
  case 91:  {
   $507 = (($q) + 8|0);
   $508 = HEAP32[$507>>2]|0;
   $509 = (_ftell(($508|0))|0);
   $510 = (($q) + 72|0);
   HEAP32[$510>>2] = $509;
   break;
  }
  case 92:  {
   $511 = (($q) + 8|0);
   $512 = HEAP32[$511>>2]|0;
   $513 = (($q) + 16|0);
   $514 = HEAP32[$513>>2]|0;
   $515 = (($q) + 24|0);
   $516 = HEAP32[$515>>2]|0;
   $517 = (($q) + 32|0);
   $518 = HEAP32[$517>>2]|0;
   $519 = (_fwrite(($512|0),($514|0),($516|0),($518|0))|0);
   $520 = (($q) + 72|0);
   HEAP32[$520>>2] = $519;
   break;
  }
  case 71:  {
   $405 = (($q) + 8|0);
   $406 = HEAP32[$405>>2]|0;
   $407 = (($q) + 16|0);
   $408 = HEAP32[$407>>2]|0;
   $409 = (_getlogin_r(($406|0),($408|0))|0);
   $410 = (($q) + 72|0);
   HEAP32[$410>>2] = $409;
   break;
  }
  case 72:  {
   $411 = (($q) + 8|0);
   $412 = HEAP32[$411>>2]|0;
   $413 = (_sysconf(($412|0))|0);
   $414 = (($q) + 72|0);
   HEAP32[$414>>2] = $413;
   break;
  }
  case 73:  {
   $415 = (($q) + 8|0);
   $416 = HEAP32[$415>>2]|0;
   $417 = (_sbrk(($416|0))|0);
   $418 = (($q) + 72|0);
   HEAP32[$418>>2] = $417;
   break;
  }
  case 74:  {
   $419 = (($q) + 8|0);
   $420 = HEAP32[$419>>2]|0;
   _clearerr(($420|0));
   break;
  }
  case 75:  {
   $421 = (($q) + 8|0);
   $422 = HEAP32[$421>>2]|0;
   $423 = (($q) + 16|0);
   $424 = HEAP32[$423>>2]|0;
   $425 = (_fdopen(($422|0),($424|0))|0);
   $426 = (($q) + 72|0);
   HEAP32[$426>>2] = $425;
   break;
  }
  case 76:  {
   $427 = (($q) + 8|0);
   $428 = HEAP32[$427>>2]|0;
   $429 = (_feof(($428|0))|0);
   $430 = (($q) + 72|0);
   HEAP32[$430>>2] = $429;
   break;
  }
  case 77:  {
   $431 = (($q) + 8|0);
   $432 = HEAP32[$431>>2]|0;
   $433 = (_ferror(($432|0))|0);
   $434 = (($q) + 72|0);
   HEAP32[$434>>2] = $433;
   break;
  }
  case 78:  {
   $435 = (($q) + 8|0);
   $436 = HEAP32[$435>>2]|0;
   $437 = (_fflush(($436|0))|0);
   $438 = (($q) + 72|0);
   HEAP32[$438>>2] = $437;
   break;
  }
  case 79:  {
   $439 = (($q) + 8|0);
   $440 = HEAP32[$439>>2]|0;
   $441 = (_fgetc(($440|0))|0);
   $442 = (($q) + 72|0);
   HEAP32[$442>>2] = $441;
   break;
  }
  case 80:  {
   $443 = (_getchar()|0);
   $444 = (($q) + 72|0);
   HEAP32[$444>>2] = $443;
   break;
  }
  case 81:  {
   $445 = (($q) + 8|0);
   $446 = HEAP32[$445>>2]|0;
   $447 = (($q) + 16|0);
   $448 = HEAP32[$447>>2]|0;
   $449 = (_fgetpos(($446|0),($448|0))|0);
   $450 = (($q) + 72|0);
   HEAP32[$450>>2] = $449;
   break;
  }
  case 82:  {
   $451 = (($q) + 8|0);
   $452 = HEAP32[$451>>2]|0;
   $453 = (_gets(($452|0))|0);
   $454 = (($q) + 72|0);
   HEAP32[$454>>2] = $453;
   break;
  }
  case 83:  {
   $455 = (($q) + 8|0);
   $456 = HEAP32[$455>>2]|0;
   $457 = (_fileno(($456|0))|0);
   $458 = (($q) + 72|0);
   HEAP32[$458>>2] = $457;
   break;
  }
  case 84:  {
   $459 = (($q) + 8|0);
   $460 = HEAP32[$459>>2]|0;
   $461 = (($q) + 16|0);
   $462 = HEAP32[$461>>2]|0;
   $463 = (_fputc(($460|0),($462|0))|0);
   $464 = (($q) + 72|0);
   HEAP32[$464>>2] = $463;
   break;
  }
  case 85:  {
   $465 = (($q) + 8|0);
   $466 = HEAP32[$465>>2]|0;
   $467 = (_putchar(($466|0))|0);
   $468 = (($q) + 72|0);
   HEAP32[$468>>2] = $467;
   break;
  }
  case 86:  {
   $469 = (($q) + 8|0);
   $470 = HEAP32[$469>>2]|0;
   $471 = (($q) + 16|0);
   $472 = HEAP32[$471>>2]|0;
   $473 = (_putc(($470|0),($472|0))|0);
   $474 = (($q) + 72|0);
   HEAP32[$474>>2] = $473;
   break;
  }
  case 55:  {
   $301 = (($q) + 8|0);
   $302 = HEAP32[$301>>2]|0;
   $303 = (($q) + 16|0);
   $304 = HEAP32[$303>>2]|0;
   $305 = (($q) + 24|0);
   $306 = HEAP32[$305>>2]|0;
   $307 = (_lockf(($302|0),($304|0),($306|0))|0);
   $308 = (($q) + 72|0);
   HEAP32[$308>>2] = $307;
   break;
  }
  case 56:  {
   $309 = (($q) + 8|0);
   $310 = HEAP32[$309>>2]|0;
   $311 = (($q) + 16|0);
   $312 = HEAP32[$311>>2]|0;
   $313 = (($q) + 24|0);
   $314 = HEAP32[$313>>2]|0;
   $315 = (_lseek(($310|0),($312|0),($314|0))|0);
   $316 = (($q) + 72|0);
   HEAP32[$316>>2] = $315;
   break;
  }
  case 57:  {
   $317 = (($q) + 8|0);
   $318 = HEAP32[$317>>2]|0;
   $319 = (_pipe(($318|0))|0);
   $320 = (($q) + 72|0);
   HEAP32[$320>>2] = $319;
   break;
  }
  case 58:  {
   $321 = (($q) + 8|0);
   $322 = HEAP32[$321>>2]|0;
   $323 = (($q) + 16|0);
   $324 = HEAP32[$323>>2]|0;
   $325 = (($q) + 24|0);
   $326 = HEAP32[$325>>2]|0;
   $327 = (_pread(($322|0),($324|0),($326|0),($326|0))|0);
   $328 = (($q) + 72|0);
   HEAP32[$328>>2] = $327;
   break;
  }
  case 59:  {
   $329 = (($q) + 8|0);
   $330 = HEAP32[$329>>2]|0;
   $331 = (($q) + 16|0);
   $332 = HEAP32[$331>>2]|0;
   $333 = (($q) + 24|0);
   $334 = HEAP32[$333>>2]|0;
   $335 = (_read(($330|0),($332|0),($334|0))|0);
   $336 = (($q) + 72|0);
   HEAP32[$336>>2] = $335;
   break;
  }
  case 60:  {
   $337 = (($q) + 8|0);
   $338 = HEAP32[$337>>2]|0;
   $339 = (_rmdir(($338|0))|0);
   $340 = (($q) + 72|0);
   HEAP32[$340>>2] = $339;
   break;
  }
  case 61:  {
   $341 = (($q) + 8|0);
   $342 = HEAP32[$341>>2]|0;
   $343 = (_unlink(($342|0))|0);
   $344 = (($q) + 72|0);
   HEAP32[$344>>2] = $343;
   break;
  }
  case 62:  {
   $345 = (($q) + 8|0);
   $346 = HEAP32[$345>>2]|0;
   $347 = (_ttyname(($346|0))|0);
   $348 = (($q) + 72|0);
   HEAP32[$348>>2] = $347;
   break;
  }
  case 63:  {
   $349 = (($q) + 8|0);
   $350 = HEAP32[$349>>2]|0;
   $351 = (($q) + 16|0);
   $352 = HEAP32[$351>>2]|0;
   $353 = (($q) + 24|0);
   $354 = HEAP32[$353>>2]|0;
   $355 = (_ttyname_r(($350|0),($352|0),($354|0))|0);
   $356 = (($q) + 72|0);
   HEAP32[$356>>2] = $355;
   break;
  }
  case 64:  {
   $357 = (($q) + 8|0);
   $358 = HEAP32[$357>>2]|0;
   $359 = (($q) + 16|0);
   $360 = HEAP32[$359>>2]|0;
   $361 = (_symlink(($358|0),($360|0))|0);
   $362 = (($q) + 72|0);
   HEAP32[$362>>2] = $361;
   break;
  }
  case 65:  {
   $363 = (($q) + 8|0);
   $364 = HEAP32[$363>>2]|0;
   $365 = (($q) + 16|0);
   $366 = HEAP32[$365>>2]|0;
   $367 = (($q) + 24|0);
   $368 = HEAP32[$367>>2]|0;
   $369 = (_readlink(($364|0),($366|0),($368|0))|0);
   $370 = (($q) + 72|0);
   HEAP32[$370>>2] = $369;
   break;
  }
  case 66:  {
   $371 = (($q) + 8|0);
   $372 = HEAP32[$371>>2]|0;
   $373 = (($q) + 16|0);
   $374 = HEAP32[$373>>2]|0;
   $375 = (($q) + 24|0);
   $376 = HEAP32[$375>>2]|0;
   $377 = (($q) + 32|0);
   $378 = HEAP32[$377>>2]|0;
   $379 = (_pwrite(($372|0),($374|0),($376|0),($378|0))|0);
   $380 = (($q) + 72|0);
   HEAP32[$380>>2] = $379;
   break;
  }
  case 67:  {
   $381 = (($q) + 8|0);
   $382 = HEAP32[$381>>2]|0;
   $383 = (($q) + 16|0);
   $384 = HEAP32[$383>>2]|0;
   $385 = (($q) + 24|0);
   $386 = HEAP32[$385>>2]|0;
   $387 = (_write(($382|0),($384|0),($386|0))|0);
   $388 = (($q) + 72|0);
   HEAP32[$388>>2] = $387;
   break;
  }
  case 68:  {
   $389 = (($q) + 8|0);
   $390 = HEAP32[$389>>2]|0;
   $391 = (($q) + 16|0);
   $392 = HEAP32[$391>>2]|0;
   $393 = (($q) + 24|0);
   $394 = HEAP32[$393>>2]|0;
   $395 = (_confstr(($390|0),($392|0),($394|0))|0);
   $396 = (($q) + 72|0);
   HEAP32[$396>>2] = $395;
   break;
  }
  case 69:  {
   $397 = (($q) + 8|0);
   $398 = HEAP32[$397>>2]|0;
   $399 = (($q) + 16|0);
   $400 = HEAP32[$399>>2]|0;
   $401 = (_gethostname(($398|0),($400|0))|0);
   $402 = (($q) + 72|0);
   HEAP32[$402>>2] = $401;
   break;
  }
  case 70:  {
   $403 = (_getlogin()|0);
   $404 = (($q) + 72|0);
   HEAP32[$404>>2] = $403;
   break;
  }
  case 131:  {
   $752 = (($q) + 8|0);
   $753 = HEAP32[$752>>2]|0;
   $754 = (($q) + 16|0);
   $755 = HEAP32[$754>>2]|0;
   $756 = (($q) + 24|0);
   $757 = HEAP32[$756>>2]|0;
   $758 = (_getpeername(($753|0),($755|0),($757|0))|0);
   $759 = (($q) + 72|0);
   HEAP32[$759>>2] = $758;
   break;
  }
  case 6:  {
   $29 = (($q) + 8|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = (_closedir(($30|0))|0);
   $32 = (($q) + 72|0);
   HEAP32[$32>>2] = $31;
   break;
  }
  case 134:  {
   $780 = (($q) + 8|0);
   $781 = HEAP32[$780>>2]|0;
   $782 = (($q) + 16|0);
   $783 = HEAP32[$782>>2]|0;
   $784 = (($q) + 24|0);
   $785 = HEAP32[$784>>2]|0;
   $786 = (($q) + 32|0);
   $787 = HEAP32[$786>>2]|0;
   $788 = (($q) + 40|0);
   $789 = HEAP32[$788>>2]|0;
   $790 = (($q) + 48|0);
   $791 = HEAP32[$790>>2]|0;
   $792 = (_sendto(($781|0),($783|0),($785|0),($787|0),($789|0),($791|0))|0);
   $793 = (($q) + 72|0);
   HEAP32[$793>>2] = $792;
   break;
  }
  case 135:  {
   $794 = (($q) + 8|0);
   $795 = HEAP32[$794>>2]|0;
   $796 = (($q) + 16|0);
   $797 = HEAP32[$796>>2]|0;
   $798 = (($q) + 24|0);
   $799 = HEAP32[$798>>2]|0;
   $800 = (($q) + 32|0);
   $801 = HEAP32[$800>>2]|0;
   $802 = (($q) + 40|0);
   $803 = HEAP32[$802>>2]|0;
   $804 = (($q) + 48|0);
   $805 = HEAP32[$804>>2]|0;
   $806 = (_recvfrom(($795|0),($797|0),($799|0),($801|0),($803|0),($805|0))|0);
   $807 = (($q) + 72|0);
   HEAP32[$807>>2] = $806;
   break;
  }
  case 136:  {
   $808 = (($q) + 8|0);
   $809 = HEAP32[$808>>2]|0;
   $810 = (($q) + 16|0);
   $811 = HEAP32[$810>>2]|0;
   $812 = (($q) + 24|0);
   $813 = HEAP32[$812>>2]|0;
   $814 = (($q) + 32|0);
   $815 = HEAP32[$814>>2]|0;
   $816 = (($q) + 40|0);
   $817 = HEAP32[$816>>2]|0;
   $818 = (_getsockopt(($809|0),($811|0),($813|0),($815|0),($817|0))|0);
   $819 = (($q) + 72|0);
   HEAP32[$819>>2] = $818;
   break;
  }
  case 8:  {
   $37 = (($q) + 8|0);
   $38 = HEAP32[$37>>2]|0;
   $39 = (($q) + 16|0);
   $40 = HEAP32[$39>>2]|0;
   _seekdir(($38|0),($40|0));
   break;
  }
  case 2:  {
   $7 = (($q) + 8|0);
   $8 = HEAP32[$7>>2]|0;
   $9 = (($q) + 16|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = (($q) + 24|0);
   $12 = HEAP32[$11>>2]|0;
   $13 = (_fgets(($8|0),($10|0),($12|0))|0);
   $14 = (($q) + 72|0);
   HEAP32[$14>>2] = $13;
   break;
  }
  case 54:  {
   $295 = (($q) + 8|0);
   $296 = HEAP32[$295>>2]|0;
   $297 = (($q) + 16|0);
   $298 = HEAP32[$297>>2]|0;
   $299 = (_link(($296|0),($298|0))|0);
   $300 = (($q) + 72|0);
   HEAP32[$300>>2] = $299;
   break;
  }
  case 5:  {
   $25 = (($q) + 8|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = (_opendir(($26|0))|0);
   $28 = (($q) + 72|0);
   HEAP32[$28>>2] = $27;
   break;
  }
  case 42:  {
   $233 = (($q) + 8|0);
   $234 = HEAP32[$233>>2]|0;
   $235 = (_fchdir(($234|0))|0);
   $236 = (($q) + 72|0);
   HEAP32[$236>>2] = $235;
   break;
  }
  case 43:  {
   $237 = (($q) + 8|0);
   $238 = HEAP32[$237>>2]|0;
   $239 = (_ctermid(($238|0))|0);
   $240 = (($q) + 72|0);
   HEAP32[$240>>2] = $239;
   break;
  }
  case 44:  {
   $241 = (($q) + 8|0);
   $242 = HEAP32[$241>>2]|0;
   $243 = (($q) + 16|0);
   $244 = HEAP32[$243>>2]|0;
   $245 = (_crypt(($242|0),($244|0))|0);
   $246 = (($q) + 72|0);
   HEAP32[$246>>2] = $245;
   break;
  }
  case 45:  {
   $247 = (($q) + 8|0);
   $248 = HEAP32[$247>>2]|0;
   $249 = (($q) + 16|0);
   $250 = HEAP32[$249>>2]|0;
   _encrypt(($248|0),($250|0));
   break;
  }
  case 46:  {
   $251 = (($q) + 8|0);
   $252 = HEAP32[$251>>2]|0;
   $253 = (($q) + 16|0);
   $254 = HEAP32[$253>>2]|0;
   $255 = (_fpathconf(($252|0),($254|0))|0);
   $256 = (($q) + 72|0);
   HEAP32[$256>>2] = $255;
   break;
  }
  case 47:  {
   $257 = (($q) + 8|0);
   $258 = HEAP32[$257>>2]|0;
   $259 = (_fsync(($258|0))|0);
   $260 = (($q) + 72|0);
   HEAP32[$260>>2] = $259;
   break;
  }
  case 48:  {
   $261 = (($q) + 8|0);
   $262 = HEAP32[$261>>2]|0;
   $263 = (($q) + 16|0);
   $264 = HEAP32[$263>>2]|0;
   $265 = (_truncate(($262|0),($264|0))|0);
   $266 = (($q) + 72|0);
   HEAP32[$266>>2] = $265;
   break;
  }
  case 49:  {
   $267 = (($q) + 8|0);
   $268 = HEAP32[$267>>2]|0;
   $269 = (($q) + 16|0);
   $270 = HEAP32[$269>>2]|0;
   $271 = (_ftruncate(($268|0),($270|0))|0);
   $272 = (($q) + 72|0);
   HEAP32[$272>>2] = $271;
   break;
  }
  case 50:  {
   $273 = (($q) + 8|0);
   $274 = HEAP32[$273>>2]|0;
   $275 = (($q) + 16|0);
   $276 = HEAP32[$275>>2]|0;
   $277 = (_getcwd(($274|0),($276|0))|0);
   $278 = (($q) + 72|0);
   HEAP32[$278>>2] = $277;
   break;
  }
  case 51:  {
   $279 = (($q) + 8|0);
   $280 = HEAP32[$279>>2]|0;
   $281 = (_getwd(($280|0))|0);
   $282 = (($q) + 72|0);
   HEAP32[$282>>2] = $281;
   break;
  }
  case 52:  {
   $283 = (($q) + 8|0);
   $284 = HEAP32[$283>>2]|0;
   $285 = (_isatty(($284|0))|0);
   $286 = (($q) + 72|0);
   HEAP32[$286>>2] = $285;
   break;
  }
  case 53:  {
   $287 = (($q) + 8|0);
   $288 = HEAP32[$287>>2]|0;
   $289 = (($q) + 16|0);
   $290 = HEAP32[$289>>2]|0;
   $291 = (($q) + 24|0);
   $292 = HEAP32[$291>>2]|0;
   $293 = (_lchown(($288|0),($290|0),($292|0))|0);
   $294 = (($q) + 72|0);
   HEAP32[$294>>2] = $293;
   break;
  }
  case 124:  {
   $697 = (($q) + 8|0);
   $698 = HEAP32[$697>>2]|0;
   $699 = (($q) + 16|0);
   $700 = HEAP32[$699>>2]|0;
   $701 = (_shutdown(($698|0),($700|0))|0);
   $702 = (($q) + 72|0);
   HEAP32[$702>>2] = $701;
   break;
  }
  case 125:  {
   $703 = (($q) + 8|0);
   $704 = HEAP32[$703>>2]|0;
   $705 = (($q) + 16|0);
   $706 = HEAP32[$705>>2]|0;
   $707 = (($q) + 24|0);
   $708 = HEAP32[$707>>2]|0;
   HEAP32[$vararg_buffer19>>2] = $708;
   $709 = (_ioctl(($704|0),($706|0),($vararg_buffer19|0))|0);
   $710 = (($q) + 72|0);
   HEAP32[$710>>2] = $709;
   break;
  }
  case 126:  {
   $711 = (($q) + 8|0);
   $712 = HEAP32[$711>>2]|0;
   $713 = (($q) + 16|0);
   $714 = HEAP32[$713>>2]|0;
   $715 = (($q) + 24|0);
   $716 = HEAP32[$715>>2]|0;
   $717 = (_accept(($712|0),($714|0),($716|0))|0);
   $718 = (($q) + 72|0);
   HEAP32[$718>>2] = $717;
   break;
  }
  case 127:  {
   $719 = (($q) + 8|0);
   $720 = HEAP32[$719>>2]|0;
   $721 = (($q) + 16|0);
   $722 = HEAP32[$721>>2]|0;
   $723 = (($q) + 24|0);
   $724 = HEAP32[$723>>2]|0;
   $725 = (($q) + 32|0);
   $726 = HEAP32[$725>>2]|0;
   $727 = (($q) + 40|0);
   $728 = HEAP32[$727>>2]|0;
   $729 = (_select(($720|0),($722|0),($724|0),($726|0),($728|0))|0);
   $730 = (($q) + 72|0);
   HEAP32[$730>>2] = $729;
   break;
  }
  case 128:  {
   $731 = (($q) + 8|0);
   $732 = HEAP32[$731>>2]|0;
   $733 = (($q) + 16|0);
   $734 = HEAP32[$733>>2]|0;
   $735 = $734;
   $736 = (_connect(($732|0),($734|0),($735|0))|0);
   $737 = (($q) + 72|0);
   HEAP32[$737>>2] = $736;
   break;
  }
  case 129:  {
   $738 = (($q) + 8|0);
   $739 = HEAP32[$738>>2]|0;
   $740 = (($q) + 16|0);
   $741 = HEAP32[$740>>2]|0;
   $742 = (_listen(($739|0),($741|0))|0);
   $743 = (($q) + 72|0);
   HEAP32[$743>>2] = $742;
   break;
  }
  case 130:  {
   $744 = (($q) + 8|0);
   $745 = HEAP32[$744>>2]|0;
   $746 = (($q) + 16|0);
   $747 = HEAP32[$746>>2]|0;
   $748 = (($q) + 24|0);
   $749 = HEAP32[$748>>2]|0;
   $750 = (_getsockname(($745|0),($747|0),($749|0))|0);
   $751 = (($q) + 72|0);
   HEAP32[$751>>2] = $750;
   break;
  }
  case 93:  {
   $521 = (($q) + 8|0);
   $522 = HEAP32[$521>>2]|0;
   $523 = (($q) + 16|0);
   $524 = HEAP32[$523>>2]|0;
   $525 = (_popen(($522|0),($524|0))|0);
   $526 = (($q) + 72|0);
   HEAP32[$526>>2] = $525;
   break;
  }
  case 94:  {
   $527 = (($q) + 8|0);
   $528 = HEAP32[$527>>2]|0;
   $529 = (_pclose(($528|0))|0);
   $530 = (($q) + 72|0);
   HEAP32[$530>>2] = $529;
   break;
  }
  case 95:  {
   $531 = (($q) + 8|0);
   $532 = HEAP32[$531>>2]|0;
   _perror(($532|0));
   break;
  }
  case 96:  {
   $533 = (($q) + 8|0);
   $534 = HEAP32[$533>>2]|0;
   $535 = (_remove(($534|0))|0);
   $536 = (($q) + 72|0);
   HEAP32[$536>>2] = $535;
   break;
  }
  case 97:  {
   $537 = (($q) + 8|0);
   $538 = HEAP32[$537>>2]|0;
   $539 = (($q) + 16|0);
   $540 = HEAP32[$539>>2]|0;
   $541 = (_rename(($538|0),($540|0))|0);
   $542 = (($q) + 72|0);
   HEAP32[$542>>2] = $541;
   break;
  }
  case 98:  {
   $543 = (($q) + 8|0);
   $544 = HEAP32[$543>>2]|0;
   _rewind(($544|0));
   break;
  }
  case 99:  {
   $545 = (($q) + 8|0);
   $546 = HEAP32[$545>>2]|0;
   $547 = (_tmpnam(($546|0))|0);
   $548 = (($q) + 72|0);
   HEAP32[$548>>2] = $547;
   break;
  }
  case 100:  {
   $549 = (($q) + 8|0);
   $550 = HEAP32[$549>>2]|0;
   $551 = (($q) + 16|0);
   $552 = HEAP32[$551>>2]|0;
   $553 = (_tempnam(($550|0),($552|0))|0);
   $554 = (($q) + 72|0);
   HEAP32[$554>>2] = $553;
   break;
  }
  case 101:  {
   $555 = (_tmpfile()|0);
   $556 = (($q) + 72|0);
   HEAP32[$556>>2] = $555;
   break;
  }
  case 102:  {
   $557 = (($q) + 8|0);
   $558 = HEAP32[$557>>2]|0;
   $559 = (($q) + 16|0);
   $560 = HEAP32[$559>>2]|0;
   $561 = (_ungetc(($558|0),($560|0))|0);
   $562 = (($q) + 72|0);
   HEAP32[$562>>2] = $561;
   break;
  }
  case 103:  {
   $563 = (($q) + 8|0);
   $564 = HEAP32[$563>>2]|0;
   $565 = (($q) + 16|0);
   $566 = HEAP32[$565>>2]|0;
   $567 = (($q) + 24|0);
   $568 = HEAP32[$567>>2]|0;
   HEAP32[$vararg_buffer4>>2] = $568;
   $569 = (_fscanf(($564|0),($566|0),($vararg_buffer4|0))|0);
   $570 = (($q) + 72|0);
   HEAP32[$570>>2] = $569;
   break;
  }
  case 104:  {
   $571 = (($q) + 8|0);
   $572 = HEAP32[$571>>2]|0;
   $573 = (($q) + 16|0);
   $574 = HEAP32[$573>>2]|0;
   HEAP32[$vararg_buffer7>>2] = $574;
   $575 = (_scanf(($572|0),($vararg_buffer7|0))|0);
   $576 = (($q) + 72|0);
   HEAP32[$576>>2] = $575;
   break;
  }
  case 4:  {
   $21 = (($q) + 8|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = (_fclose(($22|0))|0);
   $24 = (($q) + 72|0);
   HEAP32[$24>>2] = $23;
   break;
  }
  case 7:  {
   $33 = (($q) + 8|0);
   $34 = HEAP32[$33>>2]|0;
   $35 = (_telldir(($34|0))|0);
   $36 = (($q) + 72|0);
   HEAP32[$36>>2] = $35;
   break;
  }
  case 1:  {
   $1 = (($q) + 8|0);
   $2 = HEAP32[$1>>2]|0;
   $3 = (($q) + 16|0);
   $4 = HEAP32[$3>>2]|0;
   $5 = (_fopen(($2|0),($4|0))|0);
   $6 = (($q) + 72|0);
   HEAP32[$6>>2] = $5;
   break;
  }
  case 3:  {
   $15 = (($q) + 8|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = (($q) + 16|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = (_fputs(($16|0),($18|0))|0);
   $20 = (($q) + 72|0);
   HEAP32[$20>>2] = $19;
   break;
  }
  case 33:  {
   $181 = (($q) + 8|0);
   $182 = HEAP32[$181>>2]|0;
   $183 = (($q) + 16|0);
   $184 = HEAP32[$183>>2]|0;
   $185 = (($q) + 24|0);
   $186 = HEAP32[$185>>2]|0;
   $187 = (_poll(($182|0),($184|0),($186|0))|0);
   $188 = (($q) + 72|0);
   HEAP32[$188>>2] = $187;
   break;
  }
  case 34:  {
   $189 = (($q) + 8|0);
   $190 = HEAP32[$189>>2]|0;
   $191 = (($q) + 16|0);
   $192 = HEAP32[$191>>2]|0;
   $193 = (_access(($190|0),($192|0))|0);
   $194 = (($q) + 72|0);
   HEAP32[$194>>2] = $193;
   break;
  }
  case 35:  {
   $195 = (($q) + 8|0);
   $196 = HEAP32[$195>>2]|0;
   $197 = (_chdir(($196|0))|0);
   $198 = (($q) + 72|0);
   HEAP32[$198>>2] = $197;
   break;
  }
  case 36:  {
   $199 = (($q) + 8|0);
   $200 = HEAP32[$199>>2]|0;
   $201 = (($q) + 16|0);
   $202 = HEAP32[$201>>2]|0;
   $203 = (($q) + 24|0);
   $204 = HEAP32[$203>>2]|0;
   $205 = (_chown(($200|0),($202|0),($204|0))|0);
   $206 = (($q) + 72|0);
   HEAP32[$206>>2] = $205;
   break;
  }
  case 37:  {
   $207 = (($q) + 8|0);
   $208 = HEAP32[$207>>2]|0;
   $209 = (_chroot(($208|0))|0);
   $210 = (($q) + 72|0);
   HEAP32[$210>>2] = $209;
   break;
  }
  case 38:  {
   $211 = (($q) + 8|0);
   $212 = HEAP32[$211>>2]|0;
   $213 = (_close(($212|0))|0);
   $214 = (($q) + 72|0);
   HEAP32[$214>>2] = $213;
   break;
  }
  case 39:  {
   $215 = (($q) + 8|0);
   $216 = HEAP32[$215>>2]|0;
   $217 = (_dup(($216|0))|0);
   $218 = (($q) + 72|0);
   HEAP32[$218>>2] = $217;
   break;
  }
  case 40:  {
   $219 = (($q) + 8|0);
   $220 = HEAP32[$219>>2]|0;
   $221 = (($q) + 16|0);
   $222 = HEAP32[$221>>2]|0;
   $223 = (_dup2(($220|0),($222|0))|0);
   $224 = (($q) + 72|0);
   HEAP32[$224>>2] = $223;
   break;
  }
  case 41:  {
   $225 = (($q) + 8|0);
   $226 = HEAP32[$225>>2]|0;
   $227 = (($q) + 16|0);
   $228 = HEAP32[$227>>2]|0;
   $229 = (($q) + 24|0);
   $230 = HEAP32[$229>>2]|0;
   $231 = (_fchown(($226|0),($228|0),($230|0))|0);
   $232 = (($q) + 72|0);
   HEAP32[$232>>2] = $231;
   break;
  }
  case 132:  {
   $760 = (($q) + 8|0);
   $761 = HEAP32[$760>>2]|0;
   $762 = (($q) + 16|0);
   $763 = HEAP32[$762>>2]|0;
   $764 = (($q) + 24|0);
   $765 = HEAP32[$764>>2]|0;
   $766 = (($q) + 32|0);
   $767 = HEAP32[$766>>2]|0;
   $768 = (_send(($761|0),($763|0),($765|0),($767|0))|0);
   $769 = (($q) + 72|0);
   HEAP32[$769>>2] = $768;
   break;
  }
  case 133:  {
   $770 = (($q) + 8|0);
   $771 = HEAP32[$770>>2]|0;
   $772 = (($q) + 16|0);
   $773 = HEAP32[$772>>2]|0;
   $774 = (($q) + 24|0);
   $775 = HEAP32[$774>>2]|0;
   $776 = (($q) + 32|0);
   $777 = HEAP32[$776>>2]|0;
   $778 = (_recv(($771|0),($773|0),($775|0),($777|0))|0);
   $779 = (($q) + 72|0);
   HEAP32[$779>>2] = $778;
   break;
  }
  default: {
   ___assert_fail((1800|0),(840|0),374,(1856|0));
   // unreachable;
  }
  }
 } while(0);
 $820 = (($q) + 4|0);
 HEAP32[$820>>2] = 1;
 (_emscripten_futex_wake(($820|0),2147483647)|0);
 STACKTOP = sp;return;
}
function runPostSets() {
 
}
function _strlen(ptr) {
    ptr = ptr|0;
    var curr = 0;
    curr = ptr;
    while (((HEAP8[((curr)>>0)])|0)) {
      curr = (curr + 1)|0;
    }
    return (curr - ptr)|0;
}
function _memcpy(dest, src, num) {

    dest = dest|0; src = src|0; num = num|0;
    var ret = 0;
    if ((num|0) >= 4096) return _emscripten_memcpy_big(dest|0, src|0, num|0)|0;
    ret = dest|0;
    if ((dest&3) == (src&3)) {
      while (dest & 3) {
        if ((num|0) == 0) return ret|0;
        HEAP8[((dest)>>0)]=((HEAP8[((src)>>0)])|0);
        dest = (dest+1)|0;
        src = (src+1)|0;
        num = (num-1)|0;
      }
      while ((num|0) >= 4) {
        HEAP32[((dest)>>2)]=((HEAP32[((src)>>2)])|0);
        dest = (dest+4)|0;
        src = (src+4)|0;
        num = (num-4)|0;
      }
    }
    while ((num|0) > 0) {
      HEAP8[((dest)>>0)]=((HEAP8[((src)>>0)])|0);
      dest = (dest+1)|0;
      src = (src+1)|0;
      num = (num-1)|0;
    }
    return ret|0;
}
function _memset(ptr, value, num) {
    ptr = ptr|0; value = value|0; num = num|0;
    var stop = 0, value4 = 0, stop4 = 0, unaligned = 0;
    stop = (ptr + num)|0;
    if ((num|0) >= 20) {
      // This is unaligned, but quite large, so work hard to get to aligned settings
      value = value & 0xff;
      unaligned = ptr & 3;
      value4 = value | (value << 8) | (value << 16) | (value << 24);
      stop4 = stop & ~3;
      if (unaligned) {
        unaligned = (ptr + 4 - unaligned)|0;
        while ((ptr|0) < (unaligned|0)) { // no need to check for stop, since we have large num
          HEAP8[((ptr)>>0)]=value;
          ptr = (ptr+1)|0;
        }
      }
      while ((ptr|0) < (stop4|0)) {
        HEAP32[((ptr)>>2)]=value4;
        ptr = (ptr+4)|0;
      }
    }
    while ((ptr|0) < (stop|0)) {
      HEAP8[((ptr)>>0)]=value;
      ptr = (ptr+1)|0;
    }
    return (ptr-num)|0;
}
function _strcpy(pdest, psrc) {
    pdest = pdest|0; psrc = psrc|0;
    var i = 0;
    do {
      HEAP8[(((pdest+i)|0)>>0)]=HEAP8[(((psrc+i)|0)>>0)];
      i = (i+1)|0;
    } while (((HEAP8[(((psrc)+(i-1))>>0)])|0));
    return pdest|0;
}

  
function dynCall_ii(index,a1) {
  index = index|0;
  a1=a1|0;
  return FUNCTION_TABLE_ii[index&1](a1|0)|0;
}


function dynCall_vi(index,a1) {
  index = index|0;
  a1=a1|0;
  FUNCTION_TABLE_vi[index&0](a1|0);
}


function dynCall_v(index) {
  index = index|0;
  
  FUNCTION_TABLE_v[index&3]();
}

function b0(p0) { p0 = p0|0; nullFunc_ii(0);return 0; }
function b1(p0) { p0 = p0|0; nullFunc_vi(1); }
function b2() { ; nullFunc_v(2); }

// EMSCRIPTEN_END_FUNCS
var FUNCTION_TABLE_ii = [b0,__Z10ThreadMainPv];
var FUNCTION_TABLE_vi = [b1];
var FUNCTION_TABLE_v = [b2,b2,__Z10WaitToJoinv,b2];

  return { _emscripten_main_thread_process_queued_calls: _emscripten_main_thread_process_queued_calls, _strcpy: _strcpy, _free: _free, _main: _main, __Atomics_load_HEAPF32_emulated: __Atomics_load_HEAPF32_emulated, __Atomics_store_HEAPF64_emulated: __Atomics_store_HEAPF64_emulated, ___pthread_tsd_run_dtors: ___pthread_tsd_run_dtors, _strlen: _strlen, __Atomics_store_HEAPF32_emulated: __Atomics_store_HEAPF32_emulated, _memset: _memset, _malloc: _malloc, _memcpy: _memcpy, __Atomics_load_HEAPF64_emulated: __Atomics_load_HEAPF64_emulated, _emscripten_sync_run_in_main_thread: _emscripten_sync_run_in_main_thread, _emscripten_sync_run_in_main_thread_4: _emscripten_sync_run_in_main_thread_4, _emscripten_sync_run_in_main_thread_5: _emscripten_sync_run_in_main_thread_5, _emscripten_sync_run_in_main_thread_6: _emscripten_sync_run_in_main_thread_6, _emscripten_sync_run_in_main_thread_1: _emscripten_sync_run_in_main_thread_1, _emscripten_sync_run_in_main_thread_2: _emscripten_sync_run_in_main_thread_2, _emscripten_sync_run_in_main_thread_3: _emscripten_sync_run_in_main_thread_3, runPostSets: runPostSets, stackAlloc: stackAlloc, stackSave: stackSave, stackRestore: stackRestore, establishStackSpace: establishStackSpace, setThrew: setThrew, setTempRet0: setTempRet0, getTempRet0: getTempRet0, dynCall_ii: dynCall_ii, dynCall_vi: dynCall_vi, dynCall_v: dynCall_v };
})
// EMSCRIPTEN_END_ASM
(Module.asmGlobalArg, Module.asmLibraryArg, buffer);
var real__emscripten_main_thread_process_queued_calls = asm["_emscripten_main_thread_process_queued_calls"]; asm["_emscripten_main_thread_process_queued_calls"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real__emscripten_main_thread_process_queued_calls.apply(null, arguments);
};

var real__strcpy = asm["_strcpy"]; asm["_strcpy"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real__strcpy.apply(null, arguments);
};

var real__main = asm["_main"]; asm["_main"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real__main.apply(null, arguments);
};

var real___Atomics_load_HEAPF32_emulated = asm["__Atomics_load_HEAPF32_emulated"]; asm["__Atomics_load_HEAPF32_emulated"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real___Atomics_load_HEAPF32_emulated.apply(null, arguments);
};

var real___Atomics_store_HEAPF64_emulated = asm["__Atomics_store_HEAPF64_emulated"]; asm["__Atomics_store_HEAPF64_emulated"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real___Atomics_store_HEAPF64_emulated.apply(null, arguments);
};

var real____pthread_tsd_run_dtors = asm["___pthread_tsd_run_dtors"]; asm["___pthread_tsd_run_dtors"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real____pthread_tsd_run_dtors.apply(null, arguments);
};

var real__strlen = asm["_strlen"]; asm["_strlen"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real__strlen.apply(null, arguments);
};

var real___Atomics_store_HEAPF32_emulated = asm["__Atomics_store_HEAPF32_emulated"]; asm["__Atomics_store_HEAPF32_emulated"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real___Atomics_store_HEAPF32_emulated.apply(null, arguments);
};

var real___Atomics_load_HEAPF64_emulated = asm["__Atomics_load_HEAPF64_emulated"]; asm["__Atomics_load_HEAPF64_emulated"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real___Atomics_load_HEAPF64_emulated.apply(null, arguments);
};

var real__emscripten_sync_run_in_main_thread = asm["_emscripten_sync_run_in_main_thread"]; asm["_emscripten_sync_run_in_main_thread"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real__emscripten_sync_run_in_main_thread.apply(null, arguments);
};

var real__emscripten_sync_run_in_main_thread_4 = asm["_emscripten_sync_run_in_main_thread_4"]; asm["_emscripten_sync_run_in_main_thread_4"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real__emscripten_sync_run_in_main_thread_4.apply(null, arguments);
};

var real__emscripten_sync_run_in_main_thread_5 = asm["_emscripten_sync_run_in_main_thread_5"]; asm["_emscripten_sync_run_in_main_thread_5"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real__emscripten_sync_run_in_main_thread_5.apply(null, arguments);
};

var real__emscripten_sync_run_in_main_thread_6 = asm["_emscripten_sync_run_in_main_thread_6"]; asm["_emscripten_sync_run_in_main_thread_6"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real__emscripten_sync_run_in_main_thread_6.apply(null, arguments);
};

var real__emscripten_sync_run_in_main_thread_1 = asm["_emscripten_sync_run_in_main_thread_1"]; asm["_emscripten_sync_run_in_main_thread_1"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real__emscripten_sync_run_in_main_thread_1.apply(null, arguments);
};

var real__emscripten_sync_run_in_main_thread_2 = asm["_emscripten_sync_run_in_main_thread_2"]; asm["_emscripten_sync_run_in_main_thread_2"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real__emscripten_sync_run_in_main_thread_2.apply(null, arguments);
};

var real__emscripten_sync_run_in_main_thread_3 = asm["_emscripten_sync_run_in_main_thread_3"]; asm["_emscripten_sync_run_in_main_thread_3"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real__emscripten_sync_run_in_main_thread_3.apply(null, arguments);
};

var real_runPostSets = asm["runPostSets"]; asm["runPostSets"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real_runPostSets.apply(null, arguments);
};
var _emscripten_main_thread_process_queued_calls = Module["_emscripten_main_thread_process_queued_calls"] = asm["_emscripten_main_thread_process_queued_calls"];
var _strcpy = Module["_strcpy"] = asm["_strcpy"];
var _free = Module["_free"] = asm["_free"];
var _main = Module["_main"] = asm["_main"];
var __Atomics_load_HEAPF32_emulated = Module["__Atomics_load_HEAPF32_emulated"] = asm["__Atomics_load_HEAPF32_emulated"];
var __Atomics_store_HEAPF64_emulated = Module["__Atomics_store_HEAPF64_emulated"] = asm["__Atomics_store_HEAPF64_emulated"];
var ___pthread_tsd_run_dtors = Module["___pthread_tsd_run_dtors"] = asm["___pthread_tsd_run_dtors"];
var _strlen = Module["_strlen"] = asm["_strlen"];
var __Atomics_store_HEAPF32_emulated = Module["__Atomics_store_HEAPF32_emulated"] = asm["__Atomics_store_HEAPF32_emulated"];
var _memset = Module["_memset"] = asm["_memset"];
var _malloc = Module["_malloc"] = asm["_malloc"];
var _memcpy = Module["_memcpy"] = asm["_memcpy"];
var __Atomics_load_HEAPF64_emulated = Module["__Atomics_load_HEAPF64_emulated"] = asm["__Atomics_load_HEAPF64_emulated"];
var _emscripten_sync_run_in_main_thread = Module["_emscripten_sync_run_in_main_thread"] = asm["_emscripten_sync_run_in_main_thread"];
var _emscripten_sync_run_in_main_thread_4 = Module["_emscripten_sync_run_in_main_thread_4"] = asm["_emscripten_sync_run_in_main_thread_4"];
var _emscripten_sync_run_in_main_thread_5 = Module["_emscripten_sync_run_in_main_thread_5"] = asm["_emscripten_sync_run_in_main_thread_5"];
var _emscripten_sync_run_in_main_thread_6 = Module["_emscripten_sync_run_in_main_thread_6"] = asm["_emscripten_sync_run_in_main_thread_6"];
var _emscripten_sync_run_in_main_thread_1 = Module["_emscripten_sync_run_in_main_thread_1"] = asm["_emscripten_sync_run_in_main_thread_1"];
var _emscripten_sync_run_in_main_thread_2 = Module["_emscripten_sync_run_in_main_thread_2"] = asm["_emscripten_sync_run_in_main_thread_2"];
var _emscripten_sync_run_in_main_thread_3 = Module["_emscripten_sync_run_in_main_thread_3"] = asm["_emscripten_sync_run_in_main_thread_3"];
var runPostSets = Module["runPostSets"] = asm["runPostSets"];
var dynCall_ii = Module["dynCall_ii"] = asm["dynCall_ii"];
var dynCall_vi = Module["dynCall_vi"] = asm["dynCall_vi"];
var dynCall_v = Module["dynCall_v"] = asm["dynCall_v"];

Runtime.stackAlloc = asm['stackAlloc'];
Runtime.stackSave = asm['stackSave'];
Runtime.stackRestore = asm['stackRestore'];
Runtime.establishStackSpace = asm['establishStackSpace'];
Runtime.setTempRet0 = asm['setTempRet0'];
Runtime.getTempRet0 = asm['getTempRet0'];


// Warning: printing of i64 values may be slightly rounded! No deep i64 math used, so precise i64 code not included
var i64Math = null;

// === Auto-generated postamble setup entry stuff ===

if (memoryInitializer && !ENVIRONMENT_IS_PTHREAD) {
  if (typeof Module['locateFile'] === 'function') {
    memoryInitializer = Module['locateFile'](memoryInitializer);
  } else if (Module['memoryInitializerPrefixURL']) {
    memoryInitializer = Module['memoryInitializerPrefixURL'] + memoryInitializer;
  }
  if (ENVIRONMENT_IS_NODE || ENVIRONMENT_IS_SHELL) {
    var data = Module['readBinary'](memoryInitializer);
    HEAPU8.set(data, STATIC_BASE);
  } else {
    addRunDependency('memory initializer');
    Browser.asyncLoad(memoryInitializer, function(data) {
      for (var i = 0; i < data.length; i++) {
        assert(HEAPU8[STATIC_BASE + i] === 0, "area for memory initializer should not have been touched before it's loaded");
      }
      HEAPU8.set(data, STATIC_BASE);
      removeRunDependency('memory initializer');
    }, function(data) {
      throw 'could not load memory initializer ' + memoryInitializer;
    });
  }
}

function ExitStatus(status) {
  this.name = "ExitStatus";
  this.message = "Program terminated with exit(" + status + ")";
  this.status = status;
};
ExitStatus.prototype = new Error();
ExitStatus.prototype.constructor = ExitStatus;

var initialStackTop;
var preloadStartTime = null;
var calledMain = false;

dependenciesFulfilled = function runCaller() {
  // If run has never been called, and we should call run (INVOKE_RUN is true, and Module.noInitialRun is not false)
  if (!Module['calledRun']) run();
  if (!Module['calledRun']) dependenciesFulfilled = runCaller; // try this again later, after new deps are fulfilled
}

Module['callMain'] = Module.callMain = function callMain(args) {
  assert(runDependencies == 0, 'cannot call main when async dependencies remain! (listen on __ATMAIN__)');
  assert(__ATPRERUN__.length == 0, 'cannot call main when preRun functions remain to be called');

  args = args || [];

  ensureInitRuntime();

  var argc = args.length+1;
  function pad() {
    for (var i = 0; i < 4-1; i++) {
      argv.push(0);
    }
  }
  var argv = [allocate(intArrayFromString(Module['thisProgram']), 'i8', ALLOC_NORMAL) ];
  pad();
  for (var i = 0; i < argc-1; i = i + 1) {
    argv.push(allocate(intArrayFromString(args[i]), 'i8', ALLOC_NORMAL));
    pad();
  }
  argv.push(0);
  argv = allocate(argv, 'i32', ALLOC_NORMAL);

  initialStackTop = STACKTOP;

  try {

    var ret = Module['_main'](argc, argv, 0);


    // if we're not running an evented main loop, it's time to exit
    exit(ret);
  }
  catch(e) {
    if (e instanceof ExitStatus) {
      // exit() throws this once it's done to make sure execution
      // has been stopped completely
      return;
    } else if (e == 'SimulateInfiniteLoop') {
      // running an evented main loop, don't immediately exit
      Module['noExitRuntime'] = true;
      return;
    } else {
      if (e && typeof e === 'object' && e.stack) Module.printErr('exception thrown: ' + [e, e.stack]);
      throw e;
    }
  } finally {
    calledMain = true;
  }
}




function run(args) {
  args = args || Module['arguments'];

  if (preloadStartTime === null) preloadStartTime = Date.now();

  if (runDependencies > 0) {
    Module.printErr('run() called, but dependencies remain, so not running');
    return;
  }

  preRun();

  if (runDependencies > 0) return; // a preRun added a dependency, run will be called later
  if (Module['calledRun']) return; // run may have just been called through dependencies being fulfilled just in this very frame

  function doRun() {
    if (Module['calledRun']) return; // run may have just been called while the async setStatus time below was happening
    Module['calledRun'] = true;

    if (ABORT) return; 

    ensureInitRuntime();

    preMain();

    if (ENVIRONMENT_IS_WEB && preloadStartTime !== null) {
      Module.printErr('pre-main prep time: ' + (Date.now() - preloadStartTime) + ' ms');
    }

    if (Module['onRuntimeInitialized']) Module['onRuntimeInitialized']();

    if (Module['_main'] && shouldRunNow) Module['callMain'](args);

    postRun();
  }

  if (Module['setStatus']) {
    Module['setStatus']('Running...');
    setTimeout(function() {
      setTimeout(function() {
        Module['setStatus']('');
      }, 1);
      doRun();
    }, 1);
  } else {
    doRun();
  }
}
Module['run'] = Module.run = run;

function exit(status) {
  if (Module['noExitRuntime']) {
    Module.printErr('exit(' + status + ') called, but noExitRuntime, so not exiting (you can use emscripten_force_exit, if you want to force a true shutdown)');
    return;
  }

  ABORT = true;
  EXITSTATUS = status;
  STACKTOP = initialStackTop;

  // exit the runtime
  exitRuntime();

  if (ENVIRONMENT_IS_NODE) {
    // Work around a node.js bug where stdout buffer is not flushed at process exit:
    // Instead of process.exit() directly, wait for stdout flush event.
    // See https://github.com/joyent/node/issues/1669 and https://github.com/kripken/emscripten/issues/2582
    // Workaround is based on https://github.com/RReverser/acorn/commit/50ab143cecc9ed71a2d66f78b4aec3bb2e9844f6
    process['stdout']['once']('drain', function () {
      process['exit'](status);
    });
    console.log(' '); // Make sure to print something to force the drain event to occur, in case the stdout buffer was empty.
    // Work around another node bug where sometimes 'drain' is never fired - make another effort
    // to emit the exit status, after a significant delay (if node hasn't fired drain by then, give up)
    setTimeout(function() {
      process['exit'](status);
    }, 500);
  } else
  if (ENVIRONMENT_IS_SHELL && typeof quit === 'function') {
    quit(status);
  }
  // if we reach here, we must throw an exception to halt the current execution
  throw new ExitStatus(status);
}
Module['exit'] = Module.exit = exit;

var abortDecorators = [];

function abort(what) {
  if (what !== undefined) {
    Module.print(what);
    Module.printErr(what);
    what = JSON.stringify(what)
  } else {
    what = '';
  }

  ABORT = true;
  EXITSTATUS = 1;

  var extra = '';

  var output = 'abort(' + what + ') at ' + stackTrace() + extra;
  abortDecorators.forEach(function(decorator) {
    output = decorator(output, what);
  });
  throw output;
}
Module['abort'] = Module.abort = abort;

// {{PRE_RUN_ADDITIONS}}

if (Module['preInit']) {
  if (typeof Module['preInit'] == 'function') Module['preInit'] = [Module['preInit']];
  while (Module['preInit'].length > 0) {
    Module['preInit'].pop()();
  }
}

// shouldRunNow refers to calling main(), not run().
var shouldRunNow = true;
if (Module['noInitialRun']) {
  shouldRunNow = false;
}


if (!ENVIRONMENT_IS_PTHREAD) run();

// {{POST_RUN_ADDITIONS}}






// {{MODULE_ADDITIONS}}



